
% Script for ADC analysis. Calculates ENOB.
% 2011 Even Brenden

clear all
close all
clf

% Load files.
all_sines_mode_1 = wavread('prime_sines_14k_8bit.wav');
all_sines_mode_2 = wavread('prime_sines_14k_9bit.wav');
all_sines_mode_3 = wavread('prime_sines_14k_10bit.wav');


% Sine frequency bins.
% f_mode = [99 297 891 1089 2673 3267 5049 6831];
f = [495 1482 4445 5432 13332 16295 25183 34070];
secs=5;

no_sines=8; % Number of sines.
lobes=30; % Side lobes compensation.
dc=8;	% DC bins

sinad_mode_1 = zeros(1, no_sines);
sinad_mode_2 = zeros(1, no_sines);
sinad_mode_3 = zeros(1, no_sines);

for mode=1:3

	fs = 14000;
	M = secs*fs;
	
	if mode==1
		all_sines = all_sines_mode_1;
	elseif mode==2
		all_sines = all_sines_mode_2;
	else
		all_sines = all_sines_mode_3;
	end
	
	% disp(strcat('mode ', num2str(mode)));

	for i=1:8
		
		% Extract current sine.
		signal = all_sines((i-1)*2*M+1:(i-1)*2*M+M).*hann(M);
		
		% M-point FFT.
		signal_ft = abs(fft(signal, M))(1:M/2);
		
		% Extract bins from sines and noise+distortion.
		sine_bins = zeros(1, M/2);
		sine_bins(max(dc, f(i)-lobes):min(f(i)+lobes, M/2)) = signal_ft(max(dc, f(i)-lobes):min(f(i)+lobes, M/2));
		noise_bins = signal_ft;
		noise_bins(1:dc)=0;
		noise_bins(max(1, f(i)-lobes):min(f(i)+lobes, M/2))=0;

		% Calculate powers.
		sine_p =  sum(sine_bins.^2);
		noise_p = sum(noise_bins.^2);
		
		% Plotting.
		plot_sig=0;
		if(plot_sig)
			subplot(3, 1, mode);
			plot(20*log10(noise_bins));
			hold on;
			plot(20*log10(sine_bins), 'r');
			title(num2str(mode));
		end

		% Calculate SINAD.
		if mode==1
			sinad_mode_1(i) = sine_p/noise_p;
		elseif mode==2
			sinad_mode_2(i) = sine_p/noise_p;
		else
			sinad_mode_3(i) = sine_p/noise_p;
		end
	end


end

% Display ENOB results.
disp('ENOB');
ENOB(sinad_mode_1)
ENOB(sinad_mode_2)
ENOB(sinad_mode_3)

% Plotting ENOB and SINAD versus sine frequencies.
yy_draw=1;
if(yy_draw)

	close all
	figure
	[AX,H1,H2] = plotyy(f, SINADdb(sinad_mode_1), f, ENOB(sinad_mode_1), 'semilogx');
	set(get(AX(1), 'Ylabel'), 'String', 'SINAD (dB)');
	set(get(AX(2), 'Ylabel'), 'String', 'ENOB (bits)');
	xlabel('Hz');

	delete(H1)
	delete(H2)

	set(AX,'NextPlot','add')
	plot(AX(1), f, SINADdb(sinad_mode_1), '-*k');
	plot(AX(1), f, SINADdb(sinad_mode_2), '-^k');
	plot(AX(1), f, SINADdb(sinad_mode_3), '-ok');

	legend(AX(1), 'Mode 1 (B_A=8 bits)', 'Mode 2 (B_A=9 bits)', 'Mode 3 (B_A=10 bits)');
	legend(AX(1), 'boxon');

	print 'sinad_enob.eps' '-deps'
end