
% Calculate and draw PSD estimation from ADC test results. Both silence and sine tests are plotted.


% Extract silence part from ADC test results.
silence_mode_1 = wavread('prime_sines_14k_8bit.wav')(80001:140000);
silence_mode_2 = wavread('prime_sines_14k_9bit.wav')(80001:140000);
silence_mode_3 = wavread('prime_sines_14k_10bit.wav')(80001:140000);

M=length(silence_mode_1);

% Estimate PSD.
sm1 = spectral_xdf(silence_mode_1);
sm2 = spectral_xdf(silence_mode_2);
sm3 = spectral_xdf(silence_mode_3);

sm1(:, 2)=10*log10(sm1(:, 2));
sm2(:, 2)=10*log10(sm2(:, 2));
sm3(:, 2)=10*log10(sm3(:, 2));

close all
n=0:1/M:0.5-1/M;

figure(1)
plot(sm1(1:M/2, 1), sm1(1:M/2, 2), 'b')
hold on;
plot(sm1(1:M/2, 1), sm2(1:M/2, 2), 'r')
plot(sm1(1:M/2, 1), sm3(1:M/2, 2), 'k')
ylabel('P_x(k) [dB]');
xlabel('k (normalized to [0, 0.5])');
%axis([0 0.5 0 5e-005]);
text(0.09, -50, '(peaks)');
text(0.09, -57, '(peaks)');
legend('Mode 1 (B_A=8 bits)', 'Mode 2 (B_A=9 bits)', 'Mode 3 (B_A=10 bits)');
legend('boxon');

print 'adc_noise_psd.eps' '-deps';


% Extract sine part from ADC test results.
silence_mode_1 = wavread('n+d_14k_8bit.wav');
silence_mode_2 = wavread('n+d_14k_9bit.wav');
silence_mode_3 = wavread('n+d_14k_10bit.wav');

M=length(silence_mode_1);

% Estimate PSD.
sm1 = spectral_xdf(silence_mode_1);
sm2 = spectral_xdf(silence_mode_2);
sm3 = spectral_xdf(silence_mode_3);

sm1(:, 2)=10*log10(sm1(:, 2));
sm2(:, 2)=10*log10(sm2(:, 2));
sm3(:, 2)=10*log10(sm3(:, 2));

n=0:1/M:0.5-1/M;

figure(2)
plot(sm1(1:M/2, 1), sm1(1:M/2, 2), 'b')
hold on;
plot(sm1(1:M/2, 1), sm2(1:M/2, 2), 'r')
plot(sm1(1:M/2, 1), sm3(1:M/2, 2), 'k')
ylabel('P_x(k) [dB]');
xlabel('k (normalized to [0, 0.5])');
axis([0 0.5 -65 -35]);
text(0.025, -36, 'fundamental frequency f of sine');
text(0.3, -42, '(peaks)');
legend('Mode 1 (B_A=8 bits)', 'Mode 2 (B_A=9 bits)', 'Mode 3 (B_A=10 bits)');
legend('boxon');

print 'adc_sine_psd.eps' '-deps';