
% Noise gate/expander. Attenuation is attenuation factor for frames under tresh, att_rel is attack and release (on frame basis) in samples.
% 2011 Even Brenden

function [y, history] = expander(x, tresh, attenuation, history, F);

	N = length(x);

	% Determine slope sign
	former_history=history;

	avg = sum(abs(x))/N;
	if avg<tresh
		history=min(F, history+1);
	else
		history=max(0, history-1);
	end
	if history % Linear attack/release slope

		current_attenuation = attenuation+(1-history/F)*(1-attenuation);
		former_attenuation = attenuation+(1-former_history/F)*(1-attenuation);

		slope_sign = history-former_history;

		if slope_sign!=0 % Linearly interpolate
			for n=1:N
				x(n) *= ((N-n+1)/N)*former_attenuation + (n/N)*current_attenuation;
			end
		else % Max/flat
			x *= current_attenuation;
		end
	end

	y=x;
