
% Midtread quantizer
%
% -Range: [-a, a), assuming no DC bias
% -Scale: 0 if function should find scale itself
%
% 2011 Even Brenden

function [output, scale] = midrise_q(input, bits, scale)

	% Find scale
	if nargin < 3
		maxi = max(abs(input));
	else
		maxi = 1/scale;
	end

	% Drop the upper and lower [drop]% of the dynamic range of the signal when quantizing.
	drop = 0.01;

	% Find log of maximal value to adapt Q to dynamic range.
	sbits = floor(-log2(maxi*(1-drop)));
	
	% Quantize by rounding
	output = zeros(1, length(input));
	shift = 16-bits-sbits+1;
	for n=1:length(input)

		integer = floor(input(n)*2^16);		% Emulating 16-bit precision with sign (i.e. 17 bit) for fractional part of Q16.
		div = floor(integer*2^-(shift));	% Cut precision by shifting.
		qed = div*2^(shift);			% Shift up again with loss in precision.
		output(n) = qed/2^16;			% Back to float.
	
	end
end
