#ifndef _NRF51502_OVERLAY_H
#define _NRF51502_OVERLAY_H

#include "nrf.h"


typedef uint32_t PPITask_t;         /*!< PPI task type  */
typedef uint32_t PPIEvent_t;        /*!< PPI event type */


/**
 * PPI channel
 */
typedef struct
{
  __IO PPIEvent_t * EEP;  ///< Event end-point
  __IO PPITask_t * TEP;   ///< Task end-point
}PPIChannel_Type;


/**
 * PPI channel types
 */
typedef struct
{
  __IO uint8_t RFU[0x500];            /*!< Reserved  */
   PPIChannel_Type CH[8];             /*!< PPI channel 0-7  */
}PPI_Type;

/* Radio */

/**
 * nRF5 Transceiver SHORTCUTS
 */
typedef enum 
{
  RADIO_SHORT_READY_START_IDn   = RADIO_SHORTS_READY_START_Pos,    /*!< Shortcut between ready event and start task */
  RADIO_SHORT_END_DISABLE_IDn   = RADIO_SHORTS_END_DISABLE_Pos,    /*!< Shortcut between end event and enable task */
  RADIO_SHORT_DISABLED_TXEN_IDn = RADIO_SHORTS_DISABLED_TXEN_Pos,  /*!< Shortcut between end event and enable task */
  RADIO_SHORT_DISABLED_RXEN_IDn = RADIO_SHORTS_DISABLED_RXEN_Pos   /*!< Shortcut between end event and disable task */
}RADIO_SHORTS_IDn_Type;

/**
 * nRF5 Transceiver datarate
 */
typedef enum
{
  RADIO_DATARATE_1MBPS_S   = RADIO_MODE_MODE_1MbitFSK,   /*!<  1Mbps data rate (ShockBurst modulation) */
  RADIO_DATARATE_2MBPS_S   = RADIO_MODE_MODE_2MbitFSK,   /*!< 2Mbps data rate (ShockBurst modulation) */
  RADIO_DATARATE_250KBPS_S = RADIO_MODE_MODE_250KbitFSK, /*!<  250kbps data rate (ShockBurst modulation) */
  RADIO_DATARATE_1MBPS_B   = RADIO_MODE_MODE_1MbitQPSK   /*!<  1Mbps data rate (Blue tooth modulation) */
}RADIO_DATARATE_Type;

/**
 * nRF5 Transceiver base address length.
 */
typedef enum
{
  RADIO_BASE_ADR_LENGTH_2B = 2,  /*!<  2 bytes base address length = 3 bytes address length */
  RADIO_BASE_ADR_LENGTH_3B,      /*!<  3 bytes base address length = 4 bytes address length */
  RADIO_BASE_ADR_LENGTH_4B,      /*!<  4 bytes base address length = 5 bytes address length */
}RADIO_BASE_ADR_LENGTH_Type;

/**
 * nRF5 Transceiver TXPOWER values
 */
typedef enum
{
  RADIO_TXPOWER_4    = RADIO_TXPOWER_TXPOWER_Pos4dBm,  /*!<  +4dBm transmitter output power */
  RADIO_TXPOWER_0    = RADIO_TXPOWER_TXPOWER_0dBm,      /*!<   0dBm transmitter output power */
  RADIO_TXPOWER_N4   = RADIO_TXPOWER_TXPOWER_Neg4dBm,  /*!<  -4dBm transmitter output power */
  RADIO_TXPOWER_N8   = RADIO_TXPOWER_TXPOWER_Neg8dBm,  /*!<  -8dBm transmitter output power */
  RADIO_TXPOWER_N12  = RADIO_TXPOWER_TXPOWER_Neg12dBm, /*!< -12dBm transmitter output power */
  RADIO_TXPOWER_N16  = RADIO_TXPOWER_TXPOWER_Neg16dBm, /*!< -16dBm transmitter output power */
  RADIO_TXPOWER_N20  = RADIO_TXPOWER_TXPOWER_Neg20dBm  /*!< -20dBm transmitter output power */
}RADIO_TXPOWER_Type;



/**
 * nRF5 Transceiver EVENTS ID numbers
 */
typedef enum
{
  RADIO_EVENT_READY_IDn     = RADIO_INTENSET_READY_Pos,    /*!< Ready event ID number */
  RADIO_EVENT_ADDRESS_IDn   = RADIO_INTENSET_ADDRESS_Pos,  /*!< Address event ID number */
  RADIO_EVENT_PAYLOAD_IDn   = RADIO_INTENSET_PAYLOAD_Pos,  /*!< Payload event ID number */
  RADIO_EVENT_END_IDn       = RADIO_INTENSET_END_Pos,      /*!< End event ID number */
  RADIO_EVENT_DISABLED_IDn  = RADIO_INTENSET_DISABLED_Pos  /*!< Disabled event ID number */
}RADIO_EVENTS_IDn_Type;



/**
 * nRF5 Transceiver CRC start position indicator
 */
typedef enum
{
  RADIO_CRCSTART_FIRST      = RADIO_CRCCNF_SKIP_ADR_Include, /*!< CRC calculation starts at first byte of the address field */
  RADIO_CRCSTART_FIRSTAFTER = RADIO_CRCCNF_SKIP_ADR_Skip     /*!< CRC calculation starts at first byte after the address field */
}RADIO_CRCSTART_Type;


/**
 * nRF5 Transceiver CRC length
 */
typedef enum
{
  RADIO_CRCLENGTH_0   = RADIO_CRCCNF_LEN_Disabled,  /*!< CRC disabled */
  RADIO_CRCLENGTH_8   = RADIO_CRCCNF_LEN_One,       /*!< 8 bit  CRC */
  RADIO_CRCLENGTH_16  = RADIO_CRCCNF_LEN_Two,       /*!< 16 bit CRC */
  RADIO_CRCLENGTH_24  = RADIO_CRCCNF_LEN_Three      /*!< 24 bit CRC */
}RADIO_CRCLENGTH_Type;

/**
 * nRF5 Transceiver CRC result for the last received packet (every END event).
 */
typedef enum
{
  RADIO_CRCSTATUS_CRCERROR  = RADIO_CRCSTATUS_CRCSTATUS_CRCError,  /*!< Packet received with CRC error */
  RADIO_CRCSTATUS_CRCOK     = RADIO_CRCSTATUS_CRCSTATUS_CRCOk      /*!< Packet received with CRC ok */
}RADIO_CRCSTATUS_Type;

/**
 * nRF5 Transceiver bit endianness of length, slice S0, slice S1, and payload fields.
 */
typedef enum
{
  RADIO_ENDIAN_LITTLE = RADIO_PCNF1_ENDIAN_Little, /*!< Little endian bit ordering */
  RADIO_ENDIAN_BIG    = RADIO_PCNF1_ENDIAN_Big  /*!< Big endian bit ordering */
}RADIO_ENDIAN_Type;

/**
 * nRF5 Transceiver test ID numbers.
 */
typedef enum
{
  RADIO_TEST_CONT_IDn = RADIO_TEST_CONST_CARRIER_Pos, /*!< Continuous wave */
  RADIO_TEST_LOCK_IDn = RADIO_TEST_PLL_LOCK_Pos       /*!< PLL Lock */
}RADIO_TEST_IDn_Type;

/**
 * nRF5 Transceiver state.
 */
typedef enum
{
  RADIO_STATE_DISABLED   = RADIO_STATE_STATE_Disabled,   /*!< Radio is in the disabled state. */
  RADIO_STATE_RXENABLING = RADIO_STATE_STATE_RxEnabling, /*!< Radio is in the RxEnabling state. */
  RADIO_STATE_RXREADY    = RADIO_STATE_STATE_RxReady,    /*!< Radio is in the RxReady state. */
  RADIO_STATE_RXSTARTED  = RADIO_STATE_STATE_RxStarted,  /*!< Radio is in the RxStarted state. */
  RADIO_STATE_TXENABLING = RADIO_STATE_STATE_TxEnabling, /*!< Radio is in the TxEnabling state. */
  RADIO_STATE_TXREADY    = RADIO_STATE_STATE_TxReady,    /*!< Radio is in the TxReady state. */
  RADIO_STATE_TXSTARTED  = RADIO_STATE_STATE_TxStarted   /*!< Radio is in the TxStarted state. */
}RADIO_STATE_Type;


/* TIMER */

/**
 * TIMER shortcuts
 */
typedef enum 
{
  TIMER_SHORT_COMPAREVENT0_CLEAR_IDn = TIMER_SHORTS_COMPARE0_CLEAR_Pos,    /*!< Shortcut between compare match 0 event and clear task */
  TIMER_SHORT_COMPAREVENT1_CLEAR_IDn = TIMER_SHORTS_COMPARE1_CLEAR_Pos,    /*!< Shortcut between compare match 1 event and clear task */
  TIMER_SHORT_COMPAREVENT0_STOP_IDn  = TIMER_SHORTS_COMPARE0_STOP_Pos,     /*!< Shortcut between compare match 0 event and stop task */
  TIMER_SHORT_COMPAREVENT1_STOP_IDn  = TIMER_SHORTS_COMPARE1_STOP_Pos      /*!< Shortcut between compare match 1 event and stop task */
}TIMER_SHORTS_IDn_Type;


/**
 * TIMER compare events
 */
typedef enum
{
  TIMER_COMPAREVENT0_IDn = TIMER_INTEN_COMPARE0_Pos, /*!< COMPAREEVENT0 ID number*/
  TIMER_COMPAREVENT1_IDn = TIMER_INTEN_COMPARE1_Pos  /*!< COMPAREEVENT1 ID number*/
}TIMER_EVENTS_IDn_Type;


#endif
