/*   Copyright (c) 2011 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 133 $
 */

#ifndef MPU6050_H
#define MPU6050_H

/*lint ++flb "Enter library region */

#include <stdbool.h>
#include <stdint.h>

/** @file
* @brief MPU6050 gyro/accelerometer driver.
*
*
* @defgroup lib_driver_mpu6050 MPU6050 gyro/accelerometer driver.
* @{
* @ingroup nrf_libs
* @brief MPU6050 gyro/accelerometer driver.
*/

/**
 * Initializes MPU6050 and verifies it's on the bus.
 *
 * @param device_address Device I2C address in bits [6:0].
 * @return
 * @retval true MPU6050 found on the bus and ready for operation.
 * @retval false MPU6050 not found on the bus or communication failure.
 */
bool mpu6050_init(uint8_t device_address);

/**
  @brief Writes MPU6050 register contents over I2C.
  Writes one or more consecutive registers.
  @param[in]  register_address Register address to start writing to
  @param[in]  number_of_bytes Number of bytes to write
  @param[out] source Pointer to a data buffer to be written
  @retval true Register write succeeded
  @retval false Register write failed
*/
bool mpu6050_register_write(uint8_t register_address, const uint8_t *source, uint8_t number_of_bytes);

/**
  @brief Function to read MPU6050 register contents over I2C.
  Reads one or more consecutive registers.
  @param[in]  register_address Register address to start reading from
  @param[in]  number_of_bytes Number of bytes to read
  @param[out] destination Pointer to a data buffer where read data will be stored
  @retval true Register read succeeded
  @retval false Register read failed
*/
bool mpu6050_register_read(uint8_t register_address, uint8_t *destination, uint8_t number_of_bytes);

/**
  @brief Reads and verifies MPU6050 product ID.
  @retval true Product ID is what was expected
  @retval false Product ID was not what was expected
*/
bool mpu6050_verify_product_id(void);

/**
 *@}
 **/

/*lint --flb "Leave library region" */

#endif /* MPU6050_H */
