 /* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
  *
  * The information contained herein is confidential property of Nordic
  * Semiconductor ASA.Terms and conditions of usage are described in detail
  * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
  *
  * Licensees are granted free, non-transferable use of the information. NO
  * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
  * the file.
  *
  */

#ifndef SIMPLE_UART_H
#define SIMPLE_UART_H

/*lint ++flb "Enter library region */

#include <stdbool.h>
#include <stdint.h>

/** @file
* @brief Simple UART driver
*
*
* @defgroup lib_driver_simple_uart Simple UART driver
* @{
* @ingroup nrf_libs
* @brief Simple UART driver
*/

/** Sends a character to UART.
Execution is blocked until UART peripheral reports character to have been send.
@param cr Character to send.
*/
void simple_uart_put(char cr);

/** Sends a string to UART.
Execution is blocked until UART peripheral reports all characters to have been send.
Maximum string length is 254 characters including null character in the end.
@param str Null terminated string to send.
*/
void simple_uart_putstring(const char *str);

/** Configures UART to use 9600 baud rate.
@param txd_pin_number Chip pin number to be used for UART TXD
@param rxd_pin_number Chip pin number to be used for UART RXD
*/
void simple_uart_config(uint8_t txd_pin_number, uint8_t rxd_pin_number);

/**
 *@}
 **/

/*lint --flb "Leave library region" */ 
#endif
