
/*
	ADC driver.
	2012 Even brenden
*/

#include "adc.h"

static bool next_sample_is_ready;
static Q16 next_sample;

// #define PRINT

void TIMER2_IRQHandler(void) {

	#ifdef PROF_ADDA
		NRF_GPIO->OUTSET = ADC_SMP_PIN;
	#endif

	if (NRF_TIMER2->EVENTS_COMPARE[0]) {

		NRF_TIMER2->TASKS_CLEAR = 1;

		next_sample = (NRF_ADC->SAMPLE)<<NORMALIZE;
		next_sample_is_ready=true;

		NRF_ADC->TASKS_STARTCONV = 1;

		#ifdef PRINT
			uart_put16(next_sample);
		#endif

	    NRF_TIMER2->EVENTS_COMPARE[0] = 0;
	}

	#ifdef PROF_ADDA
		NRF_GPIO->OUTCLR = ADC_SMP_PIN;
	#endif
}

void adc_init(void) {

	next_sample_is_ready = false;
	next_sample = 0UL;

	#ifndef UART_AS_ADC
		real_adc_config();
	#endif
}

void real_adc_config() {

	/* Power. */

	NRF_POWER->PERPOWER |= POWER_PERPOWER_ADC_Msk;
	while ((NRF_POWER->PERRDY & POWER_PERPOWER_ADC_Msk) == 0);

	/* Enable ADC, config. */
	NRF_ADC->CONFIG = 	(ADC_CONFIG_RES_10bit << ADC_CONFIG_RES_Pos) |
						(ADC_CONFIG_PSEL_AnalogInput2 << ADC_CONFIG_PSEL_Pos);

	NRF_ADC->ENABLE = (ADC_ENABLE_ENABLE_Pos<<ADC_ENABLE_ENABLE_Enabled);

	/* Timer. */
	NRF_POWER->PERPOWER |= (1UL << POWER_PERPOWER_TIMER2_Pos);
	while ((NRF_POWER->PERRDY & (1UL << POWER_PERPOWER_TIMER2_Pos)) == 0);
	NRF_TIMER2->MODE = TIMER_MODE_MODE_Timer; /* Set the timer in Timer Mode */
	NRF_TIMER2->PRESCALER = TIMER2_PRESCALER;
	NRF_TIMER2->BITMODE = TIMER_BITMODE_BITMODE_24Bit;
	NRF_TIMER2->INTENCLR = 0xFFFFFFFFUL;
	NRF_TIMER2->INTENSET = (TIMER_INTEN_COMPARE0_Enabled<<TIMER_INTEN_COMPARE0_Pos);
	NVIC_EnableIRQ(TIMER2_IRQn);
	NRF_TIMER2->CC[0] = SAMPLE_DURATION;
	NRF_TIMER2->TASKS_CLEAR = 1;

	/* Start timer and conversion. */
	NRF_TIMER2->TASKS_START = 1;
}

void adc_fetch_next_sample(Q16 *sample) {

	#ifndef UART_AS_ADC
		*sample = next_sample;
	#else
		*sample = uart_get32();

		#ifdef PRINT
			uart_put32(*sample);
		#endif

	#endif

	next_sample_is_ready=false;
}

bool adc_next_sample_is_ready() {

	#ifdef UART_AS_ADC
		next_sample_is_ready = true;
	#endif

	return next_sample_is_ready;
}
