
/*
	ADPCM encoder and decoder, not including LPC calculation.
	2012 Even brenden
*/

#include "adpcm.h"

// #define PRINT

/* Globals. */
static Q16 x_tilde[P], s_tilde[P], x_hat, s_hat, prev_delta_enc, prev_e_enc, prev_delta_dec, prev_e_dec;
static int32_t xt_pointer, st_pointer;

void adpcm_init() {

	dpcm_init();

	prev_delta_enc = DELTA_MAX;
	prev_delta_dec = DELTA_MAX;
	#ifdef A_AQB
		prev_e_enc = 0;
		prev_e_dec = 0;
	#endif
	#ifdef SSM_AQB
		prev_e_enc = (1<<(DBITS-1)); /* Because prev_e_enc is codeword when using SSM-AQB. */
		prev_e_dec = (1<<(DBITS-1));
	#endif
}

void dpcm_init() {

	uint32_t n;
	for(n=0;n<P;n++) {
		x_tilde[n]=0UL;
		s_tilde[n]=0UL;
	}

	x_hat = 0UL;
	s_hat = 0UL;
	xt_pointer = 0UL;
	st_pointer = 0UL;
}

void adpcm_encode_Q16(Q16 *x, Q16 *a, int32_t *codeword) {

	Q16 delta, e;
	uint32_t k;

	#ifdef PROFILE
		NRF_GPIO->OUTSET = (Q_PIN);
	#endif

	/* Quantization. */
	#ifdef A_AQB
		aqb_Q16(&prev_e_enc, &prev_delta_enc, &delta);
		aqb_encode_Q16(*x-x_hat, &delta, codeword);
		aqb_decode_Q16(&delta, codeword, &e);
		prev_e_enc=e;
	#endif
	#ifdef SSM_AQB
		maqb_Q16(&prev_e_enc, &prev_delta_enc, &delta);
		maqb_encode_Q16(*x-x_hat, &delta, codeword);
		maqb_decode_Q16(&delta, codeword, &e);
		prev_e_enc=*codeword;
	#endif
	prev_delta_enc=delta;

	#ifdef PRINT
		uart_put32(e);
	#endif

	#ifdef PROFILE
		NRF_GPIO->OUTCLR = (Q_PIN);
	#endif

	/* DPCM. */
    x_tilde[xt_pointer]=e+x_hat;

    x_hat=0;

	for(k=0;k<P;k++) {
			x_hat+=MULQ16(a[k], x_tilde[modulo((xt_pointer-k), P)]);
	}

	xt_pointer=(xt_pointer+1)%P;
}

void adpcm_decode_Q16(int32_t *codeword, Q16 *a, Q16 *y) {

	Q16 delta, e;
	uint32_t k;

	#ifdef PROFILE
		NRF_GPIO->OUTSET = (DQ_PIN);
	#endif

	/* Dequantization. */
	#ifdef A_AQB
		aqb_Q16(&prev_e_dec, &prev_delta_dec, &delta);
		aqb_decode_Q16(&delta, codeword, &e);
		prev_e_dec=e;
	#endif
	#ifdef SSM_AQB
		maqb_Q16(&prev_e_dec, &prev_delta_dec, &delta);
		maqb_decode_Q16(&delta, codeword, &e);
		prev_e_dec=*codeword;
	#endif
	prev_delta_dec=delta;

	#ifdef PROFILE
		NRF_GPIO->OUTCLR = (DQ_PIN);
	#endif

	/* DPCM. */
    s_tilde[st_pointer] = e+s_hat;
    *y = s_tilde[st_pointer];

    s_hat=0;
	for(k=0;k<P;k++) {
		s_hat+=MULQ16(a[k], s_tilde[modulo(st_pointer-k, P)]);
	}

	st_pointer=(st_pointer+1)%P;
}

void dpcm_encode_Q16(Q16 *x, Q16 *a, int32_t *codeword) {

	Q16 residual;
	uint32_t k;

	/* Quantize. Here codeword is used as placeholder for residual.*/
	midrise_Q16(*x-x_hat, &residual, codeword);
    x_tilde[xt_pointer]=residual+x_hat;

    x_hat=0;

	for(k=0;k<P;k++) {
			x_hat+=MULQ16(a[k], x_tilde[modulo((xt_pointer-k), P)]);
	}

	xt_pointer=(xt_pointer+1)%P;
}

void dpcm_decode_Q16(Q16 *codeword, Q16 *a, Q16 *d) {

	uint32_t k;

	Q16 residual = (*codeword<<SHIFT)-Q16_1;

    s_tilde[st_pointer] = residual+s_hat;
    *d = s_tilde[st_pointer];

    s_hat=0;
	for(k=0;k<P;k++) {
                s_hat+=MULQ16(a[k], s_tilde[modulo(st_pointer-k, P)]);
        }

	st_pointer=(st_pointer+1)%P;
}
