
/*
	Adaptive backward quantizer.
	2012 Even Brenden
*/

#include <stdio.h>
#include "aqb.h"

//#define PRINT

void aqb_Q16(Q16 *prev_y, Q16 *prev_delta, Q16 *delta) {
	
	*delta = DIVQ16(*prev_y, *prev_delta);
	*delta = MULQ16(*delta, *delta);
	*delta = MULQ16(AQB_CONST_Q16, *delta)+ALPHA;
	*delta = SQTQ16(*delta);
	*delta = MULQ16(*delta, *prev_delta);

	/* Prevent zero and uneccessary large values of delta. */
	if(*delta<DELTA_MIN) *delta=DELTA_MIN;
	else if(*delta>DELTA_MAX) *delta=DELTA_MAX;
	
	#ifdef PRINT
		uart_put32(*delta);
		uart_put32(*prev_y);
		uart_put32(*prev_delta);
	#endif
}

void aqb_encode_Q16(Q16 x, Q16 *delta, int32_t *codeword) {
	
	(*codeword)=DIVQ16(x, *delta)>>16;
	
	/* Shift codeword from [-2^(bits-1), 2^(bits-1)-1] to [0, 2^bits-1] for easier packing. */
	(*codeword)+=(1<<(DBITS-1));

	/* Limit codeword to [0, 2^DBITS-1]. */
	if (*codeword<0L) *codeword=0L;
	if (*codeword>((1<<DBITS)-1)) *codeword=((1<<DBITS)-1);
	
	#ifdef PRINT
		uart_put32(x);
		uart_put32(*delta);	
		uart_put32(*codeword);
	#endif
}

void aqb_decode_Q16(Q16 *delta, int32_t *codeword, Q16 *y) {

	(*y) = MULQ16(*delta, (*codeword-(1<<(DBITS-1)))<<16);
	
	/* Limit y. */
	if(*y<Q16_M1) *y=Q16_M1;
	else if(*y>=Q16_1) *y=Q16_1;	
	
	#ifdef PRINT
		uart_put32(*delta);	
		uart_put32(*codeword);
		uart_put32(*y);
	#endif
}
