
/*
	DAC/PWM driver.
	2012 Even brenden
*/

#include "dac.h"

static uint8_t dac_buffer[DACN];	/* Buffer. */
static uint32_t write_ptr;			/* For toggling which part of ping-pong buffer to use for writing. */
static uint32_t read_ptr; 			/* Reading index. */
static uint32_t i_count;			/* Counter for oversampling. */
static uint32_t last_cc;			/* For toggling CC[0] and CC[2]. */
static Q16 current_sample;

#define PRINT

void dac_init() {

	#ifndef UART_AS_DAC

		write_ptr=DACN>>1; /* Start halfway into buffer for maximal distance between read_ptr and write_ptr. */
		read_ptr=0;
		i_count=0;

		pwm_init();

	#endif
}

/* Send a block of samples to DAC. */
void sample_to_dac(Q16 *sample) {

	#ifndef UART_AS_DAC

		#ifdef PROF_ADDA
			NRF_GPIO->OUTSET = DAC_BLK_PIN;
		#endif

		dac_buffer[write_ptr] = Q16_TO_PWM(*sample);
		write_ptr++;
		if(write_ptr>=DACN) write_ptr=0;

		#ifdef PROF_ADDA
			NRF_GPIO->OUTCLR = DAC_BLK_PIN;
		#endif

	#else

		#ifdef PRINT
			for(n=0;n<N;n++) {
				uart_put32(samples[n]);
			}
		#endif

	#endif
}

/* Called every time the PWM needs a new sample to output. */
Q16 next_pwm_sample(void) {

	read_ptr++;
	if(read_ptr>=DACN) {
		read_ptr=0;
	}
	return dac_buffer[read_ptr];
}

void TIMER1_IRQHandler(void) {

	if (NRF_TIMER1->EVENTS_COMPARE[1]) {

		#ifdef PROF_ADDA
	   		NRF_GPIO->OUTSET = DAC_CC1_PIN;
		#endif

		NRF_TIMER1->CC[1] += CARRIER_DUR;

		/* Interpolate. */
		i_count++;
		if (i_count>(UPSAMPLE_FACTOR-1)) {

			#ifdef PROF_ADDA
				Q16 temp = current_sample;
			#endif

			current_sample=next_pwm_sample();
			i_count=0;

			#ifdef PROF_ADDA
				if(current_sample!=temp)
					NRF_GPIO->OUTSET = (DAC_SNC_PIN);
				else
					NRF_GPIO->OUTCLR = (DAC_SNC_PIN);
			#endif
		}

		if (last_cc) {
			NRF_TIMER1->CC[0] = (NRF_TIMER1->CC[1])+current_sample;
			last_cc=0;
		} else {
			NRF_TIMER1->CC[2] = (NRF_TIMER1->CC[1])+current_sample;
			last_cc=1;
		}

		#ifdef PROF_ADDA
			NRF_GPIO->OUTCLR = DAC_CC1_PIN;
		#endif

		NRF_TIMER1->EVENTS_COMPARE[1]=0;

	} else if (NRF_TIMER1->EVENTS_COMPARE[0]) {

			#ifdef PROF_ADDA
				NRF_GPIO->OUTSET = DAC_CC0_PIN;
				NRF_GPIO->OUTCLR = DAC_CC0_PIN;
			#endif

			NRF_TIMER1->EVENTS_COMPARE[0]=0;

	} else if (NRF_TIMER1->EVENTS_COMPARE[2]) {

			#ifdef PROF_ADDA
				NRF_GPIO->OUTSET = DAC_CC2_PIN;
				NRF_GPIO->OUTCLR = DAC_CC2_PIN;
			#endif

			NRF_TIMER1->EVENTS_COMPARE[2]=0;
	}
}

static void gpiote_init(void)
{
  /* Power ON the GPIOTE peripheral */
  NRF_POWER->PERPOWER |= POWER_PERPOWER_GPIOTE_Msk;

  /* Wait for the power up */
  while (!(NRF_POWER->PERRDY & POWER_PERPOWER_GPIOTE_Msk));

  NRF_GPIO->DIRCLR = 0xFFFFFFFFUL;
  NRF_GPIO->DIRSET = (1UL << PWM_PIN)|(0x07);

  NRF_GPIO->PIN18_CNF = (GPIO_PIN18_CNF_SENSE_Disabled << GPIO_PIN18_CNF_SENSE_Pos)
   | (GPIO_PIN18_CNF_DRIVE_S0S1 << GPIO_PIN18_CNF_DRIVE_Pos)
   | (GPIO_PIN18_CNF_PULL_Disabled << GPIO_PIN18_CNF_PULL_Pos)
   | (GPIO_PIN18_CNF_INPUT_Disconnect << GPIO_PIN18_CNF_INPUT_Pos)
   | (GPIO_PIN18_CNF_DIR_Output << GPIO_PIN18_CNF_DIR_Pos);

  NRF_GPIO->OUT = 0x00000000UL;

  /* Configure GPIOTE channel 0 to toggle the PWM pin state */
  /* Note that we can only connect one GPIOTE task to an output pin */
  NRF_GPIOTE->CONFIG[0] =
    (GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos) |
    (PWM_PIN << GPIOTE_CONFIG_PSEL_Pos) |
    (GPIOTE_CONFIG_MODE_Task << GPIOTE_CONFIG_MODE_Pos);
}

static void ppi_init(void)
{
  /* Power ON the PPI peripheral */
  NRF_POWER->PERPOWER |= (1UL << POWER_PERPOWER_PPI_Pos);

  /* Wait for the power up */
  while ((NRF_POWER->PERRDY & (1UL << POWER_PERPOWER_PPI_Pos)) == 0);

  /* Configure PPI channel 0 to toggle PWM_OUTPUT_PIN on every TIMER2 COMPARE[0] match */
  NRF_PPI->CH0_EEP = (uint32_t)&NRF_TIMER1->EVENTS_COMPARE[0];
  NRF_PPI->CH0_TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[0];

  /* Configure PPI channel 1 to toggle PWM_OUTPUT_PIN on every TIMER2 COMPARE[1] match */
  NRF_PPI->CH1_EEP = (uint32_t)&NRF_TIMER1->EVENTS_COMPARE[1];
  NRF_PPI->CH1_TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[0];

  /* Configure PPI channel 2 to toggle PWM_OUTPUT_PIN on every TIMER2 COMPARE[2] match */
  NRF_PPI->CH2_EEP = (uint32_t)&NRF_TIMER1->EVENTS_COMPARE[2];
  NRF_PPI->CH2_TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[0];

  /* Enable only PPI channels 0 and 1 */
  NRF_PPI->CHEN =
    (PPI_CHEN_CH0_Enabled << PPI_CHEN_CH0_Pos) |
    (PPI_CHEN_CH1_Enabled << PPI_CHEN_CH1_Pos) |
    (PPI_CHEN_CH2_Enabled << PPI_CHEN_CH2_Pos);
}

void pwm_init(void) {

	/* Inits.*/
	gpiote_init();
	ppi_init();

	/* Start to power up TIMER1 peripheral */
	NRF_POWER->PERPOWER |= (1UL << POWER_PERPOWER_TIMER1_Pos);

	/* Wait for TIMER1 to power up */
	while ((NRF_POWER->PERRDY & (1UL << POWER_PERPOWER_TIMER1_Pos)) == 0);

	/* Config. */
	NRF_TIMER1->MODE = TIMER_MODE_MODE_Timer; /* Set the timer in Timer Mode */
	NRF_TIMER1->PRESCALER = TIMER1_PRESCALER;
	NRF_TIMER1->BITMODE = TIMER_BITMODE_BITMODE_32Bit;

	/* Set interrupt enables. */
	NRF_TIMER1->INTENCLR = 0xFFFFFFFFUL;
	NRF_TIMER1->INTENSET = ((1UL<<TIMER_INTEN_COMPARE0_Pos)|(1UL<<TIMER_INTEN_COMPARE1_Pos)|(1UL<<TIMER_INTEN_COMPARE2_Pos));
	NVIC_EnableIRQ(TIMER1_IRQn);
  	__enable_irq();

	/* Clear timer. */
	NRF_TIMER1->TASKS_CLEAR = 1;

	/* Init CCs. */
	NRF_TIMER1->CC[1] = CARRIER_DUR;
	NRF_TIMER1->CC[0] = (NRF_TIMER1->CC[1])+next_pwm_sample();
	last_cc=0;

	/* Start clock. */
	NRF_TIMER1->TASKS_START = 1;
}

