
/*
	Levinson-Durbin algorithm. Reflection coefficients are not calulated. LPCs are flipped at the end wrt. usage.
	2011 Even Brenden. Modified from http://kbs.cs.tu-berlin.de/~jutta/gsm/lpc.html.
*/

#include <stdint.h>
#include "l-d.h"

Q16 levinson_durbin_Q16(Q16 *ac, Q16 *lpc) {

	uint32_t i, j;
	Q16 r, error;

	#ifdef PROFILE
		NRF_GPIO->OUTSET = (L_D_PIN);
	#endif

	error = ac[0];

	if (ac[0]==0) {
		for(i=0;i<P;i++)
			lpc[i]=0;
		return 0;
	}

	for (i=0; i<P; i++) {

	/* Sum up this iteration's reflection coefficient.
	*/
		r = -ac[i+1];
		for (j=0;j<i;j++) {
			r -= MULQ16(lpc[j], ac[i-j]);
		}

		/* Avoid divison by zero. */
		if(error==0)
			error=Q16_SMALLEST;

		r=DIVQ16(r, error);;

	/*  Update LPC coefficients and total error.
	*/

		lpc[i] = r;
		for (j=0; j<i/2; j++) {
			float tmp  = lpc[j];
			lpc[j]     += MULQ16(r, lpc[i-1-j]);
			lpc[i-1-j] += MULQ16(r, tmp);
		}
		if (i%2) {
			lpc[j] += MULQ16(lpc[j], r);
		}
		error = MULQ16(error, Q16_1 - MULQ16(r, r));
	}

	for(i=0;i<P;i++) {
		lpc[i]*=-1;
	}

	#ifdef PROFILE
		NRF_GPIO->OUTCLR = (L_D_PIN);
	#endif

	return error;
}
