
/*
	One-word-memory AQB as described in section 2.2.
	2012 Even Brenden
*/

#include <stdio.h>
#include "maqb.h"

// #define PRINT

void maqb_Q16(int32_t *prev_codeword, Q16 *prev_delta, Q16 *delta) {

	/* Shift codeword back from [0, 2^bits-1] to [-2^(bits-1), 2^(bits-1)-1] and then to Q16 for decoding. */
	(*prev_codeword)-=(1<<(DBITS-1));

	/* So that negative values will index MX[] correctly. */
	if (*prev_codeword<0) (*prev_codeword)=-(*prev_codeword)-1;

	#if DBITS == 4
		*delta=MULQ16((*prev_delta), M4[*prev_codeword]);
	#endif

	#if DBITS == 3
		*delta=MULQ16((*prev_delta), M3[*prev_codeword]);
	#endif

	#if DBITS == 2
		*delta=MULQ16((*prev_delta), M2[*prev_codeword]);
	#endif

	/* Prevent zero and uneccessary large values of delta. */
	if(*delta<DELTA_MIN) *delta=DELTA_MIN;
	else if(*delta>DELTA_MAX) *delta=DELTA_MAX;
}

void maqb_encode_Q16(Q16 x, Q16 *delta, int32_t *codeword) {

	/* Calculate codeword. */
	*codeword=DIVQ16(x, *delta)>>16;

	/* Shift codeword from [-2^(bits-1), 2^(bits-1)-1] to [0, 2^bits-1] for easier packing. */
	(*codeword)+=(1<<(DBITS-1));

	/* Limit codeword to [0, 2^DBITS-1]. */
	if (*codeword<0L) *codeword=0L;
	if (*codeword>((1<<DBITS)-1)) *codeword=((1<<DBITS)-1);
}

void maqb_decode_Q16(Q16 *delta, int32_t *codeword, Q16 *y) {

	/* Calculate reconstructed value. Need to shift codeword back from [0, 2^bits-1] to [-2^(bits-1), 2^(bits-1)-1] and then to Q16 for decoding. */
	*y = MULQ16(*delta, (*codeword-(1<<(DBITS-1)))<<16);
}
