/*
	Radio configuration.
	Author is confidential.
*/

#include "nrf.h"
#include "radio_config.h"
#include "nrf_delay.h"
#include "config.h"

void radio_configure()
{
    // Radio config
    NRF_RADIO->TXPOWER = (RADIO_TXPOWER_TXPOWER_0dBm << RADIO_TXPOWER_TXPOWER_Pos);
    NRF_RADIO->FREQUENCY = 63UL;                // Frequency bin 7, 2407MHz
    NRF_RADIO->MODE = (RADIO_MODE_MODE_Nrf_2Mbit << RADIO_MODE_MODE_Pos);

    // Radio address config
    NRF_RADIO->PREFIX0 = 0xC4C3C2E7UL;   // Prefix byte of addresses 3 to 0
    NRF_RADIO->PREFIX1 = 0xC5C6C7C8UL;   // Prefix byte of addresses 7 to 4
    NRF_RADIO->BASE0   = 0xE7E7E7E7UL;   // Base address for prefix 0
    NRF_RADIO->BASE1   = 0x00C2C2C2UL;   // Base address for prefix 1-7
    NRF_RADIO->TXADDRESS = 0x00UL;       // Set device address 0 to use when transmitting
    NRF_RADIO->RXADDRESSES = 0x01UL;     // Enable device address 0 to use which receiving

    // Packet configuration
    // S1 size = 0 bits, S0 size = 0 bytes, payload length size = 8 bits
    NRF_RADIO->PCNF0 = (0UL << RADIO_PCNF0_S1LEN_Pos) |
                       (0UL << RADIO_PCNF0_S0LEN_Pos) |
                       (8UL << RADIO_PCNF0_LFLEN_Pos); //lint !e845 "The right argument to operator '|' is certain to be 0"

    // Packet configuration
    // Bit 25: 0 Whitening disabled 1 Whitening enabled
    // Bit 24: 0 Little endian 1 Big endian,
    // 4 byte base address length (5 byte full address length),
    // 0 byte static length, max 255 byte payload
     NRF_RADIO->PCNF1 = (RADIO_PCNF1_WHITEEN_Disabled << RADIO_PCNF1_WHITEEN_Pos) |
                        (RADIO_PCNF1_ENDIAN_Big << RADIO_PCNF1_ENDIAN_Pos) |
                        (4UL << RADIO_PCNF1_BALEN_Pos) |
                        (0UL << RADIO_PCNF1_STATLEN_Pos) |
                        (NP << RADIO_PCNF1_MAXLEN_Pos); //lint !e845 "The right argument to operator '|' is certain to be 0"

    // CRC Config
    NRF_RADIO->CRCCNF = (RADIO_CRCCNF_LEN_Two << RADIO_CRCCNF_LEN_Pos); // Number of checksum bits
    if ((NRF_RADIO->CRCCNF & RADIO_CRCCNF_LEN_Msk) == (RADIO_CRCCNF_LEN_Two << RADIO_CRCCNF_LEN_Pos))
    {
      NRF_RADIO -> CRCINIT = 0xFFFFUL;         // Initial value
      NRF_RADIO -> CRCPOLY = 0x11021UL;        // CRC poly: x^16+x^12^x^5+1
    }
    else if ((NRF_RADIO->CRCCNF & RADIO_CRCCNF_LEN_Msk) == (RADIO_CRCCNF_LEN_One << RADIO_CRCCNF_LEN_Pos))
    {
      NRF_RADIO -> CRCINIT = 0xFFUL;         // Initial value
      NRF_RADIO -> CRCPOLY = 0x107UL;         // CRC poly: x^8+x^2^x^1+1
    }

    nrf_delay_ms(3);       // Add delay to ensure configuration is clocked to radio
}


