
/*
	RX API.
	2012 Even brenden
*/

#include "rx.h"

#ifndef USE_GZLL

	// #define PRINT

	uint8_t rx_packet[NP];			/* The one that the radio writes to. Volatile. */
	uint8_t rx_buffer[RXTX_BUFFER_SIZE];
	uint32_t rx_buffer_write_ptr;
	uint32_t rx_buffer_read_ptr;
	bool rx_packet_ready;			/* Flag to indicate that there are unread packets. */

	void rx_init()
	{

	   	/* Inits. */
		rx_packet_ready=false;
		rx_buffer_write_ptr = 0;
		rx_buffer_read_ptr = 0;

		#ifndef NO_RF

			/* Turn peripherals on */
			NRF_POWER->PERPOWER |= (POWER_PERPOWER_PPI_Msk | POWER_PERPOWER_RADIO_Msk);
			/* Wait for power up */
			while ((NRF_POWER->PERRDY & POWER_PERPOWER_PPI_Msk) == 0);
			/* Wait for power up */
			while ((NRF_POWER->PERRDY & POWER_PERPOWER_RADIO_Msk) == 0);

			/* Set interrupt enables. */
			NRF_RADIO->INTENCLR = 0xFFFFFFFFUL;
			NRF_RADIO->INTENSET = (1<<RADIO_INTEN_END_Pos);
			NVIC_EnableIRQ(RADIO_IRQn);

			/* Set radio configuration parameters. */
			radio_configure();

		    /* Set payload pointer. */
		    NRF_RADIO->PACKETPTR = (uint32_t)rx_packet;

			/* Enable radio. */
			NRF_RADIO->TASKS_DISABLE = 1U;
			nrf_delay_us(120);
		    NRF_RADIO->TASKS_RXEN = 1U;

			/* Wait for READY state. */
			while((NRF_RADIO->EVENTS_READY)==0);
			NRF_RADIO->EVENTS_READY = 0U;
			nrf_delay_us(120);
			NRF_RADIO->TASKS_START = 1U;

		#endif
	}

	#ifdef IS_RX
		void RADIO_IRQHandler(void) {

			if(NRF_RADIO->EVENTS_END != 0U) {

				#ifdef PROFILE
					NRF_GPIO->OUTSET = (RX_PIN);
				#endif

				NRF_RADIO->EVENTS_END = 0U;
				NRF_RADIO->TASKS_START = 1U;

				#ifdef PRINT
					for(uint32_t n=0;n<NP;n++) {
						uart_put(rx_packet[n]);
					}
				#endif

				/* May call buffer_up_rx_packet() here to be sure that packet is saved/copied before an eventual new call
					to RADIO_IRQHandler(), but at the cost of slowing down this ISR. */

				rx_packet_ready=true;

				#ifdef PROFILE
					NRF_GPIO->OUTCLR = (RX_PIN);
				#endif
			}
		}
	#endif

	/* Equal to set_tx_packet(). */
	void buffer_up_rx_packet() {
			uint32_t n;

			for(n=0;n<NP;n++) {
				rx_buffer[n+rx_buffer_write_ptr] = rx_packet[n];
			}
			rx_buffer_write_ptr+=NP;
			if (rx_buffer_write_ptr==RXTX_BUFFER_SIZE-1) rx_buffer_write_ptr=0;
	}

	/* Check flag from main(). */
	bool rx_next_packet_is_ready() {
		return rx_packet_ready;
	}

	/* Fetch newly received	packet. */
	void fetch_new_codewords(int32_t *codewords) {

		/* Unpack and write to *samples. */
		unpack_packet(&rx_buffer[rx_buffer_read_ptr], &codewords[0]);
		
		rx_buffer_read_ptr+=NP;
		if (rx_buffer_read_ptr==RXTX_BUFFER_SIZE-1) rx_buffer_read_ptr=0;
		if (rx_buffer_read_ptr==rx_buffer_write_ptr) {
			rx_packet_ready=false;
		}
	}

	/* Extract all samples from a packet. */
	void unpack_packet(uint8_t *packet, int32_t *codewords) {

		uint32_t rp_bit_ptr;
		int32_t n, m;

		rp_bit_ptr=0;

		for(n=0;n<N;n++) {

			/* Unpack DBITS bits at a time. */
			codewords[n]=0UL;
			for(m=DBITS-1; m>-1; m--) {
				codewords[n] |= BIT_IS_SET_MOD_BYTE(packet[rp_bit_ptr/8], rp_bit_ptr)<<m;
				rp_bit_ptr++;
			}
		}
	}

	#ifdef NO_RF
	void set_rx_packet(uint8_t *packet) {
		uint32_t n;

		for(n=0;n<NP;n++) {
			rx_packet[n] = packet[n];
		}
		rx_packet_ready=true;
	}
	#endif

#endif
