
/*
	TX API.
	2012 Even brenden
*/

#include "tx.h"

#ifndef USE_GZLL

	// #define PRINT

	#ifdef NO_RF
		bool next_packet_is_ready;
	#endif

	uint8_t tx_buffer[RXTX_BUFFER_SIZE];
	uint32_t tx_buffer_write_ptr;
	uint32_t tx_buffer_read_ptr;
	uint8_t tx_packet[NP];
	uint8_t building_packet[NP];
	uint32_t sp_bit_ptr;

	void tx_init()
	{
		/* Inits. */
		sp_bit_ptr = 0;
		tx_buffer_write_ptr = 0;
		tx_buffer_read_ptr = 0;

		#ifndef NO_RF

			/* Turn peripherals on. */
			NRF_POWER->PERPOWER |= (POWER_PERPOWER_PPI_Msk | POWER_PERPOWER_RADIO_Msk);
			/* Wait for power up. */
			while ((NRF_POWER->PERRDY & POWER_PERPOWER_PPI_Msk) == 0);
			/* Wait for power up. */
			while ((NRF_POWER->PERRDY & POWER_PERPOWER_RADIO_Msk) == 0);

			/* Set radio configuration parameters. */
			radio_configure();

			/* Enable ISR for monitoring transmission end. */
			NRF_RADIO->INTENCLR = 0xFFFFFFFFUL;
			NRF_RADIO->INTENSET = (1<<RADIO_INTEN_END_Pos);
			NVIC_EnableIRQ(RADIO_IRQn);

			/* Enable (first disable) radio and keep it open during program. */
			NRF_RADIO->TASKS_DISABLE = 1U;
			while(NRF_RADIO->EVENTS_DISABLED == 0U);
			NRF_RADIO->EVENTS_DISABLED = 0U;

			NRF_RADIO->TASKS_TXEN = 1;
			while (NRF_RADIO->EVENTS_READY == 0U);
			NRF_RADIO->EVENTS_READY = 0U;

			/* Set payload pointer. */
			NRF_RADIO->PACKETPTR = (uint32_t)tx_packet;

		#endif
	}

	#ifdef IS_TX
		void RADIO_IRQHandler(void) {

			if(NRF_RADIO->EVENTS_END != 0U) {

				#ifdef PROFILE
					NRF_GPIO->OUTSET = (TX_PIN);
					NRF_GPIO->OUTCLR = (TX_PIN);
				#endif

				NRF_RADIO->EVENTS_END = 0U;
			}
		}
	#endif

	/* Add a single codeword to packet and updates sp_bit_ptr. */
	/* *codeword must be in [0, 2^DBITS-1]. */
	void pack_codeword(int32_t *codeword) {

		#ifdef PROFILE
			NRF_GPIO->OUTSET = (PACK_PIN);
		#endif

		int32_t n;

		/* Append DBITS bits to packet. */
		for(n=DBITS-1; n>-1; n--) {
			building_packet[sp_bit_ptr/8] |= SET_BIT_MOD_BYTE(BIT_IS_SET(*codeword, n), sp_bit_ptr);
			sp_bit_ptr++;
		}

		/* If packet is full, move to queue and start a new one. */
		if (sp_bit_ptr>=(NP*8-1)) {

			/* 	Copy to tx_buffer and off it goes. Reset building_packet while at it. This must be achieved within 1/fs time!
				Notice that this is not real TX buffer operation as functionality for checking if the current tx_packet
				has been sent has not been implemented.
			*/
			for(n=0;n<NP;n++) {
				tx_buffer[n+tx_buffer_write_ptr]=building_packet[n];
				tx_packet[n] = tx_buffer[n+tx_buffer_read_ptr];
				building_packet[n]=0UL;
			}
			tx_buffer_write_ptr+=NP;
			if (tx_buffer_write_ptr==RXTX_BUFFER_SIZE-1) tx_buffer_write_ptr=0;
			tx_buffer_read_ptr+=NP;
			if (tx_buffer_read_ptr==RXTX_BUFFER_SIZE-1) tx_buffer_read_ptr=0;

			#ifdef PRINT
				for(uint32_t n=0;n<NP;n++) {
					uart_put(tx_packet[n]);
				}
			#endif

			#ifndef NO_RF
				/* Wait for end of last packet before sending a new one? */
// 				while(NRF_RADIO->EVENTS_END == 0U);
// 				NRF_RADIO->EVENTS_END = 0U;

				/* Start transmission. */
				NRF_RADIO->TASKS_START = 1U;
			#else
				next_packet_is_ready=true;
			#endif

			sp_bit_ptr = 0;
		}

	#ifdef PROFILE
		NRF_GPIO->OUTCLR = (PACK_PIN);
	#endif
	}

	#ifdef NO_RF

		bool tx_next_packet_is_ready() {
			return next_packet_is_ready;
		}

		uint8_t *get_tx_packet() {
			next_packet_is_ready=false;
			return &tx_buffer[read_ptr];
		}

	#endif

#endif
