
/*
	UART AI. For debugging.
	2012 Even brenden
*/

#include "uart.h"

/** Sends a character to UART.
Execution is blocked until UART peripheral reports character to have been send.
\param cr Character to send.
*/
void uart_put(uint8_t cr)
{
  NRF_UART0->TXD = cr;
  while(NRF_UART0->EVENTS_TXDRDY!=1){}  // Wait for TXD data to be sent
  NRF_UART0->EVENTS_TXDRDY=0;
}

/** Reads a character from UART.
Execution is blocked until UART peripheral detects character has been received.
\return cr Received character.
*/
uint8_t uart_get()
{
  while(NRF_UART0->EVENTS_RXDRDY!=1){}  // Wait for RXD data to be received
  NRF_UART0->EVENTS_RXDRDY=0;
  return (uint8_t)NRF_UART0->RXD;
}

/** Sends ' Exit!' string to UART.
Execution is blocked until UART peripheral detects all characters have been sent.
*/
void uart_quit(void)
{
  uart_put('0');
}

/** Sends 'Start: ' string to UART.
Execution is blocked until UART peripheral detects all characters have been sent.
*/
void uart_start(void)
{
  uart_put('1');
}

/** Configures UART.
\returns Never returns anything.
*/
void uart_config(void)
{
  /* Power ON the UART0 peripheral */
  NRF_POWER->PERPOWER |= (1UL << POWER_PERRDY_UART0_Pos);

  /* Wait for the power up */
  while (!(NRF_POWER->PERRDY & (1UL << POWER_PERRDY_UART0_Pos)))
  {
  }

 // NRF_UART0->PSELRTS = 0; //no flow control yet
  NRF_UART0->PSELTXD = 16;
 // NRF_UART0->PSELCTS = 2; //no flow control yet
  NRF_UART0->PSELRXD = 17;

  NRF_UART0->ENABLE           = (UART_ENABLE_ENABLE_Enabled << UART_ENABLE_ENABLE_Pos);
  NRF_UART0->BAUDRATE         = (UART_BAUDRATE_BAUDRATE_Baud460800 << UART_BAUDRATE_BAUDRATE_Pos);
  NRF_UART0->TASKS_STARTTX    = 1;
  NRF_UART0->TASKS_STARTRX    = 1;
  NRF_UART0->EVENTS_RXDRDY    = 0;
}

void uart_put32(int32_t cr) {

	uart_put((uint8_t) (cr>>24));
	uart_put((uint8_t) (cr>>16));
	uart_put((uint8_t) (cr>>8));
	uart_put((uint8_t) (cr));
}

int32_t uart_get32(void) {

	int32_t data = 0L, temp;
	uint8_t n;

	/* Written "inconvenient" bco. some error i didn't manage to fix. */
	for(n=0;n<4;n++) {
		temp = uart_get();
		data |= (((int32_t) temp)<< ((3-n)*8) );
	}

	return data;
}

void uart_put16(int16_t cr) {

	uart_put((uint8_t) (cr>>8));
	uart_put((uint8_t) (cr));
}

int16_t uart_get16(void) {

	int16_t data = 0;

	data |= ((uart_get())<<8);
	data |= (uart_get());

	return data;
}

