
/*
	Q15.16 type. Defintion and operators.
	2011 Even Brenden
*/

#include <stdio.h>
#include "Q16.h"

/*
	Definition:

	Bit 32: [sign bit | bits 31-17 integer bits | bits 1-16 fractional bits]. 2's complement.
	Functions for muliplication and division is implemented but does not warn for over- or underflow and does not saturate. For addition and subtraction this should be configured in
	compiler.
*/

// #define PRINT

/* Construct 2's comp. Q16 from sign bit, unsigned integer part and unsigned fractional part. Saturation arithmetic. */
Q16 MAKEQ16(uint8_t sign, uint16_t integer, uint16_t fraction) {

	/* Saturate. */
	if((integer>32767) & (!sign)) {
		integer = 32767;
	} else if((integer>32768) & (sign)) {
		integer = 32768;
		fraction = 0;
	}

	/* Make Q16. */
	if (sign) {
		return ~(((Q16) (integer<<16))|fraction)+1;
	}

	return ((Q16) (integer<<16))|fraction;
}

/* Saturated addition. */
Q16 ADDQ16(Q16 a, Q16 b) {

	Q16 temp = a+b;

	if((temp > 0) && (a < 0) && (b < 0)) {
		return Q16MIN;
	}
	if((temp < 0) && (a > 0) && (b > 0)) {
		return Q16MAX;
	}
	return temp;
}

/* Saturated sign toggle. */
Q16 TOGQ16(Q16 a) {

	// -32768 -> 32767
	if (a==Q16MIN) {
		return Q16MAX;
	} else {
		return -a;
	}
}

/* Saturated subtraction. */
Q16 SUBQ16(Q16 a, Q16 b) {

	return ADDQ16(a, TOGQ16(b));
}

/* Saturated multiplication. */
Q16 MULQ16(Q16 a, Q16 b) {

	int32_t temp;

	/* Calculate integer part. */
	int16_t hi[2] = {(a >> 16), (b >> 16)};			// Extract it.
	int32_t r_hi = hi[0] * hi[1];				// Calculate it.

	/* Calculate fractional part. */
	uint16_t lo[2] = {(a & 0xFFFF), (b & 0xFFFF)}; 		// Extract it.
	int32_t r_md = (hi[0] * lo[1]) + (hi[1] * lo[0]);	// Calculate mid part.
	uint32_t r_lo = lo[0] * lo[1];				// Calculate fractional only part.

	/* Saturate. */
	temp  = r_hi+(r_md>>16);
	if((temp>=32768) | (temp<-32768)) {
		return ((temp<0) ? Q16MIN : Q16MAX);
	}

	/* Join parts. */
	r_md += (r_hi & 0xFFFF) << 16;
	r_md += (r_lo >> 16);

	return r_md;
}

/* Q16 divison with precision of 16 bits (1 sign, 7 integer, 8 fractional). Saturated. */
Q16 DIVQ16(Q16 a, Q16 b) {

	if(a==0)
		return 0x00;

	if(b==0)
		printf("DIVQ16: division by zero!");

	uint8_t n, sign;
	uint32_t answer, remainder, divisor;
	Q16 sanswer;

	answer = 0;

	/* Get sign and save it for the end. */
	sign = ((a<0) != (b<0));
	if (sign) {
		a = (a<0 ? -a : a);
		b = (b<0 ? -b : b);
	}

	/* Saturate by limiting input. */
	a = (a>Q16_127 ? a = Q16_127 : a);
	a = (a<Q16_9TH_BIT ? a = Q16_9TH_BIT : a);
	b = (b>Q16_127 ? b = Q16_127 : b);
	b = (b<Q16_9TH_BIT ? b = Q16_9TH_BIT : b);

	/* Begin division. */
	remainder = (uint32_t) a >> 8;
	divisor = (uint32_t) b << 8;

	/* Calculate integer part. */
	for (n = 0; n < 16; n++) {
		divisor = divisor >> 1;
		if (remainder >= divisor) {
			remainder -= divisor;
			answer |= 1 << (31 - n);
		}
	}

	/* Calculate fractional part. */
	for (n = 0; n < 16; n++) {
		remainder = remainder << 1;
		if (remainder >= divisor) {
			remainder -= divisor;
			answer |= 1 << (15 - n);
		}
	}

	/* Insert sign. */
	sanswer = (sign ? TOGQ16((Q16) answer) : (Q16) answer);
	return sanswer;
}

/* Square root. */
Q16 SQTQ16(Q16 number) {

	/* Get and toggle sign. */
	uint8_t sign;
	if(sign = (number<0))
		number=TOGQ16(number);

	int64_t tempOp = number;
	tempOp <<= 16;
	int64_t tempOut = 0;
	int64_t tempOne = 0x4000000000000000ULL;

	while(tempOne > tempOp)
		tempOne >>= 2;

	while(tempOne != 0) {
		if(tempOp >= tempOut + tempOne) {
			tempOp -= tempOut + tempOne;
			tempOut += tempOne << 1;
		}
		tempOut >>= 1;
		tempOne >>= 2;
	}

	if(sign)
		return TOGQ16(tempOut);
	else
		return tempOut;
}


/* Alt. square root (doesn't work). */
Q16 _SQTQ16(Q16 number) {

	/* Get sign. */
	uint8_t sign = (number<0);

	/* Remove sign. */
	number=TOGQ16(number);

	uint32_t root, remHi, remLo, testDiv, count;

	root = 0;
	remHi = 0;
	remLo = number;
	count = 30;

	/* Calc root. */
	do {
		remHi = (remHi<<2) | (remLo>>30);
		remLo <<= 2;

		#ifdef PRINT
			printf("\n\ncount:\t%d", count);
			printf("\nremHi:\t");
			printbin(remHi, 32);
			printf("\t%d", remHi);
			printf("\nremLo:\t");
			printbin(remLo, 32);
			printf("\t%d", remLo);
		#endif

		root <<= 1;
		testDiv = (root << 1) + 1;

		#ifdef PRINT
			printf("\ntestD:\t");
			printbin(testDiv, 32);
			printf("\t%d", testDiv);
		#endif

		if (remHi >= testDiv) {

			remHi -= testDiv;
			root++;

			#ifdef PRINT
				printf("\nremHi:\t");
				printbin(remHi, 32);
				printf("\t%d", remHi);
				printf("\nroot:\t");
				printbin(root, 32);
				printf("\t%d", remLo);
			#endif
		}
	} while (count-- != 0);

	/* Shift to Q16 and insert sign. */
	root>>=7;
	root *= (-sign);

	#ifdef PRINT
		printf("\n\nroot:\t");
		PRINTQ16(root);
	#endif
}


/* Return integer part by zeroing out fractional part. Add one for negative numbers. */
Q16 INTQ16(Q16 number) {

	if (number<0)
		return (number&0xFFFF0000)+0x00010000;
	return (number&0xFFFF0000);
}

/* Return fractional part by subtracting integer part. */
Q16 FRAQ16(Q16 number) {

	return (number-INTQ16(number));
}

/* Q16 floating point representation printer */
void PRINTQ16(Q16 number) {

	uint32_t i;
	int16_t integer=0;
	double fraction=0;

	integer=(number>>16);

	for(i=0; i<16; i++) {
		if (number&(1<<i)) {
			fraction+=(double) 1/(1<<(16-i));
		}
	}

	printf("%f\n", ((double) integer)+fraction);

}

/* Q16 floating point representation sprinter */
int SPRINTQ16(Q16 number, char *q16_string) {

	uint32_t i;
	int16_t integer=0;
	double fraction=0;

	integer=(number>>16);

	for(i=0; i<16; i++) {
		if (number&(1<<i)) {
			fraction+=(double) 1/(1<<(16-i));
		}
	}

	return sprintf(q16_string, "%f", ((double) integer)+fraction);
}

/* Convert int16 to Q16 */
Q16 int16_t2Q16(int16_t number) {

	uint8_t sign = (number<0);
	if (sign)
		number*=-1;

	if (number>-32768)
		return MAKEQ16(sign, 0, number<<1);
	else
		return Q16_MINUS1;
}

/* Convert Q16 to int16. */
int16_t Q162int16_t(Q16 number) {

	return (int16_t) (FRAQ16(number)>>1);
}


/* Binary representation printer for debugging */
void printbin(int number, uint8_t size) {

	uint8_t i;
	char buf[size];

	for(i=0;i<size;i++) {
		if (number&(1<<i))
			buf[size-i-1]='1';
		else
			buf[size-i-1]='0';
	}

	for(i=0;i<size;i++) {
		printf("%c", buf[i]);
	}
}
