
/*
	ADPCM encoder and decoder, not including LPC calculation.
	2012 Even brenden
*/

#include <stdio.h>
#include "adpcm.h"

// #define PRINT

void adpcm_encode_Q16(Q16 *x, Q16 *a, Q16 *x_hat, Q16 *x_tilde, int8_t *xt_pointer, Q16 *prev_delta, Q16 *prev_e, int32_t *codeword) {

	#ifdef PRINT
	printf("\nadpcm_encode_Q16\n");
    printf("\nprev_e:\t\t");
	PRINTQ16(*prev_e);
    printf("prev_delta:\t");
	PRINTQ16(*prev_delta);
	#endif

		Q16 *delta=malloc(sizeof(Q16));
		Q16 *e=malloc(sizeof(Q16));

	#ifndef SSM_AQB
        caqb_Q16(prev_e, prev_delta, delta);
		caqb_encode_Q16(*x-*x_hat, delta, codeword);
		caqb_decode_Q16(delta, codeword, e);
		*prev_e=*e;
	#else
		maqb_Q16(prev_delta, prev_e, delta);
		maqb_encode_Q16(*x-*x_hat, delta, codeword);
		maqb_decode_Q16(delta, codeword, e);
		*prev_e=*codeword;
	#endif

		*prev_delta=*delta;

        x_tilde[*xt_pointer]=*e+*x_hat;

	#ifdef PRINT
    printf("e:\t\t");
	PRINTQ16(*e);
    printf("delta:\t\t");
	PRINTQ16(*delta);
    printf("codeword:\t%d\n", *codeword);
	printf("x:\t\t");
	PRINTQ16(*x);
	printf("x_hat:\t\t");
	PRINTQ16(*x_hat);
	printf("x-x_hat:\t");
	PRINTQ16(*x-*x_hat);
	printf("xt_pointer:\t%d\n", *xt_pointer);
	int i;
	for(i=0;i<P;i++) {
		printf("x_tilde[%d]:\t", i);
		PRINTQ16(x_tilde[i]);
	}
	#endif


        *x_hat=0;

	uint8_t k;
        for(k=0;k<P;k++) {
		#ifdef PRINT
		printf("a[%d]*x_tilde[%d]:", k, modulo((*xt_pointer-k), P));
		PRINTQ16(MULQ16(a[k], x_tilde[modulo((*xt_pointer-k), P)]));
		#endif
                *x_hat+=MULQ16(a[k], x_tilde[modulo((*xt_pointer-k), P)]);
        }

	#ifdef PRINT
	printf("x_hat:\t\t");
	PRINTQ16(*x_hat);
	#endif

	*xt_pointer=(*xt_pointer+1)%P;
}

void adpcm_decode_Q16(Q16 *codeword, Q16 *a, Q16 *s_hat, Q16 *s_tilde, int8_t *st_pointer, Q16 *prev_delta, Q16 *prev_e, Q16 *y) {

	#ifdef PRINT
	printf("\nadpcm_decode_Q16\n");
    printf("\nprev_e:\t\t");
	PRINTQ16(*prev_e);
    printf("prev_delta:\t");
	PRINTQ16(*prev_delta);
	#endif

	Q16 *delta=malloc(sizeof(Q16));
	Q16 *e=malloc(sizeof(Q16));

	#ifndef SSM_AQB
		caqb_Q16(prev_e, prev_delta, delta);
		caqb_decode_Q16(delta, codeword, e);
		*prev_e=*e;
	#else
		maqb_Q16(prev_delta, prev_e, delta);
		maqb_decode_Q16(delta, codeword, e);
		*prev_e=*codeword;
	#endif

	*prev_delta=*delta;

	s_tilde[*st_pointer]=*e+*s_hat;
	*y = s_tilde[*st_pointer];

	*s_hat=0;

	uint8_t k;
	for(k=0;k<P;k++) {
                *s_hat+=MULQ16(a[k], s_tilde[modulo(*st_pointer-k, P)]);
        }

	*st_pointer=(*st_pointer+1)%P;

	#ifdef PRINT
    printf("e:\t\t");
	PRINTQ16(*e);
    printf("delta:\t\t");
	PRINTQ16(*prev_delta);
    printf("codeword:\t%d\n", *codeword);
	printf("y:\t\t");
	PRINTQ16(*y);
	#endif
}

void dpcm_encode(float *x, uint8_t p, float *a, int8_t dbits, float scale, float *x_hat, float *x_tilde, int8_t *xt_pointer, float *d) {

	#ifdef PRINT
	printf("\nx: %f\n", *x);
	printf("x_hat: %f\n", *x_hat);
	printf("x-x_hat: %f\n", *x-*x_hat);
	#endif

        *d = midrise(*x-*x_hat, scale);
	/* Done, rest is for next step */

        x_tilde[*xt_pointer]=*d+*x_hat;

	#ifdef PRINT
	printf("xt_pointer: %d\n", *xt_pointer);
	printf("d (after q): %f\n", *d);
	int i;
	for(i=0;i<p;i++)
		printf("x_tilde[%d]: %f\n", i, x_tilde[i]);
	#endif

        *x_hat=0;

	/* Note the minus sign in front of the a's! */
	uint8_t k;
        for(k=0;k<p;k++) {
		#ifdef PRINT
		printf("a[%d]*x_tilde[%d]: %f\n", k, modulo((*xt_pointer-k), p), a[k]*x_tilde[(*xt_pointer-k)%p]);
		#endif
                *x_hat+=a[k]*x_tilde[modulo((*xt_pointer-k), p)];
        }

	#ifdef PRINT
	printf("x_hat: %f\n\n", *x_hat);
	#endif

	*xt_pointer=(*xt_pointer+1)%p;
}

void dpcm_decode(float *x, uint8_t p, float *a, float *s_hat, float *s_tilde, int8_t *st_pointer, float *d) {

        s_tilde[*st_pointer]=*x+*s_hat;
        *d = s_tilde[*st_pointer];

        *s_hat=0;
	uint8_t k;
	for(k=0;k<p;k++) {
                *s_hat+=a[k]*s_tilde[modulo(*st_pointer-k, p)];
        }

	*st_pointer=(*st_pointer+1)%p;
}

void dpcm_encode_Q16(Q16 *x, uint8_t p, Q16 *a, Q16 dbits, Q16 *x_hat, Q16 *x_tilde, int8_t *xt_pointer, int32_t *d) {

        *d = midrise_Q16(*x-*x_hat);

        x_tilde[*xt_pointer]=*d+*x_hat;

	#ifdef PRINT
	printf("\nx:\t\t");
	PRINTQ16(*x);
	printf("x_hat:\t\t");
	PRINTQ16(*x_hat);
	printf("x-x_hat:\t");
	PRINTQ16(*x-*x_hat);
	printf("xt_pointer:\t%d\n", *xt_pointer);
	printf("d (after q):\t");
	PRINTQ16(*d);
	int i;
	for(i=0;i<p;i++) {
		printf("x_tilde[%d]:\t", i);
		PRINTQ16(x_tilde[i]);
	}
	#endif

        *x_hat=0;

	uint8_t k;
        for(k=0;k<p;k++) {
		#ifdef PRINT
		printf("a[%d]*x_tilde[%d]:", k, modulo((*xt_pointer-k), p));
		PRINTQ16(MULQ16(a[k], x_tilde[modulo((*xt_pointer-k), p)]));
		#endif
                *x_hat+=MULQ16(a[k], x_tilde[modulo((*xt_pointer-k), p)]);
        }

	#ifdef PRINT
	printf("x_hat:\t\t");
	PRINTQ16(*x_hat);
	#endif

	*xt_pointer=(*xt_pointer+1)%p;
}

void dpcm_decode_Q16(Q16 *x, uint8_t p, Q16 *a, Q16 *s_hat, Q16 *s_tilde, int8_t *st_pointer, Q16 *d) {

	s_tilde[*st_pointer]=*x+*s_hat;
	*d = s_tilde[*st_pointer];

	*s_hat=0;

	uint8_t k;
	for(k=0;k<p;k++) {
                *s_hat+=MULQ16(a[k], s_tilde[modulo(*st_pointer-k, p)]);
        }

	*st_pointer=(*st_pointer+1)%p;
}
