
/*
	Complex AQB encoder.
	2012 Even Brenden
*/

#include <stdio.h>
#include "caqb.h"

// #define PRINT

void caqb_Q16(Q16 *prev_y, Q16 *prev_delta, Q16 *delta) {

	#ifdef PRINT
	printf("prev_y:\t\t");
	PRINTQ16(*prev_y);
	printf("prev_delta:\t");
	PRINTQ16(*prev_delta);
	#endif

	(*delta) = DIVQ16(*prev_y, *prev_delta);
	#ifdef PRINT
	PRINTQ16(*delta);
	#endif
	(*delta) = MULQ16(*delta, *delta);
	#ifdef PRINT
	PRINTQ16(*delta);
	#endif
	(*delta) = MULQ16(AQB_CONST_Q16, *delta)+ALPHA;
	#ifdef PRINT
	PRINTQ16(*delta);
	#endif
	(*delta) = SQTQ16(*delta);
	#ifdef PRINT
	PRINTQ16(*delta);
	#endif
	(*delta) = MULQ16(*delta, *prev_delta);
	#ifdef PRINT
	PRINTQ16(*delta);
	printf("a-delta:\t");
	PRINTQ16(*delta);
	fflush(stdout);
	#endif

	/* Prevent zero delta. */
	if(*delta<DELTA_MIN)
		*delta=DELTA_MIN;

	/* Limit delta. */
	if(*delta>DELTA_MAX)
		*delta=DELTA_MAX;
}

void caqb_encode_Q16(Q16 x, Q16 *delta, int32_t *codeword) {

	(*codeword)=DIVQ16(x, *delta)>>16;

	/* Shift codeword from [-2^(bits-1), 2^(bits-1)-1] to [0, 2^bits-1] for easier packing. */
	(*codeword)+=(1<<(DBITS-1));

	/* Limit codeword to [0, 2^DBITS-1]. */
	if ((*codeword)<0) *codeword=0;
	if ((*codeword)>((1<<DBITS)-1)) *codeword=((1<<DBITS)-1);

	#ifdef PRINT
    printf("e-delta:\t");
	PRINTQ16(*delta);
    printf("e-x:\t\t");
	PRINTQ16(x);
	printf("e-codeword:\t%d\n", *codeword);
	fflush(stdout);
	#endif
}

void caqb_decode_Q16(Q16 *delta, int32_t *codeword, Q16 *y) {

	(*y) = MULQ16(*delta, (*codeword-(1<<(DBITS-1)))<<16);

	#ifdef PRINT
    printf("d-delta:\t");
	PRINTQ16(*delta);
	printf("d-codeword:\t%d\n", *codeword);
    printf("d-y:\t\t");
	PRINTQ16(*y);
	fflush(stdout);
	#endif

	/* Limit y. */
	if((*y<0) && (TOGQ16(*y)>Q16_1)) *y=Q16_M1;
	else if(*y>=Q16_1) *y=Q16_1;
}