
/*
	All aspects of encoding and decoding.
	2011 Even Brenden
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "Q16.h"
#include "maths.h"
#include "l-d.h"
#include "adpcm.h"
#include "coding.h"
#include "config.h"

// #define PRINT

/* Encoder and decoder for N size block of samples. */

/* Global variables */
Q16 *x_tilde_Q16, *s_tilde_Q16, *x_hat_Q16, *s_hat_Q16, *prev_delta_enc_Q16, *prev_e_enc_Q16, *prev_delta_dec_Q16, *prev_e_dec_Q16;
int8_t *xt_pointer_Q16, *st_pointer_Q16;
float *x_tilde, *s_tilde, *x_hat, *s_hat;
int8_t *xt_pointer, *st_pointer;

/* For initializing global vars. */
void init_aqb_coder_Q16() {

	x_tilde_Q16 = malloc(P*sizeof(Q16));
	s_tilde_Q16 = malloc(P*sizeof(Q16));
	x_hat_Q16 = malloc(sizeof(Q16));
	s_hat_Q16 = malloc(sizeof(Q16));
	xt_pointer_Q16 = malloc(sizeof(int8_t));
	st_pointer_Q16 = malloc(sizeof(int8_t));
	prev_delta_enc_Q16 = malloc(sizeof(Q16));
	prev_e_enc_Q16 = malloc(sizeof(Q16));
	prev_delta_dec_Q16 = malloc(sizeof(Q16));
	prev_e_dec_Q16 = malloc(sizeof(Q16));

	int n;
	for(n=0;n<P;n++) {
		x_tilde_Q16[n] = 0;
		s_tilde_Q16[n] = 0;
	}

	*x_hat_Q16 = 0;
	*s_hat_Q16 = 0;
	*xt_pointer_Q16 = 0;
	*st_pointer_Q16 = 0;
	*prev_delta_enc_Q16 = DELTA_MAX;
	#ifndef SSM_AQB
		*prev_e_enc_Q16 = 0;
		*prev_e_dec_Q16 = 0;
	#else
		*prev_e_enc_Q16 = (1<<(DBITS-1)); /* Because prev_e_enc is codeword when using SSM-AQB. */
		*prev_e_dec_Q16 = (1<<(DBITS-1));
	#endif
	*prev_delta_dec_Q16 = DELTA_MAX;
}

/* For initializing global vars. */
void init_coder_Q16() {

	x_tilde_Q16 = malloc(P*sizeof(Q16));
	s_tilde_Q16 = malloc(P*sizeof(Q16));
	x_hat_Q16 = malloc(sizeof(Q16));
	s_hat_Q16 = malloc(sizeof(Q16));
	xt_pointer_Q16 = malloc(sizeof(int8_t));
	st_pointer_Q16 = malloc(sizeof(int8_t));

	int n;
	for(n=0;n<P;n++) {
		x_tilde_Q16[n] = 0;
		s_tilde_Q16[n] = 0;
	}

	*x_hat_Q16 = 0;
	*s_hat_Q16 = 0;
	*xt_pointer_Q16 = 0;
	*st_pointer_Q16 = 0;
}

/* For initializing global vars. */
void init_coder() {

	x_tilde = malloc(P*sizeof(float));
	s_tilde = malloc(P*sizeof(float));
	x_hat = malloc(sizeof(float));
	s_hat = malloc(sizeof(float));
	xt_pointer = malloc(sizeof(int8_t));
	st_pointer = malloc(sizeof(int8_t));

	int n;
	for(n=0;n<P;n++) {
		x_tilde[n] = 0;
		s_tilde[n] = 0;
	}

	*x_hat = 0;
	*s_hat = 0;
	*xt_pointer = 0;
	*st_pointer = 0;
}

/* AQB encoder for Q16. */
void adaptive_encode_Q16(Q16 *samples, Q16 *a, int32_t *codewords) {

		int n;
		for(n=0;n<N;n++) {
			adpcm_encode_Q16(&samples[n], &a[0], x_hat_Q16, &x_tilde_Q16[0], xt_pointer_Q16, prev_delta_enc_Q16, prev_e_enc_Q16, &codewords[n]);
		}
}

/* AQB decoder for Q16. */
void adaptive_decode_Q16(int32_t *codewords, Q16 *a, Q16 *decoded) {

		int n;
		for(n=0;n<N;n++) {
			adpcm_decode_Q16(&codewords[n], &a[0], s_hat_Q16, s_tilde_Q16, st_pointer_Q16, prev_delta_dec_Q16, prev_e_dec_Q16, &decoded[n]);
		}
}

/* Encoder for Q16. */
void encode_Q16(Q16 *samples, Q16 *a, Q16 *residue) {

		/* General init */
		uint16_t n;

		for(n=0;n<N;n++) {
			dpcm_encode_Q16(&samples[n], P, &a[0], DBITS, x_hat_Q16, &x_tilde_Q16[0], xt_pointer_Q16, &residue[n]);
		}
}

/* Decoder for Q16. */
void decode_Q16(Q16 *residue, Q16 *a, Q16 *decoded) {

		uint16_t n;
		for(n=0;n<N;n++) {
			dpcm_decode_Q16(&residue[n], P, &a[0], s_hat_Q16, &s_tilde_Q16[0], st_pointer_Q16, &decoded[n]);
		}
}

/* Encoder for float. */
void encode_float(float *samples, float *a, float *residue) {

		/* General init */
		uint16_t n;
		float rxx[P+1];
		float scale;

		scale = get_scale(&samples[0], N);

		/* ADPCM encoder */
		acf(&samples[0], N, P+1, &rxx[0]);
	        levinson_durbin(P, &rxx[0], &a[0]);

		if(rxx[0]>0) {

			#ifdef PRINT
			printf("\nrxx:\n");
			for(n=0;n<P+1;n++)
				printf("%f\n", rxx[n]);
			printf("\nlpc:\n");
			for(n=0;n<P;n++)
				printf("%f\n", a[n]);
			#endif

			#ifdef PRINT
			printf("\nscale: %f\n", scale);
			#endif

			for(n=0;n<N;n++) {
				// printf("samples: %f", samples[n]);
				dpcm_encode(&samples[n], P, &a[0], DBITS, scale, x_hat, &x_tilde[0], xt_pointer, &residue[n]);
			}
		} else {

			#ifdef PRINT
			printf("\nzero block!\n");
			#endif

			for(n=0;n<N;n++)
				residue[n]=0;
			for(n=0;n<P;n++)
				x_tilde[n]=0;
			*x_hat=0;
		}
}

/* Decoder for float. */

void decode_float(float *residue, float *a, float *decoded) {

		uint16_t n;
		for(n=0;n<N;n++) {
			dpcm_decode(&residue[n], P, &a[0], s_hat, &s_tilde[0], st_pointer, &decoded[n]);
		}
}
