
/*
	Noise gate.
	2011 Even Brenden
*/

#include <stdio.h>
#include "expander.h"
#include "maths.h"

// #define PRINT

/* Global vars. */
int8_t history;
uint8_t F;

void reset_expander(uint8_t att_rel) {

	F = att_rel; /* Number of frames to reach full attenuation and back. */
	history=0;
}

void expander_Q16(Q16 *buffer, uint16_t size, Q16 tresh, Q16 attenuation) {

        int8_t former_history, n;
        uint8_t slope_sign;
        Q16 average, stop_attenuation, start_attenuation;

	Q16 QN = MAKEQ16(0, size, 0);
	Q16 QF = MAKEQ16(0, F, 0);

        /* Compute average of absolute values. */
	average = 0;
	for(n=0;n<size;n++) {
		average += absolute_Q16(buffer[n]);
	}
	average = DIVQ16(average, QN);

	/* Save former history to calc slope. */
	former_history = history;

	/* Determine factor for slope. */
        if (average<=tresh) {
                history+=1;
		if (history>F) {
			history=F;
		}
        } else {
                history-=1;
		if (history<0) {
			history=0;
		}
        }

	#ifdef PRINT
	printf("\nhistory:\t%d", history);
	#endif

	/* Expand. */
        if (history) {

                stop_attenuation = attenuation+MULQ16(Q16_1-DIVQ16(MAKEQ16(0, history, 0), QF), Q16_1-attenuation);
		start_attenuation = attenuation+MULQ16(Q16_1-DIVQ16(MAKEQ16(0, former_history, 0), QF), Q16_1-attenuation); /* Optimize this! */

                slope_sign = history-former_history;

                for(n=0;n<size;n++) {
			Q16 Qn = MAKEQ16(0, n, 0);
                        if (slope_sign!=0) {
                                buffer[n] = MULQ16(buffer[n], MULQ16(DIVQ16(QN-Qn-Q16_1, QN), start_attenuation) + MULQ16(DIVQ16(Qn+Q16_1, QN), stop_attenuation));
                        } else {
                                buffer[n] = MULQ16(buffer[n], stop_attenuation);
                        }
		}
        }
}

void expander(float *buffer, uint16_t size, float tresh, float attenuation) {

        uint8_t former_history, n;
        int8_t slope_sign;
        float avg, stop_attenuation, start_attenuation;

        /* Compute avg of absolute values. Not using maths.c function because it is faster to do absolute() on the fly. */

	avg = 0;
	for(n=0;n<size;n++) {
		avg += absolute(buffer[n]);
	}
	avg /= size;

	former_history = history;

	/* Determine factor for slope */
	if (avg<=tresh) {
			history+=1;
		if (history>F) {
			history=F;
		} else {
				history-=1;
		}
		if (history<0) {
			history=0;
		}
	}

	/* Expand */
	if (history) {
		stop_attenuation = attenuation+(1.0-(float) history/F)*(1.0-attenuation);
		start_attenuation = attenuation+(1.0-(float) former_history/F)*(1.0-attenuation); /* Optimize! */

		slope_sign = history-former_history;

		for(n=0;n<size;n++) {
				if (slope_sign!=0) {
						buffer[n] *= (((float) (size-n-1.0)/size)*start_attenuation + ((float) (n+1.0)/size)*stop_attenuation);
				} else {
						buffer[n] *= stop_attenuation;
				}
		}
	}
}
