
/*
	Levinson-Durbin algorithm. Reflection coefficients are not calulated. LPCs are flipped at the end wrt. usage.
	2011 Even Brenden. Rewritten from http://kbs.cs.tu-berlin.de/~jutta/gsm/lpc.html.
*/

#include <stdio.h>
#include <stdint.h>
#include "l-d.h"

// #define PRINT

Q16 levinson_durbin_Q16(uint8_t p, Q16 *ac, Q16 *lpc) {

	uint8_t i, j;
	Q16 r, error = ac[0];

	if (ac[0]==0) {
		for(i=0;i<p;i++)
			lpc[i]=0;
		return 0;
	}

	for (i=0; i<p; i++) {

	/* Sum up this iteration's reflection coefficient.
	*/
		r = -ac[i+1];
		for (j=0;j<i;j++) {
			r -= MULQ16(lpc[j], ac[i-j]);

			#ifdef PRINT
			printf("\nlpc[%d]:\t", j);
			PRINTQ16(lpc[j]);
			printf("ac[%d]:\t", i-j);
			PRINTQ16(ac[i-j]);;
			printf("MULQ16(lpc[%d], ac[%d]):\t", j, i-j);
			PRINTQ16(MULQ16(lpc[j], ac[i-j]));
			#endif
		}

		/* Avoid divison by zero. */
		if(error==0)
			error=Q16_SMALLEST;

		r=DIVQ16(r, error);

		#ifdef PRINT
		printf("\nr:\t");
		PRINTQ16(r);
		#endif

	/*  Update LPC coefficients and total error.
	*/

		lpc[i] = r;
		for (j=0; j<i/2; j++) {
			float tmp  = lpc[j];
			lpc[j]     += MULQ16(r, lpc[i-1-j]);
			lpc[i-1-j] += MULQ16(r, tmp);
		}
		if (i%2) {
			lpc[j] += MULQ16(lpc[j], r);
		}
		error = MULQ16(error, Q16_1 - MULQ16(r, r));
	}

	#ifdef PRINT
	printf("\nerror:\t");
	PRINTQ16(error);
	#endif

	for(i=0;i<p;i++) {
		lpc[i]*=-1;
	}

	return error;
}

float levinson_durbin(uint8_t p, float const * ac, float * lpc) {

	uint8_t i, j;
	float r, error = ac[0];

	if (ac[0]==0) {
		return 0;
	}

	for (i=0; i<p; i++) {

	/* Sum up this iteration's reflection coefficient.
	*/
		r = -ac[i+1];
		for (j=0; j<i; j++) {
			r -= lpc[j]*ac[i-j];
		}
		r /= error;

	/*  Update LPC coefficients and total error.
	*/

		lpc[i] = r;
		for (j=0; j<i/2; j++) {
			float tmp  = lpc[j];
			lpc[j]     += r*lpc[i-1-j];
			lpc[i-1-j] += r*tmp;
		}
		if (i%2) {
			lpc[j] += lpc[j]*r;
		}
		error *= 1.0-r*r;
	}

        for(i=0;i<p;i++) {
                lpc[i]*=-1;
        }
	return error;
}
