
/*
	One-word-memory AQB for Q16.
	2012 Even Brenden
*/

#include <stdio.h>
#include "maqb.h"

// #define PRINT

/* *codeword must be in [0, 2^bits-1]. */
void maqb_Q16(Q16 *prev_delta, int32_t *prev_codeword, Q16 *delta) {

	/* Shift codeword back from [0, 2^bits-1] to [-2^(bits-1), 2^(bits-1)-1] and then to Q16 for decoding. */
	(*prev_codeword)-=(1<<(DBITS-1));

	/* So that negative values will index MX[] correctly. */
	if (*prev_codeword<0) (*prev_codeword)=-(*prev_codeword)-1;

	#if DBITS == 4
		*delta=MULQ16((*prev_delta), M4[*prev_codeword]);
	#endif

	#if DBITS == 3
		*delta=MULQ16((*prev_delta), M3[*prev_codeword]);
	#endif

	#if DBITS == 2
		*delta=MULQ16((*prev_delta), M2[*prev_codeword]);
	#endif

	/* Prevent zero delta. */
	if(*delta<DELTA_MIN)
		*delta=DELTA_MIN;

	/* Limit delta. */
	if(*delta>DELTA_MAX)
		*delta=DELTA_MAX;

	#ifdef PRINT
		printf("\nprev_delta:\t");
		PRINTQ16(*prev_delta);
		printf("prev_cw:\t%d\n", *prev_codeword);
		printf("M4[prev_cw]:\t");
		PRINTQ16(M4[*prev_codeword]);
		printf("delta:\t\t");
		PRINTQ16(*delta);
	#endif
}

void maqb_encode_Q16(Q16 x, Q16 *delta, int32_t *codeword) {

	/* Calculate codeword. */
	*codeword=DIVQ16(x, *delta)>>16;

	/* Shift codeword from [-2^(bits-1), 2^(bits-1)-1] to [0, 2^bits-1] for easier packing. */
	(*codeword)+=(1<<(DBITS-1));

	/* Limit codeword to [0, 2^DBITS-1]. */
	if (*codeword<0L) *codeword=0L;
	if (*codeword>((1<<DBITS)-1)) *codeword=((1<<DBITS)-1);

	#ifdef PRINT
		printf("\nx:\t\t");
		PRINTQ16(x);
		printf("delta:\t\t");
		PRINTQ16(*delta);
		printf("codeword:\t%d\n", *codeword);
	#endif
}

void maqb_decode_Q16(Q16 *delta, int32_t *codeword, Q16 *y) {

	/* Calculate reconstructed value. Need to shift codeword back from [0, 2^bits-1] to [-2^(bits-1), 2^(bits-1)-1] and then to Q16 for decoding. */
	*y = MULQ16(*delta, (*codeword-(1<<(DBITS-1)))<<16);

	#ifdef PRINT
		printf("\ny:\t\t");
		PRINTQ16(*y);
	#endif
}
