
/*
	Midtread quantizer.
	2011 Even Brenden
*/

#include "midrise.h"

// #define PRINT

#define DROP 1.11 // Drop the A% in 1/(1-A) of the dynamic range of the signal when quantizing. (Awkward formula is for speed.). A as of now is approx. 0.1.
# define DROP_Q16 0x11C00 // Current Q16 equals 1.094, which gives a A of approx. 0.1.

Q16 midrise_Q16(Q16 input) {

	Q16 result, shifted;
	int shift;

	/* Calculate shift for truncation. */
	shift = 16-DBITS+1;
	shift = (shift<0 ? shift=0 : shift);

	/* Truncate. */
	shifted = input>>shift;
	result = shifted<<shift;

	#ifdef PRINT
	printf("\ninput:\t\t");
	PRINTQ16(input);
	printf("\nshift:\t\t%d\n", shift);
	printf("\nresult:\t\t");
	PRINTQ16(result);
	#endif

	return result;
}

float midrise(float input, float scale) {

	float result;
	int32_t integer, shifted, quantized;
	uint8_t sbits;
	int8_t shift;

	/* Constrain dynamic range. */
	sbits = (uint8_t) log_2(scale*DROP);

	/* Calculate shift for truncation. */
	shift = 16-DBITS-sbits+1;
	shift = (shift<0 ? shift=0 : shift);

	/* Truncate. */
	input *= 65536.0;
	integer = (int32_t) input;
	shifted = integer>>shift;
	quantized = shifted<<shift;
	result = (float) quantized;
	result /= 65536.0;

	#ifdef PRINT
	printf("\ninput:\t\t%f", input);
	printf("\nscale:\t\t%f", scale);
	printf("\nsbits:\t\t%d", sbits);
	printf("\nshift:\t\t%d", shift);
	printf("\ninteger:\t%d", integer);
	printf("\nshifted:\t%d", shifted);
	printf("\nq-tized:\t%d", quantized);
	printf("\nresult:\t\t%f\n", result);
	#endif

	return result;
}

