
/*
	This is a set of tests for each of the major functions of the system, as well as full system tests.
	They are merely tests by observation, and do not represent a proper testing framework.
	Each function has a test for floating point numbers and an equivalent Q15.16 test.

	2011 Even Brenden
*/

#include <stdio.h>
#include <stdint.h>
#include "expander.h"
#include "maths.h"
#include "midrise.h"
#include "l-d.h"
#include "coding.h"
#include "config.h"
#include "aqb.h"

/* For analyzing number formats in raw audio files. */
void analyze_raw_audio() {

        FILE *file_in;
        file_in = fopen("testaudio.raw", "rb");

        fseek(file_in, 0L, SEEK_END);
        long size = ftell(file_in);
        size /= sizeof(int16_t);
        fseek(file_in, 0L, SEEK_SET);

       	int16_t  input_buffer[size];
        fread(&input_buffer[0], sizeof(int16_t), size, file_in);
        fclose(file_in);

	printf("\nint16 input:\n");
	int n;
	for(n=0;n<size;n++) {
		printbin(input_buffer[n], sizeof(int16_t)*8);
		printf("%d\n", input_buffer[n]);
	}

	printf("\nQ16:\n");
	/* Convert to Q16 */
	for(n=0;n<size;n++) {
		printbin(int16_t2Q16(input_buffer[n]), sizeof(int32_t)*8);
		PRINTQ16(int16_t2Q16(input_buffer[n]));
	}

	printf("\nint16 again:\n");
	/* Convert back to int16 */
	for(n=0;n<size;n++) {
		printbin(Q162int16_t(int16_t2Q16(input_buffer[n])), sizeof(int16_t)*8);
		printf("%d\n", Q162int16_t(int16_t2Q16(input_buffer[n])));
	}
	printf("\n");
}

/* Full adaptive system test. */

#define ATEST_NQ 4

void system_Q16_adaptive() {

	Q16 samples[ATEST_NQ] =
	{
		Q16_0P5,
		Q16_0P5,
		Q16_0P5,
		Q16_0P5
	};

	int32_t codewords[ATEST_NQ];
	Q16 decoded[ATEST_NQ];
	Q16 lpc = A_1;

	/* Go! */

	int n;
	printf("\npre-test:\n");
	for(n=0;n<ATEST_NQ;n++) {
		PRINTQ16(samples[n]);
	}

	init_aqb_coder_Q16();
	for(n=0;n<ATEST_NQ;n++) {
		adaptive_encode_Q16(&samples[n], &lpc, &codewords[n]);
	}
	for(n=0;n<ATEST_NQ;n++) {
		adaptive_decode_Q16(&codewords[n], &lpc, &decoded[n]);
	}

	printf("\npost-C:\n");
	for(n=0;n<ATEST_NQ;n++) {
		PRINTQ16(decoded[n]);
	}

	printf("\n");
}

/* Full system test. */

#define TEST_NQ 4
#define BLOCK_SIZEQ 4
#define PQ 1
#define DBITSQ 16

void system_Q16_() {

		Q16 samples[TEST_NQ] = {MAKEQ16(0, 1, 0),
					MAKEQ16(0, 1, 0),
					MAKEQ16(0, 1, 0),
					MAKEQ16(0, 1, 0)
					};

	        Q16 residue[TEST_NQ];
		Q16 decoded[TEST_NQ];
		Q16 lpc[PQ];

		/* Make sure all new arrays are set to zero */
		int z;
		for(z=0;z<TEST_NQ;z++) {
			residue[z]=0;
			decoded[z]=0;
		}

                Q16 rxx[PQ+1];
                Q16 scale;

		Q16 x_tilde[PQ], s_tilde[PQ];
		Q16 *x_hat, *s_hat;
		int8_t *xt_pointer, *st_pointer;

	        x_hat=malloc(sizeof(Q16));
	        s_hat=malloc(sizeof(Q16));
	        xt_pointer=malloc(sizeof(Q16));
	        st_pointer=malloc(sizeof(Q16));

	        Q16 tresh = MAKEQ16(0, 0, 0);
	        Q16 attenuation = MAKEQ16(0, 0, (1<<15));
	        uint8_t F=2;

		/* Go! */

		reset_expander(F);

		int n, i;
		printf("\npre-test:\n");
		for(n=0;n<TEST_NQ;n++) {
			PRINTQ16(samples[n]);
		}

		/* Tests on block basis */
		reset_expander(F);
		for(n=0;n<TEST_NQ/BLOCK_SIZEQ;n++) {
			expander_Q16(&samples[n*BLOCK_SIZEQ], BLOCK_SIZEQ, tresh, attenuation);
	                acf_Q16(&samples[n*BLOCK_SIZEQ], BLOCK_SIZEQ, PQ+1, &rxx[0]);
	                levinson_durbin_Q16(PQ, &rxx[0], &lpc[0]);
	        	scale = get_scale_Q16(&samples[n*BLOCK_SIZEQ], BLOCK_SIZEQ);
			for(i=n*BLOCK_SIZEQ;i<BLOCK_SIZEQ*(n+1);i++) {
				dpcm_encode_Q16(&samples[i], PQ, &lpc[0], DBITSQ, scale, x_hat, &x_tilde[0], xt_pointer, &residue[n]);
				dpcm_decode_Q16(&residue[i], PQ, &lpc[0], s_hat, &s_tilde[0], st_pointer, &decoded[i]);
			}
		}

		printf("\npost-NR:\n");
		for(n=0;n<TEST_NQ;n++) {
			PRINTQ16(samples[n]);
		}

		printf("\npost-C:\n");
		for(n=0;n<TEST_NQ;n++) {
			PRINTQ16(decoded[n]);
		}

		printf("\n");
}

#define TEST_N 8
#define BLOCK_SIZE 4
#define TEST_P 1
#define TEST_DBITS 16

void system_() {

		float samples[TEST_N] = { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
	        float residue[TEST_N];
		float decoded[TEST_N];

		/* Make sure all new arrays are set to zero */
		int z;
		for(z=0;z<TEST_N;z++) {
			residue[z]=0;
			decoded[z]=0;
		}

	        float tresh = 0.0;
	        float attenuation = 0.5;
	        uint8_t F=2;

	        float rxx[TEST_P+1];
	        float lpc[TEST_P];

		float scale;

	        float x_tilde[TEST_P];
	        float *x_hat=malloc(sizeof(float));
	        int8_t *xt_pointer=malloc(sizeof(float));

	        float s_tilde[TEST_P];
	        float *s_hat=malloc(sizeof(float));
	        int8_t *st_pointer=malloc(sizeof(float));

		int n, i;
		printf("\npre-test:\n");
		for(n=0;n<TEST_N;n++) {
			printf("%f\n", samples[n]);
		}

		/* Tests on block basis */
		reset_expander(F);
		for(n=0;n<TEST_N/BLOCK_SIZE;n++) {
			expander(&samples[n*BLOCK_SIZE], BLOCK_SIZE, tresh, attenuation);
			acf(&samples[n*BLOCK_SIZE], BLOCK_SIZE, TEST_P+1, &rxx[0]);
			levinson_durbin(TEST_P, &rxx[0], &lpc[0]);
			scale = get_scale(&samples[n*BLOCK_SIZE], BLOCK_SIZE);
			for(i=n*BLOCK_SIZE;i<BLOCK_SIZE*(n+1);i++) {
				dpcm_encode(&samples[i], TEST_P, &lpc[0], TEST_DBITS, scale, x_hat, &x_tilde[0], xt_pointer, &residue[i]);
				dpcm_decode(&residue[i], TEST_P, &lpc[0], s_hat, &s_tilde[0], st_pointer, &decoded[i]);
			}
		}

		printf("\npost-NR:\n");
		for(n=0;n<TEST_N;n++) {
			printf("%f\n", samples[n]);
		}

		printf("\npost-C:\n");
		for(n=0;n<TEST_N;n++) {
			printf("%f\n", decoded[n]);
		}

		printf("\n");
}


/* Levinson-Durbin test. */

#define P_MAX 3

void ld_Q16_() {

	Q16 acf[P_MAX+1] = {MAKEQ16(0, 1, 0), MAKEQ16(0, 0, (1<<15)), MAKEQ16(0, 0, (1<<14)), MAKEQ16(0, 0, (1<<13))};
	Q16 lpc[P_MAX];
	Q16 error;

	levinson_durbin_Q16(P_MAX, &acf[0], &lpc[0]);

	printf("\nrxx:\n");
	uint8_t i;
	for(i=0; i<P_MAX+1; i++) {
		PRINTQ16(acf[i]);
	}
	printf("\nlpc:\n");
	for(i=0; i<P_MAX; i++) {
		PRINTQ16(lpc[i]);
	}
	printf("\nerror:");
	PRINTQ16(error);
	printf("\n\n");
}

void ld_() {

	float const acf[P_MAX+1] =  {1.00000, 0.50000, 0.25000, 0.12500};
	float lpc[P_MAX];
	float error;

	error = levinson_durbin(P_MAX, &acf[0], &lpc[0]);

	printf("rxx\t\tLPC\n");
	uint8_t i;
	for(i=0; i<P_MAX; i++) {
		printf("%f\t%f\n", acf[i], lpc[i]);
	}
	printf("%f\nerror: %f\n", acf[P_MAX], error);
}


/* ADPCM encoding and decoding test. */

#define ADPCM_SIZE 6
#define ADPCM_P 3

void adpcm_Q16_() {

	Q16 x[ADPCM_SIZE] = {	MAKEQ16(1, 0, (1<<15)),
				MAKEQ16(1, 0, (1<<14)),
				MAKEQ16(1, 0, (1<<13)),
				MAKEQ16(0, 0, 0),
				MAKEQ16(0, 0, (1<<14)),
				MAKEQ16(0, 0, (1<<15))
				};
	Q16 d[ADPCM_SIZE];
	Q16 dec[ADPCM_SIZE];
	uint8_t dbits = 14;
	Q16 scale;
	scale = get_scale_Q16(&x[0], ADPCM_SIZE);

	uint8_t p=ADPCM_P;
	Q16 a[ADPCM_P] = {MAKEQ16(0, 0, (1<<15)), MAKEQ16(1, 0, (1<<15)), MAKEQ16(0, 0, (1<<13))};
	Q16 x_tilde[ADPCM_P] = {0, 0, 0};
	Q16 s_tilde[ADPCM_P] = {0, 0, 0};

	Q16 *x_hat=malloc(sizeof(Q16));
	int8_t *xt_pointer=malloc(sizeof(Q16));
	Q16 *s_hat=malloc(sizeof(Q16));
	int8_t *st_pointer=malloc(sizeof(Q16));
	*x_hat=0;
	*xt_pointer=0;
	*s_hat=0;
	*st_pointer=0;

	uint8_t n;
	printf("\ninput:\n");
	for(n=0;n<ADPCM_SIZE;n++) {
		PRINTQ16(x[n]);
	}
	printf("\nencoded:\n");
	for(n=0;n<ADPCM_SIZE;n++) {
		dpcm_encode_Q16(&x[n], p, &a[0], dbits, scale, x_hat, &x_tilde[0], xt_pointer, &d[n]);
		PRINTQ16(d[n]);
	}
	printf("\ndecode:\n");
	for(n=0;n<ADPCM_SIZE;n++) {
		dpcm_decode_Q16(&d[n], p, &a[0], s_hat, &s_tilde[0], st_pointer, &dec[n]);
		PRINTQ16(dec[n]);
	}
}

void dpcm_() {

	float x[ADPCM_SIZE] = {-0.5, -0.25, -0.125, 0.0, 0.25, 0.5};
	float d[ADPCM_SIZE];
	float dec[ADPCM_SIZE];
	uint8_t dbits = 14;
	float scale = 1/0.7;

	uint8_t p=ADPCM_P;
	float a[ADPCM_P] = {0.5, -0.5, 0.125};
	float x_tilde[ADPCM_P] = {0.0, 0.0, 0.0};
	float s_tilde[ADPCM_P] = {0.0, 0.0, 0.0};

	float *x_hat=malloc(sizeof(float));
	int8_t *xt_pointer=malloc(sizeof(float));
	float *s_hat=malloc(sizeof(float));
	int8_t *st_pointer=malloc(sizeof(float));
	*x_hat=0;
	*xt_pointer=0;
	*s_hat=0;
	*st_pointer=0;

	uint8_t n;
	printf("\n");
	printf("encode:\nin\t\tout\n");
	for(n=0;n<ADPCM_SIZE;n++) {
		dpcm_encode(&x[n], p, &a[0], dbits, scale, x_hat, &x_tilde[0], xt_pointer, &d[n]);
		printf("%f\t%f\n", x[n], d[n]);
	}

	printf("\n");
	printf("decode:\nin\t\tout\n");
	uint8_t m;
	for(m=0;m<ADPCM_SIZE;m++) {
		dpcm_decode(&d[m], p, &a[0], s_hat, &s_tilde[0], st_pointer, &dec[m]);
		printf("%f\t%f\n", d[m], dec[m]);
	}
}


/* Midtread quantizer test. */

#define Q_SIZE_Q16 8

void midrise_Q16_() {

	Q16 ten = MAKEQ16(0, 10, 0);

	Q16 samples[Q_SIZE_Q16] = {
					DIVQ16(MAKEQ16(1, 7, 0), ten),
					DIVQ16(MAKEQ16(1, 4, 0), ten),
					DIVQ16(MAKEQ16(1, 2, 0), ten),
					DIVQ16(MAKEQ16(1, 1, 0), ten),
					MAKEQ16(0, 0, 0),
					DIVQ16(MAKEQ16(1, 1, 0), ten),
					DIVQ16(MAKEQ16(1, 2, 0), ten),
					DIVQ16(MAKEQ16(0, 4, 0), ten),
				};
	Q16 scale = MAKEQ16(0, 2, 0);

	uint8_t n;
	Q16 result;
	printf("in:\n");
	for(n=0;n<Q_SIZE_Q16;n++) {
		PRINTQ16(samples[n]);
	}
	printf("out:\n");
	for(n=0;n<Q_SIZE_Q16;n++) {
		result = midrise_Q16(samples[n], scale);
		PRINTQ16(result);
	}
	printf("\n");
}

#define Q_SIZE 10

void midrise_() {

	float samples[Q_SIZE] = {-0.5, -0.4, -0.3, -0.2, -0.1, 0.0, 0.1, 0.2, 0.3, 0.4};
	float scale = 2;

	int n;
	float result;
	printf("in\t\tout\n");
	for(n=0;n<Q_SIZE;n++) {
		result = midrise(samples[n], scale);
		printf("%f\t%f\n", samples[n], result);
	}
	printf("\n");
}


/* Autocorrelation calculation test. */

#define ACF_SIZE 4
#define ACF_LAG 4

void acf_Q16_() {

	Q16 samples[ACF_SIZE] = {MAKEQ16(0, 1, 0), MAKEQ16(0, 0, (1<<15)), MAKEQ16(0, 0, (1<<14)), MAKEQ16(0, 0, (1<<13))};

	Q16 rxx[ACF_LAG];

	acf_Q16(&samples[0], ACF_SIZE, ACF_LAG, &rxx[0]);

	printf("\n");
	int n=0;
	for(n=0;n<ACF_SIZE;n++) {
		PRINTQ16(rxx[n]);
	}
	printf("\n");
}

void acf_() {

	float samples[ACF_SIZE] = {1.0, 0.5, 0.25, 0.125};

	float rxx[ACF_LAG];
	uint8_t lag = ACF_LAG;

	acf(&samples[0], ACF_SIZE, lag, &rxx[0]);

	int n=0;
	for(n=0;n<ACF_SIZE;n++) {
		printf("%f\t", rxx[n]);
	}
	printf("\n");
}


/* Test for find scaling constant. */

void get_scale_Q16_() {

	Q16 ten = MAKEQ16(0, 10, 0);

	Q16 samples[4] = {	MAKEQ16(0, 0, (1<<15)),
				MAKEQ16(0, 0, (1<<15)|(1<<14)),
				MAKEQ16(0, 0, 0),
				DIVQ16(MAKEQ16(1, 8, 0), ten)
				};

	Q16 scale;

	scale = get_scale_Q16(&samples[0], 4);

	PRINTQ16(scale);
}

void get_scale_() {

	float samples[4] = {0.5, 0.75, 0.0, -0.8};
	float result;

	result = get_scale(&samples[0], 4);

	printf("\n%f\n", result);
}


/* Expander test. */

#define EXPANDER_SIZE 12

void expander_Q16_() {

	Q16 half = MAKEQ16(0, 0, (1<<15));
	Q16 one = MAKEQ16(0, 1, 0);

	Q16 samples[EXPANDER_SIZE] = {half, half, half, half, half, half, one, one, one, one, one, one};
	Q16 tresh = MAKEQ16(0, 0, (1<<15)|(1<<14)); // 0.75
	Q16 attenuation = half;
	uint8_t F=2;

	int n;
	printf("\ninput:\n");
	for (n=0;n<EXPANDER_SIZE;n++) {
		PRINTQ16(samples[n]);
	}

	reset_expander(F);

	expander_Q16(&samples[0], 2, tresh, attenuation);
	expander_Q16(&samples[2], 2, tresh, attenuation);
	expander_Q16(&samples[4], 2, tresh, attenuation);
	expander_Q16(&samples[6], 2, tresh, attenuation);
	expander_Q16(&samples[8], 2, tresh, attenuation);
	expander_Q16(&samples[10], 2, tresh, attenuation);

	printf("\noutput:\n");
	for (n=0;n<EXPANDER_SIZE;n++) {
		PRINTQ16(samples[n]);
	}
	printf("\n");
}

void expander_() {

	float samples[EXPANDER_SIZE] = {0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0};
	float tresh = 0.75;
	float attenuation = 0.5;
	uint8_t F=2;

	int n;
	printf("\ninput:\t");
	for (n=0;n<EXPANDER_SIZE;n++) {
		printf("%f\t", samples[n]);
	}

	reset_expander(F);

	expander(&samples[0], 2, tresh, attenuation);
	expander(&samples[2], 2, tresh, attenuation);
	expander(&samples[4], 2, tresh, attenuation);
	expander(&samples[6], 2, tresh, attenuation);
	expander(&samples[8], 2, tresh, attenuation);
	expander(&samples[10], 2, tresh, attenuation);

	printf("\noutput:\t");
	for (n=0;n<EXPANDER_SIZE;n++) {
		printf("%f\t", samples[n]);
	}
	printf("\n");
}

#define ASIG 0
#define AINT 0
#define AFRA (1<<15)
#define BSIG 0
#define BINT 0
#define BFRA 0x1E5A
#define CSIG 0
#define CINT 0
#define CFRA 0xFFFF
#define DSIG 1
#define DINT 0
#define DFRA 1

/* Q15.16 arithmetics test. */
void Q16_() {

	Q16 a = MAKEQ16(ASIG, AINT, AFRA);
	Q16 b = MAKEQ16(BSIG, BINT, BFRA);
	Q16 c = MAKEQ16(CSIG, CINT, CFRA);
	Q16 d = MAKEQ16(DSIG, DINT, DFRA);

	Q16 e = TOGQ16(a);
	Q16 f = TOGQ16(b);
	Q16 g = TOGQ16(c);
	Q16 h = TOGQ16(d);

	Q16 i = ADDQ16(a, b);
	Q16 j = ADDQ16(c, d);

	Q16 k = SUBQ16(a, b);
	Q16 l = SUBQ16(c, d);

	Q16 m = MULQ16(a, b);
	Q16 n = MULQ16(c, d);
	Q16 y = MULQ16(a, c);
	Q16 z = MULQ16(b, d);

	Q16 o = DIVQ16(a, b);
	Q16 p = DIVQ16(c, d);
	Q16 ae = DIVQ16(a, c);
	Q16 oe = DIVQ16(b, d);

	Q16 q = INTQ16(a);
	Q16 r = FRAQ16(a);
	Q16 s = INTQ16(b);
	Q16 t = FRAQ16(b);
	Q16 u = INTQ16(c);
	Q16 v = FRAQ16(c);
	Q16 w = INTQ16(d);
	Q16 x = FRAQ16(d);

	printf("\n");
	printf("a:\t");
	PRINTQ16(a);
	printf("b:\t");
	PRINTQ16(b);
	printf("c:\t");
	PRINTQ16(c);
	printf("d:\t");
	PRINTQ16(d);
	printf("-a:\t");
	PRINTQ16(e);
	printf("-b:\t");
	PRINTQ16(f);
	printf("-c:\t");
	PRINTQ16(g);
	printf("-d:\t");
	PRINTQ16(h);
	printf("a+b:\t");
	PRINTQ16(i);
	printf("c+d:\t");
	PRINTQ16(j);
	printf("a-b:\t");
	PRINTQ16(k);
	printf("c-d:\t");
	PRINTQ16(l);
	printf("a*b:\t");
	PRINTQ16(m);
	printf("c*d:\t");
	PRINTQ16(n);
	printf("a*c:\t");
	PRINTQ16(y);
	printf("b*d:\t");
	PRINTQ16(z);
	printf("a/b:\t");
	PRINTQ16(o);
	printf("c/d:\t");
	PRINTQ16(p);
	printf("a/c:\t");
	PRINTQ16(ae);
	printf("b/d:\t");
	PRINTQ16(oe);
	printf("int(a):\t");
	PRINTQ16(q);
	printf("fra(a):\t");
	PRINTQ16(r);
	printf("int(b):\t");
	PRINTQ16(s);
	printf("fra(b):\t");
	PRINTQ16(t);
	printf("int(c):\t");
	PRINTQ16(u);
	printf("fra(c):\t");
	PRINTQ16(v);
	printf("int(d):\t");
	PRINTQ16(w);
	printf("fra(d):\t");
	PRINTQ16(x);
	printf("\n");
}

/* FIR filter test. */

#define FF_X_SIZE 4
#define FF_H_SIZE 4 // Must be L (see fir_filter.c)
void fir_filter_() {

	float x[FF_X_SIZE] = {1.0, 1.0, 1.0, 1.0};

	float *y = malloc(FF_X_SIZE*sizeof(float));
	float *h = malloc(FF_H_SIZE*sizeof(float));
	uint8_t n;

	for(n=0;n<FF_X_SIZE;n++) {

		fir_filter(&x[n], h, n, &y[n]);
	}

	printf("\n\ninn:");
	for(n=0;n<FF_X_SIZE;n++) {
		printf("\t%f", x[n]);
	}
	printf("\n\nut:");
	for(n=0;n<FF_X_SIZE;n++) {
		printf("\t%f", y[n]);
	}
	printf("\n\n");
}

#define FF_X_SIZE_Q16 4
#define FF_H_SIZE_Q16 4 // Must be L (see fir_filter.c)
void fir_filter_Q16_() {

	Q16 x[FF_X_SIZE_Q16] = {Q16_1, Q16_1, Q16_1, Q16_1};

	Q16 *y = malloc(FF_X_SIZE_Q16*sizeof(Q16));
	Q16 *h = malloc(FF_H_SIZE_Q16*sizeof(Q16));
	uint8_t n;

	for(n=0;n<FF_X_SIZE_Q16;n++) {

		fir_filter(&x[n], h, n, &y[n]);
	}

	printf("\n\ninn:\n");
	for(n=0;n<FF_X_SIZE_Q16;n++) {
		PRINTQ16(x[n]);
	}
	printf("\nut:\n");
	for(n=0;n<FF_X_SIZE_Q16;n++) {
		PRINTQ16(y[n]);
	}
	printf("\n");
}

/* Complex AQB test. */

#define AQB_N 4
#define AQB_DBITS 4

void AQB_Q16() {

	Q16 x[AQB_N] = {Q16_0P5, Q16_0P5, Q16_0P5, Q16_0P5};
	Q16 y[AQB_N] = {0, 0, 0, 0};
	Q16 delta[AQB_N+1] = {(1<<16+1-AQB_DBITS), 0, 0, 0, 0};
	int32_t codewords[AQB_N] = {0, 0, 0, 0};

	Q16 *zero = malloc(sizeof(Q16));
	(*zero)=0x00;

	printf("\ndelta[0]:\t");
	PRINTQ16(delta[0]);
	printf("x[-1]:\t\t");
	PRINTQ16(*zero);
	printf("\n");

	int i=0;
	for(i=0;i<AQB_N;i++) {

		if(i>0)
			aqb_Q16(&y[i-1], &delta[i], &delta[i+1]);
		else
			aqb_Q16(zero, &delta[i], &delta[i+1]);
		aqb_encode_Q16(x[i], &delta[i+1], &codewords[i]);
		aqb_decode_Q16(&delta[i+1], &codewords[i], &y[i]);

		printf("x[%d]:\t\t", i);
		PRINTQ16(x[i]);
		printf("delta[%d]:\t", i+1);
		PRINTQ16(delta[i+1]);
		printf("codewords[%d]:\t%d\n", i, codewords[i]);
		printf("y[%d]:\t\t", i);
		PRINTQ16(y[i]);
		printf("\n");
	}
}

/* Q16 square root test. */

void SQT_Q16() {

	Q16 x = Q16_0P9;
	Q16 y = SQTQ16(x);
	PRINTQ16(y);

}

/* Main method. */
int main() {

	AQB_Q16();
	return 0;
}
