#ifndef __NRF51_BITS_H
#define __NRF51_BITS_H

#include <core_cm0.h>

/* Peripheral: ADC */
/* Description: Analog to digital converter. */

/* Register: ADC_INTENSET */
/* Description: ADC interrupt enable set register. */

/* Bit 0 : Enable interrupt on ENDCONV event. */
#define ADC_INTENSET_ENDCONV_Pos (0UL) /*!< Position of ENDCONV field. */
#define ADC_INTENSET_ENDCONV_Msk (0x1UL << ADC_INTENSET_ENDCONV_Pos) /*!< Bit mask of ENDCONV field. */
#define ADC_INTENSET_ENDCONV_Disabled (0UL) /*!< Interrupt disabled. */
#define ADC_INTENSET_ENDCONV_Enabled (1UL) /*!< Interrupt enabled. */
#define ADC_INTENSET_ENDCONV_Set (1UL) /*!< Enable interrupt on write. */

/* Register: ADC_INTENCLR */
/* Description: ADC interrupt enable clear register. */

/* Bit 0 : Disable interrupt on ENDCONV event. */
#define ADC_INTENCLR_ENDCONV_Pos (0UL) /*!< Position of ENDCONV field. */
#define ADC_INTENCLR_ENDCONV_Msk (0x1UL << ADC_INTENCLR_ENDCONV_Pos) /*!< Bit mask of ENDCONV field. */
#define ADC_INTENCLR_ENDCONV_Disabled (0UL) /*!< Interrupt disabled. */
#define ADC_INTENCLR_ENDCONV_Enabled (1UL) /*!< Interrupt enabled. */
#define ADC_INTENCLR_ENDCONV_Clear (1UL) /*!< Disable interrupt on write. */

/* Register: ADC_BUSY */
/* Description: ADC busy register. */

/* Bit 0 : ADC busy register. */
#define ADC_BUSY_BUSY_Pos (0UL) /*!< Position of BUSY field. */
#define ADC_BUSY_BUSY_Msk (0x1UL << ADC_BUSY_BUSY_Pos) /*!< Bit mask of BUSY field. */
#define ADC_BUSY_BUSY_Ready (0UL) /*!< No ongoing ADC conversion is taking place. ADC is ready. */
#define ADC_BUSY_BUSY_Busy (1UL) /*!< An ADC conversion is taking place. ADC is busy. */

/* Register: ADC_ENABLE */
/* Description: ADC enable. */

/* Bit 0 : ADC enable. */
#define ADC_ENABLE_ENABLE_Pos (0UL) /*!< Position of ENABLE field. */
#define ADC_ENABLE_ENABLE_Msk (0x1UL << ADC_ENABLE_ENABLE_Pos) /*!< Bit mask of ENABLE field. */
#define ADC_ENABLE_ENABLE_Disabled (0UL) /*!< ADC is disabled. */
#define ADC_ENABLE_ENABLE_Enabled (1UL) /*!< ADC is enabled. If an analog input pin is selected as source of the conversion, the selected pin is configured as an analog input. */

/* Register: ADC_CONFIG */
/* Description: ADC configuration register. */

/* Bits 17..16 : ADC external reference pin selection. */
#define ADC_CONFIG_EXTREFSEL_Pos (16UL) /*!< Position of EXTREFSEL field. */
#define ADC_CONFIG_EXTREFSEL_Msk (0x3UL << ADC_CONFIG_EXTREFSEL_Pos) /*!< Bit mask of EXTREFSEL field. */
#define ADC_CONFIG_EXTREFSEL_None (0UL) /*!< Analog external reference inputs disabled. */
#define ADC_CONFIG_EXTREFSEL_AnalogReference0 (1UL) /*!< Use analog reference 0 as reference. */
#define ADC_CONFIG_EXTREFSEL_AnalogReference1 (2UL) /*!< Use analog reference 1 as reference. */

/* Bits 15..8 : ADC analog pin selection. */
#define ADC_CONFIG_PSEL_Pos (8UL) /*!< Position of PSEL field. */
#define ADC_CONFIG_PSEL_Msk (0xFFUL << ADC_CONFIG_PSEL_Pos) /*!< Bit mask of PSEL field. */
#define ADC_CONFIG_PSEL_Disabled (0UL) /*!< Analog input pins disabled. */
#define ADC_CONFIG_PSEL_AnalogInput0 (1UL) /*!< Use analog input 0 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput1 (2UL) /*!< Use analog input 1 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput2 (4UL) /*!< Use analog input 2 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput3 (8UL) /*!< Use analog input 3 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput4 (16UL) /*!< Use analog input 4 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput5 (32UL) /*!< Use analog input 5 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput6 (64UL) /*!< Use analog input 6 as analog input. */
#define ADC_CONFIG_PSEL_AnalogInput7 (128UL) /*!< Use analog input 7 as analog input. */

/* Bits 6..5 : ADC reference selection. */
#define ADC_CONFIG_REFSEL_Pos (5UL) /*!< Position of REFSEL field. */
#define ADC_CONFIG_REFSEL_Msk (0x3UL << ADC_CONFIG_REFSEL_Pos) /*!< Bit mask of REFSEL field. */
#define ADC_CONFIG_REFSEL_Internal1Point2 (0x00UL) /*!< Use internal 1.2V as reference for conversion. */
#define ADC_CONFIG_REFSEL_External (0x01UL) /*!< Use external source configured by EXTREFSEL as reference for conversion. */
#define ADC_CONFIG_REFSEL_SupplyTwoThirdsPrescaling (0x02UL) /*!< Use supply voltage with 2/3 prescaling as reference for conversion. Only usable when supply voltage is between 1.7V and 2.6V. */
#define ADC_CONFIG_REFSEL_SupplyOneThirdPrescaling (0x03UL) /*!< Use supply voltage with 1/3 prescaling as reference for conversion. Only usable when supply voltage is between 2.5V and 3.6V. */

/* Bits 4..2 : ADC input selection. */
#define ADC_CONFIG_INPSEL_Pos (2UL) /*!< Position of INPSEL field. */
#define ADC_CONFIG_INPSEL_Msk (0x7UL << ADC_CONFIG_INPSEL_Pos) /*!< Bit mask of INPSEL field. */
#define ADC_CONFIG_INPSEL_AnalogInputNoPrescaling (0x00UL) /*!< Analog input specified by PSEL with no prescaling used as input for the conversion. */
#define ADC_CONFIG_INPSEL_AnalogInputTwoThirdsPrescaling (0x01UL) /*!< Analog input specified by PSEL with 2/3 prescaling used as input for the conversion. */
#define ADC_CONFIG_INPSEL_AnalogInputOneThirdPrescaling (0x02UL) /*!< Analog input specified by PSEL with 1/3 prescaling used as input for the conversion. */
#define ADC_CONFIG_INPSEL_SupplyTwoThirdsPrescaling (0x05UL) /*!< Supply voltage with 2/3 prescaling used as input for the conversion. */
#define ADC_CONFIG_INPSEL_SupplyOneThirdPrescaling (0x06UL) /*!< Supply voltage with 1/3 prescaling used as input for the conversion. */

/* Bits 1..0 : ADC resolution. */
#define ADC_CONFIG_RES_Pos (0UL) /*!< Position of RES field. */
#define ADC_CONFIG_RES_Msk (0x3UL << ADC_CONFIG_RES_Pos) /*!< Bit mask of RES field. */
#define ADC_CONFIG_RES_8bit (0x00UL) /*!< 8bit ADC resolution. */
#define ADC_CONFIG_RES_9bit (0x01UL) /*!< 9bit ADC resolution. */
#define ADC_CONFIG_RES_10bit (0x02UL) /*!< 10bit ADC resolution. */

/* Register: ADC_SAMPLE */
/* Description: Result of ADC conversion. */

/* Bits 9..0 : Result of ADC conversion. */
#define ADC_SAMPLE_SAMPLE_Pos (0UL) /*!< Position of SAMPLE field. */
#define ADC_SAMPLE_SAMPLE_Msk (0x3FFUL << ADC_SAMPLE_SAMPLE_Pos) /*!< Bit mask of SAMPLE field. */


/* Peripheral: CCM */
/* Description: AES CCM Mode Encryption. */

/* Register: CCM_SHORTS */
/* Description: Shortcut for the CCM. */

/* Bit 0 : Short-cut between ENDKSGEN event and CRYPT task. */
#define CCM_SHORTS_ENDKSGEN_CRYPT_Pos (0U) /*!< Position of ENDKSGEN_CRYPT field. */
#define CCM_SHORTS_ENDKSGEN_CRYPT_Msk (0x1UL << CCM_SHORTS_ENDKSGEN_CRYPT_Pos) /*!< Bit mask of ENDKSGEN_CRYPT field. */
#define CCM_SHORTS_ENDKSGEN_CRYPT_Disabled (0UL) /*!< Shortcut disabled. */
#define CCM_SHORTS_ENDKSGEN_CRYPT_Enabled (1UL) /*!< Shortcut enabled. */

/* Register: CCM_INTEN */
/* Description: Interrupt enable register. */

/* Bit 1 : Interrupt enable on ENDCRYPT event. */
#define CCM_INTEN_ENDCRYPT_Pos (1U) /*!< Position of ENDCRYPT field. */
#define CCM_INTEN_ENDCRYPT_Msk (0x1UL << CCM_INTEN_ENDCRYPT_Pos) /*!< Bit mask of ENDCRYPT field. */
#define CCM_INTEN_ENDCRYPT_Disabled (0UL) /*!< Interrupt disabled. */
#define CCM_INTEN_ENDCRYPT_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 0 : Interrupt enable on ENDKSGEN event. */
#define CCM_INTEN_ENDKSGEN_Pos (0U) /*!< Position of ENDKSGEN field. */
#define CCM_INTEN_ENDKSGEN_Msk (0x1UL << CCM_INTEN_ENDKSGEN_Pos) /*!< Bit mask of ENDKSGEN field. */
#define CCM_INTEN_ENDKSGEN_Disabled (0UL) /*!< Interrupt disabled. */
#define CCM_INTEN_ENDKSGEN_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: CCM_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 1 : Enable interrupt on ENDCRYPT event. */
#define CCM_INTENSET_ENDCRYPT_Pos (1U) /*!< Position of ENDCRYPT field. */
#define CCM_INTENSET_ENDCRYPT_Msk (0x1UL << CCM_INTENSET_ENDCRYPT_Pos) /*!< Bit mask of ENDCRYPT field. */

/* Bit 0 : Enable interrupt on ENDKSGEN event. */
#define CCM_INTENSET_ENDKSGEN_Pos (0U) /*!< Position of ENDKSGEN field. */
#define CCM_INTENSET_ENDKSGEN_Msk (0x1UL << CCM_INTENSET_ENDKSGEN_Pos) /*!< Bit mask of ENDKSGEN field. */

/* Register: CCM_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 1 : Disable interrupt on ENDCRYPT event. */
#define CCM_INTENCLR_ENDCRYPT_Pos (1U) /*!< Position of ENDCRYPT field. */
#define CCM_INTENCLR_ENDCRYPT_Msk (0x1UL << CCM_INTENCLR_ENDCRYPT_Pos) /*!< Bit mask of ENDCRYPT field. */

/* Bit 0 : Disable interrupt on ENDKSGEN event. */
#define CCM_INTENCLR_ENDKSGEN_Pos (0U) /*!< Position of ENDKSGEN field. */
#define CCM_INTENCLR_ENDKSGEN_Msk (0x1UL << CCM_INTENCLR_ENDKSGEN_Pos) /*!< Bit mask of ENDKSGEN field. */

/* Register: CCM_MICSTATUS */
/* Description: CCM RX MIC check result. */

/* Bit 0 : Result of the MIC check performed during the previous CCM RX STARTCRYPT */
#define CCM_MICSTATUS_MICSTATUS_Pos (0U) /*!< Position of MICSTATUS field. */
#define CCM_MICSTATUS_MICSTATUS_Msk (0x1UL << CCM_MICSTATUS_MICSTATUS_Pos) /*!< Bit mask of MICSTATUS field. */
#define CCM_MICSTATUS_MICSTATUS_CheckFailed (0UL) /*!< MIC check failed. */
#define CCM_MICSTATUS_MICSTATUS_CheckPassed (1UL) /*!< MIC check passed. */

/* Register: CCM_MODE */
/* Description: Operation mode. */

/* Bit 0 : CCM mode operation. */
#define CCM_MODE_MODE_Pos (0U) /*!< Position of MODE field. */
#define CCM_MODE_MODE_Msk (0x1UL << CCM_MODE_MODE_Pos) /*!< Bit mask of MODE field. */
#define CCM_MODE_MODE_Encryption (0UL) /*!< CCM mode TX */
#define CCM_MODE_MODE_Decryption (1UL) /*!< CCM mode TX */


/* Peripheral: CLOCK */
/* Description: Clock control. */

/* Register: CLOCK_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 1 : Enable interrupt on KI32STARTED event. */
#define CLOCK_INTENSET_KI32STARTED_Pos (1U) /*!< Position of KI32STARTED field. */
#define CLOCK_INTENSET_KI32STARTED_Msk (0x1UL << CLOCK_INTENSET_KI32STARTED_Pos) /*!< Bit mask of KI32STARTED field. */

/* Bit 0 : Enable interrupt on M16STARTED event. */
#define CLOCK_INTENSET_M16STARTED_Pos (0U) /*!< Position of M16STARTED field. */
#define CLOCK_INTENSET_M16STARTED_Msk (0x1UL << CLOCK_INTENSET_M16STARTED_Pos) /*!< Bit mask of M16STARTED field. */

/* Register: CLOCK_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 1 : Disable interrupt on KI32STARTED event. */
#define CLOCK_INTENCLR_KI32STARTED_Pos (1U) /*!< Position of KI32STARTED field. */
#define CLOCK_INTENCLR_KI32STARTED_Msk (0x1UL << CLOCK_INTENCLR_KI32STARTED_Pos) /*!< Bit mask of KI32STARTED field. */

/* Bit 0 : Disable interrupt on M16STARTED event. */
#define CLOCK_INTENCLR_M16STARTED_Pos (0U) /*!< Position of M16STARTED field. */
#define CLOCK_INTENCLR_M16STARTED_Msk (0x1UL << CLOCK_INTENCLR_M16STARTED_Pos) /*!< Bit mask of M16STARTED field. */

/* Register: CLOCK_KI32RUN */
/* Description: 32kiHz source available. */

/* Bit 0 : Clock status. */
#define CLOCK_KI32RUN_STAT_Pos (0U) /*!< Position of STAT field. */
#define CLOCK_KI32RUN_STAT_Msk (0x1UL << CLOCK_KI32RUN_STAT_Pos) /*!< Bit mask of STAT field. */
#define CLOCK_KI32RUN_STAT_NOT_AVAILABLE (0x00UL) /*!< 32kiHz clock source not available. */
#define CLOCK_KI32RUN_STAT_AVAILABLE (0x01UL) /*!< 32kiHz clock source available. */

/* Register: CLOCK_KI32STAT */
/* Description: Which 32kiHz source is running. */

/* Bit 0 : Clock source. */
#define CLOCK_KI32STAT_SRC_Pos (0U) /*!< Position of SRC field. */
#define CLOCK_KI32STAT_SRC_Msk (0x1UL << CLOCK_KI32STAT_SRC_Pos) /*!< Bit mask of SRC field. */
#define CLOCK_KI32STAT_SRC_RC (0x00UL) /*!< I32kiHz RC oscillator running and generating the 32kiHz clock. */
#define CLOCK_KI32STAT_SRC_Xtal (0x01UL) /*!< 32kiHz crystal oscillator running and generating the 32kiHz clock. */

/* Register: CLOCK_KI32SRC */
/* Description: Clock source for the 32kiHz clock. */

/* Bit 0 : Clock source. */
#define CLOCK_KI32SRC_SRC_Pos (0U) /*!< Position of SRC field. */
#define CLOCK_KI32SRC_SRC_Msk (0x1UL << CLOCK_KI32SRC_SRC_Pos) /*!< Bit mask of SRC field. */
#define CLOCK_KI32SRC_SRC_RC (0x00UL) /*!< Internal 32kiHz RC oscillator. */
#define CLOCK_KI32SRC_SRC_Xtal (0x01UL) /*!< External 32kiHz crystal. */


/* Peripheral: ECB */
/* Description: AES ECB Mode Encryption. */

/* Register: ECB_INTEN */
/* Description: Interrupt enable register. */

/* Bit 1 : Enable interrupt on ERRORECB event. */
#define ECB_INTEN_ERRORECB_Pos (1U) /*!< Position of ERRORECB field. */
#define ECB_INTEN_ERRORECB_Msk (0x1UL << ECB_INTEN_ERRORECB_Pos) /*!< Bit mask of ERRORECB field. */
#define ECB_INTEN_ERRORECB_Disabled (0UL) /*!< Interrupt disabled. */
#define ECB_INTEN_ERRORECB_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 0 : Enable interrupt on ENDECB event. */
#define ECB_INTEN_ENDECB_Pos (0U) /*!< Position of ENDECB field. */
#define ECB_INTEN_ENDECB_Msk (0x1UL << ECB_INTEN_ENDECB_Pos) /*!< Bit mask of ENDECB field. */
#define ECB_INTEN_ENDECB_Disabled (0UL) /*!< Interrupt disabled. */
#define ECB_INTEN_ENDECB_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: ECB_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 1 : Enable interrupt on ERRORECB event. */
#define ECB_INTENSET_ERRORECB_Pos (1U) /*!< Position of ERRORECB field. */
#define ECB_INTENSET_ERRORECB_Msk (0x1UL << ECB_INTENSET_ERRORECB_Pos) /*!< Bit mask of ERRORECB field. */

/* Bit 0 : Enable interrupt on ENDECB event. */
#define ECB_INTENSET_ENDECB_Pos (0U) /*!< Position of ENDECB field. */
#define ECB_INTENSET_ENDECB_Msk (0x1UL << ECB_INTENSET_ENDECB_Pos) /*!< Bit mask of ENDECB field. */

/* Register: ECB_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 1 : Disable interrupt on ERRORECB event. */
#define ECB_INTENCLR_ERRORECB_Pos (1U) /*!< Position of ERRORECB field. */
#define ECB_INTENCLR_ERRORECB_Msk (0x1UL << ECB_INTENCLR_ERRORECB_Pos) /*!< Bit mask of ERRORECB field. */

/* Bit 0 : Disable interrupt on ENDECB event. */
#define ECB_INTENCLR_ENDECB_Pos (0U) /*!< Position of ENDECB field. */
#define ECB_INTENCLR_ENDECB_Msk (0x1UL << ECB_INTENCLR_ENDECB_Pos) /*!< Bit mask of ENDECB field. */


/* Peripheral: FICR */
/* Description: Factory Information Configuration */

/* Register: FICR_PPFC */
/* Description: Pre-programmed factory code present. */

/* Bits 7..0 : Pre-programmed factory code present. */
#define FICR_PPFC_PPFC_Pos (0U) /*!< Position of PPFC field. */
#define FICR_PPFC_PPFC_Msk (0xFFUL << FICR_PPFC_PPFC_Pos) /*!< Bit mask of PPFC field. */
#define FICR_PPFC_PPFC_NotPresent (0xFFUL) /*!< Not present. */
#define FICR_PPFC_PPFC_Present (0x00UL) /*!< Present. */


/* Peripheral: GPIO */
/* Description: General purpose input and output */

/* Register: GPIO_OUT */
/* Description: Write GPIO port. */

/* Bit 31 : Pin 31. */
#define GPIO_OUT_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_OUT_PIN31_Msk (0x1UL << GPIO_OUT_PIN31_Pos) /*!< Bit mask of PIN31 field. */

/* Bit 30 : Pin 30. */
#define GPIO_OUT_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_OUT_PIN30_Msk (0x1UL << GPIO_OUT_PIN30_Pos) /*!< Bit mask of PIN30 field. */

/* Bit 29 : Pin 29. */
#define GPIO_OUT_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_OUT_PIN29_Msk (0x1UL << GPIO_OUT_PIN29_Pos) /*!< Bit mask of PIN29 field. */

/* Bit 28 : Pin 28. */
#define GPIO_OUT_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_OUT_PIN28_Msk (0x1UL << GPIO_OUT_PIN28_Pos) /*!< Bit mask of PIN28 field. */

/* Bit 27 : Pin 27. */
#define GPIO_OUT_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_OUT_PIN27_Msk (0x1UL << GPIO_OUT_PIN27_Pos) /*!< Bit mask of PIN27 field. */

/* Bit 26 : Pin 26. */
#define GPIO_OUT_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_OUT_PIN26_Msk (0x1UL << GPIO_OUT_PIN26_Pos) /*!< Bit mask of PIN26 field. */

/* Bit 25 : Pin 25. */
#define GPIO_OUT_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_OUT_PIN25_Msk (0x1UL << GPIO_OUT_PIN25_Pos) /*!< Bit mask of PIN25 field. */

/* Bit 24 : Pin 24. */
#define GPIO_OUT_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_OUT_PIN24_Msk (0x1UL << GPIO_OUT_PIN24_Pos) /*!< Bit mask of PIN24 field. */

/* Bit 23 : Pin 23. */
#define GPIO_OUT_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_OUT_PIN23_Msk (0x1UL << GPIO_OUT_PIN23_Pos) /*!< Bit mask of PIN23 field. */

/* Bit 22 : Pin 22. */
#define GPIO_OUT_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_OUT_PIN22_Msk (0x1UL << GPIO_OUT_PIN22_Pos) /*!< Bit mask of PIN22 field. */

/* Bit 21 : Pin 21. */
#define GPIO_OUT_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_OUT_PIN21_Msk (0x1UL << GPIO_OUT_PIN21_Pos) /*!< Bit mask of PIN21 field. */

/* Bit 20 : Pin 20. */
#define GPIO_OUT_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_OUT_PIN20_Msk (0x1UL << GPIO_OUT_PIN20_Pos) /*!< Bit mask of PIN20 field. */

/* Bit 19 : Pin 19. */
#define GPIO_OUT_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_OUT_PIN19_Msk (0x1UL << GPIO_OUT_PIN19_Pos) /*!< Bit mask of PIN19 field. */

/* Bit 18 : Pin 18. */
#define GPIO_OUT_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_OUT_PIN18_Msk (0x1UL << GPIO_OUT_PIN18_Pos) /*!< Bit mask of PIN18 field. */

/* Bit 17 : Pin 17. */
#define GPIO_OUT_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_OUT_PIN17_Msk (0x1UL << GPIO_OUT_PIN17_Pos) /*!< Bit mask of PIN17 field. */

/* Bit 16 : Pin 16. */
#define GPIO_OUT_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_OUT_PIN16_Msk (0x1UL << GPIO_OUT_PIN16_Pos) /*!< Bit mask of PIN16 field. */

/* Bit 15 : Pin 15. */
#define GPIO_OUT_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_OUT_PIN15_Msk (0x1UL << GPIO_OUT_PIN15_Pos) /*!< Bit mask of PIN15 field. */

/* Bit 14 : Pin 14. */
#define GPIO_OUT_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_OUT_PIN14_Msk (0x1UL << GPIO_OUT_PIN14_Pos) /*!< Bit mask of PIN14 field. */

/* Bit 13 : Pin 13. */
#define GPIO_OUT_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_OUT_PIN13_Msk (0x1UL << GPIO_OUT_PIN13_Pos) /*!< Bit mask of PIN13 field. */

/* Bit 12 : Pin 12. */
#define GPIO_OUT_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_OUT_PIN12_Msk (0x1UL << GPIO_OUT_PIN12_Pos) /*!< Bit mask of PIN12 field. */

/* Bit 11 : Pin 11. */
#define GPIO_OUT_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_OUT_PIN11_Msk (0x1UL << GPIO_OUT_PIN11_Pos) /*!< Bit mask of PIN11 field. */

/* Bit 10 : Pin 10. */
#define GPIO_OUT_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_OUT_PIN10_Msk (0x1UL << GPIO_OUT_PIN10_Pos) /*!< Bit mask of PIN10 field. */

/* Bit 9 : Pin 9. */
#define GPIO_OUT_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_OUT_PIN9_Msk (0x1UL << GPIO_OUT_PIN9_Pos) /*!< Bit mask of PIN9 field. */

/* Bit 8 : Pin 8. */
#define GPIO_OUT_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_OUT_PIN8_Msk (0x1UL << GPIO_OUT_PIN8_Pos) /*!< Bit mask of PIN8 field. */

/* Bit 7 : Pin 7. */
#define GPIO_OUT_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_OUT_PIN7_Msk (0x1UL << GPIO_OUT_PIN7_Pos) /*!< Bit mask of PIN7 field. */

/* Bit 6 : Pin 6. */
#define GPIO_OUT_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_OUT_PIN6_Msk (0x1UL << GPIO_OUT_PIN6_Pos) /*!< Bit mask of PIN6 field. */

/* Bit 5 : Pin 5. */
#define GPIO_OUT_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_OUT_PIN5_Msk (0x1UL << GPIO_OUT_PIN5_Pos) /*!< Bit mask of PIN5 field. */

/* Bit 4 : Pin 4. */
#define GPIO_OUT_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_OUT_PIN4_Msk (0x1UL << GPIO_OUT_PIN4_Pos) /*!< Bit mask of PIN4 field. */

/* Bit 3 : Pin 3. */
#define GPIO_OUT_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_OUT_PIN3_Msk (0x1UL << GPIO_OUT_PIN3_Pos) /*!< Bit mask of PIN3 field. */

/* Bit 2 : Pin 2. */
#define GPIO_OUT_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_OUT_PIN2_Msk (0x1UL << GPIO_OUT_PIN2_Pos) /*!< Bit mask of PIN2 field. */

/* Bit 1 : Pin 1. */
#define GPIO_OUT_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_OUT_PIN1_Msk (0x1UL << GPIO_OUT_PIN1_Pos) /*!< Bit mask of PIN1 field. */

/* Bit 0 : Pin 0. */
#define GPIO_OUT_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_OUT_PIN0_Msk (0x1UL << GPIO_OUT_PIN0_Pos) /*!< Bit mask of PIN0 field. */

/* Register: GPIO_IN */
/* Description: Read GPIO port. */

/* Bit 31 : Pin 31. */
#define GPIO_IN_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_IN_PIN31_Msk (0x1UL << GPIO_IN_PIN31_Pos) /*!< Bit mask of PIN31 field. */

/* Bit 30 : Pin 30. */
#define GPIO_IN_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_IN_PIN30_Msk (0x1UL << GPIO_IN_PIN30_Pos) /*!< Bit mask of PIN30 field. */

/* Bit 29 : Pin 29. */
#define GPIO_IN_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_IN_PIN29_Msk (0x1UL << GPIO_IN_PIN29_Pos) /*!< Bit mask of PIN29 field. */

/* Bit 28 : Pin 28. */
#define GPIO_IN_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_IN_PIN28_Msk (0x1UL << GPIO_IN_PIN28_Pos) /*!< Bit mask of PIN28 field. */

/* Bit 27 : Pin 27. */
#define GPIO_IN_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_IN_PIN27_Msk (0x1UL << GPIO_IN_PIN27_Pos) /*!< Bit mask of PIN27 field. */

/* Bit 26 : Pin 26. */
#define GPIO_IN_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_IN_PIN26_Msk (0x1UL << GPIO_IN_PIN26_Pos) /*!< Bit mask of PIN26 field. */

/* Bit 25 : Pin 25. */
#define GPIO_IN_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_IN_PIN25_Msk (0x1UL << GPIO_IN_PIN25_Pos) /*!< Bit mask of PIN25 field. */

/* Bit 24 : Pin 24. */
#define GPIO_IN_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_IN_PIN24_Msk (0x1UL << GPIO_IN_PIN24_Pos) /*!< Bit mask of PIN24 field. */

/* Bit 23 : Pin 23. */
#define GPIO_IN_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_IN_PIN23_Msk (0x1UL << GPIO_IN_PIN23_Pos) /*!< Bit mask of PIN23 field. */

/* Bit 22 : Pin 22. */
#define GPIO_IN_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_IN_PIN22_Msk (0x1UL << GPIO_IN_PIN22_Pos) /*!< Bit mask of PIN22 field. */

/* Bit 21 : Pin 21. */
#define GPIO_IN_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_IN_PIN21_Msk (0x1UL << GPIO_IN_PIN21_Pos) /*!< Bit mask of PIN21 field. */

/* Bit 20 : Pin 20. */
#define GPIO_IN_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_IN_PIN20_Msk (0x1UL << GPIO_IN_PIN20_Pos) /*!< Bit mask of PIN20 field. */

/* Bit 19 : Pin 19. */
#define GPIO_IN_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_IN_PIN19_Msk (0x1UL << GPIO_IN_PIN19_Pos) /*!< Bit mask of PIN19 field. */

/* Bit 18 : Pin 18. */
#define GPIO_IN_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_IN_PIN18_Msk (0x1UL << GPIO_IN_PIN18_Pos) /*!< Bit mask of PIN18 field. */

/* Bit 17 : Pin 17. */
#define GPIO_IN_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_IN_PIN17_Msk (0x1UL << GPIO_IN_PIN17_Pos) /*!< Bit mask of PIN17 field. */

/* Bit 16 : Pin 16. */
#define GPIO_IN_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_IN_PIN16_Msk (0x1UL << GPIO_IN_PIN16_Pos) /*!< Bit mask of PIN16 field. */

/* Bit 15 : Pin 15. */
#define GPIO_IN_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_IN_PIN15_Msk (0x1UL << GPIO_IN_PIN15_Pos) /*!< Bit mask of PIN15 field. */

/* Bit 14 : Pin 14. */
#define GPIO_IN_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_IN_PIN14_Msk (0x1UL << GPIO_IN_PIN14_Pos) /*!< Bit mask of PIN14 field. */

/* Bit 13 : Pin 13. */
#define GPIO_IN_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_IN_PIN13_Msk (0x1UL << GPIO_IN_PIN13_Pos) /*!< Bit mask of PIN13 field. */

/* Bit 12 : Pin 12. */
#define GPIO_IN_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_IN_PIN12_Msk (0x1UL << GPIO_IN_PIN12_Pos) /*!< Bit mask of PIN12 field. */

/* Bit 11 : Pin 11. */
#define GPIO_IN_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_IN_PIN11_Msk (0x1UL << GPIO_IN_PIN11_Pos) /*!< Bit mask of PIN11 field. */

/* Bit 10 : Pin 10. */
#define GPIO_IN_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_IN_PIN10_Msk (0x1UL << GPIO_IN_PIN10_Pos) /*!< Bit mask of PIN10 field. */

/* Bit 9 : Pin 9. */
#define GPIO_IN_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_IN_PIN9_Msk (0x1UL << GPIO_IN_PIN9_Pos) /*!< Bit mask of PIN9 field. */

/* Bit 8 : Pin 8. */
#define GPIO_IN_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_IN_PIN8_Msk (0x1UL << GPIO_IN_PIN8_Pos) /*!< Bit mask of PIN8 field. */

/* Bit 7 : Pin 7. */
#define GPIO_IN_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_IN_PIN7_Msk (0x1UL << GPIO_IN_PIN7_Pos) /*!< Bit mask of PIN7 field. */

/* Bit 6 : Pin 6. */
#define GPIO_IN_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_IN_PIN6_Msk (0x1UL << GPIO_IN_PIN6_Pos) /*!< Bit mask of PIN6 field. */

/* Bit 5 : Pin 5. */
#define GPIO_IN_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_IN_PIN5_Msk (0x1UL << GPIO_IN_PIN5_Pos) /*!< Bit mask of PIN5 field. */

/* Bit 4 : Pin 4. */
#define GPIO_IN_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_IN_PIN4_Msk (0x1UL << GPIO_IN_PIN4_Pos) /*!< Bit mask of PIN4 field. */

/* Bit 3 : Pin 3. */
#define GPIO_IN_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_IN_PIN3_Msk (0x1UL << GPIO_IN_PIN3_Pos) /*!< Bit mask of PIN3 field. */

/* Bit 2 : Pin 2. */
#define GPIO_IN_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_IN_PIN2_Msk (0x1UL << GPIO_IN_PIN2_Pos) /*!< Bit mask of PIN2 field. */

/* Bit 1 : Pin 1. */
#define GPIO_IN_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_IN_PIN1_Msk (0x1UL << GPIO_IN_PIN1_Pos) /*!< Bit mask of PIN1 field. */

/* Bit 0 : Pin 0. */
#define GPIO_IN_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_IN_PIN0_Msk (0x1UL << GPIO_IN_PIN0_Pos) /*!< Bit mask of PIN0 field. */

/* Register: GPIO_OUTSET */
/* Description: Set individual bits in GPIO port. */

/* Bit 31 : Pin 31. */
#define GPIO_OUTSET_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_OUTSET_PIN31_Msk (0x1UL << GPIO_OUTSET_PIN31_Pos) /*!< Bit mask of PIN31 field. */

/* Bit 30 : Pin 30. */
#define GPIO_OUTSET_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_OUTSET_PIN30_Msk (0x1UL << GPIO_OUTSET_PIN30_Pos) /*!< Bit mask of PIN30 field. */

/* Bit 29 : Pin 29. */
#define GPIO_OUTSET_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_OUTSET_PIN29_Msk (0x1UL << GPIO_OUTSET_PIN29_Pos) /*!< Bit mask of PIN29 field. */

/* Bit 28 : Pin 28. */
#define GPIO_OUTSET_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_OUTSET_PIN28_Msk (0x1UL << GPIO_OUTSET_PIN28_Pos) /*!< Bit mask of PIN28 field. */

/* Bit 27 : Pin 27. */
#define GPIO_OUTSET_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_OUTSET_PIN27_Msk (0x1UL << GPIO_OUTSET_PIN27_Pos) /*!< Bit mask of PIN27 field. */

/* Bit 26 : Pin 26. */
#define GPIO_OUTSET_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_OUTSET_PIN26_Msk (0x1UL << GPIO_OUTSET_PIN26_Pos) /*!< Bit mask of PIN26 field. */

/* Bit 25 : Pin 25. */
#define GPIO_OUTSET_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_OUTSET_PIN25_Msk (0x1UL << GPIO_OUTSET_PIN25_Pos) /*!< Bit mask of PIN25 field. */

/* Bit 24 : Pin 24. */
#define GPIO_OUTSET_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_OUTSET_PIN24_Msk (0x1UL << GPIO_OUTSET_PIN24_Pos) /*!< Bit mask of PIN24 field. */

/* Bit 23 : Pin 23. */
#define GPIO_OUTSET_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_OUTSET_PIN23_Msk (0x1UL << GPIO_OUTSET_PIN23_Pos) /*!< Bit mask of PIN23 field. */

/* Bit 22 : Pin 22. */
#define GPIO_OUTSET_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_OUTSET_PIN22_Msk (0x1UL << GPIO_OUTSET_PIN22_Pos) /*!< Bit mask of PIN22 field. */

/* Bit 21 : Pin 21. */
#define GPIO_OUTSET_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_OUTSET_PIN21_Msk (0x1UL << GPIO_OUTSET_PIN21_Pos) /*!< Bit mask of PIN21 field. */

/* Bit 20 : Pin 20. */
#define GPIO_OUTSET_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_OUTSET_PIN20_Msk (0x1UL << GPIO_OUTSET_PIN20_Pos) /*!< Bit mask of PIN20 field. */

/* Bit 19 : Pin 19. */
#define GPIO_OUTSET_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_OUTSET_PIN19_Msk (0x1UL << GPIO_OUTSET_PIN19_Pos) /*!< Bit mask of PIN19 field. */

/* Bit 18 : Pin 18. */
#define GPIO_OUTSET_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_OUTSET_PIN18_Msk (0x1UL << GPIO_OUTSET_PIN18_Pos) /*!< Bit mask of PIN18 field. */

/* Bit 17 : Pin 17. */
#define GPIO_OUTSET_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_OUTSET_PIN17_Msk (0x1UL << GPIO_OUTSET_PIN17_Pos) /*!< Bit mask of PIN17 field. */

/* Bit 16 : Pin 16. */
#define GPIO_OUTSET_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_OUTSET_PIN16_Msk (0x1UL << GPIO_OUTSET_PIN16_Pos) /*!< Bit mask of PIN16 field. */

/* Bit 15 : Pin 15. */
#define GPIO_OUTSET_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_OUTSET_PIN15_Msk (0x1UL << GPIO_OUTSET_PIN15_Pos) /*!< Bit mask of PIN15 field. */

/* Bit 14 : Pin 14. */
#define GPIO_OUTSET_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_OUTSET_PIN14_Msk (0x1UL << GPIO_OUTSET_PIN14_Pos) /*!< Bit mask of PIN14 field. */

/* Bit 13 : Pin 13. */
#define GPIO_OUTSET_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_OUTSET_PIN13_Msk (0x1UL << GPIO_OUTSET_PIN13_Pos) /*!< Bit mask of PIN13 field. */

/* Bit 12 : Pin 12. */
#define GPIO_OUTSET_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_OUTSET_PIN12_Msk (0x1UL << GPIO_OUTSET_PIN12_Pos) /*!< Bit mask of PIN12 field. */

/* Bit 11 : Pin 11. */
#define GPIO_OUTSET_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_OUTSET_PIN11_Msk (0x1UL << GPIO_OUTSET_PIN11_Pos) /*!< Bit mask of PIN11 field. */

/* Bit 10 : Pin 10. */
#define GPIO_OUTSET_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_OUTSET_PIN10_Msk (0x1UL << GPIO_OUTSET_PIN10_Pos) /*!< Bit mask of PIN10 field. */

/* Bit 9 : Pin 9. */
#define GPIO_OUTSET_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_OUTSET_PIN9_Msk (0x1UL << GPIO_OUTSET_PIN9_Pos) /*!< Bit mask of PIN9 field. */

/* Bit 8 : Pin 8. */
#define GPIO_OUTSET_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_OUTSET_PIN8_Msk (0x1UL << GPIO_OUTSET_PIN8_Pos) /*!< Bit mask of PIN8 field. */

/* Bit 7 : Pin 7. */
#define GPIO_OUTSET_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_OUTSET_PIN7_Msk (0x1UL << GPIO_OUTSET_PIN7_Pos) /*!< Bit mask of PIN7 field. */

/* Bit 6 : Pin 6. */
#define GPIO_OUTSET_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_OUTSET_PIN6_Msk (0x1UL << GPIO_OUTSET_PIN6_Pos) /*!< Bit mask of PIN6 field. */

/* Bit 5 : Pin 5. */
#define GPIO_OUTSET_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_OUTSET_PIN5_Msk (0x1UL << GPIO_OUTSET_PIN5_Pos) /*!< Bit mask of PIN5 field. */

/* Bit 4 : Pin 4. */
#define GPIO_OUTSET_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_OUTSET_PIN4_Msk (0x1UL << GPIO_OUTSET_PIN4_Pos) /*!< Bit mask of PIN4 field. */

/* Bit 3 : Pin 3. */
#define GPIO_OUTSET_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_OUTSET_PIN3_Msk (0x1UL << GPIO_OUTSET_PIN3_Pos) /*!< Bit mask of PIN3 field. */

/* Bit 2 : Pin 2. */
#define GPIO_OUTSET_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_OUTSET_PIN2_Msk (0x1UL << GPIO_OUTSET_PIN2_Pos) /*!< Bit mask of PIN2 field. */

/* Bit 1 : Pin 1. */
#define GPIO_OUTSET_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_OUTSET_PIN1_Msk (0x1UL << GPIO_OUTSET_PIN1_Pos) /*!< Bit mask of PIN1 field. */

/* Bit 0 : Pin 0. */
#define GPIO_OUTSET_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_OUTSET_PIN0_Msk (0x1UL << GPIO_OUTSET_PIN0_Pos) /*!< Bit mask of PIN0 field. */

/* Register: GPIO_OUTCLR */
/* Description: Clear individual bits in GPIO port. */

/* Bit 31 : Pin 31. */
#define GPIO_OUTCLR_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_OUTCLR_PIN31_Msk (0x1UL << GPIO_OUTCLR_PIN31_Pos) /*!< Bit mask of PIN31 field. */

/* Bit 30 : Pin 30. */
#define GPIO_OUTCLR_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_OUTCLR_PIN30_Msk (0x1UL << GPIO_OUTCLR_PIN30_Pos) /*!< Bit mask of PIN30 field. */

/* Bit 29 : Pin 29. */
#define GPIO_OUTCLR_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_OUTCLR_PIN29_Msk (0x1UL << GPIO_OUTCLR_PIN29_Pos) /*!< Bit mask of PIN29 field. */

/* Bit 28 : Pin 28. */
#define GPIO_OUTCLR_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_OUTCLR_PIN28_Msk (0x1UL << GPIO_OUTCLR_PIN28_Pos) /*!< Bit mask of PIN28 field. */

/* Bit 27 : Pin 27. */
#define GPIO_OUTCLR_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_OUTCLR_PIN27_Msk (0x1UL << GPIO_OUTCLR_PIN27_Pos) /*!< Bit mask of PIN27 field. */

/* Bit 26 : Pin 26. */
#define GPIO_OUTCLR_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_OUTCLR_PIN26_Msk (0x1UL << GPIO_OUTCLR_PIN26_Pos) /*!< Bit mask of PIN26 field. */

/* Bit 25 : Pin 25. */
#define GPIO_OUTCLR_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_OUTCLR_PIN25_Msk (0x1UL << GPIO_OUTCLR_PIN25_Pos) /*!< Bit mask of PIN25 field. */

/* Bit 24 : Pin 24. */
#define GPIO_OUTCLR_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_OUTCLR_PIN24_Msk (0x1UL << GPIO_OUTCLR_PIN24_Pos) /*!< Bit mask of PIN24 field. */

/* Bit 23 : Pin 23. */
#define GPIO_OUTCLR_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_OUTCLR_PIN23_Msk (0x1UL << GPIO_OUTCLR_PIN23_Pos) /*!< Bit mask of PIN23 field. */

/* Bit 22 : Pin 22. */
#define GPIO_OUTCLR_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_OUTCLR_PIN22_Msk (0x1UL << GPIO_OUTCLR_PIN22_Pos) /*!< Bit mask of PIN22 field. */

/* Bit 21 : Pin 21. */
#define GPIO_OUTCLR_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_OUTCLR_PIN21_Msk (0x1UL << GPIO_OUTCLR_PIN21_Pos) /*!< Bit mask of PIN21 field. */

/* Bit 20 : Pin 20. */
#define GPIO_OUTCLR_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_OUTCLR_PIN20_Msk (0x1UL << GPIO_OUTCLR_PIN20_Pos) /*!< Bit mask of PIN20 field. */

/* Bit 19 : Pin 19. */
#define GPIO_OUTCLR_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_OUTCLR_PIN19_Msk (0x1UL << GPIO_OUTCLR_PIN19_Pos) /*!< Bit mask of PIN19 field. */

/* Bit 18 : Pin 18. */
#define GPIO_OUTCLR_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_OUTCLR_PIN18_Msk (0x1UL << GPIO_OUTCLR_PIN18_Pos) /*!< Bit mask of PIN18 field. */

/* Bit 17 : Pin 17. */
#define GPIO_OUTCLR_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_OUTCLR_PIN17_Msk (0x1UL << GPIO_OUTCLR_PIN17_Pos) /*!< Bit mask of PIN17 field. */

/* Bit 16 : Pin 16. */
#define GPIO_OUTCLR_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_OUTCLR_PIN16_Msk (0x1UL << GPIO_OUTCLR_PIN16_Pos) /*!< Bit mask of PIN16 field. */

/* Bit 15 : Pin 15. */
#define GPIO_OUTCLR_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_OUTCLR_PIN15_Msk (0x1UL << GPIO_OUTCLR_PIN15_Pos) /*!< Bit mask of PIN15 field. */

/* Bit 14 : Pin 14. */
#define GPIO_OUTCLR_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_OUTCLR_PIN14_Msk (0x1UL << GPIO_OUTCLR_PIN14_Pos) /*!< Bit mask of PIN14 field. */

/* Bit 13 : Pin 13. */
#define GPIO_OUTCLR_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_OUTCLR_PIN13_Msk (0x1UL << GPIO_OUTCLR_PIN13_Pos) /*!< Bit mask of PIN13 field. */

/* Bit 12 : Pin 12. */
#define GPIO_OUTCLR_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_OUTCLR_PIN12_Msk (0x1UL << GPIO_OUTCLR_PIN12_Pos) /*!< Bit mask of PIN12 field. */

/* Bit 11 : Pin 11. */
#define GPIO_OUTCLR_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_OUTCLR_PIN11_Msk (0x1UL << GPIO_OUTCLR_PIN11_Pos) /*!< Bit mask of PIN11 field. */

/* Bit 10 : Pin 10. */
#define GPIO_OUTCLR_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_OUTCLR_PIN10_Msk (0x1UL << GPIO_OUTCLR_PIN10_Pos) /*!< Bit mask of PIN10 field. */

/* Bit 9 : Pin 9. */
#define GPIO_OUTCLR_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_OUTCLR_PIN9_Msk (0x1UL << GPIO_OUTCLR_PIN9_Pos) /*!< Bit mask of PIN9 field. */

/* Bit 8 : Pin 8. */
#define GPIO_OUTCLR_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_OUTCLR_PIN8_Msk (0x1UL << GPIO_OUTCLR_PIN8_Pos) /*!< Bit mask of PIN8 field. */

/* Bit 7 : Pin 7. */
#define GPIO_OUTCLR_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_OUTCLR_PIN7_Msk (0x1UL << GPIO_OUTCLR_PIN7_Pos) /*!< Bit mask of PIN7 field. */

/* Bit 6 : Pin 6. */
#define GPIO_OUTCLR_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_OUTCLR_PIN6_Msk (0x1UL << GPIO_OUTCLR_PIN6_Pos) /*!< Bit mask of PIN6 field. */

/* Bit 5 : Pin 5. */
#define GPIO_OUTCLR_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_OUTCLR_PIN5_Msk (0x1UL << GPIO_OUTCLR_PIN5_Pos) /*!< Bit mask of PIN5 field. */

/* Bit 4 : Pin 4. */
#define GPIO_OUTCLR_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_OUTCLR_PIN4_Msk (0x1UL << GPIO_OUTCLR_PIN4_Pos) /*!< Bit mask of PIN4 field. */

/* Bit 3 : Pin 3. */
#define GPIO_OUTCLR_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_OUTCLR_PIN3_Msk (0x1UL << GPIO_OUTCLR_PIN3_Pos) /*!< Bit mask of PIN3 field. */

/* Bit 2 : Pin 2. */
#define GPIO_OUTCLR_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_OUTCLR_PIN2_Msk (0x1UL << GPIO_OUTCLR_PIN2_Pos) /*!< Bit mask of PIN2 field. */

/* Bit 1 : Pin 1. */
#define GPIO_OUTCLR_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_OUTCLR_PIN1_Msk (0x1UL << GPIO_OUTCLR_PIN1_Pos) /*!< Bit mask of PIN1 field. */

/* Bit 0 : Pin 0. */
#define GPIO_OUTCLR_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_OUTCLR_PIN0_Msk (0x1UL << GPIO_OUTCLR_PIN0_Pos) /*!< Bit mask of PIN0 field. */

/* Register: GPIO_DIR */
/* Description: Direction of GPIO pins. */

/* Bit 31 : Pin 31. */
#define GPIO_DIR_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_DIR_PIN31_Msk (0x1UL << GPIO_DIR_PIN31_Pos) /*!< Bit mask of PIN31 field. */
#define GPIO_DIR_PIN31_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN31_Output (1UL) /*!< Pin set as output. */

/* Bit 30 : Pin 30. */
#define GPIO_DIR_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_DIR_PIN30_Msk (0x1UL << GPIO_DIR_PIN30_Pos) /*!< Bit mask of PIN30 field. */
#define GPIO_DIR_PIN30_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN30_Output (1UL) /*!< Pin set as output. */

/* Bit 29 : Pin 29. */
#define GPIO_DIR_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_DIR_PIN29_Msk (0x1UL << GPIO_DIR_PIN29_Pos) /*!< Bit mask of PIN29 field. */
#define GPIO_DIR_PIN29_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN29_Output (1UL) /*!< Pin set as output. */

/* Bit 28 : Pin 28. */
#define GPIO_DIR_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_DIR_PIN28_Msk (0x1UL << GPIO_DIR_PIN28_Pos) /*!< Bit mask of PIN28 field. */
#define GPIO_DIR_PIN28_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN28_Output (1UL) /*!< Pin set as output. */

/* Bit 27 : Pin 27. */
#define GPIO_DIR_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_DIR_PIN27_Msk (0x1UL << GPIO_DIR_PIN27_Pos) /*!< Bit mask of PIN27 field. */
#define GPIO_DIR_PIN27_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN27_Output (1UL) /*!< Pin set as output. */

/* Bit 26 : Pin 26. */
#define GPIO_DIR_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_DIR_PIN26_Msk (0x1UL << GPIO_DIR_PIN26_Pos) /*!< Bit mask of PIN26 field. */
#define GPIO_DIR_PIN26_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN26_Output (1UL) /*!< Pin set as output. */

/* Bit 25 : Pin 25. */
#define GPIO_DIR_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_DIR_PIN25_Msk (0x1UL << GPIO_DIR_PIN25_Pos) /*!< Bit mask of PIN25 field. */
#define GPIO_DIR_PIN25_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN25_Output (1UL) /*!< Pin set as output. */

/* Bit 24 : Pin 24. */
#define GPIO_DIR_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_DIR_PIN24_Msk (0x1UL << GPIO_DIR_PIN24_Pos) /*!< Bit mask of PIN24 field. */
#define GPIO_DIR_PIN24_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN24_Output (1UL) /*!< Pin set as output. */

/* Bit 23 : Pin 23. */
#define GPIO_DIR_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_DIR_PIN23_Msk (0x1UL << GPIO_DIR_PIN23_Pos) /*!< Bit mask of PIN23 field. */
#define GPIO_DIR_PIN23_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN23_Output (1UL) /*!< Pin set as output. */

/* Bit 22 : Pin 22. */
#define GPIO_DIR_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_DIR_PIN22_Msk (0x1UL << GPIO_DIR_PIN22_Pos) /*!< Bit mask of PIN22 field. */
#define GPIO_DIR_PIN22_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN22_Output (1UL) /*!< Pin set as output. */

/* Bit 21 : Pin 21. */
#define GPIO_DIR_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_DIR_PIN21_Msk (0x1UL << GPIO_DIR_PIN21_Pos) /*!< Bit mask of PIN21 field. */
#define GPIO_DIR_PIN21_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN21_Output (1UL) /*!< Pin set as output. */

/* Bit 20 : Pin 20. */
#define GPIO_DIR_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_DIR_PIN20_Msk (0x1UL << GPIO_DIR_PIN20_Pos) /*!< Bit mask of PIN20 field. */
#define GPIO_DIR_PIN20_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN20_Output (1UL) /*!< Pin set as output. */

/* Bit 19 : Pin 19. */
#define GPIO_DIR_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_DIR_PIN19_Msk (0x1UL << GPIO_DIR_PIN19_Pos) /*!< Bit mask of PIN19 field. */
#define GPIO_DIR_PIN19_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN19_Output (1UL) /*!< Pin set as output. */

/* Bit 18 : Pin 18. */
#define GPIO_DIR_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_DIR_PIN18_Msk (0x1UL << GPIO_DIR_PIN18_Pos) /*!< Bit mask of PIN18 field. */
#define GPIO_DIR_PIN18_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN18_Output (1UL) /*!< Pin set as output. */

/* Bit 17 : Pin 17. */
#define GPIO_DIR_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_DIR_PIN17_Msk (0x1UL << GPIO_DIR_PIN17_Pos) /*!< Bit mask of PIN17 field. */
#define GPIO_DIR_PIN17_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN17_Output (1UL) /*!< Pin set as output. */

/* Bit 16 : Pin 16. */
#define GPIO_DIR_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_DIR_PIN16_Msk (0x1UL << GPIO_DIR_PIN16_Pos) /*!< Bit mask of PIN16 field. */
#define GPIO_DIR_PIN16_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN16_Output (1UL) /*!< Pin set as output. */

/* Bit 15 : Pin 15. */
#define GPIO_DIR_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_DIR_PIN15_Msk (0x1UL << GPIO_DIR_PIN15_Pos) /*!< Bit mask of PIN15 field. */
#define GPIO_DIR_PIN15_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN15_Output (1UL) /*!< Pin set as output. */

/* Bit 14 : Pin 14. */
#define GPIO_DIR_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_DIR_PIN14_Msk (0x1UL << GPIO_DIR_PIN14_Pos) /*!< Bit mask of PIN14 field. */
#define GPIO_DIR_PIN14_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN14_Output (1UL) /*!< Pin set as output. */

/* Bit 13 : Pin 13. */
#define GPIO_DIR_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_DIR_PIN13_Msk (0x1UL << GPIO_DIR_PIN13_Pos) /*!< Bit mask of PIN13 field. */
#define GPIO_DIR_PIN13_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN13_Output (1UL) /*!< Pin set as output. */

/* Bit 12 : Pin 12. */
#define GPIO_DIR_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_DIR_PIN12_Msk (0x1UL << GPIO_DIR_PIN12_Pos) /*!< Bit mask of PIN12 field. */
#define GPIO_DIR_PIN12_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN12_Output (1UL) /*!< Pin set as output. */

/* Bit 11 : Pin 11. */
#define GPIO_DIR_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_DIR_PIN11_Msk (0x1UL << GPIO_DIR_PIN11_Pos) /*!< Bit mask of PIN11 field. */
#define GPIO_DIR_PIN11_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN11_Output (1UL) /*!< Pin set as output. */

/* Bit 10 : Pin 10. */
#define GPIO_DIR_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_DIR_PIN10_Msk (0x1UL << GPIO_DIR_PIN10_Pos) /*!< Bit mask of PIN10 field. */
#define GPIO_DIR_PIN10_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN10_Output (1UL) /*!< Pin set as output. */

/* Bit 9 : Pin 9. */
#define GPIO_DIR_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_DIR_PIN9_Msk (0x1UL << GPIO_DIR_PIN9_Pos) /*!< Bit mask of PIN9 field. */
#define GPIO_DIR_PIN9_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN9_Output (1UL) /*!< Pin set as output. */

/* Bit 8 : Pin 8. */
#define GPIO_DIR_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_DIR_PIN8_Msk (0x1UL << GPIO_DIR_PIN8_Pos) /*!< Bit mask of PIN8 field. */
#define GPIO_DIR_PIN8_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN8_Output (1UL) /*!< Pin set as output. */

/* Bit 7 : Pin 7. */
#define GPIO_DIR_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_DIR_PIN7_Msk (0x1UL << GPIO_DIR_PIN7_Pos) /*!< Bit mask of PIN7 field. */
#define GPIO_DIR_PIN7_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN7_Output (1UL) /*!< Pin set as output. */

/* Bit 6 : Pin 6. */
#define GPIO_DIR_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_DIR_PIN6_Msk (0x1UL << GPIO_DIR_PIN6_Pos) /*!< Bit mask of PIN6 field. */
#define GPIO_DIR_PIN6_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN6_Output (1UL) /*!< Pin set as output. */

/* Bit 5 : Pin 5. */
#define GPIO_DIR_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_DIR_PIN5_Msk (0x1UL << GPIO_DIR_PIN5_Pos) /*!< Bit mask of PIN5 field. */
#define GPIO_DIR_PIN5_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN5_Output (1UL) /*!< Pin set as output. */

/* Bit 4 : Pin 4. */
#define GPIO_DIR_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_DIR_PIN4_Msk (0x1UL << GPIO_DIR_PIN4_Pos) /*!< Bit mask of PIN4 field. */
#define GPIO_DIR_PIN4_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN4_Output (1UL) /*!< Pin set as output. */

/* Bit 3 : Pin 3. */
#define GPIO_DIR_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_DIR_PIN3_Msk (0x1UL << GPIO_DIR_PIN3_Pos) /*!< Bit mask of PIN3 field. */
#define GPIO_DIR_PIN3_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN3_Output (1UL) /*!< Pin set as output. */

/* Bit 2 : Pin 2. */
#define GPIO_DIR_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_DIR_PIN2_Msk (0x1UL << GPIO_DIR_PIN2_Pos) /*!< Bit mask of PIN2 field. */
#define GPIO_DIR_PIN2_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN2_Output (1UL) /*!< Pin set as output. */

/* Bit 1 : Pin 1. */
#define GPIO_DIR_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_DIR_PIN1_Msk (0x1UL << GPIO_DIR_PIN1_Pos) /*!< Bit mask of PIN1 field. */
#define GPIO_DIR_PIN1_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN1_Output (1UL) /*!< Pin set as output. */

/* Bit 0 : Pin 0. */
#define GPIO_DIR_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_DIR_PIN0_Msk (0x1UL << GPIO_DIR_PIN0_Pos) /*!< Bit mask of PIN0 field. */
#define GPIO_DIR_PIN0_Input (0UL) /*!< Pin set as input. */
#define GPIO_DIR_PIN0_Output (1UL) /*!< Pin set as output. */

/* Register: GPIO_DIRSET */
/* Description: Setting DIR register. */

/* Bit 31 : Set as output pin 31. */
#define GPIO_DIRSET_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_DIRSET_PIN31_Msk (0x1UL << GPIO_DIRSET_PIN31_Pos) /*!< Bit mask of PIN31 field. */

/* Bit 30 : Set as output pin 30. */
#define GPIO_DIRSET_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_DIRSET_PIN30_Msk (0x1UL << GPIO_DIRSET_PIN30_Pos) /*!< Bit mask of PIN30 field. */

/* Bit 29 : Set as output pin 29. */
#define GPIO_DIRSET_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_DIRSET_PIN29_Msk (0x1UL << GPIO_DIRSET_PIN29_Pos) /*!< Bit mask of PIN29 field. */

/* Bit 28 : Set as output pin 28. */
#define GPIO_DIRSET_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_DIRSET_PIN28_Msk (0x1UL << GPIO_DIRSET_PIN28_Pos) /*!< Bit mask of PIN28 field. */

/* Bit 27 : Set as output pin 27. */
#define GPIO_DIRSET_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_DIRSET_PIN27_Msk (0x1UL << GPIO_DIRSET_PIN27_Pos) /*!< Bit mask of PIN27 field. */

/* Bit 26 : Set as output pin 26. */
#define GPIO_DIRSET_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_DIRSET_PIN26_Msk (0x1UL << GPIO_DIRSET_PIN26_Pos) /*!< Bit mask of PIN26 field. */

/* Bit 25 : Set as output pin 25. */
#define GPIO_DIRSET_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_DIRSET_PIN25_Msk (0x1UL << GPIO_DIRSET_PIN25_Pos) /*!< Bit mask of PIN25 field. */

/* Bit 24 : Set as output pin 24. */
#define GPIO_DIRSET_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_DIRSET_PIN24_Msk (0x1UL << GPIO_DIRSET_PIN24_Pos) /*!< Bit mask of PIN24 field. */

/* Bit 23 : Set as output pin 23. */
#define GPIO_DIRSET_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_DIRSET_PIN23_Msk (0x1UL << GPIO_DIRSET_PIN23_Pos) /*!< Bit mask of PIN23 field. */

/* Bit 22 : Set as output pin 22. */
#define GPIO_DIRSET_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_DIRSET_PIN22_Msk (0x1UL << GPIO_DIRSET_PIN22_Pos) /*!< Bit mask of PIN22 field. */

/* Bit 21 : Set as output pin 21. */
#define GPIO_DIRSET_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_DIRSET_PIN21_Msk (0x1UL << GPIO_DIRSET_PIN21_Pos) /*!< Bit mask of PIN21 field. */

/* Bit 20 : Set as output pin 20. */
#define GPIO_DIRSET_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_DIRSET_PIN20_Msk (0x1UL << GPIO_DIRSET_PIN20_Pos) /*!< Bit mask of PIN20 field. */

/* Bit 19 : Set as output pin 19. */
#define GPIO_DIRSET_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_DIRSET_PIN19_Msk (0x1UL << GPIO_DIRSET_PIN19_Pos) /*!< Bit mask of PIN19 field. */

/* Bit 18 : Set as output pin 18. */
#define GPIO_DIRSET_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_DIRSET_PIN18_Msk (0x1UL << GPIO_DIRSET_PIN18_Pos) /*!< Bit mask of PIN18 field. */

/* Bit 17 : Set as output pin 17. */
#define GPIO_DIRSET_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_DIRSET_PIN17_Msk (0x1UL << GPIO_DIRSET_PIN17_Pos) /*!< Bit mask of PIN17 field. */

/* Bit 16 : Set as output pin 16. */
#define GPIO_DIRSET_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_DIRSET_PIN16_Msk (0x1UL << GPIO_DIRSET_PIN16_Pos) /*!< Bit mask of PIN16 field. */

/* Bit 15 : Set as output pin 15. */
#define GPIO_DIRSET_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_DIRSET_PIN15_Msk (0x1UL << GPIO_DIRSET_PIN15_Pos) /*!< Bit mask of PIN15 field. */

/* Bit 14 : Set as output pin 14. */
#define GPIO_DIRSET_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_DIRSET_PIN14_Msk (0x1UL << GPIO_DIRSET_PIN14_Pos) /*!< Bit mask of PIN14 field. */

/* Bit 13 : Set as output pin 13. */
#define GPIO_DIRSET_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_DIRSET_PIN13_Msk (0x1UL << GPIO_DIRSET_PIN13_Pos) /*!< Bit mask of PIN13 field. */

/* Bit 12 : Set as output pin 12. */
#define GPIO_DIRSET_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_DIRSET_PIN12_Msk (0x1UL << GPIO_DIRSET_PIN12_Pos) /*!< Bit mask of PIN12 field. */

/* Bit 11 : Set as output pin 11. */
#define GPIO_DIRSET_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_DIRSET_PIN11_Msk (0x1UL << GPIO_DIRSET_PIN11_Pos) /*!< Bit mask of PIN11 field. */

/* Bit 10 : Set as output pin 10. */
#define GPIO_DIRSET_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_DIRSET_PIN10_Msk (0x1UL << GPIO_DIRSET_PIN10_Pos) /*!< Bit mask of PIN10 field. */

/* Bit 9 : Set as output pin 9. */
#define GPIO_DIRSET_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_DIRSET_PIN9_Msk (0x1UL << GPIO_DIRSET_PIN9_Pos) /*!< Bit mask of PIN9 field. */

/* Bit 8 : Set as output pin 8. */
#define GPIO_DIRSET_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_DIRSET_PIN8_Msk (0x1UL << GPIO_DIRSET_PIN8_Pos) /*!< Bit mask of PIN8 field. */

/* Bit 7 : Set as output pin 7. */
#define GPIO_DIRSET_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_DIRSET_PIN7_Msk (0x1UL << GPIO_DIRSET_PIN7_Pos) /*!< Bit mask of PIN7 field. */

/* Bit 6 : Set as output pin 6. */
#define GPIO_DIRSET_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_DIRSET_PIN6_Msk (0x1UL << GPIO_DIRSET_PIN6_Pos) /*!< Bit mask of PIN6 field. */

/* Bit 5 : Set as output pin 5. */
#define GPIO_DIRSET_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_DIRSET_PIN5_Msk (0x1UL << GPIO_DIRSET_PIN5_Pos) /*!< Bit mask of PIN5 field. */

/* Bit 4 : Set as output pin 4. */
#define GPIO_DIRSET_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_DIRSET_PIN4_Msk (0x1UL << GPIO_DIRSET_PIN4_Pos) /*!< Bit mask of PIN4 field. */

/* Bit 3 : Set as output pin 3. */
#define GPIO_DIRSET_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_DIRSET_PIN3_Msk (0x1UL << GPIO_DIRSET_PIN3_Pos) /*!< Bit mask of PIN3 field. */

/* Bit 2 : Set as output pin 2. */
#define GPIO_DIRSET_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_DIRSET_PIN2_Msk (0x1UL << GPIO_DIRSET_PIN2_Pos) /*!< Bit mask of PIN2 field. */

/* Bit 1 : Set as output pin 1. */
#define GPIO_DIRSET_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_DIRSET_PIN1_Msk (0x1UL << GPIO_DIRSET_PIN1_Pos) /*!< Bit mask of PIN1 field. */

/* Bit 0 : Set as output pin 0. */
#define GPIO_DIRSET_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_DIRSET_PIN0_Msk (0x1UL << GPIO_DIRSET_PIN0_Pos) /*!< Bit mask of PIN0 field. */

/* Register: GPIO_DIRCLR */
/* Description: Clearing DIR register. */

/* Bit 31 : Set as input pin 31. */
#define GPIO_DIRCLR_PIN31_Pos (31U) /*!< Position of PIN31 field. */
#define GPIO_DIRCLR_PIN31_Msk (0x1UL << GPIO_DIRCLR_PIN31_Pos) /*!< Bit mask of PIN31 field. */

/* Bit 30 : Set as input pin 30. */
#define GPIO_DIRCLR_PIN30_Pos (30U) /*!< Position of PIN30 field. */
#define GPIO_DIRCLR_PIN30_Msk (0x1UL << GPIO_DIRCLR_PIN30_Pos) /*!< Bit mask of PIN30 field. */

/* Bit 29 : Set as input pin 29. */
#define GPIO_DIRCLR_PIN29_Pos (29U) /*!< Position of PIN29 field. */
#define GPIO_DIRCLR_PIN29_Msk (0x1UL << GPIO_DIRCLR_PIN29_Pos) /*!< Bit mask of PIN29 field. */

/* Bit 28 : Set as input pin 28. */
#define GPIO_DIRCLR_PIN28_Pos (28U) /*!< Position of PIN28 field. */
#define GPIO_DIRCLR_PIN28_Msk (0x1UL << GPIO_DIRCLR_PIN28_Pos) /*!< Bit mask of PIN28 field. */

/* Bit 27 : Set as input pin 27. */
#define GPIO_DIRCLR_PIN27_Pos (27U) /*!< Position of PIN27 field. */
#define GPIO_DIRCLR_PIN27_Msk (0x1UL << GPIO_DIRCLR_PIN27_Pos) /*!< Bit mask of PIN27 field. */

/* Bit 26 : Set as input pin 26. */
#define GPIO_DIRCLR_PIN26_Pos (26U) /*!< Position of PIN26 field. */
#define GPIO_DIRCLR_PIN26_Msk (0x1UL << GPIO_DIRCLR_PIN26_Pos) /*!< Bit mask of PIN26 field. */

/* Bit 25 : Set as input pin 25. */
#define GPIO_DIRCLR_PIN25_Pos (25U) /*!< Position of PIN25 field. */
#define GPIO_DIRCLR_PIN25_Msk (0x1UL << GPIO_DIRCLR_PIN25_Pos) /*!< Bit mask of PIN25 field. */

/* Bit 24 : Set as input pin 24. */
#define GPIO_DIRCLR_PIN24_Pos (24U) /*!< Position of PIN24 field. */
#define GPIO_DIRCLR_PIN24_Msk (0x1UL << GPIO_DIRCLR_PIN24_Pos) /*!< Bit mask of PIN24 field. */

/* Bit 23 : Set as input pin 23. */
#define GPIO_DIRCLR_PIN23_Pos (23U) /*!< Position of PIN23 field. */
#define GPIO_DIRCLR_PIN23_Msk (0x1UL << GPIO_DIRCLR_PIN23_Pos) /*!< Bit mask of PIN23 field. */

/* Bit 22 : Set as input pin 22. */
#define GPIO_DIRCLR_PIN22_Pos (22U) /*!< Position of PIN22 field. */
#define GPIO_DIRCLR_PIN22_Msk (0x1UL << GPIO_DIRCLR_PIN22_Pos) /*!< Bit mask of PIN22 field. */

/* Bit 21 : Set as input pin 21. */
#define GPIO_DIRCLR_PIN21_Pos (21U) /*!< Position of PIN21 field. */
#define GPIO_DIRCLR_PIN21_Msk (0x1UL << GPIO_DIRCLR_PIN21_Pos) /*!< Bit mask of PIN21 field. */

/* Bit 20 : Set as input pin 20. */
#define GPIO_DIRCLR_PIN20_Pos (20U) /*!< Position of PIN20 field. */
#define GPIO_DIRCLR_PIN20_Msk (0x1UL << GPIO_DIRCLR_PIN20_Pos) /*!< Bit mask of PIN20 field. */

/* Bit 19 : Set as input pin 19. */
#define GPIO_DIRCLR_PIN19_Pos (19U) /*!< Position of PIN19 field. */
#define GPIO_DIRCLR_PIN19_Msk (0x1UL << GPIO_DIRCLR_PIN19_Pos) /*!< Bit mask of PIN19 field. */

/* Bit 18 : Set as input pin 18. */
#define GPIO_DIRCLR_PIN18_Pos (18U) /*!< Position of PIN18 field. */
#define GPIO_DIRCLR_PIN18_Msk (0x1UL << GPIO_DIRCLR_PIN18_Pos) /*!< Bit mask of PIN18 field. */

/* Bit 17 : Set as input pin 17. */
#define GPIO_DIRCLR_PIN17_Pos (17U) /*!< Position of PIN17 field. */
#define GPIO_DIRCLR_PIN17_Msk (0x1UL << GPIO_DIRCLR_PIN17_Pos) /*!< Bit mask of PIN17 field. */

/* Bit 16 : Set as input pin 16. */
#define GPIO_DIRCLR_PIN16_Pos (16U) /*!< Position of PIN16 field. */
#define GPIO_DIRCLR_PIN16_Msk (0x1UL << GPIO_DIRCLR_PIN16_Pos) /*!< Bit mask of PIN16 field. */

/* Bit 15 : Set as input pin 15. */
#define GPIO_DIRCLR_PIN15_Pos (15U) /*!< Position of PIN15 field. */
#define GPIO_DIRCLR_PIN15_Msk (0x1UL << GPIO_DIRCLR_PIN15_Pos) /*!< Bit mask of PIN15 field. */

/* Bit 14 : Set as input pin 14. */
#define GPIO_DIRCLR_PIN14_Pos (14U) /*!< Position of PIN14 field. */
#define GPIO_DIRCLR_PIN14_Msk (0x1UL << GPIO_DIRCLR_PIN14_Pos) /*!< Bit mask of PIN14 field. */

/* Bit 13 : Set as input pin 13. */
#define GPIO_DIRCLR_PIN13_Pos (13U) /*!< Position of PIN13 field. */
#define GPIO_DIRCLR_PIN13_Msk (0x1UL << GPIO_DIRCLR_PIN13_Pos) /*!< Bit mask of PIN13 field. */

/* Bit 12 : Set as input pin 12. */
#define GPIO_DIRCLR_PIN12_Pos (12U) /*!< Position of PIN12 field. */
#define GPIO_DIRCLR_PIN12_Msk (0x1UL << GPIO_DIRCLR_PIN12_Pos) /*!< Bit mask of PIN12 field. */

/* Bit 11 : Set as input pin 11. */
#define GPIO_DIRCLR_PIN11_Pos (11U) /*!< Position of PIN11 field. */
#define GPIO_DIRCLR_PIN11_Msk (0x1UL << GPIO_DIRCLR_PIN11_Pos) /*!< Bit mask of PIN11 field. */

/* Bit 10 : Set as input pin 10. */
#define GPIO_DIRCLR_PIN10_Pos (10U) /*!< Position of PIN10 field. */
#define GPIO_DIRCLR_PIN10_Msk (0x1UL << GPIO_DIRCLR_PIN10_Pos) /*!< Bit mask of PIN10 field. */

/* Bit 9 : Set as input pin 9. */
#define GPIO_DIRCLR_PIN9_Pos (9U) /*!< Position of PIN9 field. */
#define GPIO_DIRCLR_PIN9_Msk (0x1UL << GPIO_DIRCLR_PIN9_Pos) /*!< Bit mask of PIN9 field. */

/* Bit 8 : Set as input pin 8. */
#define GPIO_DIRCLR_PIN8_Pos (8U) /*!< Position of PIN8 field. */
#define GPIO_DIRCLR_PIN8_Msk (0x1UL << GPIO_DIRCLR_PIN8_Pos) /*!< Bit mask of PIN8 field. */

/* Bit 7 : Set as input pin 7. */
#define GPIO_DIRCLR_PIN7_Pos (7U) /*!< Position of PIN7 field. */
#define GPIO_DIRCLR_PIN7_Msk (0x1UL << GPIO_DIRCLR_PIN7_Pos) /*!< Bit mask of PIN7 field. */

/* Bit 6 : Set as input pin 6. */
#define GPIO_DIRCLR_PIN6_Pos (6U) /*!< Position of PIN6 field. */
#define GPIO_DIRCLR_PIN6_Msk (0x1UL << GPIO_DIRCLR_PIN6_Pos) /*!< Bit mask of PIN6 field. */

/* Bit 5 : Set as input pin 5. */
#define GPIO_DIRCLR_PIN5_Pos (5U) /*!< Position of PIN5 field. */
#define GPIO_DIRCLR_PIN5_Msk (0x1UL << GPIO_DIRCLR_PIN5_Pos) /*!< Bit mask of PIN5 field. */

/* Bit 4 : Set as input pin 4. */
#define GPIO_DIRCLR_PIN4_Pos (4U) /*!< Position of PIN4 field. */
#define GPIO_DIRCLR_PIN4_Msk (0x1UL << GPIO_DIRCLR_PIN4_Pos) /*!< Bit mask of PIN4 field. */

/* Bit 3 : Set as input pin 3. */
#define GPIO_DIRCLR_PIN3_Pos (3U) /*!< Position of PIN3 field. */
#define GPIO_DIRCLR_PIN3_Msk (0x1UL << GPIO_DIRCLR_PIN3_Pos) /*!< Bit mask of PIN3 field. */

/* Bit 2 : Set as input pin 2. */
#define GPIO_DIRCLR_PIN2_Pos (2U) /*!< Position of PIN2 field. */
#define GPIO_DIRCLR_PIN2_Msk (0x1UL << GPIO_DIRCLR_PIN2_Pos) /*!< Bit mask of PIN2 field. */

/* Bit 1 : Set as input pin 1. */
#define GPIO_DIRCLR_PIN1_Pos (1U) /*!< Position of PIN1 field. */
#define GPIO_DIRCLR_PIN1_Msk (0x1UL << GPIO_DIRCLR_PIN1_Pos) /*!< Bit mask of PIN1 field. */

/* Bit 0 : Set as input pin 0. */
#define GPIO_DIRCLR_PIN0_Pos (0U) /*!< Position of PIN0 field. */
#define GPIO_DIRCLR_PIN0_Msk (0x1UL << GPIO_DIRCLR_PIN0_Pos) /*!< Bit mask of PIN0 field. */

/* Register: GPIO_PIN0_OUT */
/* Description: GPIO pin 0 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN0_OUT_PIN0_OUT_Pos (0U) /*!< Position of PIN0_OUT field. */
#define GPIO_PIN0_OUT_PIN0_OUT_Msk (0x1UL << GPIO_PIN0_OUT_PIN0_OUT_Pos) /*!< Bit mask of PIN0_OUT field. */

/* Register: GPIO_PIN0_IN */
/* Description: GPIO pin 0 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN0_IN_PIN0_IN_Pos (0U) /*!< Position of PIN0_IN field. */
#define GPIO_PIN0_IN_PIN0_IN_Msk (0x1UL << GPIO_PIN0_IN_PIN0_IN_Pos) /*!< Bit mask of PIN0_IN field. */

/* Register: GPIO_PIN1_OUT */
/* Description: GPIO pin 1 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN1_OUT_PIN1_OUT_Pos (0U) /*!< Position of PIN1_OUT field. */
#define GPIO_PIN1_OUT_PIN1_OUT_Msk (0x1UL << GPIO_PIN1_OUT_PIN1_OUT_Pos) /*!< Bit mask of PIN1_OUT field. */

/* Register: GPIO_PIN1_IN */
/* Description: GPIO pin 1 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN1_IN_PIN1_IN_Pos (0U) /*!< Position of PIN1_IN field. */
#define GPIO_PIN1_IN_PIN1_IN_Msk (0x1UL << GPIO_PIN1_IN_PIN1_IN_Pos) /*!< Bit mask of PIN1_IN field. */

/* Register: GPIO_PIN2_OUT */
/* Description: GPIO pin 2 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN2_OUT_PIN2_OUT_Pos (0U) /*!< Position of PIN2_OUT field. */
#define GPIO_PIN2_OUT_PIN2_OUT_Msk (0x1UL << GPIO_PIN2_OUT_PIN2_OUT_Pos) /*!< Bit mask of PIN2_OUT field. */

/* Register: GPIO_PIN2_IN */
/* Description: GPIO pin 2 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN2_IN_PIN2_IN_Pos (0U) /*!< Position of PIN2_IN field. */
#define GPIO_PIN2_IN_PIN2_IN_Msk (0x1UL << GPIO_PIN2_IN_PIN2_IN_Pos) /*!< Bit mask of PIN2_IN field. */

/* Register: GPIO_PIN3_OUT */
/* Description: GPIO pin 3 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN3_OUT_PIN3_OUT_Pos (0U) /*!< Position of PIN3_OUT field. */
#define GPIO_PIN3_OUT_PIN3_OUT_Msk (0x1UL << GPIO_PIN3_OUT_PIN3_OUT_Pos) /*!< Bit mask of PIN3_OUT field. */

/* Register: GPIO_PIN3_IN */
/* Description: GPIO pin 3 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN3_IN_PIN3_IN_Pos (0U) /*!< Position of PIN3_IN field. */
#define GPIO_PIN3_IN_PIN3_IN_Msk (0x1UL << GPIO_PIN3_IN_PIN3_IN_Pos) /*!< Bit mask of PIN3_IN field. */

/* Register: GPIO_PIN4_OUT */
/* Description: GPIO pin 4 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN4_OUT_PIN4_OUT_Pos (0U) /*!< Position of PIN4_OUT field. */
#define GPIO_PIN4_OUT_PIN4_OUT_Msk (0x1UL << GPIO_PIN4_OUT_PIN4_OUT_Pos) /*!< Bit mask of PIN4_OUT field. */

/* Register: GPIO_PIN4_IN */
/* Description: GPIO pin 4 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN4_IN_PIN4_IN_Pos (0U) /*!< Position of PIN4_IN field. */
#define GPIO_PIN4_IN_PIN4_IN_Msk (0x1UL << GPIO_PIN4_IN_PIN4_IN_Pos) /*!< Bit mask of PIN4_IN field. */

/* Register: GPIO_PIN5_OUT */
/* Description: GPIO pin 5 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN5_OUT_PIN5_OUT_Pos (0U) /*!< Position of PIN5_OUT field. */
#define GPIO_PIN5_OUT_PIN5_OUT_Msk (0x1UL << GPIO_PIN5_OUT_PIN5_OUT_Pos) /*!< Bit mask of PIN5_OUT field. */

/* Register: GPIO_PIN5_IN */
/* Description: GPIO pin 5 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN5_IN_PIN5_IN_Pos (0U) /*!< Position of PIN5_IN field. */
#define GPIO_PIN5_IN_PIN5_IN_Msk (0x1UL << GPIO_PIN5_IN_PIN5_IN_Pos) /*!< Bit mask of PIN5_IN field. */

/* Register: GPIO_PIN6_OUT */
/* Description: GPIO pin 6 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN6_OUT_PIN6_OUT_Pos (0U) /*!< Position of PIN6_OUT field. */
#define GPIO_PIN6_OUT_PIN6_OUT_Msk (0x1UL << GPIO_PIN6_OUT_PIN6_OUT_Pos) /*!< Bit mask of PIN6_OUT field. */

/* Register: GPIO_PIN6_IN */
/* Description: GPIO pin 6 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN6_IN_PIN6_IN_Pos (0U) /*!< Position of PIN6_IN field. */
#define GPIO_PIN6_IN_PIN6_IN_Msk (0x1UL << GPIO_PIN6_IN_PIN6_IN_Pos) /*!< Bit mask of PIN6_IN field. */

/* Register: GPIO_PIN7_OUT */
/* Description: GPIO pin 7 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN7_OUT_PIN7_OUT_Pos (0U) /*!< Position of PIN7_OUT field. */
#define GPIO_PIN7_OUT_PIN7_OUT_Msk (0x1UL << GPIO_PIN7_OUT_PIN7_OUT_Pos) /*!< Bit mask of PIN7_OUT field. */

/* Register: GPIO_PIN7_IN */
/* Description: GPIO pin 7 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN7_IN_PIN7_IN_Pos (0U) /*!< Position of PIN7_IN field. */
#define GPIO_PIN7_IN_PIN7_IN_Msk (0x1UL << GPIO_PIN7_IN_PIN7_IN_Pos) /*!< Bit mask of PIN7_IN field. */

/* Register: GPIO_PIN8_OUT */
/* Description: GPIO pin 8 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN8_OUT_PIN8_OUT_Pos (0U) /*!< Position of PIN8_OUT field. */
#define GPIO_PIN8_OUT_PIN8_OUT_Msk (0x1UL << GPIO_PIN8_OUT_PIN8_OUT_Pos) /*!< Bit mask of PIN8_OUT field. */

/* Register: GPIO_PIN8_IN */
/* Description: GPIO pin 8 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN8_IN_PIN8_IN_Pos (0U) /*!< Position of PIN8_IN field. */
#define GPIO_PIN8_IN_PIN8_IN_Msk (0x1UL << GPIO_PIN8_IN_PIN8_IN_Pos) /*!< Bit mask of PIN8_IN field. */

/* Register: GPIO_PIN9_OUT */
/* Description: GPIO pin 9 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN9_OUT_PIN9_OUT_Pos (0U) /*!< Position of PIN9_OUT field. */
#define GPIO_PIN9_OUT_PIN9_OUT_Msk (0x1UL << GPIO_PIN9_OUT_PIN9_OUT_Pos) /*!< Bit mask of PIN9_OUT field. */

/* Register: GPIO_PIN9_IN */
/* Description: GPIO pin 9 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN9_IN_PIN9_IN_Pos (0U) /*!< Position of PIN9_IN field. */
#define GPIO_PIN9_IN_PIN9_IN_Msk (0x1UL << GPIO_PIN9_IN_PIN9_IN_Pos) /*!< Bit mask of PIN9_IN field. */

/* Register: GPIO_PIN10_OUT */
/* Description: GPIO pin 10 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN10_OUT_PIN10_OUT_Pos (0U) /*!< Position of PIN10_OUT field. */
#define GPIO_PIN10_OUT_PIN10_OUT_Msk (0x1UL << GPIO_PIN10_OUT_PIN10_OUT_Pos) /*!< Bit mask of PIN10_OUT field. */

/* Register: GPIO_PIN10_IN */
/* Description: GPIO pin 10 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN10_IN_PIN10_IN_Pos (0U) /*!< Position of PIN10_IN field. */
#define GPIO_PIN10_IN_PIN10_IN_Msk (0x1UL << GPIO_PIN10_IN_PIN10_IN_Pos) /*!< Bit mask of PIN10_IN field. */

/* Register: GPIO_PIN11_OUT */
/* Description: GPIO pin 11 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN11_OUT_PIN11_OUT_Pos (0U) /*!< Position of PIN11_OUT field. */
#define GPIO_PIN11_OUT_PIN11_OUT_Msk (0x1UL << GPIO_PIN11_OUT_PIN11_OUT_Pos) /*!< Bit mask of PIN11_OUT field. */

/* Register: GPIO_PIN11_IN */
/* Description: GPIO pin 11 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN11_IN_PIN11_IN_Pos (0U) /*!< Position of PIN11_IN field. */
#define GPIO_PIN11_IN_PIN11_IN_Msk (0x1UL << GPIO_PIN11_IN_PIN11_IN_Pos) /*!< Bit mask of PIN11_IN field. */

/* Register: GPIO_PIN12_OUT */
/* Description: GPIO pin 12 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN12_OUT_PIN12_OUT_Pos (0U) /*!< Position of PIN12_OUT field. */
#define GPIO_PIN12_OUT_PIN12_OUT_Msk (0x1UL << GPIO_PIN12_OUT_PIN12_OUT_Pos) /*!< Bit mask of PIN12_OUT field. */

/* Register: GPIO_PIN12_IN */
/* Description: GPIO pin 12 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN12_IN_PIN12_IN_Pos (0U) /*!< Position of PIN12_IN field. */
#define GPIO_PIN12_IN_PIN12_IN_Msk (0x1UL << GPIO_PIN12_IN_PIN12_IN_Pos) /*!< Bit mask of PIN12_IN field. */

/* Register: GPIO_PIN13_OUT */
/* Description: GPIO pin 13 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN13_OUT_PIN13_OUT_Pos (0U) /*!< Position of PIN13_OUT field. */
#define GPIO_PIN13_OUT_PIN13_OUT_Msk (0x1UL << GPIO_PIN13_OUT_PIN13_OUT_Pos) /*!< Bit mask of PIN13_OUT field. */

/* Register: GPIO_PIN13_IN */
/* Description: GPIO pin 13 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN13_IN_PIN13_IN_Pos (0U) /*!< Position of PIN13_IN field. */
#define GPIO_PIN13_IN_PIN13_IN_Msk (0x1UL << GPIO_PIN13_IN_PIN13_IN_Pos) /*!< Bit mask of PIN13_IN field. */

/* Register: GPIO_PIN14_OUT */
/* Description: GPIO pin 14 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN14_OUT_PIN14_OUT_Pos (0U) /*!< Position of PIN14_OUT field. */
#define GPIO_PIN14_OUT_PIN14_OUT_Msk (0x1UL << GPIO_PIN14_OUT_PIN14_OUT_Pos) /*!< Bit mask of PIN14_OUT field. */

/* Register: GPIO_PIN14_IN */
/* Description: GPIO pin 14 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN14_IN_PIN14_IN_Pos (0U) /*!< Position of PIN14_IN field. */
#define GPIO_PIN14_IN_PIN14_IN_Msk (0x1UL << GPIO_PIN14_IN_PIN14_IN_Pos) /*!< Bit mask of PIN14_IN field. */

/* Register: GPIO_PIN15_OUT */
/* Description: GPIO pin 15 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN15_OUT_PIN15_OUT_Pos (0U) /*!< Position of PIN15_OUT field. */
#define GPIO_PIN15_OUT_PIN15_OUT_Msk (0x1UL << GPIO_PIN15_OUT_PIN15_OUT_Pos) /*!< Bit mask of PIN15_OUT field. */

/* Register: GPIO_PIN15_IN */
/* Description: GPIO pin 15 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN15_IN_PIN15_IN_Pos (0U) /*!< Position of PIN15_IN field. */
#define GPIO_PIN15_IN_PIN15_IN_Msk (0x1UL << GPIO_PIN15_IN_PIN15_IN_Pos) /*!< Bit mask of PIN15_IN field. */

/* Register: GPIO_PIN16_OUT */
/* Description: GPIO pin 16 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN16_OUT_PIN16_OUT_Pos (0U) /*!< Position of PIN16_OUT field. */
#define GPIO_PIN16_OUT_PIN16_OUT_Msk (0x1UL << GPIO_PIN16_OUT_PIN16_OUT_Pos) /*!< Bit mask of PIN16_OUT field. */

/* Register: GPIO_PIN16_IN */
/* Description: GPIO pin 16 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN16_IN_PIN16_IN_Pos (0U) /*!< Position of PIN16_IN field. */
#define GPIO_PIN16_IN_PIN16_IN_Msk (0x1UL << GPIO_PIN16_IN_PIN16_IN_Pos) /*!< Bit mask of PIN16_IN field. */

/* Register: GPIO_PIN17_OUT */
/* Description: GPIO pin 17 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN17_OUT_PIN17_OUT_Pos (0U) /*!< Position of PIN17_OUT field. */
#define GPIO_PIN17_OUT_PIN17_OUT_Msk (0x1UL << GPIO_PIN17_OUT_PIN17_OUT_Pos) /*!< Bit mask of PIN17_OUT field. */

/* Register: GPIO_PIN17_IN */
/* Description: GPIO pin 17 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN17_IN_PIN17_IN_Pos (0U) /*!< Position of PIN17_IN field. */
#define GPIO_PIN17_IN_PIN17_IN_Msk (0x1UL << GPIO_PIN17_IN_PIN17_IN_Pos) /*!< Bit mask of PIN17_IN field. */

/* Register: GPIO_PIN18_OUT */
/* Description: GPIO pin 18 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN18_OUT_PIN18_OUT_Pos (0U) /*!< Position of PIN18_OUT field. */
#define GPIO_PIN18_OUT_PIN18_OUT_Msk (0x1UL << GPIO_PIN18_OUT_PIN18_OUT_Pos) /*!< Bit mask of PIN18_OUT field. */

/* Register: GPIO_PIN18_IN */
/* Description: GPIO pin 18 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN18_IN_PIN18_IN_Pos (0U) /*!< Position of PIN18_IN field. */
#define GPIO_PIN18_IN_PIN18_IN_Msk (0x1UL << GPIO_PIN18_IN_PIN18_IN_Pos) /*!< Bit mask of PIN18_IN field. */

/* Register: GPIO_PIN19_OUT */
/* Description: GPIO pin 19 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN19_OUT_PIN19_OUT_Pos (0U) /*!< Position of PIN19_OUT field. */
#define GPIO_PIN19_OUT_PIN19_OUT_Msk (0x1UL << GPIO_PIN19_OUT_PIN19_OUT_Pos) /*!< Bit mask of PIN19_OUT field. */

/* Register: GPIO_PIN19_IN */
/* Description: GPIO pin 19 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN19_IN_PIN19_IN_Pos (0U) /*!< Position of PIN19_IN field. */
#define GPIO_PIN19_IN_PIN19_IN_Msk (0x1UL << GPIO_PIN19_IN_PIN19_IN_Pos) /*!< Bit mask of PIN19_IN field. */

/* Register: GPIO_PIN20_OUT */
/* Description: GPIO pin 20 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN20_OUT_PIN20_OUT_Pos (0U) /*!< Position of PIN20_OUT field. */
#define GPIO_PIN20_OUT_PIN20_OUT_Msk (0x1UL << GPIO_PIN20_OUT_PIN20_OUT_Pos) /*!< Bit mask of PIN20_OUT field. */

/* Register: GPIO_PIN20_IN */
/* Description: GPIO pin 20 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN20_IN_PIN20_IN_Pos (0U) /*!< Position of PIN20_IN field. */
#define GPIO_PIN20_IN_PIN20_IN_Msk (0x1UL << GPIO_PIN20_IN_PIN20_IN_Pos) /*!< Bit mask of PIN20_IN field. */

/* Register: GPIO_PIN21_OUT */
/* Description: GPIO pin 21 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN21_OUT_PIN21_OUT_Pos (0U) /*!< Position of PIN21_OUT field. */
#define GPIO_PIN21_OUT_PIN21_OUT_Msk (0x1UL << GPIO_PIN21_OUT_PIN21_OUT_Pos) /*!< Bit mask of PIN21_OUT field. */

/* Register: GPIO_PIN21_IN */
/* Description: GPIO pin 21 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN21_IN_PIN21_IN_Pos (0U) /*!< Position of PIN21_IN field. */
#define GPIO_PIN21_IN_PIN21_IN_Msk (0x1UL << GPIO_PIN21_IN_PIN21_IN_Pos) /*!< Bit mask of PIN21_IN field. */

/* Register: GPIO_PIN22_OUT */
/* Description: GPIO pin 22 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN22_OUT_PIN22_OUT_Pos (0U) /*!< Position of PIN22_OUT field. */
#define GPIO_PIN22_OUT_PIN22_OUT_Msk (0x1UL << GPIO_PIN22_OUT_PIN22_OUT_Pos) /*!< Bit mask of PIN22_OUT field. */

/* Register: GPIO_PIN22_IN */
/* Description: GPIO pin 22 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN22_IN_PIN22_IN_Pos (0U) /*!< Position of PIN22_IN field. */
#define GPIO_PIN22_IN_PIN22_IN_Msk (0x1UL << GPIO_PIN22_IN_PIN22_IN_Pos) /*!< Bit mask of PIN22_IN field. */

/* Register: GPIO_PIN23_OUT */
/* Description: GPIO pin 23 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN23_OUT_PIN23_OUT_Pos (0U) /*!< Position of PIN23_OUT field. */
#define GPIO_PIN23_OUT_PIN23_OUT_Msk (0x1UL << GPIO_PIN23_OUT_PIN23_OUT_Pos) /*!< Bit mask of PIN23_OUT field. */

/* Register: GPIO_PIN23_IN */
/* Description: GPIO pin 23 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN23_IN_PIN23_IN_Pos (0U) /*!< Position of PIN23_IN field. */
#define GPIO_PIN23_IN_PIN23_IN_Msk (0x1UL << GPIO_PIN23_IN_PIN23_IN_Pos) /*!< Bit mask of PIN23_IN field. */

/* Register: GPIO_PIN24_OUT */
/* Description: GPIO pin 24 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN24_OUT_PIN24_OUT_Pos (0U) /*!< Position of PIN24_OUT field. */
#define GPIO_PIN24_OUT_PIN24_OUT_Msk (0x1UL << GPIO_PIN24_OUT_PIN24_OUT_Pos) /*!< Bit mask of PIN24_OUT field. */

/* Register: GPIO_PIN24_IN */
/* Description: GPIO pin 24 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN24_IN_PIN24_IN_Pos (0U) /*!< Position of PIN24_IN field. */
#define GPIO_PIN24_IN_PIN24_IN_Msk (0x1UL << GPIO_PIN24_IN_PIN24_IN_Pos) /*!< Bit mask of PIN24_IN field. */

/* Register: GPIO_PIN25_OUT */
/* Description: GPIO pin 25 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN25_OUT_PIN25_OUT_Pos (0U) /*!< Position of PIN25_OUT field. */
#define GPIO_PIN25_OUT_PIN25_OUT_Msk (0x1UL << GPIO_PIN25_OUT_PIN25_OUT_Pos) /*!< Bit mask of PIN25_OUT field. */

/* Register: GPIO_PIN25_IN */
/* Description: GPIO pin 25 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN25_IN_PIN25_IN_Pos (0U) /*!< Position of PIN25_IN field. */
#define GPIO_PIN25_IN_PIN25_IN_Msk (0x1UL << GPIO_PIN25_IN_PIN25_IN_Pos) /*!< Bit mask of PIN25_IN field. */

/* Register: GPIO_PIN26_OUT */
/* Description: GPIO pin 26 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN26_OUT_PIN26_OUT_Pos (0U) /*!< Position of PIN26_OUT field. */
#define GPIO_PIN26_OUT_PIN26_OUT_Msk (0x1UL << GPIO_PIN26_OUT_PIN26_OUT_Pos) /*!< Bit mask of PIN26_OUT field. */

/* Register: GPIO_PIN26_IN */
/* Description: GPIO pin 26 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN26_IN_PIN26_IN_Pos (0U) /*!< Position of PIN26_IN field. */
#define GPIO_PIN26_IN_PIN26_IN_Msk (0x1UL << GPIO_PIN26_IN_PIN26_IN_Pos) /*!< Bit mask of PIN26_IN field. */

/* Register: GPIO_PIN27_OUT */
/* Description: GPIO pin 27 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN27_OUT_PIN27_OUT_Pos (0U) /*!< Position of PIN27_OUT field. */
#define GPIO_PIN27_OUT_PIN27_OUT_Msk (0x1UL << GPIO_PIN27_OUT_PIN27_OUT_Pos) /*!< Bit mask of PIN27_OUT field. */

/* Register: GPIO_PIN27_IN */
/* Description: GPIO pin 27 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN27_IN_PIN27_IN_Pos (0U) /*!< Position of PIN27_IN field. */
#define GPIO_PIN27_IN_PIN27_IN_Msk (0x1UL << GPIO_PIN27_IN_PIN27_IN_Pos) /*!< Bit mask of PIN27_IN field. */

/* Register: GPIO_PIN28_OUT */
/* Description: GPIO pin 28 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN28_OUT_PIN28_OUT_Pos (0U) /*!< Position of PIN28_OUT field. */
#define GPIO_PIN28_OUT_PIN28_OUT_Msk (0x1UL << GPIO_PIN28_OUT_PIN28_OUT_Pos) /*!< Bit mask of PIN28_OUT field. */

/* Register: GPIO_PIN28_IN */
/* Description: GPIO pin 28 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN28_IN_PIN28_IN_Pos (0U) /*!< Position of PIN28_IN field. */
#define GPIO_PIN28_IN_PIN28_IN_Msk (0x1UL << GPIO_PIN28_IN_PIN28_IN_Pos) /*!< Bit mask of PIN28_IN field. */

/* Register: GPIO_PIN29_OUT */
/* Description: GPIO pin 29 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN29_OUT_PIN29_OUT_Pos (0U) /*!< Position of PIN29_OUT field. */
#define GPIO_PIN29_OUT_PIN29_OUT_Msk (0x1UL << GPIO_PIN29_OUT_PIN29_OUT_Pos) /*!< Bit mask of PIN29_OUT field. */

/* Register: GPIO_PIN29_IN */
/* Description: GPIO pin 29 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN29_IN_PIN29_IN_Pos (0U) /*!< Position of PIN29_IN field. */
#define GPIO_PIN29_IN_PIN29_IN_Msk (0x1UL << GPIO_PIN29_IN_PIN29_IN_Pos) /*!< Bit mask of PIN29_IN field. */

/* Register: GPIO_PIN30_OUT */
/* Description: GPIO pin 30 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN30_OUT_PIN30_OUT_Pos (0U) /*!< Position of PIN30_OUT field. */
#define GPIO_PIN30_OUT_PIN30_OUT_Msk (0x1UL << GPIO_PIN30_OUT_PIN30_OUT_Pos) /*!< Bit mask of PIN30_OUT field. */

/* Register: GPIO_PIN30_IN */
/* Description: GPIO pin 30 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN30_IN_PIN30_IN_Pos (0U) /*!< Position of PIN30_IN field. */
#define GPIO_PIN30_IN_PIN30_IN_Msk (0x1UL << GPIO_PIN30_IN_PIN30_IN_Pos) /*!< Bit mask of PIN30_IN field. */

/* Register: GPIO_PIN31_OUT */
/* Description: GPIO pin 31 write register. */

/* Bit 0 : GPIO pin write register. */
#define GPIO_PIN31_OUT_PIN31_OUT_Pos (0U) /*!< Position of PIN31_OUT field. */
#define GPIO_PIN31_OUT_PIN31_OUT_Msk (0x1UL << GPIO_PIN31_OUT_PIN31_OUT_Pos) /*!< Bit mask of PIN31_OUT field. */

/* Register: GPIO_PIN31_IN */
/* Description: GPIO pin 31 read register. */

/* Bit 0 : GPIO pin read register. */
#define GPIO_PIN31_IN_PIN31_IN_Pos (0U) /*!< Position of PIN31_IN field. */
#define GPIO_PIN31_IN_PIN31_IN_Msk (0x1UL << GPIO_PIN31_IN_PIN31_IN_Pos) /*!< Bit mask of PIN31_IN field. */

/* Register: GPIO_PIN0_CNF */
/* Description: Configuration of pin 0. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN0_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN0_CNF_SENSE_Msk (0x3UL << GPIO_PIN0_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN0_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN0_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN0_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN0_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN0_CNF_DRIVE_Msk (0x7UL << GPIO_PIN0_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN0_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN0_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN0_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN0_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN0_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN0_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN0_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN0_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN0_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN0_CNF_PULL_Msk (0x3UL << GPIO_PIN0_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN0_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN0_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN0_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN0_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN0_CNF_INPUT_Msk (0x1UL << GPIO_PIN0_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN0_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN0_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN0_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN0_CNF_DIR_Msk (0x1UL << GPIO_PIN0_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN0_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN0_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN1_CNF */
/* Description: Configuration of pin 1. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN1_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN1_CNF_SENSE_Msk (0x3UL << GPIO_PIN1_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN1_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN1_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN1_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN1_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN1_CNF_DRIVE_Msk (0x7UL << GPIO_PIN1_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN1_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN1_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN1_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN1_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN1_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN1_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN1_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN1_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN1_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN1_CNF_PULL_Msk (0x3UL << GPIO_PIN1_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN1_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN1_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN1_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN1_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN1_CNF_INPUT_Msk (0x1UL << GPIO_PIN1_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN1_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN1_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN1_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN1_CNF_DIR_Msk (0x1UL << GPIO_PIN1_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN1_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN1_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN2_CNF */
/* Description: Configuration of pin 2. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN2_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN2_CNF_SENSE_Msk (0x3UL << GPIO_PIN2_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN2_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN2_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN2_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN2_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN2_CNF_DRIVE_Msk (0x7UL << GPIO_PIN2_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN2_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN2_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN2_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN2_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN2_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN2_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN2_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN2_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN2_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN2_CNF_PULL_Msk (0x3UL << GPIO_PIN2_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN2_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN2_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN2_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN2_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN2_CNF_INPUT_Msk (0x1UL << GPIO_PIN2_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN2_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN2_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN2_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN2_CNF_DIR_Msk (0x1UL << GPIO_PIN2_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN2_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN2_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN3_CNF */
/* Description: Configuration of pin 3. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN3_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN3_CNF_SENSE_Msk (0x3UL << GPIO_PIN3_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN3_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN3_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN3_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN3_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN3_CNF_DRIVE_Msk (0x7UL << GPIO_PIN3_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN3_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN3_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN3_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN3_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN3_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN3_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN3_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN3_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN3_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN3_CNF_PULL_Msk (0x3UL << GPIO_PIN3_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN3_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN3_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN3_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN3_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN3_CNF_INPUT_Msk (0x1UL << GPIO_PIN3_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN3_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN3_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN3_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN3_CNF_DIR_Msk (0x1UL << GPIO_PIN3_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN3_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN3_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN4_CNF */
/* Description: Configuration of pin 4. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN4_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN4_CNF_SENSE_Msk (0x3UL << GPIO_PIN4_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN4_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN4_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN4_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN4_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN4_CNF_DRIVE_Msk (0x7UL << GPIO_PIN4_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN4_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN4_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN4_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN4_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN4_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN4_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN4_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN4_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN4_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN4_CNF_PULL_Msk (0x3UL << GPIO_PIN4_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN4_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN4_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN4_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN4_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN4_CNF_INPUT_Msk (0x1UL << GPIO_PIN4_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN4_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN4_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN4_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN4_CNF_DIR_Msk (0x1UL << GPIO_PIN4_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN4_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN4_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN5_CNF */
/* Description: Configuration of pin 5. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN5_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN5_CNF_SENSE_Msk (0x3UL << GPIO_PIN5_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN5_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN5_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN5_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN5_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN5_CNF_DRIVE_Msk (0x7UL << GPIO_PIN5_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN5_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN5_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN5_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN5_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN5_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN5_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN5_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN5_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN5_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN5_CNF_PULL_Msk (0x3UL << GPIO_PIN5_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN5_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN5_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN5_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN5_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN5_CNF_INPUT_Msk (0x1UL << GPIO_PIN5_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN5_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN5_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN5_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN5_CNF_DIR_Msk (0x1UL << GPIO_PIN5_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN5_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN5_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN6_CNF */
/* Description: Configuration of pin 6. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN6_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN6_CNF_SENSE_Msk (0x3UL << GPIO_PIN6_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN6_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN6_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN6_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN6_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN6_CNF_DRIVE_Msk (0x7UL << GPIO_PIN6_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN6_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN6_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN6_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN6_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN6_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN6_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN6_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN6_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN6_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN6_CNF_PULL_Msk (0x3UL << GPIO_PIN6_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN6_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN6_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN6_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN6_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN6_CNF_INPUT_Msk (0x1UL << GPIO_PIN6_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN6_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN6_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN6_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN6_CNF_DIR_Msk (0x1UL << GPIO_PIN6_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN6_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN6_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN7_CNF */
/* Description: Configuration of pin 7. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN7_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN7_CNF_SENSE_Msk (0x3UL << GPIO_PIN7_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN7_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN7_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN7_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN7_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN7_CNF_DRIVE_Msk (0x7UL << GPIO_PIN7_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN7_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN7_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN7_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN7_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN7_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN7_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN7_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN7_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN7_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN7_CNF_PULL_Msk (0x3UL << GPIO_PIN7_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN7_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN7_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN7_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN7_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN7_CNF_INPUT_Msk (0x1UL << GPIO_PIN7_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN7_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN7_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN7_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN7_CNF_DIR_Msk (0x1UL << GPIO_PIN7_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN7_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN7_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN8_CNF */
/* Description: Configuration of pin 8. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN8_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN8_CNF_SENSE_Msk (0x3UL << GPIO_PIN8_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN8_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN8_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN8_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN8_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN8_CNF_DRIVE_Msk (0x7UL << GPIO_PIN8_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN8_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN8_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN8_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN8_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN8_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN8_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN8_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN8_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN8_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN8_CNF_PULL_Msk (0x3UL << GPIO_PIN8_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN8_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN8_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN8_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN8_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN8_CNF_INPUT_Msk (0x1UL << GPIO_PIN8_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN8_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN8_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN8_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN8_CNF_DIR_Msk (0x1UL << GPIO_PIN8_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN8_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN8_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN9_CNF */
/* Description: Configuration of pin 9. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN9_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN9_CNF_SENSE_Msk (0x3UL << GPIO_PIN9_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN9_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN9_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN9_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN9_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN9_CNF_DRIVE_Msk (0x7UL << GPIO_PIN9_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN9_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN9_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN9_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN9_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN9_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN9_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN9_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN9_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN9_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN9_CNF_PULL_Msk (0x3UL << GPIO_PIN9_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN9_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN9_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN9_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN9_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN9_CNF_INPUT_Msk (0x1UL << GPIO_PIN9_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN9_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN9_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN9_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN9_CNF_DIR_Msk (0x1UL << GPIO_PIN9_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN9_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN9_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN10_CNF */
/* Description: Configuration of pin 10. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN10_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN10_CNF_SENSE_Msk (0x3UL << GPIO_PIN10_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN10_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN10_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN10_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN10_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN10_CNF_DRIVE_Msk (0x7UL << GPIO_PIN10_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN10_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN10_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN10_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN10_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN10_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN10_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN10_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN10_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN10_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN10_CNF_PULL_Msk (0x3UL << GPIO_PIN10_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN10_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN10_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN10_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN10_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN10_CNF_INPUT_Msk (0x1UL << GPIO_PIN10_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN10_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN10_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN10_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN10_CNF_DIR_Msk (0x1UL << GPIO_PIN10_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN10_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN10_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN11_CNF */
/* Description: Configuration of pin 11. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN11_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN11_CNF_SENSE_Msk (0x3UL << GPIO_PIN11_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN11_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN11_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN11_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN11_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN11_CNF_DRIVE_Msk (0x7UL << GPIO_PIN11_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN11_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN11_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN11_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN11_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN11_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN11_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN11_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN11_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN11_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN11_CNF_PULL_Msk (0x3UL << GPIO_PIN11_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN11_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN11_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN11_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN11_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN11_CNF_INPUT_Msk (0x1UL << GPIO_PIN11_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN11_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN11_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN11_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN11_CNF_DIR_Msk (0x1UL << GPIO_PIN11_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN11_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN11_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN12_CNF */
/* Description: Configuration of pin 12. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN12_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN12_CNF_SENSE_Msk (0x3UL << GPIO_PIN12_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN12_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN12_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN12_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN12_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN12_CNF_DRIVE_Msk (0x7UL << GPIO_PIN12_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN12_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN12_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN12_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN12_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN12_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN12_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN12_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN12_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN12_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN12_CNF_PULL_Msk (0x3UL << GPIO_PIN12_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN12_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN12_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN12_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN12_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN12_CNF_INPUT_Msk (0x1UL << GPIO_PIN12_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN12_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN12_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN12_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN12_CNF_DIR_Msk (0x1UL << GPIO_PIN12_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN12_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN12_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN13_CNF */
/* Description: Configuration of pin 13. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN13_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN13_CNF_SENSE_Msk (0x3UL << GPIO_PIN13_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN13_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN13_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN13_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN13_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN13_CNF_DRIVE_Msk (0x7UL << GPIO_PIN13_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN13_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN13_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN13_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN13_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN13_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN13_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN13_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN13_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN13_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN13_CNF_PULL_Msk (0x3UL << GPIO_PIN13_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN13_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN13_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN13_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN13_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN13_CNF_INPUT_Msk (0x1UL << GPIO_PIN13_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN13_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN13_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN13_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN13_CNF_DIR_Msk (0x1UL << GPIO_PIN13_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN13_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN13_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN14_CNF */
/* Description: Configuration of pin 14. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN14_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN14_CNF_SENSE_Msk (0x3UL << GPIO_PIN14_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN14_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN14_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN14_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN14_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN14_CNF_DRIVE_Msk (0x7UL << GPIO_PIN14_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN14_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN14_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN14_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN14_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN14_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN14_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN14_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN14_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN14_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN14_CNF_PULL_Msk (0x3UL << GPIO_PIN14_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN14_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN14_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN14_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN14_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN14_CNF_INPUT_Msk (0x1UL << GPIO_PIN14_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN14_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN14_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN14_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN14_CNF_DIR_Msk (0x1UL << GPIO_PIN14_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN14_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN14_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN15_CNF */
/* Description: Configuration of pin 15. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN15_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN15_CNF_SENSE_Msk (0x3UL << GPIO_PIN15_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN15_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN15_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN15_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN15_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN15_CNF_DRIVE_Msk (0x7UL << GPIO_PIN15_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN15_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN15_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN15_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN15_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN15_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN15_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN15_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN15_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN15_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN15_CNF_PULL_Msk (0x3UL << GPIO_PIN15_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN15_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN15_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN15_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN15_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN15_CNF_INPUT_Msk (0x1UL << GPIO_PIN15_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN15_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN15_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN15_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN15_CNF_DIR_Msk (0x1UL << GPIO_PIN15_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN15_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN15_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN16_CNF */
/* Description: Configuration of pin 16. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN16_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN16_CNF_SENSE_Msk (0x3UL << GPIO_PIN16_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN16_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN16_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN16_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN16_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN16_CNF_DRIVE_Msk (0x7UL << GPIO_PIN16_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN16_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN16_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN16_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN16_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN16_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN16_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN16_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN16_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN16_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN16_CNF_PULL_Msk (0x3UL << GPIO_PIN16_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN16_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN16_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN16_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN16_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN16_CNF_INPUT_Msk (0x1UL << GPIO_PIN16_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN16_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN16_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN16_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN16_CNF_DIR_Msk (0x1UL << GPIO_PIN16_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN16_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN16_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN17_CNF */
/* Description: Configuration of pin 17. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN17_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN17_CNF_SENSE_Msk (0x3UL << GPIO_PIN17_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN17_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN17_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN17_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN17_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN17_CNF_DRIVE_Msk (0x7UL << GPIO_PIN17_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN17_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN17_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN17_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN17_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN17_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN17_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN17_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN17_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN17_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN17_CNF_PULL_Msk (0x3UL << GPIO_PIN17_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN17_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN17_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN17_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN17_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN17_CNF_INPUT_Msk (0x1UL << GPIO_PIN17_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN17_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN17_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN17_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN17_CNF_DIR_Msk (0x1UL << GPIO_PIN17_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN17_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN17_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN18_CNF */
/* Description: Configuration of pin 18. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN18_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN18_CNF_SENSE_Msk (0x3UL << GPIO_PIN18_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN18_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN18_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN18_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN18_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN18_CNF_DRIVE_Msk (0x7UL << GPIO_PIN18_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN18_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN18_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN18_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN18_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN18_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN18_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN18_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN18_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN18_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN18_CNF_PULL_Msk (0x3UL << GPIO_PIN18_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN18_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN18_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN18_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN18_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN18_CNF_INPUT_Msk (0x1UL << GPIO_PIN18_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN18_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN18_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN18_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN18_CNF_DIR_Msk (0x1UL << GPIO_PIN18_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN18_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN18_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN19_CNF */
/* Description: Configuration of pin 19. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN19_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN19_CNF_SENSE_Msk (0x3UL << GPIO_PIN19_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN19_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN19_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN19_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN19_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN19_CNF_DRIVE_Msk (0x7UL << GPIO_PIN19_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN19_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN19_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN19_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN19_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN19_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN19_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN19_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN19_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN19_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN19_CNF_PULL_Msk (0x3UL << GPIO_PIN19_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN19_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN19_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN19_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN19_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN19_CNF_INPUT_Msk (0x1UL << GPIO_PIN19_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN19_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN19_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN19_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN19_CNF_DIR_Msk (0x1UL << GPIO_PIN19_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN19_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN19_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN20_CNF */
/* Description: Configuration of pin 20. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN20_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN20_CNF_SENSE_Msk (0x3UL << GPIO_PIN20_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN20_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN20_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN20_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN20_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN20_CNF_DRIVE_Msk (0x7UL << GPIO_PIN20_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN20_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN20_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN20_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN20_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN20_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN20_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN20_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN20_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN20_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN20_CNF_PULL_Msk (0x3UL << GPIO_PIN20_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN20_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN20_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN20_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN20_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN20_CNF_INPUT_Msk (0x1UL << GPIO_PIN20_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN20_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN20_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN20_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN20_CNF_DIR_Msk (0x1UL << GPIO_PIN20_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN20_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN20_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN21_CNF */
/* Description: Configuration of pin 21. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN21_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN21_CNF_SENSE_Msk (0x3UL << GPIO_PIN21_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN21_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN21_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN21_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN21_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN21_CNF_DRIVE_Msk (0x7UL << GPIO_PIN21_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN21_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN21_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN21_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN21_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN21_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN21_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN21_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN21_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN21_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN21_CNF_PULL_Msk (0x3UL << GPIO_PIN21_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN21_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN21_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN21_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN21_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN21_CNF_INPUT_Msk (0x1UL << GPIO_PIN21_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN21_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN21_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN21_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN21_CNF_DIR_Msk (0x1UL << GPIO_PIN21_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN21_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN21_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN22_CNF */
/* Description: Configuration of pin 22. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN22_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN22_CNF_SENSE_Msk (0x3UL << GPIO_PIN22_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN22_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN22_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN22_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN22_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN22_CNF_DRIVE_Msk (0x7UL << GPIO_PIN22_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN22_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN22_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN22_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN22_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN22_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN22_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN22_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN22_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN22_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN22_CNF_PULL_Msk (0x3UL << GPIO_PIN22_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN22_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN22_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN22_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN22_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN22_CNF_INPUT_Msk (0x1UL << GPIO_PIN22_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN22_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN22_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN22_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN22_CNF_DIR_Msk (0x1UL << GPIO_PIN22_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN22_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN22_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN23_CNF */
/* Description: Configuration of pin 23. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN23_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN23_CNF_SENSE_Msk (0x3UL << GPIO_PIN23_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN23_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN23_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN23_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN23_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN23_CNF_DRIVE_Msk (0x7UL << GPIO_PIN23_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN23_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN23_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN23_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN23_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN23_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN23_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN23_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN23_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN23_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN23_CNF_PULL_Msk (0x3UL << GPIO_PIN23_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN23_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN23_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN23_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN23_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN23_CNF_INPUT_Msk (0x1UL << GPIO_PIN23_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN23_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN23_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN23_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN23_CNF_DIR_Msk (0x1UL << GPIO_PIN23_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN23_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN23_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN24_CNF */
/* Description: Configuration of pin 24. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN24_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN24_CNF_SENSE_Msk (0x3UL << GPIO_PIN24_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN24_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN24_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN24_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN24_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN24_CNF_DRIVE_Msk (0x7UL << GPIO_PIN24_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN24_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN24_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN24_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN24_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN24_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN24_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN24_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN24_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN24_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN24_CNF_PULL_Msk (0x3UL << GPIO_PIN24_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN24_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN24_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN24_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN24_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN24_CNF_INPUT_Msk (0x1UL << GPIO_PIN24_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN24_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN24_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN24_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN24_CNF_DIR_Msk (0x1UL << GPIO_PIN24_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN24_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN24_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN25_CNF */
/* Description: Configuration of pin 25. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN25_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN25_CNF_SENSE_Msk (0x3UL << GPIO_PIN25_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN25_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN25_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN25_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN25_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN25_CNF_DRIVE_Msk (0x7UL << GPIO_PIN25_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN25_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN25_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN25_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN25_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN25_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN25_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN25_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN25_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN25_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN25_CNF_PULL_Msk (0x3UL << GPIO_PIN25_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN25_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN25_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN25_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN25_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN25_CNF_INPUT_Msk (0x1UL << GPIO_PIN25_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN25_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN25_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN25_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN25_CNF_DIR_Msk (0x1UL << GPIO_PIN25_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN25_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN25_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN26_CNF */
/* Description: Configuration of pin 26. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN26_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN26_CNF_SENSE_Msk (0x3UL << GPIO_PIN26_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN26_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN26_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN26_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN26_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN26_CNF_DRIVE_Msk (0x7UL << GPIO_PIN26_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN26_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN26_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN26_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN26_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN26_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN26_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN26_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN26_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN26_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN26_CNF_PULL_Msk (0x3UL << GPIO_PIN26_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN26_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN26_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN26_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN26_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN26_CNF_INPUT_Msk (0x1UL << GPIO_PIN26_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN26_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN26_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN26_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN26_CNF_DIR_Msk (0x1UL << GPIO_PIN26_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN26_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN26_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN27_CNF */
/* Description: Configuration of pin 27. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN27_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN27_CNF_SENSE_Msk (0x3UL << GPIO_PIN27_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN27_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN27_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN27_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN27_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN27_CNF_DRIVE_Msk (0x7UL << GPIO_PIN27_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN27_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN27_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN27_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN27_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN27_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN27_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN27_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN27_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN27_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN27_CNF_PULL_Msk (0x3UL << GPIO_PIN27_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN27_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN27_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN27_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN27_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN27_CNF_INPUT_Msk (0x1UL << GPIO_PIN27_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN27_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN27_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN27_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN27_CNF_DIR_Msk (0x1UL << GPIO_PIN27_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN27_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN27_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN28_CNF */
/* Description: Configuration of pin 28. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN28_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN28_CNF_SENSE_Msk (0x3UL << GPIO_PIN28_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN28_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN28_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN28_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN28_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN28_CNF_DRIVE_Msk (0x7UL << GPIO_PIN28_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN28_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN28_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN28_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN28_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN28_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN28_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN28_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN28_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN28_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN28_CNF_PULL_Msk (0x3UL << GPIO_PIN28_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN28_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN28_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN28_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN28_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN28_CNF_INPUT_Msk (0x1UL << GPIO_PIN28_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN28_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN28_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN28_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN28_CNF_DIR_Msk (0x1UL << GPIO_PIN28_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN28_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN28_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN29_CNF */
/* Description: Configuration of pin 29. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN29_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN29_CNF_SENSE_Msk (0x3UL << GPIO_PIN29_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN29_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN29_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN29_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN29_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN29_CNF_DRIVE_Msk (0x7UL << GPIO_PIN29_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN29_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN29_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN29_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN29_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN29_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN29_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN29_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN29_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN29_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN29_CNF_PULL_Msk (0x3UL << GPIO_PIN29_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN29_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN29_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN29_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN29_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN29_CNF_INPUT_Msk (0x1UL << GPIO_PIN29_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN29_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN29_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN29_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN29_CNF_DIR_Msk (0x1UL << GPIO_PIN29_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN29_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN29_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN30_CNF */
/* Description: Configuration of pin 30. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN30_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN30_CNF_SENSE_Msk (0x3UL << GPIO_PIN30_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN30_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN30_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN30_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN30_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN30_CNF_DRIVE_Msk (0x7UL << GPIO_PIN30_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN30_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN30_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN30_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN30_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN30_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN30_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN30_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN30_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN30_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN30_CNF_PULL_Msk (0x3UL << GPIO_PIN30_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN30_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN30_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN30_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN30_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN30_CNF_INPUT_Msk (0x1UL << GPIO_PIN30_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN30_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN30_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN30_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN30_CNF_DIR_Msk (0x1UL << GPIO_PIN30_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN30_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN30_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */

/* Register: GPIO_PIN31_CNF */
/* Description: Configuration of pin 31. */

/* Bits 17..16 : Pin sensing mechanism. */
#define GPIO_PIN31_CNF_SENSE_Pos (16U) /*!< Position of SENSE field. */
#define GPIO_PIN31_CNF_SENSE_Msk (0x3UL << GPIO_PIN31_CNF_SENSE_Pos) /*!< Bit mask of SENSE field. */
#define GPIO_PIN31_CNF_SENSE_Disabled (0x00UL) /*!< Disabled. */
#define GPIO_PIN31_CNF_SENSE_High (0x02UL) /*!< Wakeup on high level. */
#define GPIO_PIN31_CNF_SENSE_Low (0x03UL) /*!< Wakeup on low level. */

/* Bits 10..8 : Drive configuration. */
#define GPIO_PIN31_CNF_DRIVE_Pos (8U) /*!< Position of DRIVE field. */
#define GPIO_PIN31_CNF_DRIVE_Msk (0x7UL << GPIO_PIN31_CNF_DRIVE_Pos) /*!< Bit mask of DRIVE field. */
#define GPIO_PIN31_CNF_DRIVE_S0S1 (0x00UL) /*!< Standard 0, standard 1. */
#define GPIO_PIN31_CNF_DRIVE_H0S1 (0x01UL) /*!< High 0, standard 1. */
#define GPIO_PIN31_CNF_DRIVE_S0H1 (0x02UL) /*!< Standard 0, high 1. */
#define GPIO_PIN31_CNF_DRIVE_H0H1 (0x03UL) /*!< High 0, high 1. */
#define GPIO_PIN31_CNF_DRIVE_D0S1 (0x04UL) /*!< Open drain. */
#define GPIO_PIN31_CNF_DRIVE_D0H1 (0x05UL) /*!< High drive open drain. */
#define GPIO_PIN31_CNF_DRIVE_S0D1 (0x06UL) /*!< Open source. */
#define GPIO_PIN31_CNF_DRIVE_H0D1 (0x07UL) /*!< High drive open source. */

/* Bits 3..2 : Pull-up or -down configuration. */
#define GPIO_PIN31_CNF_PULL_Pos (2U) /*!< Position of PULL field. */
#define GPIO_PIN31_CNF_PULL_Msk (0x3UL << GPIO_PIN31_CNF_PULL_Pos) /*!< Bit mask of PULL field. */
#define GPIO_PIN31_CNF_PULL_Disabled (0x00UL) /*!< No pull. */
#define GPIO_PIN31_CNF_PULL_Pulldown (0x01UL) /*!< Pulldown on pin. */
#define GPIO_PIN31_CNF_PULL_Pullup (0x03UL) /*!< Pullup on pin. */

/* Bit 1 : Connect or disconnect input path. */
#define GPIO_PIN31_CNF_INPUT_Pos (1U) /*!< Position of INPUT field. */
#define GPIO_PIN31_CNF_INPUT_Msk (0x1UL << GPIO_PIN31_CNF_INPUT_Pos) /*!< Bit mask of INPUT field. */
#define GPIO_PIN31_CNF_INPUT_Connect (0UL) /*!< Connect input pin. */
#define GPIO_PIN31_CNF_INPUT_Disconnect (1UL) /*!< Disconnect input pin. */

/* Bit 0 : Pin direction. */
#define GPIO_PIN31_CNF_DIR_Pos (0U) /*!< Position of DIR field. */
#define GPIO_PIN31_CNF_DIR_Msk (0x1UL << GPIO_PIN31_CNF_DIR_Pos) /*!< Bit mask of DIR field. */
#define GPIO_PIN31_CNF_DIR_Input (0UL) /*!< Configure pin as an input pin. */
#define GPIO_PIN31_CNF_DIR_Output (1UL) /*!< Configure pin as an output pin. */


/* Peripheral: GPIOTE */
/* Description: GPIO tasks and events */

/* Register: GPIOTE_INTEN */
/* Description: GPIOTE interrupt enable register. */

/* Bit 31 : Interrupt enable on PORT event. */
#define GPIOTE_INTEN_PORT_Pos (31U) /*!< Position of PORT field. */
#define GPIOTE_INTEN_PORT_Msk (0x1UL << GPIOTE_INTEN_PORT_Pos) /*!< Bit mask of PORT field. */
#define GPIOTE_INTEN_PORT_Disabled (0UL) /*!< Interrupt disabled. */
#define GPIOTE_INTEN_PORT_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 3 : Interrupt enable on IN[3] event. */
#define GPIOTE_INTEN_IN3_Pos (3U) /*!< Position of IN3 field. */
#define GPIOTE_INTEN_IN3_Msk (0x1UL << GPIOTE_INTEN_IN3_Pos) /*!< Bit mask of IN3 field. */
#define GPIOTE_INTEN_IN3_Disabled (0UL) /*!< Interrupt disabled. */
#define GPIOTE_INTEN_IN3_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 2 : Interrupt enable on IN[2] event. */
#define GPIOTE_INTEN_IN2_Pos (2U) /*!< Position of IN2 field. */
#define GPIOTE_INTEN_IN2_Msk (0x1UL << GPIOTE_INTEN_IN2_Pos) /*!< Bit mask of IN2 field. */
#define GPIOTE_INTEN_IN2_Disabled (0UL) /*!< Interrupt disabled. */
#define GPIOTE_INTEN_IN2_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 1 : Interrupt enable on IN[1] event. */
#define GPIOTE_INTEN_IN1_Pos (1U) /*!< Position of IN1 field. */
#define GPIOTE_INTEN_IN1_Msk (0x1UL << GPIOTE_INTEN_IN1_Pos) /*!< Bit mask of IN1 field. */
#define GPIOTE_INTEN_IN1_Disabled (0UL) /*!< Interrupt disabled. */
#define GPIOTE_INTEN_IN1_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 0 : Interrupt enable on IN[0] event. */
#define GPIOTE_INTEN_IN0_Pos (0U) /*!< Position of IN0 field. */
#define GPIOTE_INTEN_IN0_Msk (0x1UL << GPIOTE_INTEN_IN0_Pos) /*!< Bit mask of IN0 field. */
#define GPIOTE_INTEN_IN0_Disabled (0UL) /*!< Interrupt disabled. */
#define GPIOTE_INTEN_IN0_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: GPIOTE_INTENSET */
/* Description: GPIOTE interrupt enable set register. */

/* Bit 31 : Enable interrupt on PORT event. */
#define GPIOTE_INTENSET_PORT_Pos (31U) /*!< Position of PORT field. */
#define GPIOTE_INTENSET_PORT_Msk (0x1UL << GPIOTE_INTENSET_PORT_Pos) /*!< Bit mask of PORT field. */

/* Bit 3 : Enable interrupt on IN[3] event. */
#define GPIOTE_INTENSET_IN3_Pos (3U) /*!< Position of IN3 field. */
#define GPIOTE_INTENSET_IN3_Msk (0x1UL << GPIOTE_INTENSET_IN3_Pos) /*!< Bit mask of IN3 field. */

/* Bit 2 : Enable interrupt on IN[2] event. */
#define GPIOTE_INTENSET_IN2_Pos (2U) /*!< Position of IN2 field. */
#define GPIOTE_INTENSET_IN2_Msk (0x1UL << GPIOTE_INTENSET_IN2_Pos) /*!< Bit mask of IN2 field. */

/* Bit 1 : Enable interrupt on IN[1] event. */
#define GPIOTE_INTENSET_IN1_Pos (1U) /*!< Position of IN1 field. */
#define GPIOTE_INTENSET_IN1_Msk (0x1UL << GPIOTE_INTENSET_IN1_Pos) /*!< Bit mask of IN1 field. */

/* Bit 0 : Enable interrupt on IN[0] event. */
#define GPIOTE_INTENSET_IN0_Pos (0U) /*!< Position of IN0 field. */
#define GPIOTE_INTENSET_IN0_Msk (0x1UL << GPIOTE_INTENSET_IN0_Pos) /*!< Bit mask of IN0 field. */

/* Register: GPIOTE_INTENCLR */
/* Description: GPIOTE interrupt enable clear register. */

/* Bit 31 : Disable interrupt on PORT event. */
#define GPIOTE_INTENCLR_PORT_Pos (31U) /*!< Position of PORT field. */
#define GPIOTE_INTENCLR_PORT_Msk (0x1UL << GPIOTE_INTENCLR_PORT_Pos) /*!< Bit mask of PORT field. */

/* Bit 3 : Disable interrupt on IN[3] event. */
#define GPIOTE_INTENCLR_IN3_Pos (3U) /*!< Position of IN3 field. */
#define GPIOTE_INTENCLR_IN3_Msk (0x1UL << GPIOTE_INTENCLR_IN3_Pos) /*!< Bit mask of IN3 field. */

/* Bit 2 : Disable interrupt on IN[2] event. */
#define GPIOTE_INTENCLR_IN2_Pos (2U) /*!< Position of IN2 field. */
#define GPIOTE_INTENCLR_IN2_Msk (0x1UL << GPIOTE_INTENCLR_IN2_Pos) /*!< Bit mask of IN2 field. */

/* Bit 1 : Disable interrupt on IN[1] event. */
#define GPIOTE_INTENCLR_IN1_Pos (1U) /*!< Position of IN1 field. */
#define GPIOTE_INTENCLR_IN1_Msk (0x1UL << GPIOTE_INTENCLR_IN1_Pos) /*!< Bit mask of IN1 field. */

/* Bit 0 : Disable interrupt on IN[0] event. */
#define GPIOTE_INTENCLR_IN0_Pos (0U) /*!< Position of IN0 field. */
#define GPIOTE_INTENCLR_IN0_Msk (0x1UL << GPIOTE_INTENCLR_IN0_Pos) /*!< Bit mask of IN0 field. */

/* Register: GPIOTE_CONFIG */
/* Description: Channel configuration registers. */

/* Bits 17..16 : Effects on output when in Task mode, or events on input that generates an event. */
#define GPIOTE_CONFIG_POLARITY_Pos (16U) /*!< Position of POLARITY field. */
#define GPIOTE_CONFIG_POLARITY_Msk (0x3UL << GPIOTE_CONFIG_POLARITY_Pos) /*!< Bit mask of POLARITY field. */
#define GPIOTE_CONFIG_POLARITY_LoToHi (0x01UL) /*!< Low to high. */
#define GPIOTE_CONFIG_POLARITY_HiToLo (0x02UL) /*!< High to low. */
#define GPIOTE_CONFIG_POLARITY_Toggle (0x03UL) /*!< Toggle. */

/* Bits 12..8 : Pin select. */
#define GPIOTE_CONFIG_PSEL_Pos (8U) /*!< Position of PSEL field. */
#define GPIOTE_CONFIG_PSEL_Msk (0x1FUL << GPIOTE_CONFIG_PSEL_Pos) /*!< Bit mask of PSEL field. */

/* Bits 1..0 : Mode */
#define GPIOTE_CONFIG_MODE_Pos (0U) /*!< Position of MODE field. */
#define GPIOTE_CONFIG_MODE_Msk (0x3UL << GPIOTE_CONFIG_MODE_Pos) /*!< Bit mask of MODE field. */
#define GPIOTE_CONFIG_MODE_Disabled (0x00UL) /*!< Disabled. */
#define GPIOTE_CONFIG_MODE_Event (0x01UL) /*!< Channel configure in event mode. */
#define GPIOTE_CONFIG_MODE_Task (0x03UL) /*!< Channel configure in task mode. */


/* Peripheral: MEM */
/* Description: Memory Control */

/* Register: MEM_PERR0 */
/* Description: Definition of peripherals in memory region 0. */

/* Bit 31 : PPI configured in RAM region 0. */
#define MEM_PERR0_PPI_Pos (31U) /*!< Position of PPI field. */
#define MEM_PERR0_PPI_Msk (0x1UL << MEM_PERR0_PPI_Pos) /*!< Bit mask of PPI field. */
#define MEM_PERR0_PPI_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_PPI_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 30 : NVMC configured in RAM region 0. */
#define MEM_PERR0_NVMC_Pos (30U) /*!< Position of NVMC field. */
#define MEM_PERR0_NVMC_Msk (0x1UL << MEM_PERR0_NVMC_Pos) /*!< Bit mask of NVMC field. */
#define MEM_PERR0_NVMC_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_NVMC_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 18 : QDEC configured in RAM region 0. */
#define MEM_PERR0_QDEC_Pos (18U) /*!< Position of QDEC field. */
#define MEM_PERR0_QDEC_Msk (0x1UL << MEM_PERR0_QDEC_Pos) /*!< Bit mask of QDEC field. */
#define MEM_PERR0_QDEC_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_QDEC_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 17 : RTC1 configured in RAM region 0. */
#define MEM_PERR0_RTC1_Pos (17U) /*!< Position of RTC1 field. */
#define MEM_PERR0_RTC1_Msk (0x1UL << MEM_PERR0_RTC1_Pos) /*!< Bit mask of RTC1 field. */
#define MEM_PERR0_RTC1_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_RTC1_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 16 : WDOG configured in RAM region 0. */
#define MEM_PERR0_WDOG_Pos (16U) /*!< Position of WDOG field. */
#define MEM_PERR0_WDOG_Msk (0x1UL << MEM_PERR0_WDOG_Pos) /*!< Bit mask of WDOG field. */
#define MEM_PERR0_WDOG_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_WDOG_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 15 : CCM configured in RAM region 0. */
#define MEM_PERR0_CCM_Pos (15U) /*!< Position of CCM field. */
#define MEM_PERR0_CCM_Msk (0x1UL << MEM_PERR0_CCM_Pos) /*!< Bit mask of CCM field. */
#define MEM_PERR0_CCM_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_CCM_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 14 : ECB configured in RAM region 0. */
#define MEM_PERR0_ECB_Pos (14U) /*!< Position of ECB field. */
#define MEM_PERR0_ECB_Msk (0x1UL << MEM_PERR0_ECB_Pos) /*!< Bit mask of ECB field. */
#define MEM_PERR0_ECB_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_ECB_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 13 : RNG configured in RAM region 0. */
#define MEM_PERR0_RNG_Pos (13U) /*!< Position of RNG field. */
#define MEM_PERR0_RNG_Msk (0x1UL << MEM_PERR0_RNG_Pos) /*!< Bit mask of RNG field. */
#define MEM_PERR0_RNG_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_RNG_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 12 : TEMP configured in RAM region 0. */
#define MEM_PERR0_TEMP_Pos (12U) /*!< Position of TEMP field. */
#define MEM_PERR0_TEMP_Msk (0x1UL << MEM_PERR0_TEMP_Pos) /*!< Bit mask of TEMP field. */
#define MEM_PERR0_TEMP_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_TEMP_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 11 : RTC0 configured in RAM region 0. */
#define MEM_PERR0_RTC0_Pos (11U) /*!< Position of RTC0 field. */
#define MEM_PERR0_RTC0_Msk (0x1UL << MEM_PERR0_RTC0_Pos) /*!< Bit mask of RTC0 field. */
#define MEM_PERR0_RTC0_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_RTC0_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 10 : TIMER2 configured in RAM region 0. */
#define MEM_PERR0_TIMER2_Pos (10U) /*!< Position of TIMER2 field. */
#define MEM_PERR0_TIMER2_Msk (0x1UL << MEM_PERR0_TIMER2_Pos) /*!< Bit mask of TIMER2 field. */
#define MEM_PERR0_TIMER2_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_TIMER2_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 9 : TIMER1 configured in RAM region 0. */
#define MEM_PERR0_TIMER1_Pos (9U) /*!< Position of TIMER1 field. */
#define MEM_PERR0_TIMER1_Msk (0x1UL << MEM_PERR0_TIMER1_Pos) /*!< Bit mask of TIMER1 field. */
#define MEM_PERR0_TIMER1_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_TIMER1_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 8 : TIMER0 configured in RAM region 0. */
#define MEM_PERR0_TIMER0_Pos (8U) /*!< Position of TIMER0 field. */
#define MEM_PERR0_TIMER0_Msk (0x1UL << MEM_PERR0_TIMER0_Pos) /*!< Bit mask of TIMER0 field. */
#define MEM_PERR0_TIMER0_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_TIMER0_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 7 : ADC configured in RAM region 0. */
#define MEM_PERR0_ADC_Pos (7U) /*!< Position of ADC field. */
#define MEM_PERR0_ADC_Msk (0x1UL << MEM_PERR0_ADC_Pos) /*!< Bit mask of ADC field. */
#define MEM_PERR0_ADC_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_ADC_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 6 : GPIOTE configured in RAM region 0. */
#define MEM_PERR0_GPIOTE_Pos (6U) /*!< Position of GPIOTE field. */
#define MEM_PERR0_GPIOTE_Msk (0x1UL << MEM_PERR0_GPIOTE_Pos) /*!< Bit mask of GPIOTE field. */
#define MEM_PERR0_GPIOTE_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_GPIOTE_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 4 : SPI1 configured in RAM region 0. */
#define MEM_PERR0_SPI1_Pos (4U) /*!< Position of SPI1 field. */
#define MEM_PERR0_SPI1_Msk (0x1UL << MEM_PERR0_SPI1_Pos) /*!< Bit mask of SPI1 field. */
#define MEM_PERR0_SPI1_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_SPI1_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 3 : SPI0 configured in RAM region 0. */
#define MEM_PERR0_SPI0_Pos (3U) /*!< Position of SPI0 field. */
#define MEM_PERR0_SPI0_Msk (0x1UL << MEM_PERR0_SPI0_Pos) /*!< Bit mask of SPI0 field. */
#define MEM_PERR0_SPI0_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_SPI0_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 2 : UART0 configured in RAM region 0. */
#define MEM_PERR0_UART0_Pos (2U) /*!< Position of UART0 field. */
#define MEM_PERR0_UART0_Msk (0x1UL << MEM_PERR0_UART0_Pos) /*!< Bit mask of UART0 field. */
#define MEM_PERR0_UART0_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_UART0_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 1 : RADIO configured in RAM region 0. */
#define MEM_PERR0_RADIO_Pos (1U) /*!< Position of RADIO field. */
#define MEM_PERR0_RADIO_Msk (0x1UL << MEM_PERR0_RADIO_Pos) /*!< Bit mask of RADIO field. */
#define MEM_PERR0_RADIO_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_RADIO_InRegion0 (1UL) /*!< Peripheral configured in region 0. */

/* Bit 0 : MEM_POWER_CLOCK configured in RAM region 0. */
#define MEM_PERR0_MEM_POWER_CLOCK_Pos (0U) /*!< Position of MEM_POWER_CLOCK field. */
#define MEM_PERR0_MEM_POWER_CLOCK_Msk (0x1UL << MEM_PERR0_MEM_POWER_CLOCK_Pos) /*!< Bit mask of MEM_POWER_CLOCK field. */
#define MEM_PERR0_MEM_POWER_CLOCK_InRegion1 (0UL) /*!< Peripheral configured in region 1. */
#define MEM_PERR0_MEM_POWER_CLOCK_InRegion0 (1UL) /*!< Peripheral configured in region 0. */


/* Peripheral: NVMC */
/* Description: Non Volatile Memory Controller */

/* Register: NVMC_READY */
/* Description: Ready flag. */

/* Bit 0 : NVMC ready. */
#define NVMC_READY_READY_Pos (0U) /*!< Position of READY field. */
#define NVMC_READY_READY_Msk (0x1UL << NVMC_READY_READY_Pos) /*!< Bit mask of READY field. */
#define NVMC_READY_READY_Busy (0UL) /*!< NVMC is busy (on-going write or erase operation). */
#define NVMC_READY_READY_Ready (1UL) /*!< NVMC is ready. */

/* Register: NVMC_CONFIG */
/* Description: Configuration register. */

/* Bit 3 : Emulate OTP behaviour on flash device. */
#define NVMC_CONFIG_OTPMODE_Pos (3U) /*!< Position of OTPMODE field. */
#define NVMC_CONFIG_OTPMODE_Msk (0x1UL << NVMC_CONFIG_OTPMODE_Pos) /*!< Bit mask of OTPMODE field. */
#define NVMC_CONFIG_OTPMODE_Disabled (0UL) /*!< OTP mode disabled. */
#define NVMC_CONFIG_OTPMODE_Enabled (1UL) /*!< OTP mode enabled. */

/* Bit 2 : Programming mode. */
#define NVMC_CONFIG_PROGMODE_Pos (2U) /*!< Position of PROGMODE field. */
#define NVMC_CONFIG_PROGMODE_Msk (0x1UL << NVMC_CONFIG_PROGMODE_Pos) /*!< Bit mask of PROGMODE field. */
#define NVMC_CONFIG_PROGMODE_Halt (0UL) /*!< CPU is halted by NVMC while busy (READY == 0). */
#define NVMC_CONFIG_PROGMODE_NoHalt (1UL) /*!< CPU is not halted by NVMC while busy (READY == 0). */

/* Bits 1..0 : Program write enable. */
#define NVMC_CONFIG_WEN_Pos (0U) /*!< Position of WEN field. */
#define NVMC_CONFIG_WEN_Msk (0x3UL << NVMC_CONFIG_WEN_Pos) /*!< Bit mask of WEN field. */
#define NVMC_CONFIG_WEN_Ren (0x00UL) /*!< Read only access. */
#define NVMC_CONFIG_WEN_Wen (0x01UL) /*!< Write enabled. */
#define NVMC_CONFIG_WEN_Een (0x02UL) /*!< Erase enabled. */

/* Register: NVMC_ERASEALL */
/* Description: Register for erasing all non-volatile user memory. */

/* Bit 0 : Starts the erasing of all user NVM (code region 0/1 and UICR registers). */
#define NVMC_ERASEALL_ERASEALL_Pos (0U) /*!< Position of ERASEALL field. */
#define NVMC_ERASEALL_ERASEALL_Msk (0x1UL << NVMC_ERASEALL_ERASEALL_Pos) /*!< Bit mask of ERASEALL field. */
#define NVMC_ERASEALL_ERASEALL_NoOperation (0UL) /*!< No operation. */
#define NVMC_ERASEALL_ERASEALL_Erase (1UL) /*!< Start chip erase. */

/* Register: NVMC_ERASEUICR */
/* Description: Register for start erasing User Information Congfiguration Registers. */

/* Bit 0 : It can only be used when all contents of code region 1 are erased. */
#define NVMC_ERASEUICR_ERASEUICR_Pos (0U) /*!< Position of ERASEUICR field. */
#define NVMC_ERASEUICR_ERASEUICR_Msk (0x1UL << NVMC_ERASEUICR_ERASEUICR_Pos) /*!< Bit mask of ERASEUICR field. */
#define NVMC_ERASEUICR_ERASEUICR_NoOperation (0UL) /*!< No operation. */
#define NVMC_ERASEUICR_ERASEUICR_Erase (1UL) /*!< Start UICR erase. */


/* Peripheral: POWER */
/* Description: Power Control */

/* Register: POWER_RESETREAS */
/* Description: Reset reason. */

/* Bit 16 : Reset from wake-up from OFF mode. */
#define POWER_RESETREAS_OFF_Pos (16U) /*!< Position of OFF field. */
#define POWER_RESETREAS_OFF_Msk (0x1UL << POWER_RESETREAS_OFF_Pos) /*!< Bit mask of OFF field. */

/* Bit 3 : Reset from CPU lock-up. */
#define POWER_RESETREAS_LOCKUP_Pos (3U) /*!< Position of LOCKUP field. */
#define POWER_RESETREAS_LOCKUP_Msk (0x1UL << POWER_RESETREAS_LOCKUP_Pos) /*!< Bit mask of LOCKUP field. */

/* Bit 2 : Reset from AIRCR.SYSRESETREQ. */
#define POWER_RESETREAS_SREQ_Pos (2U) /*!< Position of SREQ field. */
#define POWER_RESETREAS_SREQ_Msk (0x1UL << POWER_RESETREAS_SREQ_Pos) /*!< Bit mask of SREQ field. */

/* Bit 1 : Reset from watchdog. */
#define POWER_RESETREAS_DOG_Pos (1U) /*!< Position of DOG field. */
#define POWER_RESETREAS_DOG_Msk (0x1UL << POWER_RESETREAS_DOG_Pos) /*!< Bit mask of DOG field. */

/* Bit 0 : Reset from pin. */
#define POWER_RESETREAS_RESETPIN_Pos (0U) /*!< Position of RESETPIN field. */
#define POWER_RESETREAS_RESETPIN_Msk (0x1UL << POWER_RESETREAS_RESETPIN_Pos) /*!< Bit mask of RESETPIN field. */

/* Register: POWER_PERRDY */
/* Description: Periferal on/off status. */

/* Bit 31 : PPI powered up and ready to use. */
#define POWER_PERRDY_PPI_Pos (31U) /*!< Position of PPI field. */
#define POWER_PERRDY_PPI_Msk (0x1UL << POWER_PERRDY_PPI_Pos) /*!< Bit mask of PPI field. */

/* Bit 30 : NVMC powered up and ready to use. */
#define POWER_PERRDY_NVMC_Pos (30U) /*!< Position of NVMC field. */
#define POWER_PERRDY_NVMC_Msk (0x1UL << POWER_PERRDY_NVMC_Pos) /*!< Bit mask of NVMC field. */

/* Bit 18 : QDEC powered up and ready to use. */
#define POWER_PERRDY_QDEC_Pos (18U) /*!< Position of QDEC field. */
#define POWER_PERRDY_QDEC_Msk (0x1UL << POWER_PERRDY_QDEC_Pos) /*!< Bit mask of QDEC field. */

/* Bit 17 : RTC1 powered up and ready to use. */
#define POWER_PERRDY_RTC1_Pos (17U) /*!< Position of RTC1 field. */
#define POWER_PERRDY_RTC1_Msk (0x1UL << POWER_PERRDY_RTC1_Pos) /*!< Bit mask of RTC1 field. */

/* Bit 16 : WDOG powered up and ready to use. */
#define POWER_PERRDY_WDOG_Pos (16U) /*!< Position of WDOG field. */
#define POWER_PERRDY_WDOG_Msk (0x1UL << POWER_PERRDY_WDOG_Pos) /*!< Bit mask of WDOG field. */

/* Bit 15 : CCM powered up and ready to use. */
#define POWER_PERRDY_CCM_Pos (15U) /*!< Position of CCM field. */
#define POWER_PERRDY_CCM_Msk (0x1UL << POWER_PERRDY_CCM_Pos) /*!< Bit mask of CCM field. */

/* Bit 14 : ECB powered up and ready to use. */
#define POWER_PERRDY_ECB_Pos (14U) /*!< Position of ECB field. */
#define POWER_PERRDY_ECB_Msk (0x1UL << POWER_PERRDY_ECB_Pos) /*!< Bit mask of ECB field. */

/* Bit 13 : RNG powered up and ready to use. */
#define POWER_PERRDY_RNG_Pos (13U) /*!< Position of RNG field. */
#define POWER_PERRDY_RNG_Msk (0x1UL << POWER_PERRDY_RNG_Pos) /*!< Bit mask of RNG field. */

/* Bit 12 : TEMP powered up and ready to use. */
#define POWER_PERRDY_TEMP_Pos (12U) /*!< Position of TEMP field. */
#define POWER_PERRDY_TEMP_Msk (0x1UL << POWER_PERRDY_TEMP_Pos) /*!< Bit mask of TEMP field. */

/* Bit 11 : RTC0 powered up and ready to use. */
#define POWER_PERRDY_RTC0_Pos (11U) /*!< Position of RTC0 field. */
#define POWER_PERRDY_RTC0_Msk (0x1UL << POWER_PERRDY_RTC0_Pos) /*!< Bit mask of RTC0 field. */

/* Bit 10 : TIMER2 powered up and ready to use. */
#define POWER_PERRDY_TIMER2_Pos (10U) /*!< Position of TIMER2 field. */
#define POWER_PERRDY_TIMER2_Msk (0x1UL << POWER_PERRDY_TIMER2_Pos) /*!< Bit mask of TIMER2 field. */

/* Bit 9 : TIMER1 powered up and ready to use. */
#define POWER_PERRDY_TIMER1_Pos (9U) /*!< Position of TIMER1 field. */
#define POWER_PERRDY_TIMER1_Msk (0x1UL << POWER_PERRDY_TIMER1_Pos) /*!< Bit mask of TIMER1 field. */

/* Bit 8 : TIMER0 powered up and ready to use. */
#define POWER_PERRDY_TIMER0_Pos (8U) /*!< Position of TIMER0 field. */
#define POWER_PERRDY_TIMER0_Msk (0x1UL << POWER_PERRDY_TIMER0_Pos) /*!< Bit mask of TIMER0 field. */

/* Bit 7 : ADC powered up and ready to use. */
#define POWER_PERRDY_ADC_Pos (7U) /*!< Position of ADC field. */
#define POWER_PERRDY_ADC_Msk (0x1UL << POWER_PERRDY_ADC_Pos) /*!< Bit mask of ADC field. */

/* Bit 6 : GPIOTE powered up and ready to use. */
#define POWER_PERRDY_GPIOTE_Pos (6U) /*!< Position of GPIOTE field. */
#define POWER_PERRDY_GPIOTE_Msk (0x1UL << POWER_PERRDY_GPIOTE_Pos) /*!< Bit mask of GPIOTE field. */

/* Bit 4 : SPI1 powered up and ready to use. */
#define POWER_PERRDY_SPI1_Pos (4U) /*!< Position of SPI1 field. */
#define POWER_PERRDY_SPI1_Msk (0x1UL << POWER_PERRDY_SPI1_Pos) /*!< Bit mask of SPI1 field. */

/* Bit 3 : SPI0 powered up and ready to use. */
#define POWER_PERRDY_SPI0_Pos (3U) /*!< Position of SPI0 field. */
#define POWER_PERRDY_SPI0_Msk (0x1UL << POWER_PERRDY_SPI0_Pos) /*!< Bit mask of SPI0 field. */

/* Bit 2 : UART0 powered up and ready to use. */
#define POWER_PERRDY_UART0_Pos (2U) /*!< Position of UART0 field. */
#define POWER_PERRDY_UART0_Msk (0x1UL << POWER_PERRDY_UART0_Pos) /*!< Bit mask of UART0 field. */

/* Bit 1 : RADIO powered up and ready to use. */
#define POWER_PERRDY_RADIO_Pos (1U) /*!< Position of RADIO field. */
#define POWER_PERRDY_RADIO_Msk (0x1UL << POWER_PERRDY_RADIO_Pos) /*!< Bit mask of RADIO field. */

/* Bit 0 : MEM_POWER_CLOCK powered up and ready to use. */
#define POWER_PERRDY_MEM_POWER_CLOCK_Pos (0U) /*!< Position of MEM_POWER_CLOCK field. */
#define POWER_PERRDY_MEM_POWER_CLOCK_Msk (0x1UL << POWER_PERRDY_MEM_POWER_CLOCK_Pos) /*!< Bit mask of MEM_POWER_CLOCK field. */

/* Register: POWER_SYSTEMOFF */
/* Description: System off register. */

/* Bit 0 : Enter system off mode. */
#define POWER_SYSTEMOFF_SYSTEMOFF_Pos (0U) /*!< Position of SYSTEMOFF field. */
#define POWER_SYSTEMOFF_SYSTEMOFF_Msk (0x1UL << POWER_SYSTEMOFF_SYSTEMOFF_Pos) /*!< Bit mask of SYSTEMOFF field. */
#define POWER_SYSTEMOFF_SYSTEMOFF_Enter (1UL) /*!< Enter system off mode. */

/* Register: POWER_PERPOWER */
/* Description: Peripheral power configuration. */

/* Bit 31 : Turn on PPI peripheral. */
#define POWER_PERPOWER_PPI_Pos (31U) /*!< Position of PPI field. */
#define POWER_PERPOWER_PPI_Msk (0x1UL << POWER_PERPOWER_PPI_Pos) /*!< Bit mask of PPI field. */

/* Bit 30 : Turn on NVMC peripheral. */
#define POWER_PERPOWER_NVMC_Pos (30U) /*!< Position of NVMC field. */
#define POWER_PERPOWER_NVMC_Msk (0x1UL << POWER_PERPOWER_NVMC_Pos) /*!< Bit mask of NVMC field. */

/* Bit 18 : Turn on QDEC peripheral. */
#define POWER_PERPOWER_QDEC_Pos (18U) /*!< Position of QDEC field. */
#define POWER_PERPOWER_QDEC_Msk (0x1UL << POWER_PERPOWER_QDEC_Pos) /*!< Bit mask of QDEC field. */

/* Bit 17 : Turn on RTC1 peripheral. */
#define POWER_PERPOWER_RTC1_Pos (17U) /*!< Position of RTC1 field. */
#define POWER_PERPOWER_RTC1_Msk (0x1UL << POWER_PERPOWER_RTC1_Pos) /*!< Bit mask of RTC1 field. */

/* Bit 16 : Turn on WDOG peripheral. */
#define POWER_PERPOWER_WDOG_Pos (16U) /*!< Position of WDOG field. */
#define POWER_PERPOWER_WDOG_Msk (0x1UL << POWER_PERPOWER_WDOG_Pos) /*!< Bit mask of WDOG field. */

/* Bit 15 : Turn on CCM peripheral. */
#define POWER_PERPOWER_CCM_Pos (15U) /*!< Position of CCM field. */
#define POWER_PERPOWER_CCM_Msk (0x1UL << POWER_PERPOWER_CCM_Pos) /*!< Bit mask of CCM field. */

/* Bit 14 : Turn on ECB peripheral. */
#define POWER_PERPOWER_ECB_Pos (14U) /*!< Position of ECB field. */
#define POWER_PERPOWER_ECB_Msk (0x1UL << POWER_PERPOWER_ECB_Pos) /*!< Bit mask of ECB field. */

/* Bit 13 : Turn on RNG peripheral. */
#define POWER_PERPOWER_RNG_Pos (13U) /*!< Position of RNG field. */
#define POWER_PERPOWER_RNG_Msk (0x1UL << POWER_PERPOWER_RNG_Pos) /*!< Bit mask of RNG field. */

/* Bit 12 : Turn on TEMP peripheral. */
#define POWER_PERPOWER_TEMP_Pos (12U) /*!< Position of TEMP field. */
#define POWER_PERPOWER_TEMP_Msk (0x1UL << POWER_PERPOWER_TEMP_Pos) /*!< Bit mask of TEMP field. */

/* Bit 11 : Turn on RTC0 peripheral. */
#define POWER_PERPOWER_RTC0_Pos (11U) /*!< Position of RTC0 field. */
#define POWER_PERPOWER_RTC0_Msk (0x1UL << POWER_PERPOWER_RTC0_Pos) /*!< Bit mask of RTC0 field. */

/* Bit 10 : Turn on TIMER2 peripheral. */
#define POWER_PERPOWER_TIMER2_Pos (10U) /*!< Position of TIMER2 field. */
#define POWER_PERPOWER_TIMER2_Msk (0x1UL << POWER_PERPOWER_TIMER2_Pos) /*!< Bit mask of TIMER2 field. */

/* Bit 9 : Turn on TIMER1 peripheral. */
#define POWER_PERPOWER_TIMER1_Pos (9U) /*!< Position of TIMER1 field. */
#define POWER_PERPOWER_TIMER1_Msk (0x1UL << POWER_PERPOWER_TIMER1_Pos) /*!< Bit mask of TIMER1 field. */

/* Bit 8 : Turn on TIMER0 peripheral. */
#define POWER_PERPOWER_TIMER0_Pos (8U) /*!< Position of TIMER0 field. */
#define POWER_PERPOWER_TIMER0_Msk (0x1UL << POWER_PERPOWER_TIMER0_Pos) /*!< Bit mask of TIMER0 field. */

/* Bit 7 : Turn on ADC peripheral. */
#define POWER_PERPOWER_ADC_Pos (7U) /*!< Position of ADC field. */
#define POWER_PERPOWER_ADC_Msk (0x1UL << POWER_PERPOWER_ADC_Pos) /*!< Bit mask of ADC field. */

/* Bit 6 : Turn on GPIOTE peripheral. */
#define POWER_PERPOWER_GPIOTE_Pos (6U) /*!< Position of GPIOTE field. */
#define POWER_PERPOWER_GPIOTE_Msk (0x1UL << POWER_PERPOWER_GPIOTE_Pos) /*!< Bit mask of GPIOTE field. */

/* Bit 4 : Turn on SPI1 peripheral. */
#define POWER_PERPOWER_SPI1_Pos (4U) /*!< Position of SPI1 field. */
#define POWER_PERPOWER_SPI1_Msk (0x1UL << POWER_PERPOWER_SPI1_Pos) /*!< Bit mask of SPI1 field. */

/* Bit 3 : Turn on SPI0 peripheral. */
#define POWER_PERPOWER_SPI0_Pos (3U) /*!< Position of SPI0 field. */
#define POWER_PERPOWER_SPI0_Msk (0x1UL << POWER_PERPOWER_SPI0_Pos) /*!< Bit mask of SPI0 field. */

/* Bit 2 : Turn on UART0 peripheral. */
#define POWER_PERPOWER_UART0_Pos (2U) /*!< Position of UART0 field. */
#define POWER_PERPOWER_UART0_Msk (0x1UL << POWER_PERPOWER_UART0_Pos) /*!< Bit mask of UART0 field. */

/* Bit 1 : Turn on RADIO peripheral. */
#define POWER_PERPOWER_RADIO_Pos (1U) /*!< Position of RADIO field. */
#define POWER_PERPOWER_RADIO_Msk (0x1UL << POWER_PERPOWER_RADIO_Pos) /*!< Bit mask of RADIO field. */

/* Bit 0 : Turn on MEM_POWER_CLOCK peripheral. */
#define POWER_PERPOWER_MEM_POWER_CLOCK_Pos (0U) /*!< Position of MEM_POWER_CLOCK field. */
#define POWER_PERPOWER_MEM_POWER_CLOCK_Msk (0x1UL << POWER_PERPOWER_MEM_POWER_CLOCK_Pos) /*!< Bit mask of MEM_POWER_CLOCK field. */

/* Register: POWER_POFCON */
/* Description: Power failure configuration. */

/* Bits 2..1 : Set threshold level. */
#define POWER_POFCON_THRESHOLD_Pos (1U) /*!< Position of THRESHOLD field. */
#define POWER_POFCON_THRESHOLD_Msk (0x3UL << POWER_POFCON_THRESHOLD_Pos) /*!< Bit mask of THRESHOLD field. */
#define POWER_POFCON_THRESHOLD_V21 (0x00UL) /*!< Set threshold to 2.1Volts. */
#define POWER_POFCON_THRESHOLD_V23 (0x01UL) /*!< Set threshold to 2.3Volts. */
#define POWER_POFCON_THRESHOLD_V25 (0x02UL) /*!< Set threshold to 2.5Volts. */
#define POWER_POFCON_THRESHOLD_V27 (0x03UL) /*!< Set threshold to 2.7Volts. */

/* Bit 0 : Power failure comparator enable. */
#define POWER_POFCON_POF_Pos (0U) /*!< Position of POF field. */
#define POWER_POFCON_POF_Msk (0x1UL << POWER_POFCON_POF_Pos) /*!< Bit mask of POF field. */
#define POWER_POFCON_POF_Disabled (0UL) /*!< Disabled. */
#define POWER_POFCON_POF_Enabled (1UL) /*!< Enabled. */

/* Register: POWER_RAMON */
/* Description: Ram on/off. */

/* Bit 17 : RAM block 1 behaviour in OFF mode. */
#define POWER_RAMON_OFFRAM1_Pos (17U) /*!< Position of OFFRAM1 field. */
#define POWER_RAMON_OFFRAM1_Msk (0x1UL << POWER_RAMON_OFFRAM1_Pos) /*!< Bit mask of OFFRAM1 field. */
#define POWER_RAMON_OFFRAM1_RAM1Off (0UL) /*!< RAM block 1 off in OFF mode. */
#define POWER_RAMON_OFFRAM1_RAM1On (1UL) /*!< RAM block 1 on in OFF mode. */

/* Bit 16 : RAM block 0 behaviour in OFF mode. */
#define POWER_RAMON_OFFRAM0_Pos (16U) /*!< Position of OFFRAM0 field. */
#define POWER_RAMON_OFFRAM0_Msk (0x1UL << POWER_RAMON_OFFRAM0_Pos) /*!< Bit mask of OFFRAM0 field. */
#define POWER_RAMON_OFFRAM0_RAM0Off (0UL) /*!< RAM block 0 off in OFF mode. */
#define POWER_RAMON_OFFRAM0_RAM0On (1UL) /*!< RAM block 0 on in OFF mode. */

/* Bit 1 : RAM block 1 behaviour in ON mode. */
#define POWER_RAMON_ONRAM1_Pos (1U) /*!< Position of ONRAM1 field. */
#define POWER_RAMON_ONRAM1_Msk (0x1UL << POWER_RAMON_ONRAM1_Pos) /*!< Bit mask of ONRAM1 field. */
#define POWER_RAMON_ONRAM1_RAM1Off (0UL) /*!< RAM block 1 off in ON mode. */
#define POWER_RAMON_ONRAM1_RAM1On (1UL) /*!< RAM block 1 on in ON mode. */

/* Bit 0 : RAM block 0 behaviour in ON mode. */
#define POWER_RAMON_ONRAM0_Pos (0U) /*!< Position of ONRAM0 field. */
#define POWER_RAMON_ONRAM0_Msk (0x1UL << POWER_RAMON_ONRAM0_Pos) /*!< Bit mask of ONRAM0 field. */
#define POWER_RAMON_ONRAM0_RAM0Off (0UL) /*!< RAM block 0 off in ON mode. */
#define POWER_RAMON_ONRAM0_RAM0On (1UL) /*!< RAM block 0 on in ON mode. */


/* Peripheral: PPI */
/* Description: PPI controller */

/* Register: PPI_CHEN */
/* Description: Channel enable. */

/* Bit 15 : Enable PPI channel 15. */
#define PPI_CHEN_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHEN_CH15_Msk (0x1UL << PPI_CHEN_CH15_Pos) /*!< Bit mask of CH15 field. */
#define PPI_CHEN_CH15_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH15_Enabled (1UL) /*!< Channel enabled. */

/* Bit 14 : Enable PPI channel 14. */
#define PPI_CHEN_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHEN_CH14_Msk (0x1UL << PPI_CHEN_CH14_Pos) /*!< Bit mask of CH14 field. */
#define PPI_CHEN_CH14_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH14_Enabled (1UL) /*!< Channel enabled. */

/* Bit 13 : Enable PPI channel 13. */
#define PPI_CHEN_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHEN_CH13_Msk (0x1UL << PPI_CHEN_CH13_Pos) /*!< Bit mask of CH13 field. */
#define PPI_CHEN_CH13_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH13_Enabled (1UL) /*!< Channel enabled. */

/* Bit 12 : Enable PPI channel 12. */
#define PPI_CHEN_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHEN_CH12_Msk (0x1UL << PPI_CHEN_CH12_Pos) /*!< Bit mask of CH12 field. */
#define PPI_CHEN_CH12_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH12_Enabled (1UL) /*!< Channel enabled. */

/* Bit 11 : Enable PPI channel 11. */
#define PPI_CHEN_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHEN_CH11_Msk (0x1UL << PPI_CHEN_CH11_Pos) /*!< Bit mask of CH11 field. */
#define PPI_CHEN_CH11_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH11_Enabled (1UL) /*!< Channel enabled. */

/* Bit 10 : Enable PPI channel 10. */
#define PPI_CHEN_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHEN_CH10_Msk (0x1UL << PPI_CHEN_CH10_Pos) /*!< Bit mask of CH10 field. */
#define PPI_CHEN_CH10_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH10_Enabled (1UL) /*!< Channel enabled. */

/* Bit 9 : Enable PPI channel 9. */
#define PPI_CHEN_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHEN_CH9_Msk (0x1UL << PPI_CHEN_CH9_Pos) /*!< Bit mask of CH9 field. */
#define PPI_CHEN_CH9_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH9_Enabled (1UL) /*!< Channel enabled. */

/* Bit 8 : Enable PPI channel 8. */
#define PPI_CHEN_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHEN_CH8_Msk (0x1UL << PPI_CHEN_CH8_Pos) /*!< Bit mask of CH8 field. */
#define PPI_CHEN_CH8_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH8_Enabled (1UL) /*!< Channel enabled. */

/* Bit 7 : Enable PPI channel 7. */
#define PPI_CHEN_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHEN_CH7_Msk (0x1UL << PPI_CHEN_CH7_Pos) /*!< Bit mask of CH7 field. */
#define PPI_CHEN_CH7_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH7_Enabled (1UL) /*!< Channel enabled. */

/* Bit 6 : Enable PPI channel 6. */
#define PPI_CHEN_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHEN_CH6_Msk (0x1UL << PPI_CHEN_CH6_Pos) /*!< Bit mask of CH6 field. */
#define PPI_CHEN_CH6_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH6_Enabled (1UL) /*!< Channel enabled. */

/* Bit 5 : Enable PPI channel 5. */
#define PPI_CHEN_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHEN_CH5_Msk (0x1UL << PPI_CHEN_CH5_Pos) /*!< Bit mask of CH5 field. */
#define PPI_CHEN_CH5_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH5_Enabled (1UL) /*!< Channel enabled. */

/* Bit 4 : Enable PPI channel 4. */
#define PPI_CHEN_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHEN_CH4_Msk (0x1UL << PPI_CHEN_CH4_Pos) /*!< Bit mask of CH4 field. */
#define PPI_CHEN_CH4_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH4_Enabled (1UL) /*!< Channel enabled. */

/* Bit 3 : Enable PPI channel 3. */
#define PPI_CHEN_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHEN_CH3_Msk (0x1UL << PPI_CHEN_CH3_Pos) /*!< Bit mask of CH3 field. */
#define PPI_CHEN_CH3_Disabled (0UL) /*!< Channel disabled */
#define PPI_CHEN_CH3_Enabled (1UL) /*!< Channel enabled */

/* Bit 2 : Enable PPI channel 2. */
#define PPI_CHEN_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHEN_CH2_Msk (0x1UL << PPI_CHEN_CH2_Pos) /*!< Bit mask of CH2 field. */
#define PPI_CHEN_CH2_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH2_Enabled (1UL) /*!< Channel enabled. */

/* Bit 1 : Enable PPI channel 1. */
#define PPI_CHEN_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHEN_CH1_Msk (0x1UL << PPI_CHEN_CH1_Pos) /*!< Bit mask of CH1 field. */
#define PPI_CHEN_CH1_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH1_Enabled (1UL) /*!< Channel enabled. */

/* Bit 0 : Enable PPI channel 0. */
#define PPI_CHEN_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHEN_CH0_Msk (0x1UL << PPI_CHEN_CH0_Pos) /*!< Bit mask of CH0 field. */
#define PPI_CHEN_CH0_Disabled (0UL) /*!< Channel disabled. */
#define PPI_CHEN_CH0_Enabled (1UL) /*!< Channel enabled. */

/* Register: PPI_CHENSET */
/* Description: Channel enable set. */

/* Bit 15 : Enable PPI channel 15. */
#define PPI_CHENSET_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHENSET_CH15_Msk (0x1UL << PPI_CHENSET_CH15_Pos) /*!< Bit mask of CH15 field. */

/* Bit 14 : Enable PPI channel 14. */
#define PPI_CHENSET_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHENSET_CH14_Msk (0x1UL << PPI_CHENSET_CH14_Pos) /*!< Bit mask of CH14 field. */

/* Bit 13 : Enable PPI channel 13. */
#define PPI_CHENSET_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHENSET_CH13_Msk (0x1UL << PPI_CHENSET_CH13_Pos) /*!< Bit mask of CH13 field. */

/* Bit 12 : Enable PPI channel 12. */
#define PPI_CHENSET_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHENSET_CH12_Msk (0x1UL << PPI_CHENSET_CH12_Pos) /*!< Bit mask of CH12 field. */

/* Bit 11 : Enable PPI channel 11. */
#define PPI_CHENSET_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHENSET_CH11_Msk (0x1UL << PPI_CHENSET_CH11_Pos) /*!< Bit mask of CH11 field. */

/* Bit 10 : Enable PPI channel 10. */
#define PPI_CHENSET_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHENSET_CH10_Msk (0x1UL << PPI_CHENSET_CH10_Pos) /*!< Bit mask of CH10 field. */

/* Bit 9 : Enable PPI channel 9. */
#define PPI_CHENSET_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHENSET_CH9_Msk (0x1UL << PPI_CHENSET_CH9_Pos) /*!< Bit mask of CH9 field. */

/* Bit 8 : Enable PPI channel 8. */
#define PPI_CHENSET_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHENSET_CH8_Msk (0x1UL << PPI_CHENSET_CH8_Pos) /*!< Bit mask of CH8 field. */

/* Bit 7 : Enable PPI channel 7. */
#define PPI_CHENSET_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHENSET_CH7_Msk (0x1UL << PPI_CHENSET_CH7_Pos) /*!< Bit mask of CH7 field. */

/* Bit 6 : Enable PPI channel 6. */
#define PPI_CHENSET_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHENSET_CH6_Msk (0x1UL << PPI_CHENSET_CH6_Pos) /*!< Bit mask of CH6 field. */

/* Bit 5 : Enable PPI channel 5. */
#define PPI_CHENSET_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHENSET_CH5_Msk (0x1UL << PPI_CHENSET_CH5_Pos) /*!< Bit mask of CH5 field. */

/* Bit 4 : Enable PPI channel 4. */
#define PPI_CHENSET_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHENSET_CH4_Msk (0x1UL << PPI_CHENSET_CH4_Pos) /*!< Bit mask of CH4 field. */

/* Bit 3 : Enable PPI channel 3. */
#define PPI_CHENSET_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHENSET_CH3_Msk (0x1UL << PPI_CHENSET_CH3_Pos) /*!< Bit mask of CH3 field. */

/* Bit 2 : Enable PPI channel 2. */
#define PPI_CHENSET_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHENSET_CH2_Msk (0x1UL << PPI_CHENSET_CH2_Pos) /*!< Bit mask of CH2 field. */

/* Bit 1 : Enable PPI channel 1. */
#define PPI_CHENSET_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHENSET_CH1_Msk (0x1UL << PPI_CHENSET_CH1_Pos) /*!< Bit mask of CH1 field. */

/* Bit 0 : Enable PPI channel 0. */
#define PPI_CHENSET_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHENSET_CH0_Msk (0x1UL << PPI_CHENSET_CH0_Pos) /*!< Bit mask of CH0 field. */

/* Register: PPI_CHENCLR */
/* Description: Channel enable clear. */

/* Bit 15 : Disable PPI channel 15. */
#define PPI_CHENCLR_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHENCLR_CH15_Msk (0x1UL << PPI_CHENCLR_CH15_Pos) /*!< Bit mask of CH15 field. */

/* Bit 14 : Disable PPI channel 14. */
#define PPI_CHENCLR_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHENCLR_CH14_Msk (0x1UL << PPI_CHENCLR_CH14_Pos) /*!< Bit mask of CH14 field. */

/* Bit 13 : Disable PPI channel 13. */
#define PPI_CHENCLR_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHENCLR_CH13_Msk (0x1UL << PPI_CHENCLR_CH13_Pos) /*!< Bit mask of CH13 field. */

/* Bit 12 : Disable PPI channel 12. */
#define PPI_CHENCLR_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHENCLR_CH12_Msk (0x1UL << PPI_CHENCLR_CH12_Pos) /*!< Bit mask of CH12 field. */

/* Bit 11 : Disable PPI channel 11. */
#define PPI_CHENCLR_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHENCLR_CH11_Msk (0x1UL << PPI_CHENCLR_CH11_Pos) /*!< Bit mask of CH11 field. */

/* Bit 10 : Disable PPI channel 10. */
#define PPI_CHENCLR_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHENCLR_CH10_Msk (0x1UL << PPI_CHENCLR_CH10_Pos) /*!< Bit mask of CH10 field. */

/* Bit 9 : Disable PPI channel 9. */
#define PPI_CHENCLR_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHENCLR_CH9_Msk (0x1UL << PPI_CHENCLR_CH9_Pos) /*!< Bit mask of CH9 field. */

/* Bit 8 : Disable PPI channel 8. */
#define PPI_CHENCLR_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHENCLR_CH8_Msk (0x1UL << PPI_CHENCLR_CH8_Pos) /*!< Bit mask of CH8 field. */

/* Bit 7 : Disable PPI channel 7. */
#define PPI_CHENCLR_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHENCLR_CH7_Msk (0x1UL << PPI_CHENCLR_CH7_Pos) /*!< Bit mask of CH7 field. */

/* Bit 6 : Disable PPI channel 6. */
#define PPI_CHENCLR_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHENCLR_CH6_Msk (0x1UL << PPI_CHENCLR_CH6_Pos) /*!< Bit mask of CH6 field. */

/* Bit 5 : Disable PPI channel 5. */
#define PPI_CHENCLR_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHENCLR_CH5_Msk (0x1UL << PPI_CHENCLR_CH5_Pos) /*!< Bit mask of CH5 field. */

/* Bit 4 : Disable PPI channel 4. */
#define PPI_CHENCLR_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHENCLR_CH4_Msk (0x1UL << PPI_CHENCLR_CH4_Pos) /*!< Bit mask of CH4 field. */

/* Bit 3 : Disable PPI channel 3. */
#define PPI_CHENCLR_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHENCLR_CH3_Msk (0x1UL << PPI_CHENCLR_CH3_Pos) /*!< Bit mask of CH3 field. */

/* Bit 2 : Disable PPI channel 2. */
#define PPI_CHENCLR_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHENCLR_CH2_Msk (0x1UL << PPI_CHENCLR_CH2_Pos) /*!< Bit mask of CH2 field. */

/* Bit 1 : Disable PPI channel 1. */
#define PPI_CHENCLR_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHENCLR_CH1_Msk (0x1UL << PPI_CHENCLR_CH1_Pos) /*!< Bit mask of CH1 field. */

/* Bit 0 : Disable PPI channel 0. */
#define PPI_CHENCLR_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHENCLR_CH0_Msk (0x1UL << PPI_CHENCLR_CH0_Pos) /*!< Bit mask of CH0 field. */

/* Register: PPI_CHG0 */
/* Description: Channel group 0. */

/* Bit 15 : Include CH15 in channel group 0. */
#define PPI_CHG0_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHG0_CH15_Msk (0x1UL << PPI_CHG0_CH15_Pos) /*!< Bit mask of CH15 field. */
#define PPI_CHG0_CH15_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH15_Included (1UL) /*!< Channel included. */

/* Bit 14 : Include CH14 in channel group 0. */
#define PPI_CHG0_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHG0_CH14_Msk (0x1UL << PPI_CHG0_CH14_Pos) /*!< Bit mask of CH14 field. */
#define PPI_CHG0_CH14_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH14_Included (1UL) /*!< Channel included. */

/* Bit 13 : Include CH13 in channel group 0. */
#define PPI_CHG0_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHG0_CH13_Msk (0x1UL << PPI_CHG0_CH13_Pos) /*!< Bit mask of CH13 field. */
#define PPI_CHG0_CH13_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH13_Included (1UL) /*!< Channel included. */

/* Bit 12 : Include CH12 in channel group 0. */
#define PPI_CHG0_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHG0_CH12_Msk (0x1UL << PPI_CHG0_CH12_Pos) /*!< Bit mask of CH12 field. */
#define PPI_CHG0_CH12_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH12_Included (1UL) /*!< Channel included. */

/* Bit 11 : Include CH11 in channel group 0. */
#define PPI_CHG0_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHG0_CH11_Msk (0x1UL << PPI_CHG0_CH11_Pos) /*!< Bit mask of CH11 field. */
#define PPI_CHG0_CH11_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH11_Included (1UL) /*!< Channel included. */

/* Bit 10 : Include CH10 in channel group 0. */
#define PPI_CHG0_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHG0_CH10_Msk (0x1UL << PPI_CHG0_CH10_Pos) /*!< Bit mask of CH10 field. */
#define PPI_CHG0_CH10_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH10_Included (1UL) /*!< Channel included. */

/* Bit 9 : Include CH9 in channel group 0. */
#define PPI_CHG0_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHG0_CH9_Msk (0x1UL << PPI_CHG0_CH9_Pos) /*!< Bit mask of CH9 field. */
#define PPI_CHG0_CH9_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH9_Included (1UL) /*!< Channel included. */

/* Bit 8 : Include CH8 in channel group 0. */
#define PPI_CHG0_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHG0_CH8_Msk (0x1UL << PPI_CHG0_CH8_Pos) /*!< Bit mask of CH8 field. */
#define PPI_CHG0_CH8_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH8_Included (1UL) /*!< Channel included. */

/* Bit 7 : Include CH7 in channel group 0. */
#define PPI_CHG0_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHG0_CH7_Msk (0x1UL << PPI_CHG0_CH7_Pos) /*!< Bit mask of CH7 field. */
#define PPI_CHG0_CH7_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH7_Included (1UL) /*!< Channel included. */

/* Bit 6 : Include CH6 in channel group 0. */
#define PPI_CHG0_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHG0_CH6_Msk (0x1UL << PPI_CHG0_CH6_Pos) /*!< Bit mask of CH6 field. */
#define PPI_CHG0_CH6_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH6_Included (1UL) /*!< Channel included. */

/* Bit 5 : Include CH5 in channel group 0. */
#define PPI_CHG0_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHG0_CH5_Msk (0x1UL << PPI_CHG0_CH5_Pos) /*!< Bit mask of CH5 field. */
#define PPI_CHG0_CH5_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH5_Included (1UL) /*!< Channel included. */

/* Bit 4 : Include CH4 in channel group 0. */
#define PPI_CHG0_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHG0_CH4_Msk (0x1UL << PPI_CHG0_CH4_Pos) /*!< Bit mask of CH4 field. */
#define PPI_CHG0_CH4_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH4_Included (1UL) /*!< Channel included. */

/* Bit 3 : Include CH3 in channel group 0. */
#define PPI_CHG0_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHG0_CH3_Msk (0x1UL << PPI_CHG0_CH3_Pos) /*!< Bit mask of CH3 field. */
#define PPI_CHG0_CH3_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH3_Included (1UL) /*!< Channel included. */

/* Bit 2 : Include CH2 in channel group 0. */
#define PPI_CHG0_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHG0_CH2_Msk (0x1UL << PPI_CHG0_CH2_Pos) /*!< Bit mask of CH2 field. */
#define PPI_CHG0_CH2_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH2_Included (1UL) /*!< Channel included. */

/* Bit 1 : Include CH1 in channel group 0. */
#define PPI_CHG0_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHG0_CH1_Msk (0x1UL << PPI_CHG0_CH1_Pos) /*!< Bit mask of CH1 field. */
#define PPI_CHG0_CH1_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH1_Included (1UL) /*!< Channel included. */

/* Bit 0 : Include CH0 in channel group 0. */
#define PPI_CHG0_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHG0_CH0_Msk (0x1UL << PPI_CHG0_CH0_Pos) /*!< Bit mask of CH0 field. */
#define PPI_CHG0_CH0_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG0_CH0_Included (1UL) /*!< Channel included. */

/* Register: PPI_CHG1 */
/* Description: Channel group 1. */

/* Bit 15 : Include CH15 in channel group 1. */
#define PPI_CHG1_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHG1_CH15_Msk (0x1UL << PPI_CHG1_CH15_Pos) /*!< Bit mask of CH15 field. */
#define PPI_CHG1_CH15_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH15_Included (1UL) /*!< Channel included. */

/* Bit 14 : Include CH14 in channel group 1. */
#define PPI_CHG1_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHG1_CH14_Msk (0x1UL << PPI_CHG1_CH14_Pos) /*!< Bit mask of CH14 field. */
#define PPI_CHG1_CH14_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH14_Included (1UL) /*!< Channel included. */

/* Bit 13 : Include CH13 in channel group 1. */
#define PPI_CHG1_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHG1_CH13_Msk (0x1UL << PPI_CHG1_CH13_Pos) /*!< Bit mask of CH13 field. */
#define PPI_CHG1_CH13_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH13_Included (1UL) /*!< Channel included. */

/* Bit 12 : Include CH12 in channel group 1. */
#define PPI_CHG1_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHG1_CH12_Msk (0x1UL << PPI_CHG1_CH12_Pos) /*!< Bit mask of CH12 field. */
#define PPI_CHG1_CH12_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH12_Included (1UL) /*!< Channel included. */

/* Bit 11 : Include CH11 in channel group 1. */
#define PPI_CHG1_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHG1_CH11_Msk (0x1UL << PPI_CHG1_CH11_Pos) /*!< Bit mask of CH11 field. */
#define PPI_CHG1_CH11_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH11_Included (1UL) /*!< Channel included. */

/* Bit 10 : Include CH10 in channel group 1. */
#define PPI_CHG1_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHG1_CH10_Msk (0x1UL << PPI_CHG1_CH10_Pos) /*!< Bit mask of CH10 field. */
#define PPI_CHG1_CH10_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH10_Included (1UL) /*!< Channel included. */

/* Bit 9 : Include CH9 in channel group 1. */
#define PPI_CHG1_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHG1_CH9_Msk (0x1UL << PPI_CHG1_CH9_Pos) /*!< Bit mask of CH9 field. */
#define PPI_CHG1_CH9_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH9_Included (1UL) /*!< Channel included. */

/* Bit 8 : Include CH8 in channel group 1. */
#define PPI_CHG1_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHG1_CH8_Msk (0x1UL << PPI_CHG1_CH8_Pos) /*!< Bit mask of CH8 field. */
#define PPI_CHG1_CH8_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH8_Included (1UL) /*!< Channel included. */

/* Bit 7 : Include CH7 in channel group 1. */
#define PPI_CHG1_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHG1_CH7_Msk (0x1UL << PPI_CHG1_CH7_Pos) /*!< Bit mask of CH7 field. */
#define PPI_CHG1_CH7_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH7_Included (1UL) /*!< Channel included. */

/* Bit 6 : Include CH6 in channel group 1. */
#define PPI_CHG1_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHG1_CH6_Msk (0x1UL << PPI_CHG1_CH6_Pos) /*!< Bit mask of CH6 field. */
#define PPI_CHG1_CH6_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH6_Included (1UL) /*!< Channel included. */

/* Bit 5 : Include CH5 in channel group 1. */
#define PPI_CHG1_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHG1_CH5_Msk (0x1UL << PPI_CHG1_CH5_Pos) /*!< Bit mask of CH5 field. */
#define PPI_CHG1_CH5_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH5_Included (1UL) /*!< Channel included. */

/* Bit 4 : Include CH4 in channel group 1. */
#define PPI_CHG1_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHG1_CH4_Msk (0x1UL << PPI_CHG1_CH4_Pos) /*!< Bit mask of CH4 field. */
#define PPI_CHG1_CH4_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH4_Included (1UL) /*!< Channel included. */

/* Bit 3 : Include CH3 in channel group 1. */
#define PPI_CHG1_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHG1_CH3_Msk (0x1UL << PPI_CHG1_CH3_Pos) /*!< Bit mask of CH3 field. */
#define PPI_CHG1_CH3_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH3_Included (1UL) /*!< Channel included. */

/* Bit 2 : Include CH2 in channel group 1. */
#define PPI_CHG1_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHG1_CH2_Msk (0x1UL << PPI_CHG1_CH2_Pos) /*!< Bit mask of CH2 field. */
#define PPI_CHG1_CH2_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH2_Included (1UL) /*!< Channel included. */

/* Bit 1 : Include CH1 in channel group 1. */
#define PPI_CHG1_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHG1_CH1_Msk (0x1UL << PPI_CHG1_CH1_Pos) /*!< Bit mask of CH1 field. */
#define PPI_CHG1_CH1_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH1_Included (1UL) /*!< Channel included. */

/* Bit 0 : Include CH0 in channel group 1. */
#define PPI_CHG1_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHG1_CH0_Msk (0x1UL << PPI_CHG1_CH0_Pos) /*!< Bit mask of CH0 field. */
#define PPI_CHG1_CH0_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG1_CH0_Included (1UL) /*!< Channel included. */

/* Register: PPI_CHG2 */
/* Description: Channel group 2. */

/* Bit 15 : Include CH15 in channel group 2. */
#define PPI_CHG2_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHG2_CH15_Msk (0x1UL << PPI_CHG2_CH15_Pos) /*!< Bit mask of CH15 field. */
#define PPI_CHG2_CH15_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH15_Included (1UL) /*!< Channel included. */

/* Bit 14 : Include CH14 in channel group 2. */
#define PPI_CHG2_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHG2_CH14_Msk (0x1UL << PPI_CHG2_CH14_Pos) /*!< Bit mask of CH14 field. */
#define PPI_CHG2_CH14_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH14_Included (1UL) /*!< Channel included. */

/* Bit 13 : Include CH13 in channel group 2. */
#define PPI_CHG2_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHG2_CH13_Msk (0x1UL << PPI_CHG2_CH13_Pos) /*!< Bit mask of CH13 field. */
#define PPI_CHG2_CH13_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH13_Included (1UL) /*!< Channel included. */

/* Bit 12 : Include CH12 in channel group 2. */
#define PPI_CHG2_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHG2_CH12_Msk (0x1UL << PPI_CHG2_CH12_Pos) /*!< Bit mask of CH12 field. */
#define PPI_CHG2_CH12_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH12_Included (1UL) /*!< Channel included. */

/* Bit 11 : Include CH11 in channel group 2. */
#define PPI_CHG2_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHG2_CH11_Msk (0x1UL << PPI_CHG2_CH11_Pos) /*!< Bit mask of CH11 field. */
#define PPI_CHG2_CH11_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH11_Included (1UL) /*!< Channel included. */

/* Bit 10 : Include CH10 in channel group 2. */
#define PPI_CHG2_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHG2_CH10_Msk (0x1UL << PPI_CHG2_CH10_Pos) /*!< Bit mask of CH10 field. */
#define PPI_CHG2_CH10_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH10_Included (1UL) /*!< Channel included. */

/* Bit 9 : Include CH9 in channel group 2. */
#define PPI_CHG2_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHG2_CH9_Msk (0x1UL << PPI_CHG2_CH9_Pos) /*!< Bit mask of CH9 field. */
#define PPI_CHG2_CH9_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH9_Included (1UL) /*!< Channel included. */

/* Bit 8 : Include CH8 in channel group 2. */
#define PPI_CHG2_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHG2_CH8_Msk (0x1UL << PPI_CHG2_CH8_Pos) /*!< Bit mask of CH8 field. */
#define PPI_CHG2_CH8_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH8_Included (1UL) /*!< Channel included. */

/* Bit 7 : Include CH7 in channel group 2. */
#define PPI_CHG2_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHG2_CH7_Msk (0x1UL << PPI_CHG2_CH7_Pos) /*!< Bit mask of CH7 field. */
#define PPI_CHG2_CH7_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH7_Included (1UL) /*!< Channel included. */

/* Bit 6 : Include CH6 in channel group 2. */
#define PPI_CHG2_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHG2_CH6_Msk (0x1UL << PPI_CHG2_CH6_Pos) /*!< Bit mask of CH6 field. */
#define PPI_CHG2_CH6_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH6_Included (1UL) /*!< Channel included. */

/* Bit 5 : Include CH5 in channel group 2. */
#define PPI_CHG2_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHG2_CH5_Msk (0x1UL << PPI_CHG2_CH5_Pos) /*!< Bit mask of CH5 field. */
#define PPI_CHG2_CH5_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH5_Included (1UL) /*!< Channel included. */

/* Bit 4 : Include CH4 in channel group 2. */
#define PPI_CHG2_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHG2_CH4_Msk (0x1UL << PPI_CHG2_CH4_Pos) /*!< Bit mask of CH4 field. */
#define PPI_CHG2_CH4_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH4_Included (1UL) /*!< Channel included. */

/* Bit 3 : Include CH3 in channel group 2. */
#define PPI_CHG2_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHG2_CH3_Msk (0x1UL << PPI_CHG2_CH3_Pos) /*!< Bit mask of CH3 field. */
#define PPI_CHG2_CH3_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH3_Included (1UL) /*!< Channel included. */

/* Bit 2 : Include CH2 in channel group 2. */
#define PPI_CHG2_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHG2_CH2_Msk (0x1UL << PPI_CHG2_CH2_Pos) /*!< Bit mask of CH2 field. */
#define PPI_CHG2_CH2_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH2_Included (1UL) /*!< Channel included. */

/* Bit 1 : Include CH1 in channel group 2. */
#define PPI_CHG2_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHG2_CH1_Msk (0x1UL << PPI_CHG2_CH1_Pos) /*!< Bit mask of CH1 field. */
#define PPI_CHG2_CH1_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH1_Included (1UL) /*!< Channel included. */

/* Bit 0 : Include CH0 in channel group 2. */
#define PPI_CHG2_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHG2_CH0_Msk (0x1UL << PPI_CHG2_CH0_Pos) /*!< Bit mask of CH0 field. */
#define PPI_CHG2_CH0_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG2_CH0_Included (1UL) /*!< Channel included. */

/* Register: PPI_CHG3 */
/* Description: Channel group 3. */

/* Bit 15 : Include CH15 in channel group 3. */
#define PPI_CHG3_CH15_Pos (15U) /*!< Position of CH15 field. */
#define PPI_CHG3_CH15_Msk (0x1UL << PPI_CHG3_CH15_Pos) /*!< Bit mask of CH15 field. */
#define PPI_CHG3_CH15_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH15_Included (1UL) /*!< Channel included. */

/* Bit 14 : Include CH14 in channel group 3. */
#define PPI_CHG3_CH14_Pos (14U) /*!< Position of CH14 field. */
#define PPI_CHG3_CH14_Msk (0x1UL << PPI_CHG3_CH14_Pos) /*!< Bit mask of CH14 field. */
#define PPI_CHG3_CH14_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH14_Included (1UL) /*!< Channel included. */

/* Bit 13 : Include CH13 in channel group 3. */
#define PPI_CHG3_CH13_Pos (13U) /*!< Position of CH13 field. */
#define PPI_CHG3_CH13_Msk (0x1UL << PPI_CHG3_CH13_Pos) /*!< Bit mask of CH13 field. */
#define PPI_CHG3_CH13_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH13_Included (1UL) /*!< Channel included. */

/* Bit 12 : Include CH12 in channel group 3. */
#define PPI_CHG3_CH12_Pos (12U) /*!< Position of CH12 field. */
#define PPI_CHG3_CH12_Msk (0x1UL << PPI_CHG3_CH12_Pos) /*!< Bit mask of CH12 field. */
#define PPI_CHG3_CH12_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH12_Included (1UL) /*!< Channel included. */

/* Bit 11 : Include CH11 in channel group 3. */
#define PPI_CHG3_CH11_Pos (11U) /*!< Position of CH11 field. */
#define PPI_CHG3_CH11_Msk (0x1UL << PPI_CHG3_CH11_Pos) /*!< Bit mask of CH11 field. */
#define PPI_CHG3_CH11_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH11_Included (1UL) /*!< Channel included. */

/* Bit 10 : Include CH10 in channel group 3. */
#define PPI_CHG3_CH10_Pos (10U) /*!< Position of CH10 field. */
#define PPI_CHG3_CH10_Msk (0x1UL << PPI_CHG3_CH10_Pos) /*!< Bit mask of CH10 field. */
#define PPI_CHG3_CH10_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH10_Included (1UL) /*!< Channel included. */

/* Bit 9 : Include CH9 in channel group 3. */
#define PPI_CHG3_CH9_Pos (9U) /*!< Position of CH9 field. */
#define PPI_CHG3_CH9_Msk (0x1UL << PPI_CHG3_CH9_Pos) /*!< Bit mask of CH9 field. */
#define PPI_CHG3_CH9_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH9_Included (1UL) /*!< Channel included. */

/* Bit 8 : Include CH8 in channel group 3. */
#define PPI_CHG3_CH8_Pos (8U) /*!< Position of CH8 field. */
#define PPI_CHG3_CH8_Msk (0x1UL << PPI_CHG3_CH8_Pos) /*!< Bit mask of CH8 field. */
#define PPI_CHG3_CH8_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH8_Included (1UL) /*!< Channel included. */

/* Bit 7 : Include CH7 in channel group 3. */
#define PPI_CHG3_CH7_Pos (7U) /*!< Position of CH7 field. */
#define PPI_CHG3_CH7_Msk (0x1UL << PPI_CHG3_CH7_Pos) /*!< Bit mask of CH7 field. */
#define PPI_CHG3_CH7_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH7_Included (1UL) /*!< Channel included. */

/* Bit 6 : Include CH6 in channel group 3. */
#define PPI_CHG3_CH6_Pos (6U) /*!< Position of CH6 field. */
#define PPI_CHG3_CH6_Msk (0x1UL << PPI_CHG3_CH6_Pos) /*!< Bit mask of CH6 field. */
#define PPI_CHG3_CH6_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH6_Included (1UL) /*!< Channel included. */

/* Bit 5 : Include CH5 in channel group 3. */
#define PPI_CHG3_CH5_Pos (5U) /*!< Position of CH5 field. */
#define PPI_CHG3_CH5_Msk (0x1UL << PPI_CHG3_CH5_Pos) /*!< Bit mask of CH5 field. */
#define PPI_CHG3_CH5_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH5_Included (1UL) /*!< Channel included. */

/* Bit 4 : Include CH4 in channel group 3. */
#define PPI_CHG3_CH4_Pos (4U) /*!< Position of CH4 field. */
#define PPI_CHG3_CH4_Msk (0x1UL << PPI_CHG3_CH4_Pos) /*!< Bit mask of CH4 field. */
#define PPI_CHG3_CH4_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH4_Included (1UL) /*!< Channel included. */

/* Bit 3 : Include CH3 in channel group 3. */
#define PPI_CHG3_CH3_Pos (3U) /*!< Position of CH3 field. */
#define PPI_CHG3_CH3_Msk (0x1UL << PPI_CHG3_CH3_Pos) /*!< Bit mask of CH3 field. */
#define PPI_CHG3_CH3_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH3_Included (1UL) /*!< Channel included. */

/* Bit 2 : Include CH2 in channel group 3. */
#define PPI_CHG3_CH2_Pos (2U) /*!< Position of CH2 field. */
#define PPI_CHG3_CH2_Msk (0x1UL << PPI_CHG3_CH2_Pos) /*!< Bit mask of CH2 field. */
#define PPI_CHG3_CH2_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH2_Included (1UL) /*!< Channel included. */

/* Bit 1 : Include CH1 in channel group 3. */
#define PPI_CHG3_CH1_Pos (1U) /*!< Position of CH1 field. */
#define PPI_CHG3_CH1_Msk (0x1UL << PPI_CHG3_CH1_Pos) /*!< Bit mask of CH1 field. */
#define PPI_CHG3_CH1_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH1_Included (1UL) /*!< Channel included. */

/* Bit 0 : Include CH0 in channel group 3. */
#define PPI_CHG3_CH0_Pos (0U) /*!< Position of CH0 field. */
#define PPI_CHG3_CH0_Msk (0x1UL << PPI_CHG3_CH0_Pos) /*!< Bit mask of CH0 field. */
#define PPI_CHG3_CH0_Excluded (0UL) /*!< Channel excluded. */
#define PPI_CHG3_CH0_Included (1UL) /*!< Channel included. */


/* Peripheral: QDEC */
/* Description: Rotary decoder */

/* Register: QDEC_SHORTS */
/* Description: Shortcut for the QDEC. */

/* Bit 1 : Short-cut between SAMPLERDY event and STOP task. */
#define QDEC_SHORTS_SAMPLERDY_STOP_Pos (1U) /*!< Position of SAMPLERDY_STOP field. */
#define QDEC_SHORTS_SAMPLERDY_STOP_Msk (0x1UL << QDEC_SHORTS_SAMPLERDY_STOP_Pos) /*!< Bit mask of SAMPLERDY_STOP field. */
#define QDEC_SHORTS_SAMPLERDY_STOP_Disabled (0UL) /*!< Shortcut disabled. */
#define QDEC_SHORTS_SAMPLERDY_STOP_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 0 : Short-cut between REPORTRDY event and READCLRACC task. */
#define QDEC_SHORTS_REPORTRDY_READCLRACC_Pos (0U) /*!< Position of REPORTRDY_READCLRACC field. */
#define QDEC_SHORTS_REPORTRDY_READCLRACC_Msk (0x1UL << QDEC_SHORTS_REPORTRDY_READCLRACC_Pos) /*!< Bit mask of REPORTRDY_READCLRACC field. */
#define QDEC_SHORTS_REPORTRDY_READCLRACC_Disabled (0UL) /*!< Shortcut disabled. */
#define QDEC_SHORTS_REPORTRDY_READCLRACC_Enabled (1UL) /*!< Shortcut enabled. */

/* Register: QDEC_INTEN */
/* Description: Interrupt enable register. */

/* Bit 2 : Enable ACCOF interrupt. */
#define QDEC_INTEN_ACCOF_Pos (2U) /*!< Position of ACCOF field. */
#define QDEC_INTEN_ACCOF_Msk (0x1UL << QDEC_INTEN_ACCOF_Pos) /*!< Bit mask of ACCOF field. */
#define QDEC_INTEN_ACCOF_Disabled (0UL) /*!< Interrupt disabled. */
#define QDEC_INTEN_ACCOF_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 1 : Enable REPORTRDY interrupt. */
#define QDEC_INTEN_REPORTRDY_Pos (1U) /*!< Position of REPORTRDY field. */
#define QDEC_INTEN_REPORTRDY_Msk (0x1UL << QDEC_INTEN_REPORTRDY_Pos) /*!< Bit mask of REPORTRDY field. */
#define QDEC_INTEN_REPORTRDY_Disabled (0UL) /*!< Interrupt disabled. */
#define QDEC_INTEN_REPORTRDY_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 0 : Enable SAMPLERDY interrupt. */
#define QDEC_INTEN_SAMPLERDY_Pos (0U) /*!< Position of SAMPLERDY field. */
#define QDEC_INTEN_SAMPLERDY_Msk (0x1UL << QDEC_INTEN_SAMPLERDY_Pos) /*!< Bit mask of SAMPLERDY field. */
#define QDEC_INTEN_SAMPLERDY_Disabled (0UL) /*!< Interrupt disabled. */
#define QDEC_INTEN_SAMPLERDY_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: QDEC_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 2 : Enable interrupt on ACCOF event. */
#define QDEC_INTENSET_ACCOF_Pos (2U) /*!< Position of ACCOF field. */
#define QDEC_INTENSET_ACCOF_Msk (0x1UL << QDEC_INTENSET_ACCOF_Pos) /*!< Bit mask of ACCOF field. */

/* Bit 1 : Enable interrupt on REPORTRDY event. */
#define QDEC_INTENSET_REPORTRDY_Pos (1U) /*!< Position of REPORTRDY field. */
#define QDEC_INTENSET_REPORTRDY_Msk (0x1UL << QDEC_INTENSET_REPORTRDY_Pos) /*!< Bit mask of REPORTRDY field. */

/* Bit 0 : Enable interrupt on SAMPLERDY event. */
#define QDEC_INTENSET_SAMPLERDY_Pos (0U) /*!< Position of SAMPLERDY field. */
#define QDEC_INTENSET_SAMPLERDY_Msk (0x1UL << QDEC_INTENSET_SAMPLERDY_Pos) /*!< Bit mask of SAMPLERDY field. */

/* Register: QDEC_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 2 : Disable interrupt on ACCOF event. */
#define QDEC_INTENCLR_ACCOF_Pos (2U) /*!< Position of ACCOF field. */
#define QDEC_INTENCLR_ACCOF_Msk (0x1UL << QDEC_INTENCLR_ACCOF_Pos) /*!< Bit mask of ACCOF field. */

/* Bit 1 : Disable interrupt on REPORTRDY event. */
#define QDEC_INTENCLR_REPORTRDY_Pos (1U) /*!< Position of REPORTRDY field. */
#define QDEC_INTENCLR_REPORTRDY_Msk (0x1UL << QDEC_INTENCLR_REPORTRDY_Pos) /*!< Bit mask of REPORTRDY field. */

/* Bit 0 : Disable interrupt on SAMPLERDY event. */
#define QDEC_INTENCLR_SAMPLERDY_Pos (0U) /*!< Position of SAMPLERDY field. */
#define QDEC_INTENCLR_SAMPLERDY_Msk (0x1UL << QDEC_INTENCLR_SAMPLERDY_Pos) /*!< Bit mask of SAMPLERDY field. */

/* Register: QDEC_ENABLE */
/* Description: Enable the QDEC. */

/* Bit 0 : Enable or disable QDEC. */
#define QDEC_ENABLE_ENABLE_Pos (0U) /*!< Position of ENABLE field. */
#define QDEC_ENABLE_ENABLE_Msk (0x1UL << QDEC_ENABLE_ENABLE_Pos) /*!< Bit mask of ENABLE field. */
#define QDEC_ENABLE_ENABLE_Disabled (0UL) /*!< Disabled QDEC. */
#define QDEC_ENABLE_ENABLE_Enabled (1UL) /*!< Enable QDEC. */

/* Register: QDEC_LEDPOL */
/* Description: LED output pin polarity. */

/* Bit 0 : LED output pin polarity. */
#define QDEC_LEDPOL_LEDPOL_Pos (0U) /*!< Position of LEDPOL field. */
#define QDEC_LEDPOL_LEDPOL_Msk (0x1UL << QDEC_LEDPOL_LEDPOL_Pos) /*!< Bit mask of LEDPOL field. */
#define QDEC_LEDPOL_LEDPOL_HighOn (0UL) /*!< Active high. */
#define QDEC_LEDPOL_LEDPOL_LowOn (1UL) /*!< Active low. */

/* Register: QDEC_SAMPLEPER */
/* Description: Sample period. */

/* Bits 2..0 : Sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_Pos (0U) /*!< Position of SAMPLEPER field. */
#define QDEC_SAMPLEPER_SAMPLEPER_Msk (0x7UL << QDEC_SAMPLEPER_SAMPLEPER_Pos) /*!< Bit mask of SAMPLEPER field. */
#define QDEC_SAMPLEPER_SAMPLEPER_128us (0x00UL) /*!< 128us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_256us (0x01UL) /*!< 256us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_512us (0x02UL) /*!< 512us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_1024us (0x03UL) /*!< 1024us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_2048us (0x04UL) /*!< 2048us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_4096us (0x05UL) /*!< 4096us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_8192us (0x06UL) /*!< 8192us sample period. */
#define QDEC_SAMPLEPER_SAMPLEPER_16384us (0x07UL) /*!< 16384us sample period. */

/* Register: QDEC_SAMPLE */
/* Description: Motion sample value. */

/* Bits 31..0 : Last sample taken in compliment to 2. */
#define QDEC_SAMPLE_SAMPLE_Pos (0U) /*!< Position of SAMPLE field. */
#define QDEC_SAMPLE_SAMPLE_Msk (0xFFFFFFFFUL << QDEC_SAMPLE_SAMPLE_Pos) /*!< Bit mask of SAMPLE field. */

/* Register: QDEC_REPORTPER */
/* Description: Number of samples to generate an EVENT_REPORTRDY. */

/* Bits 2..0 : Number of samples to generate an EVENT_REPORTRDY. */
#define QDEC_REPORTPER_REPORTPER_Pos (0U) /*!< Position of REPORTPER field. */
#define QDEC_REPORTPER_REPORTPER_Msk (0x7UL << QDEC_REPORTPER_REPORTPER_Pos) /*!< Bit mask of REPORTPER field. */
#define QDEC_REPORTPER_REPORTPER_10Smpl (0x00UL) /*!< 10 samples per report. */
#define QDEC_REPORTPER_REPORTPER_40Smpl (0x01UL) /*!< 40 samples per report. */
#define QDEC_REPORTPER_REPORTPER_80Smpl (0x02UL) /*!< 80 samples per report. */
#define QDEC_REPORTPER_REPORTPER_120Smpl (0x03UL) /*!< 120 samples per report. */
#define QDEC_REPORTPER_REPORTPER_160Smpl (0x04UL) /*!< 160 samples per report. */
#define QDEC_REPORTPER_REPORTPER_200Smpl (0x05UL) /*!< 200 samples per report. */
#define QDEC_REPORTPER_REPORTPER_240Smpl (0x06UL) /*!< 240 samples per report. */
#define QDEC_REPORTPER_REPORTPER_280Smpl (0x07UL) /*!< 280 samples per report. */

/* Register: QDEC_DBFEN */
/* Description: Enable debouncer input filters. */

/* Bit 0 : Enable debounce input filters. */
#define QDEC_DBFEN_DBFEN_Pos (0U) /*!< Position of DBFEN field. */
#define QDEC_DBFEN_DBFEN_Msk (0x1UL << QDEC_DBFEN_DBFEN_Pos) /*!< Bit mask of DBFEN field. */
#define QDEC_DBFEN_DBFEN_Disabled (0UL) /*!< Debounce input filters disabled. */
#define QDEC_DBFEN_DBFEN_Enabled (1UL) /*!< Debounce input filters enabled. */

/* Register: QDEC_LEDPRE */
/* Description: Time LED is switched ON before the sample. */

/* Bits 7..0 : Period in us the LED in swtiched on prior to sampling. */
#define QDEC_LEDPRE_LEDPRE_Pos (0U) /*!< Position of LEDPRE field. */
#define QDEC_LEDPRE_LEDPRE_Msk (0xFFUL << QDEC_LEDPRE_LEDPRE_Pos) /*!< Bit mask of LEDPRE field. */

/* Register: QDEC_ACCDBL */
/* Description: Accumulated double (error) transitions register. */

/* Bits 3..0 : Accumulated double (error) transitions. */
#define QDEC_ACCDBL_ACCDBL_Pos (0U) /*!< Position of ACCDBL field. */
#define QDEC_ACCDBL_ACCDBL_Msk (0xFUL << QDEC_ACCDBL_ACCDBL_Pos) /*!< Bit mask of ACCDBL field. */

/* Register: QDEC_ACCDBLREAD */
/* Description: Snapshot of ACCDBL register. Value generated by the TASKS_READCLEACC task. */

/* Bits 3..0 : Snapshot of accumulated double (error) transitions. */
#define QDEC_ACCDBLREAD_ACCDBLREAD_Pos (0U) /*!< Position of ACCDBLREAD field. */
#define QDEC_ACCDBLREAD_ACCDBLREAD_Msk (0xFUL << QDEC_ACCDBLREAD_ACCDBLREAD_Pos) /*!< Bit mask of ACCDBLREAD field. */


/* Peripheral: RADIO */
/* Description: The radio */

/* Register: RADIO_SHORTS */
/* Description: Shortcut for the radio. */

/* Bit 4 : Shortcut between ADDRESS event and RSSISTART task. */
#define RADIO_SHORTS_ADDRESS_RSSISTART_Pos (4U) /*!< Position of ADDRESS_RSSISTART field. */
#define RADIO_SHORTS_ADDRESS_RSSISTART_Msk (0x1UL << RADIO_SHORTS_ADDRESS_RSSISTART_Pos) /*!< Bit mask of ADDRESS_RSSISTART field. */
#define RADIO_SHORTS_ADDRESS_RSSISTART_Disabled (0UL) /*!< Shortcut disabled. */
#define RADIO_SHORTS_ADDRESS_RSSISTART_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 3 : Shortcut between DISABLED event and RXEN task. */
#define RADIO_SHORTS_DISABLED_RXEN_Pos (3U) /*!< Position of DISABLED_RXEN field. */
#define RADIO_SHORTS_DISABLED_RXEN_Msk (0x1UL << RADIO_SHORTS_DISABLED_RXEN_Pos) /*!< Bit mask of DISABLED_RXEN field. */
#define RADIO_SHORTS_DISABLED_RXEN_Disabled (0UL) /*!< Shortcut disabled. */
#define RADIO_SHORTS_DISABLED_RXEN_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 2 : Shortcut between DISABLED event and TXEN task.  */
#define RADIO_SHORTS_DISABLED_TXEN_Pos (2U) /*!< Position of DISABLED_TXEN field. */
#define RADIO_SHORTS_DISABLED_TXEN_Msk (0x1UL << RADIO_SHORTS_DISABLED_TXEN_Pos) /*!< Bit mask of DISABLED_TXEN field. */
#define RADIO_SHORTS_DISABLED_TXEN_Disabled (0UL) /*!< Shortcut disabled. */
#define RADIO_SHORTS_DISABLED_TXEN_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 1 : Shortcut between END event and DISABLE task. */
#define RADIO_SHORTS_END_DISABLE_Pos (1U) /*!< Position of END_DISABLE field. */
#define RADIO_SHORTS_END_DISABLE_Msk (0x1UL << RADIO_SHORTS_END_DISABLE_Pos) /*!< Bit mask of END_DISABLE field. */
#define RADIO_SHORTS_END_DISABLE_Disabled (0UL) /*!< Shortcut disabled. */
#define RADIO_SHORTS_END_DISABLE_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 0 : Shortcut between READY event and START task. */
#define RADIO_SHORTS_READY_START_Pos (0U) /*!< Position of READY_START field. */
#define RADIO_SHORTS_READY_START_Msk (0x1UL << RADIO_SHORTS_READY_START_Pos) /*!< Bit mask of READY_START field. */
#define RADIO_SHORTS_READY_START_Disabled (0UL) /*!< Shortcut disabled. */
#define RADIO_SHORTS_READY_START_Enabled (1UL) /*!< Shortcut enabled. */

/* Register: RADIO_INTEN */
/* Description: Interrupt enable register. */

/* Bit 7 : Interrupt enable on RSSIEND event. */
#define RADIO_INTEN_RSSIEND_Pos (7U) /*!< Position of RSSIEND field. */
#define RADIO_INTEN_RSSIEND_Msk (0x1UL << RADIO_INTEN_RSSIEND_Pos) /*!< Bit mask of RSSIEND field. */
#define RADIO_INTEN_RSSIEND_Disabled (0UL) /*!< Interrupt disabled. */
#define RADIO_INTEN_RSSIEND_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 4 : Interrupt enable on DISABLED event. */
#define RADIO_INTEN_DISABLED_Pos (4U) /*!< Position of DISABLED field. */
#define RADIO_INTEN_DISABLED_Msk (0x1UL << RADIO_INTEN_DISABLED_Pos) /*!< Bit mask of DISABLED field. */
#define RADIO_INTEN_DISABLED_Disabled (0UL) /*!< Interrupt disabled. */
#define RADIO_INTEN_DISABLED_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 3 : Interrupt enable on END event. */
#define RADIO_INTEN_END_Pos (3U) /*!< Position of END field. */
#define RADIO_INTEN_END_Msk (0x1UL << RADIO_INTEN_END_Pos) /*!< Bit mask of END field. */
#define RADIO_INTEN_END_Disabled (0UL) /*!< Interrupt disabled. */
#define RADIO_INTEN_END_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 2 : Interrupt enable on PAYLOAD event. */
#define RADIO_INTEN_PAYLOAD_Pos (2U) /*!< Position of PAYLOAD field. */
#define RADIO_INTEN_PAYLOAD_Msk (0x1UL << RADIO_INTEN_PAYLOAD_Pos) /*!< Bit mask of PAYLOAD field. */
#define RADIO_INTEN_PAYLOAD_Disabled (0UL) /*!< Interrupt disabled. */
#define RADIO_INTEN_PAYLOAD_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 1 : Interrupt enable on ADDRESS event. */
#define RADIO_INTEN_ADDRESS_Pos (1U) /*!< Position of ADDRESS field. */
#define RADIO_INTEN_ADDRESS_Msk (0x1UL << RADIO_INTEN_ADDRESS_Pos) /*!< Bit mask of ADDRESS field. */
#define RADIO_INTEN_ADDRESS_Disabled (0UL) /*!< Interrupt disabled. */
#define RADIO_INTEN_ADDRESS_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 0 : Interrupt enable on READY event. */
#define RADIO_INTEN_READY_Pos (0U) /*!< Position of READY field. */
#define RADIO_INTEN_READY_Msk (0x1UL << RADIO_INTEN_READY_Pos) /*!< Bit mask of READY field. */
#define RADIO_INTEN_READY_Disabled (0UL) /*!< Interrupt disabled. */
#define RADIO_INTEN_READY_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: RADIO_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 7 : Enable interrupt on RSSIEND event. */
#define RADIO_INTENSET_RSSIEND_Pos (7U) /*!< Position of RSSIEND field. */
#define RADIO_INTENSET_RSSIEND_Msk (0x1UL << RADIO_INTENSET_RSSIEND_Pos) /*!< Bit mask of RSSIEND field. */

/* Bit 4 : Enable interrupt on DISABLED event. */
#define RADIO_INTENSET_DISABLED_Pos (4U) /*!< Position of DISABLED field. */
#define RADIO_INTENSET_DISABLED_Msk (0x1UL << RADIO_INTENSET_DISABLED_Pos) /*!< Bit mask of DISABLED field. */

/* Bit 3 : Enable interrupt on END event. */
#define RADIO_INTENSET_END_Pos (3U) /*!< Position of END field. */
#define RADIO_INTENSET_END_Msk (0x1UL << RADIO_INTENSET_END_Pos) /*!< Bit mask of END field. */

/* Bit 2 : Enable interrupt on PAYLOAD event. */
#define RADIO_INTENSET_PAYLOAD_Pos (2U) /*!< Position of PAYLOAD field. */
#define RADIO_INTENSET_PAYLOAD_Msk (0x1UL << RADIO_INTENSET_PAYLOAD_Pos) /*!< Bit mask of PAYLOAD field. */

/* Bit 1 : Enable interrupt on ADDRESS event. */
#define RADIO_INTENSET_ADDRESS_Pos (1U) /*!< Position of ADDRESS field. */
#define RADIO_INTENSET_ADDRESS_Msk (0x1UL << RADIO_INTENSET_ADDRESS_Pos) /*!< Bit mask of ADDRESS field. */

/* Bit 0 : Enable interrupt on READY event. */
#define RADIO_INTENSET_READY_Pos (0U) /*!< Position of READY field. */
#define RADIO_INTENSET_READY_Msk (0x1UL << RADIO_INTENSET_READY_Pos) /*!< Bit mask of READY field. */

/* Register: RADIO_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 7 : Disable interrupt on RSSIEND event. */
#define RADIO_INTENCLR_RSSIEND_Pos (7U) /*!< Position of RSSIEND field. */
#define RADIO_INTENCLR_RSSIEND_Msk (0x1UL << RADIO_INTENCLR_RSSIEND_Pos) /*!< Bit mask of RSSIEND field. */

/* Bit 4 : Disable interrupt on DISABLED event. */
#define RADIO_INTENCLR_DISABLED_Pos (4U) /*!< Position of DISABLED field. */
#define RADIO_INTENCLR_DISABLED_Msk (0x1UL << RADIO_INTENCLR_DISABLED_Pos) /*!< Bit mask of DISABLED field. */

/* Bit 3 : Disable interrupt on END event. */
#define RADIO_INTENCLR_END_Pos (3U) /*!< Position of END field. */
#define RADIO_INTENCLR_END_Msk (0x1UL << RADIO_INTENCLR_END_Pos) /*!< Bit mask of END field. */

/* Bit 2 : Disable interrupt on PAYLOAD event. */
#define RADIO_INTENCLR_PAYLOAD_Pos (2U) /*!< Position of PAYLOAD field. */
#define RADIO_INTENCLR_PAYLOAD_Msk (0x1UL << RADIO_INTENCLR_PAYLOAD_Pos) /*!< Bit mask of PAYLOAD field. */

/* Bit 1 : Disable interrupt on ADDRESS event. */
#define RADIO_INTENCLR_ADDRESS_Pos (1U) /*!< Position of ADDRESS field. */
#define RADIO_INTENCLR_ADDRESS_Msk (0x1UL << RADIO_INTENCLR_ADDRESS_Pos) /*!< Bit mask of ADDRESS field. */

/* Bit 0 : Disable interrupt on READY event. */
#define RADIO_INTENCLR_READY_Pos (0U) /*!< Position of READY field. */
#define RADIO_INTENCLR_READY_Msk (0x1UL << RADIO_INTENCLR_READY_Pos) /*!< Bit mask of READY field. */

/* Register: RADIO_CRCSTATUS */
/* Description: CRC status register. */

/* Bit 0 : CRC status of packet received. */
#define RADIO_CRCSTATUS_CRCSTATUS_Pos (0U) /*!< Position of CRCSTATUS field. */
#define RADIO_CRCSTATUS_CRCSTATUS_Msk (0x1UL << RADIO_CRCSTATUS_CRCSTATUS_Pos) /*!< Bit mask of CRCSTATUS field. */
#define RADIO_CRCSTATUS_CRCSTATUS_CRCError (0UL) /*!< Packet received with CRC error. */
#define RADIO_CRCSTATUS_CRCSTATUS_CRCOk (1UL) /*!< Packet received with CRC ok. */

/* Register: RADIO_RXMATCH */
/* Description: Received address. */

/* Bits 2..0 : Logical address in which previous packet was received. */
#define RADIO_RXMATCH_RXMATCH_Pos (0U) /*!< Position of RXMATCH field. */
#define RADIO_RXMATCH_RXMATCH_Msk (0x7UL << RADIO_RXMATCH_RXMATCH_Pos) /*!< Bit mask of RXMATCH field. */

/* Register: RADIO_RXCRC */
/* Description: Received CRC. */

/* Bits 23..0 : CRC field of previously received packet. */
#define RADIO_RXCRC_RXCRC_Pos (0U) /*!< Position of RXCRC field. */
#define RADIO_RXCRC_RXCRC_Msk (0xFFFFFFUL << RADIO_RXCRC_RXCRC_Pos) /*!< Bit mask of RXCRC field. */

/* Register: RADIO_FREQUENCY */
/* Description: Frequency. */

/* Bits 6..0 : Radio channel frequency offset in MHz: RF Frequency = 2400 + FREQUENCY (MHz). Decision point: TXEN or RXEN task.  */
#define RADIO_FREQUENCY_FREQUENCY_Pos (0U) /*!< Position of FREQUENCY field. */
#define RADIO_FREQUENCY_FREQUENCY_Msk (0x7FUL << RADIO_FREQUENCY_FREQUENCY_Pos) /*!< Bit mask of FREQUENCY field. */

/* Register: RADIO_TXPOWER */
/* Description: Output power. */

/* Bits 7..0 : Radio output power. Decision point: TXEN task. */
#define RADIO_TXPOWER_TXPOWER_Pos (0U) /*!< Position of TXPOWER field. */
#define RADIO_TXPOWER_TXPOWER_Msk (0xFFUL << RADIO_TXPOWER_TXPOWER_Pos) /*!< Bit mask of TXPOWER field. */
#define RADIO_TXPOWER_TXPOWER_Pos4dBm (0x04UL) /*!< +4dBm. */
#define RADIO_TXPOWER_TXPOWER_0dBm (0x00UL) /*!< 0dBm. */
#define RADIO_TXPOWER_TXPOWER_Neg4dBm (0xFCUL) /*!< -4dBm. */
#define RADIO_TXPOWER_TXPOWER_Neg8dBm (0xF8UL) /*!< -8dBm. */
#define RADIO_TXPOWER_TXPOWER_Neg12dBm (0xF4UL) /*!< -12dBm. */
#define RADIO_TXPOWER_TXPOWER_Neg16dBm (0xF0UL) /*!< -16dBm. */
#define RADIO_TXPOWER_TXPOWER_Neg20dBm (0xECUL) /*!< -20dBm. */
#define RADIO_TXPOWER_TXPOWER_Neg40dBm (0xD8UL) /*!< -40dBm. */

/* Register: RADIO_MODE */
/* Description: Data rate and modulation. */

/* Bits 1..0 : Radio data rate and modulation setting. Decision point: TXEN or RXEN task. */
#define RADIO_MODE_MODE_Pos (0U) /*!< Position of MODE field. */
#define RADIO_MODE_MODE_Msk (0x3UL << RADIO_MODE_MODE_Pos) /*!< Bit mask of MODE field. */
#define RADIO_MODE_MODE_Nrf_1Mbit (0x00UL) /*!< 1Mb/s Nordic propietary radio mode. */
#define RADIO_MODE_MODE_Nrf_2Mbit (0x01UL) /*!< 2Mb/s Nordic propietary radio mode. */
#define RADIO_MODE_MODE_Nrf_250Kbit (0x02UL) /*!< 250Kb/s Nordic propietary radio mode. */
#define RADIO_MODE_MODE_Ble_1Mbit (0x03UL) /*!< 1Mb/s Bluetooth Low Energy */

/* Register: RADIO_PCNF0 */
/* Description: Packet configuration 0. */

/* Bits 19..16 : Length of S1 field in number of bits. Decision point: START task. */
#define RADIO_PCNF0_S1LEN_Pos (16U) /*!< Position of S1LEN field. */
#define RADIO_PCNF0_S1LEN_Msk (0xFUL << RADIO_PCNF0_S1LEN_Pos) /*!< Bit mask of S1LEN field. */

/* Bit 8 : Length of S0 field in number of bytes. Decision point: START task. */
#define RADIO_PCNF0_S0LEN_Pos (8U) /*!< Position of S0LEN field. */
#define RADIO_PCNF0_S0LEN_Msk (0x1UL << RADIO_PCNF0_S0LEN_Pos) /*!< Bit mask of S0LEN field. */

/* Bits 3..0 : Length of length field in number of bits. Decision point: START task. */
#define RADIO_PCNF0_LFLEN_Pos (0U) /*!< Position of LFLEN field. */
#define RADIO_PCNF0_LFLEN_Msk (0xFUL << RADIO_PCNF0_LFLEN_Pos) /*!< Bit mask of LFLEN field. */

/* Register: RADIO_PCNF1 */
/* Description: Packet configuration 1. */

/* Bit 25 : Packet whitening enable. */
#define RADIO_PCNF1_WHITEEN_Pos (25U) /*!< Position of WHITEEN field. */
#define RADIO_PCNF1_WHITEEN_Msk (0x1UL << RADIO_PCNF1_WHITEEN_Pos) /*!< Bit mask of WHITEEN field. */
#define RADIO_PCNF1_WHITEEN_Disabled (0UL) /*!< Whitening disabled. */
#define RADIO_PCNF1_WHITEEN_Enabled (1UL) /*!< Whitening enabled. */

/* Bit 24 : On air endianess of packet length field. Decision point: START task. */
#define RADIO_PCNF1_ENDIAN_Pos (24U) /*!< Position of ENDIAN field. */
#define RADIO_PCNF1_ENDIAN_Msk (0x1UL << RADIO_PCNF1_ENDIAN_Pos) /*!< Bit mask of ENDIAN field. */
#define RADIO_PCNF1_ENDIAN_Little (0UL) /*!< Least significant bit on air first */
#define RADIO_PCNF1_ENDIAN_Big (1UL) /*!< Most significant bit on air first */

/* Bits 18..16 : Base address length in number of bytes. Decision point: START task. */
#define RADIO_PCNF1_BALEN_Pos (16U) /*!< Position of BALEN field. */
#define RADIO_PCNF1_BALEN_Msk (0x7UL << RADIO_PCNF1_BALEN_Pos) /*!< Bit mask of BALEN field. */

/* Bits 15..8 : Static length in number of bytes. Decision point: START task. */
#define RADIO_PCNF1_STATLEN_Pos (8U) /*!< Position of STATLEN field. */
#define RADIO_PCNF1_STATLEN_Msk (0xFFUL << RADIO_PCNF1_STATLEN_Pos) /*!< Bit mask of STATLEN field. */

/* Bits 7..0 : Maximum length of packet payload in number of bytes. Decision point: START task. */
#define RADIO_PCNF1_MAXLEN_Pos (0U) /*!< Position of MAXLEN field. */
#define RADIO_PCNF1_MAXLEN_Msk (0xFFUL << RADIO_PCNF1_MAXLEN_Pos) /*!< Bit mask of MAXLEN field. */

/* Register: RADIO_PREFIX0 */
/* Description: Prefixes bytes for logical addresses 0 to 3. */

/* Bits 31..24 : Address prefix 3. Decision point: START task. */
#define RADIO_PREFIX0_AP3_Pos (24U) /*!< Position of AP3 field. */
#define RADIO_PREFIX0_AP3_Msk (0xFFUL << RADIO_PREFIX0_AP3_Pos) /*!< Bit mask of AP3 field. */

/* Bits 23..16 : Address prefix 2. Decision point: START task. */
#define RADIO_PREFIX0_AP2_Pos (16U) /*!< Position of AP2 field. */
#define RADIO_PREFIX0_AP2_Msk (0xFFUL << RADIO_PREFIX0_AP2_Pos) /*!< Bit mask of AP2 field. */

/* Bits 15..8 : Address prefix 1. Decision point: START task. */
#define RADIO_PREFIX0_AP1_Pos (8U) /*!< Position of AP1 field. */
#define RADIO_PREFIX0_AP1_Msk (0xFFUL << RADIO_PREFIX0_AP1_Pos) /*!< Bit mask of AP1 field. */

/* Bits 7..0 : Address prefix 0. Decision point: START task. */
#define RADIO_PREFIX0_AP0_Pos (0U) /*!< Position of AP0 field. */
#define RADIO_PREFIX0_AP0_Msk (0xFFUL << RADIO_PREFIX0_AP0_Pos) /*!< Bit mask of AP0 field. */

/* Register: RADIO_PREFIX1 */
/* Description: Prefixes bytes for logical addresses 4 to 7. */

/* Bits 31..24 : Address prefix 7. Decision point: START task. */
#define RADIO_PREFIX1_AP7_Pos (24U) /*!< Position of AP7 field. */
#define RADIO_PREFIX1_AP7_Msk (0xFFUL << RADIO_PREFIX1_AP7_Pos) /*!< Bit mask of AP7 field. */

/* Bits 23..16 : Address prefix 6. Decision point: START task. */
#define RADIO_PREFIX1_AP6_Pos (16U) /*!< Position of AP6 field. */
#define RADIO_PREFIX1_AP6_Msk (0xFFUL << RADIO_PREFIX1_AP6_Pos) /*!< Bit mask of AP6 field. */

/* Bits 15..8 : Address prefix 5. Decision point: START task. */
#define RADIO_PREFIX1_AP5_Pos (8U) /*!< Position of AP5 field. */
#define RADIO_PREFIX1_AP5_Msk (0xFFUL << RADIO_PREFIX1_AP5_Pos) /*!< Bit mask of AP5 field. */

/* Bits 7..0 : Address prefix 4. Decision point: START task. */
#define RADIO_PREFIX1_AP4_Pos (0U) /*!< Position of AP4 field. */
#define RADIO_PREFIX1_AP4_Msk (0xFFUL << RADIO_PREFIX1_AP4_Pos) /*!< Bit mask of AP4 field. */

/* Register: RADIO_TXADDRESS */
/* Description: Transmit address select. */

/* Bits 2..0 : Logical address to be used when transmitting a packet. Decision point: START task. */
#define RADIO_TXADDRESS_TXADDRESS_Pos (0U) /*!< Position of TXADDRESS field. */
#define RADIO_TXADDRESS_TXADDRESS_Msk (0x7UL << RADIO_TXADDRESS_TXADDRESS_Pos) /*!< Bit mask of TXADDRESS field. */

/* Register: RADIO_RXADDRESSES */
/* Description: Receive address select. */

/* Bit 7 : Enable reception on logical address 7. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR7_Pos (7U) /*!< Position of ADR7 field. */
#define RADIO_RXADDRESSES_ADR7_Msk (0x1UL << RADIO_RXADDRESSES_ADR7_Pos) /*!< Bit mask of ADR7 field. */
#define RADIO_RXADDRESSES_ADR7_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR7_Enabled (1UL) /*!< Reception enabled. */

/* Bit 6 : Enable reception on logical address 6. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR6_Pos (6U) /*!< Position of ADR6 field. */
#define RADIO_RXADDRESSES_ADR6_Msk (0x1UL << RADIO_RXADDRESSES_ADR6_Pos) /*!< Bit mask of ADR6 field. */
#define RADIO_RXADDRESSES_ADR6_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR6_Enabled (1UL) /*!< Reception enabled. */

/* Bit 5 : Enable reception on logical address 5. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR5_Pos (5U) /*!< Position of ADR5 field. */
#define RADIO_RXADDRESSES_ADR5_Msk (0x1UL << RADIO_RXADDRESSES_ADR5_Pos) /*!< Bit mask of ADR5 field. */
#define RADIO_RXADDRESSES_ADR5_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR5_Enabled (1UL) /*!< Reception enabled. */

/* Bit 4 : Enable reception on logical address 4. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR4_Pos (4U) /*!< Position of ADR4 field. */
#define RADIO_RXADDRESSES_ADR4_Msk (0x1UL << RADIO_RXADDRESSES_ADR4_Pos) /*!< Bit mask of ADR4 field. */
#define RADIO_RXADDRESSES_ADR4_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR4_Enabled (1UL) /*!< Reception enabled. */

/* Bit 3 : Enable reception on logical address 3. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR3_Pos (3U) /*!< Position of ADR3 field. */
#define RADIO_RXADDRESSES_ADR3_Msk (0x1UL << RADIO_RXADDRESSES_ADR3_Pos) /*!< Bit mask of ADR3 field. */
#define RADIO_RXADDRESSES_ADR3_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR3_Enabled (1UL) /*!< Reception enabled. */

/* Bit 2 : Enable reception on logical address 2. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR2_Pos (2U) /*!< Position of ADR2 field. */
#define RADIO_RXADDRESSES_ADR2_Msk (0x1UL << RADIO_RXADDRESSES_ADR2_Pos) /*!< Bit mask of ADR2 field. */
#define RADIO_RXADDRESSES_ADR2_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR2_Enabled (1UL) /*!< Reception enabled. */

/* Bit 1 : Enable reception on logical address 1. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR1_Pos (1U) /*!< Position of ADR1 field. */
#define RADIO_RXADDRESSES_ADR1_Msk (0x1UL << RADIO_RXADDRESSES_ADR1_Pos) /*!< Bit mask of ADR1 field. */
#define RADIO_RXADDRESSES_ADR1_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR1_Enabled (1UL) /*!< Reception enabled. */

/* Bit 0 : Enable reception on logical address 0. Decision point: START task. */
#define RADIO_RXADDRESSES_ADR0_Pos (0U) /*!< Position of ADR0 field. */
#define RADIO_RXADDRESSES_ADR0_Msk (0x1UL << RADIO_RXADDRESSES_ADR0_Pos) /*!< Bit mask of ADR0 field. */
#define RADIO_RXADDRESSES_ADR0_Disabled (0UL) /*!< Reception disabled. */
#define RADIO_RXADDRESSES_ADR0_Enabled (1UL) /*!< Reception enabled. */

/* Register: RADIO_CRCCNF */
/* Description: CRC configuration. */

/* Bit 8 : Leave packet address field out of the CRC calculation. Decision point: START task. */
#define RADIO_CRCCNF_SKIP_ADR_Pos (8U) /*!< Position of SKIP_ADR field. */
#define RADIO_CRCCNF_SKIP_ADR_Msk (0x1UL << RADIO_CRCCNF_SKIP_ADR_Pos) /*!< Bit mask of SKIP_ADR field. */
#define RADIO_CRCCNF_SKIP_ADR_Include (0UL) /*!< Include packet address in CRC calculation. */
#define RADIO_CRCCNF_SKIP_ADR_Skip (1UL) /*!< Packet address is skipped in CRC calculation. The CRC calculation will start at the first byte after the address. */

/* Bits 1..0 : CRC length. Decision point: START task. */
#define RADIO_CRCCNF_LEN_Pos (0U) /*!< Position of LEN field. */
#define RADIO_CRCCNF_LEN_Msk (0x3UL << RADIO_CRCCNF_LEN_Pos) /*!< Bit mask of LEN field. */
#define RADIO_CRCCNF_LEN_Disabled (0UL) /*!< CRC calculation disabled. */
#define RADIO_CRCCNF_LEN_One (1UL) /*!< One byte long CRC. */
#define RADIO_CRCCNF_LEN_Two (2UL) /*!< Two bytes long CRC. */
#define RADIO_CRCCNF_LEN_Three (3UL) /*!< Three bytes long CRC. */

/* Register: RADIO_CRCPOLY */
/* Description: CRC polynomial. */

/* Bits 23..1 : CRC polynomial. Decision point: START task. */
#define RADIO_CRCPOLY_CRCPOLY_Pos (1U) /*!< Position of CRCPOLY field. */
#define RADIO_CRCPOLY_CRCPOLY_Msk (0x7FFFFFUL << RADIO_CRCPOLY_CRCPOLY_Pos) /*!< Bit mask of CRCPOLY field. */

/* Register: RADIO_CRCINIT */
/* Description: CRC initial value. */

/* Bits 23..0 : Initial value for CRC calculation. Decision point: START task. */
#define RADIO_CRCINIT_CRCINIT_Pos (0U) /*!< Position of CRCINIT field. */
#define RADIO_CRCINIT_CRCINIT_Msk (0xFFFFFFUL << RADIO_CRCINIT_CRCINIT_Pos) /*!< Bit mask of CRCINIT field. */

/* Register: RADIO_TEST */
/* Description: Test features enable register. */

/* Bit 1 : PLL lock. Decision point: TXEN or RXEN task. */
#define RADIO_TEST_PLL_LOCK_Pos (1U) /*!< Position of PLL_LOCK field. */
#define RADIO_TEST_PLL_LOCK_Msk (0x1UL << RADIO_TEST_PLL_LOCK_Pos) /*!< Bit mask of PLL_LOCK field. */
#define RADIO_TEST_PLL_LOCK_Disabled (0UL) /*!< PLL lock disabled. */
#define RADIO_TEST_PLL_LOCK_Enabled (1UL) /*!< PLL lock enabled. */

/* Bit 0 : Constant carrier. Decision point: TXEN task. */
#define RADIO_TEST_CONST_CARRIER_Pos (0U) /*!< Position of CONST_CARRIER field. */
#define RADIO_TEST_CONST_CARRIER_Msk (0x1UL << RADIO_TEST_CONST_CARRIER_Pos) /*!< Bit mask of CONST_CARRIER field. */
#define RADIO_TEST_CONST_CARRIER_Disabled (0UL) /*!< Constant carrier disabled. */
#define RADIO_TEST_CONST_CARRIER_Enabled (1UL) /*!< Constant carrier enabled. */

/* Register: RADIO_RSSISAMPLE */
/* Description: RSSI sample. */

/* Bits 6..0 : RSSI sample result. The result is read as a positive value so that RXPOWER = -RSSISAMPLE dBm */
#define RADIO_RSSISAMPLE_RSSISAMPLE_Pos (0U) /*!< Position of RSSISAMPLE field. */
#define RADIO_RSSISAMPLE_RSSISAMPLE_Msk (0x7FUL << RADIO_RSSISAMPLE_RSSISAMPLE_Pos) /*!< Bit mask of RSSISAMPLE field. */

/* Register: RADIO_STATE */
/* Description: Current radio state. */

/* Bits 2..0 : Current radio state. */
#define RADIO_STATE_STATE_Pos (0U) /*!< Position of STATE field. */
#define RADIO_STATE_STATE_Msk (0x7UL << RADIO_STATE_STATE_Pos) /*!< Bit mask of STATE field. */
#define RADIO_STATE_STATE_Disabled (0x00UL) /*!< Radio is in the disabled state. */
#define RADIO_STATE_STATE_RxEnabling (0x01UL) /*!< Radio is in the RxEnabling state. */
#define RADIO_STATE_STATE_RxReady (0x02UL) /*!< Radio is in the RxReady state. */
#define RADIO_STATE_STATE_RxStarted (0x03UL) /*!< Radio is in the RxStarted state. */
#define RADIO_STATE_STATE_TxEnabling (0x04UL) /*!< Radio is in the TxEnabling state. */
#define RADIO_STATE_STATE_TxReady (0x05UL) /*!< Radio is in the TxReady state. */
#define RADIO_STATE_STATE_TxStarted (0x06UL) /*!< Radio is in the TxStarted state. */

/* Register: RADIO_DATAWHITEIV */
/* Description: Data whitening initial value. */

/* Bits 5..0 : Data whitening initial value. Bit 0 corresponds to Position 0 of the LSFR, Bit 1 to position 5... Decision point: TXEN or RXEN task. */
#define RADIO_DATAWHITEIV_DATAWHITEIV_Pos (0U) /*!< Position of DATAWHITEIV field. */
#define RADIO_DATAWHITEIV_DATAWHITEIV_Msk (0x3FUL << RADIO_DATAWHITEIV_DATAWHITEIV_Pos) /*!< Bit mask of DATAWHITEIV field. */


/* Peripheral: RNG */
/* Description: Random Number Generator */

/* Register: RNG_INTEN */
/* Description: RNG interrupt enable register */

/* Bit 0 : Enable VALRDY interrupt. */
#define RNG_INTEN_VALRDY_Pos (0U) /*!< Position of VALRDY field. */
#define RNG_INTEN_VALRDY_Msk (0x1UL << RNG_INTEN_VALRDY_Pos) /*!< Bit mask of VALRDY field. */
#define RNG_INTEN_VALRDY_Disabled (0UL) /*!< Interrupt disabled. */
#define RNG_INTEN_VALRDY_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: RNG_INTENSET */
/* Description: RNG interrupt enable set register */

/* Bit 0 : Enable interrupt on VALRDY event. */
#define RNG_INTENSET_VALRDY_Pos (0U) /*!< Position of VALRDY field. */
#define RNG_INTENSET_VALRDY_Msk (0x1UL << RNG_INTENSET_VALRDY_Pos) /*!< Bit mask of VALRDY field. */

/* Register: RNG_INTENCLR */
/* Description: RNG interrupt enable clear register */

/* Bit 0 : Disable interrupt on VALRDY event. */
#define RNG_INTENCLR_VALRDY_Pos (0U) /*!< Position of VALRDY field. */
#define RNG_INTENCLR_VALRDY_Msk (0x1UL << RNG_INTENCLR_VALRDY_Pos) /*!< Bit mask of VALRDY field. */

/* Register: RNG_VALUE */
/* Description: RNG random number. */

/* Bits 7..0 : Generated random number. */
#define RNG_VALUE_VALUE_Pos (0U) /*!< Position of VALUE field. */
#define RNG_VALUE_VALUE_Msk (0xFFUL << RNG_VALUE_VALUE_Pos) /*!< Bit mask of VALUE field. */


/* Peripheral: RTC */
/* Description: Real time counter 0 */

/* Register: RTC_INTEN */
/* Description: Timer interrupt enable register. */

/* Bit 19 : Interrupt enable on COMPARE[3] */
#define RTC_INTEN_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define RTC_INTEN_COMPARE3_Msk (0x1UL << RTC_INTEN_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */
#define RTC_INTEN_COMPARE3_Disabled (0UL) /*!< Interrupt disabled. */
#define RTC_INTEN_COMPARE3_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 18 : Interrupt enable on COMPARE[2] */
#define RTC_INTEN_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define RTC_INTEN_COMPARE2_Msk (0x1UL << RTC_INTEN_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */
#define RTC_INTEN_COMPARE2_Disabled (0UL) /*!< Interrupt disabled. */
#define RTC_INTEN_COMPARE2_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 17 : Interrupt enable on COMPARE[1] */
#define RTC_INTEN_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define RTC_INTEN_COMPARE1_Msk (0x1UL << RTC_INTEN_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */
#define RTC_INTEN_COMPARE1_Disabled (0UL) /*!< Interrupt disabled. */
#define RTC_INTEN_COMPARE1_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 16 : Interrupt enable on COMPARE[0] */
#define RTC_INTEN_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define RTC_INTEN_COMPARE0_Msk (0x1UL << RTC_INTEN_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */
#define RTC_INTEN_COMPARE0_Disabled (0UL) /*!< Interrupt disabled. */
#define RTC_INTEN_COMPARE0_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 1 : Interrupt enable on OVRFLW. */
#define RTC_INTEN_OVRFLW_Pos (1U) /*!< Position of OVRFLW field. */
#define RTC_INTEN_OVRFLW_Msk (0x1UL << RTC_INTEN_OVRFLW_Pos) /*!< Bit mask of OVRFLW field. */
#define RTC_INTEN_OVRFLW_Disabled (0UL) /*!< Interrupt disabled. */
#define RTC_INTEN_OVRFLW_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 0 : Interrupt enable on TICK. */
#define RTC_INTEN_TICK_Pos (0U) /*!< Position of TICK field. */
#define RTC_INTEN_TICK_Msk (0x1UL << RTC_INTEN_TICK_Pos) /*!< Bit mask of TICK field. */
#define RTC_INTEN_TICK_Disabled (0UL) /*!< Interrupt disabled. */
#define RTC_INTEN_TICK_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: RTC_INTENSET */
/* Description: Timer interrupt enable set register. */

/* Bit 19 : Enable interrupt on COMPARE[3] event. */
#define RTC_INTENSET_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define RTC_INTENSET_COMPARE3_Msk (0x1UL << RTC_INTENSET_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */

/* Bit 18 : Enable interrupt on COMPARE[2] event. */
#define RTC_INTENSET_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define RTC_INTENSET_COMPARE2_Msk (0x1UL << RTC_INTENSET_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */

/* Bit 17 : Enable interrupt on COMPARE[1] event. */
#define RTC_INTENSET_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define RTC_INTENSET_COMPARE1_Msk (0x1UL << RTC_INTENSET_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */

/* Bit 16 : Enable interrupt on COMPARE[0] event. */
#define RTC_INTENSET_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define RTC_INTENSET_COMPARE0_Msk (0x1UL << RTC_INTENSET_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */

/* Bit 1 : Enable interrupt on OVRFLW event. */
#define RTC_INTENSET_OVRFLW_Pos (1U) /*!< Position of OVRFLW field. */
#define RTC_INTENSET_OVRFLW_Msk (0x1UL << RTC_INTENSET_OVRFLW_Pos) /*!< Bit mask of OVRFLW field. */

/* Bit 0 : Enable interrupt on TICK event. */
#define RTC_INTENSET_TICK_Pos (0U) /*!< Position of TICK field. */
#define RTC_INTENSET_TICK_Msk (0x1UL << RTC_INTENSET_TICK_Pos) /*!< Bit mask of TICK field. */

/* Register: RTC_INTENCLR */
/* Description: Timer interrupt enable clear register. */

/* Bit 19 : Disable interrupt on COMPARE[3] event. */
#define RTC_INTENCLR_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define RTC_INTENCLR_COMPARE3_Msk (0x1UL << RTC_INTENCLR_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */

/* Bit 18 : Disable interrupt on COMPARE[2] event. */
#define RTC_INTENCLR_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define RTC_INTENCLR_COMPARE2_Msk (0x1UL << RTC_INTENCLR_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */

/* Bit 17 : Disable interrupt on COMPARE[1] event. */
#define RTC_INTENCLR_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define RTC_INTENCLR_COMPARE1_Msk (0x1UL << RTC_INTENCLR_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */

/* Bit 16 : Disable interrupt on COMPARE[0] event. */
#define RTC_INTENCLR_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define RTC_INTENCLR_COMPARE0_Msk (0x1UL << RTC_INTENCLR_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */

/* Bit 1 : Disable interrupt on OVRFLW event. */
#define RTC_INTENCLR_OVRFLW_Pos (1U) /*!< Position of OVRFLW field. */
#define RTC_INTENCLR_OVRFLW_Msk (0x1UL << RTC_INTENCLR_OVRFLW_Pos) /*!< Bit mask of OVRFLW field. */

/* Bit 0 : Disable interrupt on TICK event. */
#define RTC_INTENCLR_TICK_Pos (0U) /*!< Position of TICK field. */
#define RTC_INTENCLR_TICK_Msk (0x1UL << RTC_INTENCLR_TICK_Pos) /*!< Bit mask of TICK field. */

/* Register: RTC_EVTEN */
/* Description: Configures event enable state for each RTC event. */

/* Bit 19 : COMPARE[3] event enable. */
#define RTC_EVTEN_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define RTC_EVTEN_COMPARE3_Msk (0x1UL << RTC_EVTEN_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */
#define RTC_EVTEN_COMPARE3_Disabled (0UL) /*!< Event disabled. */
#define RTC_EVTEN_COMPARE3_Enabled (1UL) /*!< Event enabled. */

/* Bit 18 : COMPARE[2] event enable. */
#define RTC_EVTEN_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define RTC_EVTEN_COMPARE2_Msk (0x1UL << RTC_EVTEN_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */
#define RTC_EVTEN_COMPARE2_Disabled (0UL) /*!< Event disabled. */
#define RTC_EVTEN_COMPARE2_Enabled (1UL) /*!< Event enabled. */

/* Bit 17 : COMPARE[1] event enable. */
#define RTC_EVTEN_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define RTC_EVTEN_COMPARE1_Msk (0x1UL << RTC_EVTEN_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */
#define RTC_EVTEN_COMPARE1_Disabled (0UL) /*!< Event disabled. */
#define RTC_EVTEN_COMPARE1_Enabled (1UL) /*!< Event enabled. */

/* Bit 16 : COMPARE[0] event enable. */
#define RTC_EVTEN_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define RTC_EVTEN_COMPARE0_Msk (0x1UL << RTC_EVTEN_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */
#define RTC_EVTEN_COMPARE0_Disabled (0UL) /*!< Event disabled. */
#define RTC_EVTEN_COMPARE0_Enabled (1UL) /*!< Event enabled. */

/* Bit 1 : OVRFLW event enable. */
#define RTC_EVTEN_OVRFLW_Pos (1U) /*!< Position of OVRFLW field. */
#define RTC_EVTEN_OVRFLW_Msk (0x1UL << RTC_EVTEN_OVRFLW_Pos) /*!< Bit mask of OVRFLW field. */
#define RTC_EVTEN_OVRFLW_Disabled (0UL) /*!< Event disabled. */
#define RTC_EVTEN_OVRFLW_Enabled (1UL) /*!< Event enabled. */

/* Bit 0 : TICK event enable. */
#define RTC_EVTEN_TICK_Pos (0U) /*!< Position of TICK field. */
#define RTC_EVTEN_TICK_Msk (0x1UL << RTC_EVTEN_TICK_Pos) /*!< Bit mask of TICK field. */
#define RTC_EVTEN_TICK_Disabled (0UL) /*!< Event disabled. */
#define RTC_EVTEN_TICK_Enabled (1UL) /*!< Event enabled. */

/* Register: RTC_EVTENSET */
/* Description: Enable events. The reading of this register gives the value of EVTEN. */

/* Bit 19 : Enable COMPARE[3] event. */
#define RTC_EVTENSET_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define RTC_EVTENSET_COMPARE3_Msk (0x1UL << RTC_EVTENSET_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */

/* Bit 18 : Enable COMPARE[2] event. */
#define RTC_EVTENSET_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define RTC_EVTENSET_COMPARE2_Msk (0x1UL << RTC_EVTENSET_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */

/* Bit 17 : Enable COMPARE[1] event. */
#define RTC_EVTENSET_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define RTC_EVTENSET_COMPARE1_Msk (0x1UL << RTC_EVTENSET_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */

/* Bit 16 : Enable COMPARE[0] event. */
#define RTC_EVTENSET_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define RTC_EVTENSET_COMPARE0_Msk (0x1UL << RTC_EVTENSET_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */

/* Bit 1 : Enable OVRFLW event. */
#define RTC_EVTENSET_OVRFLW_Pos (1U) /*!< Position of OVRFLW field. */
#define RTC_EVTENSET_OVRFLW_Msk (0x1UL << RTC_EVTENSET_OVRFLW_Pos) /*!< Bit mask of OVRFLW field. */

/* Bit 0 : Enable TICK event. */
#define RTC_EVTENSET_TICK_Pos (0U) /*!< Position of TICK field. */
#define RTC_EVTENSET_TICK_Msk (0x1UL << RTC_EVTENSET_TICK_Pos) /*!< Bit mask of TICK field. */

/* Register: RTC_EVTENCLR */
/* Description: Disable events. The reading of this register gives the value of EVTEN. */

/* Bit 19 : Disable COMPARE[3] event. */
#define RTC_EVTENCLR_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define RTC_EVTENCLR_COMPARE3_Msk (0x1UL << RTC_EVTENCLR_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */

/* Bit 18 : Disable COMPARE[2] event. */
#define RTC_EVTENCLR_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define RTC_EVTENCLR_COMPARE2_Msk (0x1UL << RTC_EVTENCLR_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */

/* Bit 17 : Disable COMPARE[1] event. */
#define RTC_EVTENCLR_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define RTC_EVTENCLR_COMPARE1_Msk (0x1UL << RTC_EVTENCLR_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */

/* Bit 16 : Disable COMPARE[0] event. */
#define RTC_EVTENCLR_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define RTC_EVTENCLR_COMPARE0_Msk (0x1UL << RTC_EVTENCLR_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */

/* Bit 1 : Disable OVRFLW event. */
#define RTC_EVTENCLR_OVRFLW_Pos (1U) /*!< Position of OVRFLW field. */
#define RTC_EVTENCLR_OVRFLW_Msk (0x1UL << RTC_EVTENCLR_OVRFLW_Pos) /*!< Bit mask of OVRFLW field. */

/* Bit 0 : Disable TICK event. */
#define RTC_EVTENCLR_TICK_Pos (0U) /*!< Position of TICK field. */
#define RTC_EVTENCLR_TICK_Msk (0x1UL << RTC_EVTENCLR_TICK_Pos) /*!< Bit mask of TICK field. */

/* Register: RTC_COUNTER */
/* Description: Current COUNTER value. */

/* Bits 23..0 : Counter value. */
#define RTC_COUNTER_COUNTER_Pos (0U) /*!< Position of COUNTER field. */
#define RTC_COUNTER_COUNTER_Msk (0xFFFFFFUL << RTC_COUNTER_COUNTER_Pos) /*!< Bit mask of COUNTER field. */

/* Register: RTC_PRESCALER */
/* Description: 12-bit prescaler for COUNTER frequency (32768/(PRESCALER+1)). Must be written when RTC is STOPed. */

/* Bits 11..0 : RTC PRESCALER value. */
#define RTC_PRESCALER_PRESCALER_Pos (0U) /*!< Position of PRESCALER field. */
#define RTC_PRESCALER_PRESCALER_Msk (0xFFFUL << RTC_PRESCALER_PRESCALER_Pos) /*!< Bit mask of PRESCALER field. */

/* Register: RTC_CC */
/* Description: Capture/compare registers. */

/* Bits 23..0 : Compare value. */
#define RTC_CC_COMPARE_Pos (0U) /*!< Position of COMPARE field. */
#define RTC_CC_COMPARE_Msk (0xFFFFFFUL << RTC_CC_COMPARE_Pos) /*!< Bit mask of COMPARE field. */


/* Peripheral: S2W */
/* Description: Serial 2-Wire master 0. */

/* Register: S2W_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 14 : Enable interrupt on BB event. */
#define S2W_INTENSET_BB_Pos (14UL) /*!< Position of BB field. */
#define S2W_INTENSET_BB_Msk (0x1UL << S2W_INTENSET_BB_Pos) /*!< Bit mask of BB field. */
#define S2W_INTENSET_BB_Disabled (0UL) /*!< Interrupt disabled. */
#define S2W_INTENSET_BB_Enabled (1UL) /*!< Interrupt enabled. */
#define S2W_INTENSET_BB_Set (1UL) /*!< Enable interrupt on write. */

/* Bit 8 : Enable interrupt on TXDSENT event. */
#define S2W_INTENSET_TXDSENT_Pos (7UL) /*!< Position of TXDSENT field. */
#define S2W_INTENSET_TXDSENT_Msk (0x1UL << S2W_INTENSET_TXDSENT_Pos) /*!< Bit mask of TXDSENT field. */
#define S2W_INTENSET_TXDSENT_Disabled (0UL) /*!< Interrupt disabled. */
#define S2W_INTENSET_TXDSENT_Enabled (1UL) /*!< Interrupt enabled. */
#define S2W_INTENSET_TXDSENT_Set (1UL) /*!< Enable interrupt on write. */

/* Bit 2 : Enable interrupt on READY event. */
#define S2W_INTENSET_RXDREADY_Pos (2UL) /*!< Position of RXDREADY field. */
#define S2W_INTENSET_RXDREADY_Msk (0x1UL << S2W_INTENSET_RXDREADY_Pos) /*!< Bit mask of RXDREADY field. */
#define S2W_INTENSET_RXDREADY_Disabled (0UL) /*!< Interrupt disabled. */
#define S2W_INTENSET_RXDREADY_Enabled (1UL) /*!< Interrupt enabled. */
#define S2W_INTENSET_RXDREADY_Set (1UL) /*!< Enable interrupt on write. */

/* Register: S2W_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 14 : Disable interrupt on BB event. */
#define S2W_INTENCLR_BB_Pos (14UL) /*!< Position of BB field. */
#define S2W_INTENCLR_BB_Msk (0x1UL << S2W_INTENCLR_BB_Pos) /*!< Bit mask of BB field. */
#define S2W_INTENCLR_BB_Disabled (0UL) /*!< Interrupt disabled. */
#define S2W_INTENCLR_BB_Enabled (1UL) /*!< Interrupt enabled. */
#define S2W_INTENCLR_BB_Clear (1UL) /*!< Disable interrupt on write. */

/* Bit 8 : Disable interrupt on TXDSENT event. */
#define S2W_INTENCLR_TXDSENT_Pos (7UL) /*!< Position of TXDSENT field. */
#define S2W_INTENCLR_TXDSENT_Msk (0x1UL << S2W_INTENCLR_TXDSENT_Pos) /*!< Bit mask of TXDSENT field. */
#define S2W_INTENCLR_TXDSENT_Disabled (0UL) /*!< Interrupt disabled. */
#define S2W_INTENCLR_TXDSENT_Enabled (1UL) /*!< Interrupt enabled. */
#define S2W_INTENCLR_TXDSENT_Clear (1UL) /*!< Disable interrupt on write. */

/* Bit 2 : Disable interrupt on RXDREADY event. */
#define S2W_INTENCLR_RXDREADY_Pos (2UL) /*!< Position of RXDREADY field. */
#define S2W_INTENCLR_RXDREADY_Msk (0x1UL << S2W_INTENCLR_RXDREADY_Pos) /*!< Bit mask of RXDREADY field. */
#define S2W_INTENCLR_RXDREADY_Disabled (0UL) /*!< Interrupt disabled. */
#define S2W_INTENCLR_RXDREADY_Enabled (1UL) /*!< Interrupt enabled. */
#define S2W_INTENCLR_RXDREADY_Clear (1UL) /*!< Disable interrupt on write. */

/* Register: S2W_ERRORSRC */
/* Description: 2-wire error source. */

/* Bit 2 : NACK received after sending a data byte. */
#define S2W_ERRORSRC_DNACK_Pos (2UL) /*!< Position of DNACK field. */
#define S2W_ERRORSRC_DNACK_Msk (0x1UL << S2W_ERRORSRC_DNACK_Pos) /*!< Bit mask of DNACK field. */
#define S2W_ERRORSRC_DNACK_NotPresent (0UL) /*!< Error not present. */
#define S2W_ERRORSRC_DNACK_Present (1UL) /*!< Error present. */

/* Bit 1 : NACK received after sending the address. */
#define S2W_ERRORSRC_ANACK_Pos (1UL) /*!< Position of ANACK field. */
#define S2W_ERRORSRC_ANACK_Msk (0x1UL << S2W_ERRORSRC_ANACK_Pos) /*!< Bit mask of ANACK field. */
#define S2W_ERRORSRC_ANACK_NotPresent (0UL) /*!< Error not present. */
#define S2W_ERRORSRC_ANACK_Present (1UL) /*!< Error present. */

/* Register: S2W_ENABLE */
/* Description: Enable 2-wire master. */

/* Bits 2..0 : Enable or disable W2M */
#define S2W_ENABLE_ENABLE_Pos (0UL) /*!< Position of ENABLE field. */
#define S2W_ENABLE_ENABLE_Msk (0x7UL << S2W_ENABLE_ENABLE_Pos) /*!< Bit mask of ENABLE field. */
#define S2W_ENABLE_ENABLE_Disabled (0x00UL) /*!< Disabled. */
#define S2W_ENABLE_ENABLE_Enabled (0x05UL) /*!< Enabled. */

/* Register: S2W_RXD */
/* Description: RX data register. */

/* Bits 7..0 : RX data from last transfer. */
#define S2W_RXD_RXD_Pos (0UL) /*!< Position of RXD field. */
#define S2W_RXD_RXD_Msk (0xFFUL << S2W_RXD_RXD_Pos) /*!< Bit mask of RXD field. */

/* Register: S2W_TXD */
/* Description: TX data register. */

/* Bits 7..0 : TX data for next transfer. */
#define S2W_TXD_TXD_Pos (0UL) /*!< Position of TXD field. */
#define S2W_TXD_TXD_Msk (0xFFUL << S2W_TXD_TXD_Pos) /*!< Bit mask of TXD field. */

/* Register: S2W_FREQUENCY */
/* Description: 2-wire frequency. */

/* Bits 31..0 : 2-wire master clock frequency. */
#define S2W_FREQUENCY_FREQUENCY_Pos (0UL) /*!< Position of FREQUENCY field. */
#define S2W_FREQUENCY_FREQUENCY_Msk (0xFFFFFFFFUL << S2W_FREQUENCY_FREQUENCY_Pos) /*!< Bit mask of FREQUENCY field. */
#define S2W_FREQUENCY_FREQUENCY_K100 (0x01980000UL) /*!< 100 kbps. */
#define S2W_FREQUENCY_FREQUENCY_K400 (0x06680000UL) /*!< 400 kbps. */

/* Register: S2W_ADDRESS */
/* Description: Address used in the 2-wire transfer. */

/* Bits 6..0 : 2-wire address. */
#define S2W_ADDRESS_ADDRESS_Pos (0UL) /*!< Position of ADDRESS field. */
#define S2W_ADDRESS_ADDRESS_Msk (0x7FUL << S2W_ADDRESS_ADDRESS_Pos) /*!< Bit mask of ADDRESS field. */


/* Peripheral: SPI */
/* Description: SPI master */

/* Register: SPI_INTEN */
/* Description: Interrupt enable register. */

/* Bit 2 : Interrupt enable on READY event. */
#define SPI_INTEN_READY_Pos (2U) /*!< Position of READY field. */
#define SPI_INTEN_READY_Msk (0x1UL << SPI_INTEN_READY_Pos) /*!< Bit mask of READY field. */
#define SPI_INTEN_READY_Disabled (0UL) /*!< Interrupt disabled. */
#define SPI_INTEN_READY_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: SPI_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 2 : Enable interrupt on READY event. */
#define SPI_INTENSET_READY_Pos (2U) /*!< Position of READY field. */
#define SPI_INTENSET_READY_Msk (0x1UL << SPI_INTENSET_READY_Pos) /*!< Bit mask of READY field. */

/* Register: SPI_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 2 : Disable interrupt on READY event. */
#define SPI_INTENCLR_READY_Pos (2U) /*!< Position of READY field. */
#define SPI_INTENCLR_READY_Msk (0x1UL << SPI_INTENCLR_READY_Pos) /*!< Bit mask of READY field. */

/* Register: SPI_ENABLE */
/* Description: Enable SPI. */

/* Bit 0 : Enable or disable SPI. */
#define SPI_ENABLE_ENABLE_Pos (0U) /*!< Position of ENABLE field. */
#define SPI_ENABLE_ENABLE_Msk (0x1UL << SPI_ENABLE_ENABLE_Pos) /*!< Bit mask of ENABLE field. */
#define SPI_ENABLE_ENABLE_Disabled (0UL) /*!< Disabled SPI. */
#define SPI_ENABLE_ENABLE_Enabled (1UL) /*!< Enable SPI. */

/* Register: SPI_RXD */
/* Description: RX data. */

/* Bits 7..0 : RX data from last transfer. */
#define SPI_RXD_RXD_Pos (0U) /*!< Position of RXD field. */
#define SPI_RXD_RXD_Msk (0xFFUL << SPI_RXD_RXD_Pos) /*!< Bit mask of RXD field. */

/* Register: SPI_TXD */
/* Description: TX data. */

/* Bits 7..0 : TX data for next transfer. */
#define SPI_TXD_TXD_Pos (0U) /*!< Position of TXD field. */
#define SPI_TXD_TXD_Msk (0xFFUL << SPI_TXD_TXD_Pos) /*!< Bit mask of TXD field. */

/* Register: SPI_FREQUENCY */
/* Description: SPI frequency */

/* Bits 31..0 : SPI data rate. */
#define SPI_FREQUENCY_FREQUENCY_Pos (0U) /*!< Position of FREQUENCY field. */
#define SPI_FREQUENCY_FREQUENCY_Msk (0xFFFFFFFFUL << SPI_FREQUENCY_FREQUENCY_Pos) /*!< Bit mask of FREQUENCY field. */
#define SPI_FREQUENCY_FREQUENCY_K125 (0x02000000UL) /*!< 125kbps. */
#define SPI_FREQUENCY_FREQUENCY_K250 (0x04000000UL) /*!< 250kbps. */
#define SPI_FREQUENCY_FREQUENCY_K500 (0x08000000UL) /*!< 500kbps. */
#define SPI_FREQUENCY_FREQUENCY_M1 (0x10000000UL) /*!< 1Mbps. */
#define SPI_FREQUENCY_FREQUENCY_M2 (0x20000000UL) /*!< 2Mbps. */
#define SPI_FREQUENCY_FREQUENCY_M4 (0x40000000UL) /*!< 4Mbps. */
#define SPI_FREQUENCY_FREQUENCY_M8 (0x80000000UL) /*!< 8Mbps. */

/* Register: SPI_CONFIG */
/* Description: RX data */

/* Bit 2 : Serial clock (SCK) phase. */
#define SPI_CONFIG_CPHA_Pos (2U) /*!< Position of CPHA field. */
#define SPI_CONFIG_CPHA_Msk (0x1UL << SPI_CONFIG_CPHA_Pos) /*!< Bit mask of CPHA field. */
#define SPI_CONFIG_CPHA_Leading (0UL) /*!< Sample on leading edge of the clock. Shift serial data on trailing edge. */
#define SPI_CONFIG_CPHA_Trailing (1UL) /*!< Sample on trailing edge of the clock. Shift serial data on leading edge. */

/* Bit 1 : Serial clock (SCK) polarity. */
#define SPI_CONFIG_CPOL_Pos (1U) /*!< Position of CPOL field. */
#define SPI_CONFIG_CPOL_Msk (0x1UL << SPI_CONFIG_CPOL_Pos) /*!< Bit mask of CPOL field. */
#define SPI_CONFIG_CPOL_ActiveHigh (0UL) /*!< Active high. */
#define SPI_CONFIG_CPOL_ActiveLow (1UL) /*!< Active low. */

/* Bit 0 : Endianess of transmission. */
#define SPI_CONFIG_ORDER_Pos (0U) /*!< Position of ORDER field. */
#define SPI_CONFIG_ORDER_Msk (0x1UL << SPI_CONFIG_ORDER_Pos) /*!< Bit mask of ORDER field. */
#define SPI_CONFIG_ORDER_MsbFirst (0UL) /*!< Most significant bit transmitted out first. */
#define SPI_CONFIG_ORDER_LsbFirst (1UL) /*!< Least significant bit transmitted out first. */


/* Peripheral: TEMP */
/* Description: Temperature Sensor */

/* Register: TEMP_INTEN */
/* Description: TEMP interrupt enable register. */

/* Bit 0 : Enable DATARDY interrupt. */
#define TEMP_INTEN_DATARDY_Pos (0U) /*!< Position of DATARDY field. */
#define TEMP_INTEN_DATARDY_Msk (0x1UL << TEMP_INTEN_DATARDY_Pos) /*!< Bit mask of DATARDY field. */
#define TEMP_INTEN_DATARDY_Disabled (0UL) /*!< Interrupt disabled. */
#define TEMP_INTEN_DATARDY_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: TEMP_INTENSET */
/* Description: TEMP interrupt enable set register. */

/* Bit 0 : Enable interrupt on DATARDY event. */
#define TEMP_INTENSET_DATARDY_Pos (0U) /*!< Position of DATARDY field. */
#define TEMP_INTENSET_DATARDY_Msk (0x1UL << TEMP_INTENSET_DATARDY_Pos) /*!< Bit mask of DATARDY field. */

/* Register: TEMP_INTENCLR */
/* Description: TEMP interrupt enable clear register. */

/* Bit 0 : Disable interrupt on DATARDY event. */
#define TEMP_INTENCLR_DATARDY_Pos (0U) /*!< Position of DATARDY field. */
#define TEMP_INTENCLR_DATARDY_Msk (0x1UL << TEMP_INTENCLR_DATARDY_Pos) /*!< Bit mask of DATARDY field. */

/* Register: TEMP_TEMP */
/* Description: Temperature. */

/* Bits 9..0 : Die temperature in degC, 2's complement format, 0.25 degC pecision. Decision point: DATARDY */
#define TEMP_TEMP_TEMP_Pos (0U) /*!< Position of TEMP field. */
#define TEMP_TEMP_TEMP_Msk (0x3FFUL << TEMP_TEMP_TEMP_Pos) /*!< Bit mask of TEMP field. */


/* Peripheral: TIMER */
/* Description: Timer 0 */

/* Register: TIMER_SHORTS */
/* Description: Shortcuts for COMPAREEVENTs. */

/* Bit 11 : Shortcut between CC[3] event and the STOP task. */
#define TIMER_SHORTS_COMPARE3_STOP_Pos (11U) /*!< Position of COMPARE3_STOP field. */
#define TIMER_SHORTS_COMPARE3_STOP_Msk (0x1UL << TIMER_SHORTS_COMPARE3_STOP_Pos) /*!< Bit mask of COMPARE3_STOP field. */
#define TIMER_SHORTS_COMPARE3_STOP_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE3_STOP_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 10 : Shortcut between CC[2] event and the STOP task. */
#define TIMER_SHORTS_COMPARE2_STOP_Pos (10U) /*!< Position of COMPARE2_STOP field. */
#define TIMER_SHORTS_COMPARE2_STOP_Msk (0x1UL << TIMER_SHORTS_COMPARE2_STOP_Pos) /*!< Bit mask of COMPARE2_STOP field. */
#define TIMER_SHORTS_COMPARE2_STOP_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE2_STOP_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 9 : Shortcut between CC[1] event and the STOP task. */
#define TIMER_SHORTS_COMPARE1_STOP_Pos (9U) /*!< Position of COMPARE1_STOP field. */
#define TIMER_SHORTS_COMPARE1_STOP_Msk (0x1UL << TIMER_SHORTS_COMPARE1_STOP_Pos) /*!< Bit mask of COMPARE1_STOP field. */
#define TIMER_SHORTS_COMPARE1_STOP_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE1_STOP_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 8 : Shortcut between CC[0] event and the STOP task. */
#define TIMER_SHORTS_COMPARE0_STOP_Pos (8U) /*!< Position of COMPARE0_STOP field. */
#define TIMER_SHORTS_COMPARE0_STOP_Msk (0x1UL << TIMER_SHORTS_COMPARE0_STOP_Pos) /*!< Bit mask of COMPARE0_STOP field. */
#define TIMER_SHORTS_COMPARE0_STOP_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE0_STOP_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 3 : Shortcut between CC[3] event and the CLEAR task. */
#define TIMER_SHORTS_COMPARE3_CLEAR_Pos (3U) /*!< Position of COMPARE3_CLEAR field. */
#define TIMER_SHORTS_COMPARE3_CLEAR_Msk (0x1UL << TIMER_SHORTS_COMPARE3_CLEAR_Pos) /*!< Bit mask of COMPARE3_CLEAR field. */
#define TIMER_SHORTS_COMPARE3_CLEAR_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE3_CLEAR_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 2 : Shortcut between CC[2] event and the CLEAR task. */
#define TIMER_SHORTS_COMPARE2_CLEAR_Pos (2U) /*!< Position of COMPARE2_CLEAR field. */
#define TIMER_SHORTS_COMPARE2_CLEAR_Msk (0x1UL << TIMER_SHORTS_COMPARE2_CLEAR_Pos) /*!< Bit mask of COMPARE2_CLEAR field. */
#define TIMER_SHORTS_COMPARE2_CLEAR_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE2_CLEAR_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 1 : Shortcut between CC[1] event and the CLEAR task. */
#define TIMER_SHORTS_COMPARE1_CLEAR_Pos (1U) /*!< Position of COMPARE1_CLEAR field. */
#define TIMER_SHORTS_COMPARE1_CLEAR_Msk (0x1UL << TIMER_SHORTS_COMPARE1_CLEAR_Pos) /*!< Bit mask of COMPARE1_CLEAR field. */
#define TIMER_SHORTS_COMPARE1_CLEAR_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE1_CLEAR_Enabled (1UL) /*!< Shortcut enabled. */

/* Bit 0 : Shortcut between CC[0] event and the CLEAR task. */
#define TIMER_SHORTS_COMPARE0_CLEAR_Pos (0U) /*!< Position of COMPARE0_CLEAR field. */
#define TIMER_SHORTS_COMPARE0_CLEAR_Msk (0x1UL << TIMER_SHORTS_COMPARE0_CLEAR_Pos) /*!< Bit mask of COMPARE0_CLEAR field. */
#define TIMER_SHORTS_COMPARE0_CLEAR_Disabled (0UL) /*!< Shortcut disabled. */
#define TIMER_SHORTS_COMPARE0_CLEAR_Enabled (1UL) /*!< Shortcut enabled. */

/* Register: TIMER_INTEN */
/* Description: Timer interrupt enable register. */

/* Bit 19 : Interrupt enable on COMPARE[3] */
#define TIMER_INTEN_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define TIMER_INTEN_COMPARE3_Msk (0x1UL << TIMER_INTEN_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */
#define TIMER_INTEN_COMPARE3_Disabled (0UL) /*!< Interrupt disabled. */
#define TIMER_INTEN_COMPARE3_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 18 : Interrupt enable on COMPARE[2] */
#define TIMER_INTEN_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define TIMER_INTEN_COMPARE2_Msk (0x1UL << TIMER_INTEN_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */
#define TIMER_INTEN_COMPARE2_Disabled (0UL) /*!< Interrupt disabled. */
#define TIMER_INTEN_COMPARE2_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 17 : Interrupt enable on COMPARE[1] */
#define TIMER_INTEN_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define TIMER_INTEN_COMPARE1_Msk (0x1UL << TIMER_INTEN_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */
#define TIMER_INTEN_COMPARE1_Disabled (0UL) /*!< Interrupt disabled. */
#define TIMER_INTEN_COMPARE1_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 16 : Interrupt enable on COMPARE[0] */
#define TIMER_INTEN_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define TIMER_INTEN_COMPARE0_Msk (0x1UL << TIMER_INTEN_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */
#define TIMER_INTEN_COMPARE0_Disabled (0UL) /*!< Interrupt disabled. */
#define TIMER_INTEN_COMPARE0_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: TIMER_INTENSET */
/* Description: Timer interrupt enable set register. */

/* Bit 19 : Enable interrupt on COMPARE[3] */
#define TIMER_INTENSET_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define TIMER_INTENSET_COMPARE3_Msk (0x1UL << TIMER_INTENSET_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */

/* Bit 18 : Enable interrupt on COMPARE[2] */
#define TIMER_INTENSET_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define TIMER_INTENSET_COMPARE2_Msk (0x1UL << TIMER_INTENSET_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */

/* Bit 17 : Enable interrupt on COMPARE[1] */
#define TIMER_INTENSET_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define TIMER_INTENSET_COMPARE1_Msk (0x1UL << TIMER_INTENSET_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */

/* Bit 16 : Enable interrupt on COMPARE[0] */
#define TIMER_INTENSET_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define TIMER_INTENSET_COMPARE0_Msk (0x1UL << TIMER_INTENSET_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */

/* Register: TIMER_INTENCLR */
/* Description: Timer interrupt enable clear register. */

/* Bit 19 : Disable interrupt on COMPARE[3] */
#define TIMER_INTENCLR_COMPARE3_Pos (19U) /*!< Position of COMPARE3 field. */
#define TIMER_INTENCLR_COMPARE3_Msk (0x1UL << TIMER_INTENCLR_COMPARE3_Pos) /*!< Bit mask of COMPARE3 field. */

/* Bit 18 : Disable interrupt on COMPARE[2] */
#define TIMER_INTENCLR_COMPARE2_Pos (18U) /*!< Position of COMPARE2 field. */
#define TIMER_INTENCLR_COMPARE2_Msk (0x1UL << TIMER_INTENCLR_COMPARE2_Pos) /*!< Bit mask of COMPARE2 field. */

/* Bit 17 : Disable interrupt on COMPARE[1] */
#define TIMER_INTENCLR_COMPARE1_Pos (17U) /*!< Position of COMPARE1 field. */
#define TIMER_INTENCLR_COMPARE1_Msk (0x1UL << TIMER_INTENCLR_COMPARE1_Pos) /*!< Bit mask of COMPARE1 field. */

/* Bit 16 : Disable interrupt on COMPARE[0] */
#define TIMER_INTENCLR_COMPARE0_Pos (16U) /*!< Position of COMPARE0 field. */
#define TIMER_INTENCLR_COMPARE0_Msk (0x1UL << TIMER_INTENCLR_COMPARE0_Pos) /*!< Bit mask of COMPARE0 field. */

/* Register: TIMER_MODE */
/* Description: Timer Mode selection. */

/* Bit 0 : Select Normal or Counter mode. */
#define TIMER_MODE_MODE_Pos (0U) /*!< Position of MODE field. */
#define TIMER_MODE_MODE_Msk (0x1UL << TIMER_MODE_MODE_Pos) /*!< Bit mask of MODE field. */
#define TIMER_MODE_MODE_Timer (0UL) /*!< Timer in Normal mode. */
#define TIMER_MODE_MODE_Counter (1UL) /*!< Timer in Counter mode. */

/* Register: TIMER_BITMODE */
/* Description: Sets timer behaviour. */

/* Bits 1..0 : Sets timer behaviour ro be like the implementation of a timer with width as indicated. */
#define TIMER_BITMODE_BITMODE_Pos (0U) /*!< Position of BITMODE field. */
#define TIMER_BITMODE_BITMODE_Msk (0x3UL << TIMER_BITMODE_BITMODE_Pos) /*!< Bit mask of BITMODE field. */
#define TIMER_BITMODE_BITMODE_16Bit (0x00UL) /*!< 16-bit timer behaviour. */
#define TIMER_BITMODE_BITMODE_08Bit (0x01UL) /*!< 8-bit timer behaviour. */
#define TIMER_BITMODE_BITMODE_24Bit (0x02UL) /*!< 24-bit timer behaviour. */
#define TIMER_BITMODE_BITMODE_32Bit (0x03UL) /*!< 32-bit timer behaviour. */

/* Register: TIMER_PRESCALER */
/* Description: 4-bit prescaler to source clock frequency (max value 9). source code frequency is divided by 2^SCALE. */

/* Bits 3..0 : Timer PRESCALER value. Max value is 9. */
#define TIMER_PRESCALER_PRESCALER_Pos (0U) /*!< Position of PRESCALER field. */
#define TIMER_PRESCALER_PRESCALER_Msk (0xFUL << TIMER_PRESCALER_PRESCALER_Pos) /*!< Bit mask of PRESCALER field. */


/* Peripheral: UART */
/* Description: Universal Asynchronous Receiver/Transmitter. */

/* Register: UART_INTEN */
/* Description: Interrupt enable register. */

/* Bit 9 : Interrupt enable on ERROR event. */
#define UART_INTEN_ERROR_Pos (9U) /*!< Position of ERROR field. */
#define UART_INTEN_ERROR_Msk (0x1UL << UART_INTEN_ERROR_Pos) /*!< Bit mask of ERROR field. */
#define UART_INTEN_ERROR_Disabled (0UL) /*!< Interrupt disabled. */
#define UART_INTEN_ERROR_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 7 : Interrupt enable on TXRDY event. */
#define UART_INTEN_TXDRDY_Pos (7U) /*!< Position of TXDRDY field. */
#define UART_INTEN_TXDRDY_Msk (0x1UL << UART_INTEN_TXDRDY_Pos) /*!< Bit mask of TXDRDY field. */
#define UART_INTEN_TXDRDY_Disabled (0UL) /*!< Interrupt disabled. */
#define UART_INTEN_TXDRDY_Enabled (1UL) /*!< Interrupt enabled. */

/* Bit 2 : Interrupt enable on RXRDY event. */
#define UART_INTEN_RXDRDY_Pos (2U) /*!< Position of RXDRDY field. */
#define UART_INTEN_RXDRDY_Msk (0x1UL << UART_INTEN_RXDRDY_Pos) /*!< Bit mask of RXDRDY field. */
#define UART_INTEN_RXDRDY_Disabled (0UL) /*!< Interrupt disabled. */
#define UART_INTEN_RXDRDY_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: UART_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 9 : Enable interrupt on ERROR event. */
#define UART_INTENSET_ERROR_Pos (9U) /*!< Position of ERROR field. */
#define UART_INTENSET_ERROR_Msk (0x1UL << UART_INTENSET_ERROR_Pos) /*!< Bit mask of ERROR field. */

/* Bit 7 : Enable interrupt on TXRDY event. */
#define UART_INTENSET_TXDRDY_Pos (7U) /*!< Position of TXDRDY field. */
#define UART_INTENSET_TXDRDY_Msk (0x1UL << UART_INTENSET_TXDRDY_Pos) /*!< Bit mask of TXDRDY field. */

/* Bit 2 : Enable interrupt on RXRDY event. */
#define UART_INTENSET_RXDRDY_Pos (2U) /*!< Position of RXDRDY field. */
#define UART_INTENSET_RXDRDY_Msk (0x1UL << UART_INTENSET_RXDRDY_Pos) /*!< Bit mask of RXDRDY field. */

/* Register: UART_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 9 : Disable interrupt on ERROR event. */
#define UART_INTENCLR_ERROR_Pos (9U) /*!< Position of ERROR field. */
#define UART_INTENCLR_ERROR_Msk (0x1UL << UART_INTENCLR_ERROR_Pos) /*!< Bit mask of ERROR field. */

/* Bit 7 : Disable interrupt on TXRDY event. */
#define UART_INTENCLR_TXDRDY_Pos (7U) /*!< Position of TXDRDY field. */
#define UART_INTENCLR_TXDRDY_Msk (0x1UL << UART_INTENCLR_TXDRDY_Pos) /*!< Bit mask of TXDRDY field. */

/* Bit 2 : Disable interrupt on RXRDY event. */
#define UART_INTENCLR_RXDRDY_Pos (2U) /*!< Position of RXDRDY field. */
#define UART_INTENCLR_RXDRDY_Msk (0x1UL << UART_INTENCLR_RXDRDY_Pos) /*!< Bit mask of RXDRDY field. */

/* Register: UART_ERRORSRC */
/* Description: Error source. Write error field to 1 to clear error. */

/* Bit 3 : The serial data input is '0' for longer than the length of a data frame. */
#define UART_ERRORSRC_BREAK_Pos (3U) /*!< Position of BREAK field. */
#define UART_ERRORSRC_BREAK_Msk (0x1UL << UART_ERRORSRC_BREAK_Pos) /*!< Bit mask of BREAK field. */
#define UART_ERRORSRC_BREAK_NotPresent (0UL) /*!< Error not present. */
#define UART_ERRORSRC_BREAK_Present (1UL) /*!< Error present. */

/* Bit 2 : A valid stop bit is not detected on the serial data input after all bits in a character have been received. */
#define UART_ERRORSRC_FRAMING_Pos (2U) /*!< Position of FRAMING field. */
#define UART_ERRORSRC_FRAMING_Msk (0x1UL << UART_ERRORSRC_FRAMING_Pos) /*!< Bit mask of FRAMING field. */
#define UART_ERRORSRC_FRAMING_NotPresent (0UL) /*!< Error not present. */
#define UART_ERRORSRC_FRAMING_Present (1UL) /*!< Error present. */

/* Bit 1 : A character with bad parity is received. Only checked if HW parity control is enabled. */
#define UART_ERRORSRC_PARITY_Pos (1U) /*!< Position of PARITY field. */
#define UART_ERRORSRC_PARITY_Msk (0x1UL << UART_ERRORSRC_PARITY_Pos) /*!< Bit mask of PARITY field. */
#define UART_ERRORSRC_PARITY_NotPresent (0UL) /*!< Error not present. */
#define UART_ERRORSRC_PARITY_Present (1UL) /*!< Error present. */

/* Bit 0 : A start bit is received while the previous data still lies in RXD. (Data loss). */
#define UART_ERRORSRC_OVERRUN_Pos (0U) /*!< Position of OVERRUN field. */
#define UART_ERRORSRC_OVERRUN_Msk (0x1UL << UART_ERRORSRC_OVERRUN_Pos) /*!< Bit mask of OVERRUN field. */
#define UART_ERRORSRC_OVERRUN_NotPresent (0UL) /*!< Error not present. */
#define UART_ERRORSRC_OVERRUN_Present (1UL) /*!< Error present. */

/* Register: UART_ENABLE */
/* Description: Enable UART and acquire IOs. */

/* Bits 2..0 : Enable or disable UART and acquire IOs. */
#define UART_ENABLE_ENABLE_Pos (0U) /*!< Position of ENABLE field. */
#define UART_ENABLE_ENABLE_Msk (0x7UL << UART_ENABLE_ENABLE_Pos) /*!< Bit mask of ENABLE field. */
#define UART_ENABLE_ENABLE_Disabled (0x00UL) /*!< UART disabled. */
#define UART_ENABLE_ENABLE_Enabled (0x04UL) /*!< UART enabled. */

/* Register: UART_RXD */
/* Description: RXD register. */

/* Bits 7..0 : RX data from last transfer. */
#define UART_RXD_RXD_Pos (0U) /*!< Position of RXD field. */
#define UART_RXD_RXD_Msk (0xFFUL << UART_RXD_RXD_Pos) /*!< Bit mask of RXD field. */

/* Register: UART_TXD */
/* Description: TXD register. */

/* Bits 7..0 : TX data for next transfer. */
#define UART_TXD_TXD_Pos (0U) /*!< Position of TXD field. */
#define UART_TXD_TXD_Msk (0xFFUL << UART_TXD_TXD_Pos) /*!< Bit mask of TXD field. */

/* Register: UART_BAUDRATE */
/* Description: UART Baudrate. */

/* Bits 31..0 : UART baudrate. */
#define UART_BAUDRATE_BAUDRATE_Pos (0U) /*!< Position of BAUDRATE field. */
#define UART_BAUDRATE_BAUDRATE_Msk (0xFFFFFFFFUL << UART_BAUDRATE_BAUDRATE_Pos) /*!< Bit mask of BAUDRATE field. */
#define UART_BAUDRATE_BAUDRATE_Baud1200 (0x0004D000UL) /*!< 1200 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud2400 (0x0009D000UL) /*!< 2400 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud4800 (0x0013B000UL) /*!< 4800 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud9600 (0x00275000UL) /*!< 9600 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud14400 (0x003BA000UL) /*!< 14400 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud19200 (0x004EA000UL) /*!< 19200 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud28800 (0x0075F000UL) /*!< 28800 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud38400 (0x009D4000UL) /*!< 38400 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud57600 (0x00EBE000UL) /*!< 57600 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud76800 (0x013A9000UL) /*!< 76800 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud115200 (0x01D7D000UL) /*!< 115200 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud230400 (0x03AFB000UL) /*!< 230400 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud250000 (0x03FFF000UL) /*!< 250000 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud460800 (0x075F6000UL) /*!< 460800 baud. */
#define UART_BAUDRATE_BAUDRATE_Baud1M (0x10000000UL) /*!< 1M baud. */

/* Register: UART_CONFIG */
/* Description: Configuration of parity and hardware flow control register. */

/* Bit 3 : Configuration of parity used in transmission. */
#define UART_CONFIG_TXPARITY_Pos (3U) /*!< Position of TXPARITY field. */
#define UART_CONFIG_TXPARITY_Msk (0x1UL << UART_CONFIG_TXPARITY_Pos) /*!< Bit mask of TXPARITY field. */
#define UART_CONFIG_TXPARITY_Manual (0UL) /*!< 9th bit transmitted is taken from the TXPARITY register. */
#define UART_CONFIG_TXPARITY_Auto (1UL) /*!< 9th bit transmitted is generated from the 8 first bits. */

/* Bit 2 : Configuration of parity used in reception. */
#define UART_CONFIG_RXPARITY_Pos (2U) /*!< Position of RXPARITY field. */
#define UART_CONFIG_RXPARITY_Msk (0x1UL << UART_CONFIG_RXPARITY_Pos) /*!< Bit mask of RXPARITY field. */
#define UART_CONFIG_RXPARITY_Manual (0UL) /*!< 9th bit received is saved in RXPARITY register. */
#define UART_CONFIG_RXPARITY_Auto (1UL) /*!< 9th bit received is compared to a parity generated from the 8 first bits. */

/* Bit 1 : Include parity bit. */
#define UART_CONFIG_PARITY_Pos (1U) /*!< Position of PARITY field. */
#define UART_CONFIG_PARITY_Msk (0x1UL << UART_CONFIG_PARITY_Pos) /*!< Bit mask of PARITY field. */
#define UART_CONFIG_PARITY_Disabled (0UL) /*!< Parity bit not included. */
#define UART_CONFIG_PARITY_Enabled (1UL) /*!< Parity bit included. */

/* Bit 0 : Hardware flow control. */
#define UART_CONFIG_HWFC_Pos (0U) /*!< Position of HWFC field. */
#define UART_CONFIG_HWFC_Msk (0x1UL << UART_CONFIG_HWFC_Pos) /*!< Bit mask of HWFC field. */
#define UART_CONFIG_HWFC_Disabled (0UL) /*!< Hardware flow control disabled. */
#define UART_CONFIG_HWFC_Enabled (1UL) /*!< Hardware flow control enabled. */

/* Register: UART_RXPARITY */
/* Description: RX parity. */

/* Bit 0 : RX parity bit. */
#define UART_RXPARITY_RXPARITY_Pos (0U) /*!< Position of RXPARITY field. */
#define UART_RXPARITY_RXPARITY_Msk (0x1UL << UART_RXPARITY_RXPARITY_Pos) /*!< Bit mask of RXPARITY field. */

/* Register: UART_TXPARITY */
/* Description: TX parity. */

/* Bit 0 : TX parity bit. */
#define UART_TXPARITY_TXPARITY_Pos (0U) /*!< Position of TXPARITY field. */
#define UART_TXPARITY_TXPARITY_Msk (0x1UL << UART_TXPARITY_TXPARITY_Pos) /*!< Bit mask of TXPARITY field. */


/* Peripheral: UICR */
/* Description: User Information Configuration */

/* Register: UICR_RBPCONF */
/* Description: Readback protection configuration. */

/* Bits 15..8 : Readback protect region 0 and 1. */
#define UICR_RBPCONF_PALL_Pos (8U) /*!< Position of PALL field. */
#define UICR_RBPCONF_PALL_Msk (0xFFUL << UICR_RBPCONF_PALL_Pos) /*!< Bit mask of PALL field. */
#define UICR_RBPCONF_PALL_Disabled (0xFFUL) /*!< Disabled. */
#define UICR_RBPCONF_PALL_Enabled (0x00UL) /*!< Enabled. */

/* Bits 7..0 : Readback protect region 0. */
#define UICR_RBPCONF_PR0_Pos (0U) /*!< Position of PR0 field. */
#define UICR_RBPCONF_PR0_Msk (0xFFUL << UICR_RBPCONF_PR0_Pos) /*!< Bit mask of PR0 field. */
#define UICR_RBPCONF_PR0_Disabled (0xFFUL) /*!< Disabled. */
#define UICR_RBPCONF_PR0_Enabled (0x00UL) /*!< Enabled. */

/* Register: UICR_XTALFREQ */
/* Description: Nominal frequency of high frequency xtal used in the design. */

/* Bits 7..0 : Select the nominal frequency of external xtal. */
#define UICR_XTALFREQ_XTALFREQ_Pos (0U) /*!< Position of XTALFREQ field. */
#define UICR_XTALFREQ_XTALFREQ_Msk (0xFFUL << UICR_XTALFREQ_XTALFREQ_Pos) /*!< Bit mask of XTALFREQ field. */
#define UICR_XTALFREQ_XTALFREQ_16MHz (0xFFUL) /*!< 16MHz crystal is used. */
#define UICR_XTALFREQ_XTALFREQ_32MHz (0x00UL) /*!< 32MHz crystal is used. */


/* Peripheral: WDOG */
/* Description: Watchdog */

/* Register: WDOG_INTEN */
/* Description: Interrupt enable register. */

/* Bit 0 : Enable TIMEOUT interrupt. */
#define WDOG_INTEN_TIMEOUT_Pos (0U) /*!< Position of TIMEOUT field. */
#define WDOG_INTEN_TIMEOUT_Msk (0x1UL << WDOG_INTEN_TIMEOUT_Pos) /*!< Bit mask of TIMEOUT field. */
#define WDOG_INTEN_TIMEOUT_Disabled (0UL) /*!< Interrupt disabled. */
#define WDOG_INTEN_TIMEOUT_Enabled (1UL) /*!< Interrupt enabled. */

/* Register: WDOG_INTENSET */
/* Description: Interrupt enable set register. */

/* Bit 0 : Enable interrupt on TIMEOUT event. */
#define WDOG_INTENSET_TIMEOUT_Pos (0U) /*!< Position of TIMEOUT field. */
#define WDOG_INTENSET_TIMEOUT_Msk (0x1UL << WDOG_INTENSET_TIMEOUT_Pos) /*!< Bit mask of TIMEOUT field. */

/* Register: WDOG_INTENCLR */
/* Description: Interrupt enable clear register. */

/* Bit 0 : Disable interrupt on TIMEOUT event. */
#define WDOG_INTENCLR_TIMEOUT_Pos (0U) /*!< Position of TIMEOUT field. */
#define WDOG_INTENCLR_TIMEOUT_Msk (0x1UL << WDOG_INTENCLR_TIMEOUT_Pos) /*!< Bit mask of TIMEOUT field. */

/* Register: WDOG_REQSTATUS */
/* Description: Request status. */

/* Bit 7 : Request status for RR[7]. */
#define WDOG_REQSTATUS_RR7_Pos (7U) /*!< Position of RR7 field. */
#define WDOG_REQSTATUS_RR7_Msk (0x1UL << WDOG_REQSTATUS_RR7_Pos) /*!< Bit mask of RR7 field. */
#define WDOG_REQSTATUS_RR7_DisabledOrRequested (0UL) /*!< RR[7] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR7_EnabledAndUnrequested (1UL) /*!< RR[7] register is enabled and has not jet requested. */

/* Bit 6 : Request status for RR[6]. */
#define WDOG_REQSTATUS_RR6_Pos (6U) /*!< Position of RR6 field. */
#define WDOG_REQSTATUS_RR6_Msk (0x1UL << WDOG_REQSTATUS_RR6_Pos) /*!< Bit mask of RR6 field. */
#define WDOG_REQSTATUS_RR6_DisabledOrRequested (0UL) /*!< RR[6] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR6_EnabledAndUnrequested (1UL) /*!< RR[6] register is enabled and has not jet requested. */

/* Bit 5 : Request status for RR[5]. */
#define WDOG_REQSTATUS_RR5_Pos (5U) /*!< Position of RR5 field. */
#define WDOG_REQSTATUS_RR5_Msk (0x1UL << WDOG_REQSTATUS_RR5_Pos) /*!< Bit mask of RR5 field. */
#define WDOG_REQSTATUS_RR5_DisabledOrRequested (0UL) /*!< RR[5] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR5_EnabledAndUnrequested (1UL) /*!< RR[5] register is enabled and has not jet requested. */

/* Bit 4 : Request status for RR[4]. */
#define WDOG_REQSTATUS_RR4_Pos (4U) /*!< Position of RR4 field. */
#define WDOG_REQSTATUS_RR4_Msk (0x1UL << WDOG_REQSTATUS_RR4_Pos) /*!< Bit mask of RR4 field. */
#define WDOG_REQSTATUS_RR4_DisabledOrRequested (0UL) /*!< RR[4] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR4_EnabledAndUnrequested (1UL) /*!< RR[4] register is enabled and has not jet requested. */

/* Bit 3 : Request status for RR[3]. */
#define WDOG_REQSTATUS_RR3_Pos (3U) /*!< Position of RR3 field. */
#define WDOG_REQSTATUS_RR3_Msk (0x1UL << WDOG_REQSTATUS_RR3_Pos) /*!< Bit mask of RR3 field. */
#define WDOG_REQSTATUS_RR3_DisabledOrRequested (0UL) /*!< RR[3] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR3_EnabledAndUnrequested (1UL) /*!< RR[3] register is enabled and has not jet requested. */

/* Bit 2 : Request status for RR[2]. */
#define WDOG_REQSTATUS_RR2_Pos (2U) /*!< Position of RR2 field. */
#define WDOG_REQSTATUS_RR2_Msk (0x1UL << WDOG_REQSTATUS_RR2_Pos) /*!< Bit mask of RR2 field. */
#define WDOG_REQSTATUS_RR2_DisabledOrRequested (0UL) /*!< RR[2] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR2_EnabledAndUnrequested (1UL) /*!< RR[2] register is enabled and has not jet requested. */

/* Bit 1 : Request status for RR[1]. */
#define WDOG_REQSTATUS_RR1_Pos (1U) /*!< Position of RR1 field. */
#define WDOG_REQSTATUS_RR1_Msk (0x1UL << WDOG_REQSTATUS_RR1_Pos) /*!< Bit mask of RR1 field. */
#define WDOG_REQSTATUS_RR1_DisabledOrRequested (0UL) /*!< RR[1] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR1_EnabledAndUnrequested (1UL) /*!< RR[1] register is enabled and has not jet requested. */

/* Bit 0 : Request status for RR[0]. */
#define WDOG_REQSTATUS_RR0_Pos (0U) /*!< Position of RR0 field. */
#define WDOG_REQSTATUS_RR0_Msk (0x1UL << WDOG_REQSTATUS_RR0_Pos) /*!< Bit mask of RR0 field. */
#define WDOG_REQSTATUS_RR0_DisabledOrRequested (0UL) /*!< RR[0] register is not enabled or has already requested reload. */
#define WDOG_REQSTATUS_RR0_EnabledAndUnrequested (1UL) /*!< RR[0] register is enabled and has not jet requested. */

/* Register: WDOG_RREN */
/* Description: Reload request enable. */

/* Bit 7 : Enable or disable RR[7] register. */
#define WDOG_RREN_RR7_Pos (7U) /*!< Position of RR7 field. */
#define WDOG_RREN_RR7_Msk (0x1UL << WDOG_RREN_RR7_Pos) /*!< Bit mask of RR7 field. */
#define WDOG_RREN_RR7_Disabled (0UL) /*!< RR[7] register is disabled. */
#define WDOG_RREN_RR7_Enabled (1UL) /*!< RR[7] register is enabled. */

/* Bit 6 : Enable or disable RR[6] register. */
#define WDOG_RREN_RR6_Pos (6U) /*!< Position of RR6 field. */
#define WDOG_RREN_RR6_Msk (0x1UL << WDOG_RREN_RR6_Pos) /*!< Bit mask of RR6 field. */
#define WDOG_RREN_RR6_Disabled (0UL) /*!< RR[6] register is disabled. */
#define WDOG_RREN_RR6_Enabled (1UL) /*!< RR[6] register is enabled. */

/* Bit 5 : Enable or disable RR[5] register. */
#define WDOG_RREN_RR5_Pos (5U) /*!< Position of RR5 field. */
#define WDOG_RREN_RR5_Msk (0x1UL << WDOG_RREN_RR5_Pos) /*!< Bit mask of RR5 field. */
#define WDOG_RREN_RR5_Disabled (0UL) /*!< RR[5] register is disabled. */
#define WDOG_RREN_RR5_Enabled (1UL) /*!< RR[5] register is enabled. */

/* Bit 4 : Enable or disable RR[4] register. */
#define WDOG_RREN_RR4_Pos (4U) /*!< Position of RR4 field. */
#define WDOG_RREN_RR4_Msk (0x1UL << WDOG_RREN_RR4_Pos) /*!< Bit mask of RR4 field. */
#define WDOG_RREN_RR4_Disabled (0UL) /*!< RR[4] register is disabled. */
#define WDOG_RREN_RR4_Enabled (1UL) /*!< RR[4] register is enabled. */

/* Bit 3 : Enable or disable RR[3] register. */
#define WDOG_RREN_RR3_Pos (3U) /*!< Position of RR3 field. */
#define WDOG_RREN_RR3_Msk (0x1UL << WDOG_RREN_RR3_Pos) /*!< Bit mask of RR3 field. */
#define WDOG_RREN_RR3_Disabled (0UL) /*!< RR[3] register is disabled. */
#define WDOG_RREN_RR3_Enabled (1UL) /*!< RR[3] register is enabled. */

/* Bit 2 : Enable or disable RR[2] register. */
#define WDOG_RREN_RR2_Pos (2U) /*!< Position of RR2 field. */
#define WDOG_RREN_RR2_Msk (0x1UL << WDOG_RREN_RR2_Pos) /*!< Bit mask of RR2 field. */
#define WDOG_RREN_RR2_Disabled (0UL) /*!< RR[2] register is disabled. */
#define WDOG_RREN_RR2_Enabled (1UL) /*!< RR[2] register is enabled. */

/* Bit 1 : Enable or disable RR[1] register. */
#define WDOG_RREN_RR1_Pos (1U) /*!< Position of RR1 field. */
#define WDOG_RREN_RR1_Msk (0x1UL << WDOG_RREN_RR1_Pos) /*!< Bit mask of RR1 field. */
#define WDOG_RREN_RR1_Disabled (0UL) /*!< RR[1] register is disabled. */
#define WDOG_RREN_RR1_Enabled (1UL) /*!< RR[1] register is enabled. */

/* Bit 0 : Enable or disable RR[0] register. */
#define WDOG_RREN_RR0_Pos (0U) /*!< Position of RR0 field. */
#define WDOG_RREN_RR0_Msk (0x1UL << WDOG_RREN_RR0_Pos) /*!< Bit mask of RR0 field. */
#define WDOG_RREN_RR0_Disabled (0UL) /*!< RR[0] register is disabled. */
#define WDOG_RREN_RR0_Enabled (1UL) /*!< RR[0] register is enabled. */

/* Register: WDOG_CONFIG */
/* Description: Configuration register. */

/* Bit 3 : Configure the watchdog to pause or not while the CPU is halted by the debugger. */
#define WDOG_CONFIG_HALT_Pos (3U) /*!< Position of HALT field. */
#define WDOG_CONFIG_HALT_Msk (0x1UL << WDOG_CONFIG_HALT_Pos) /*!< Bit mask of HALT field. */
#define WDOG_CONFIG_HALT_Pause (0UL) /*!< Pause watchdog while the CPU is halted by the debugger. */
#define WDOG_CONFIG_HALT_Run (1UL) /*!< Do not pause watchdog while the CPU is halted by the debugger. */

/* Bit 0 : Configure the watchdog to pause or not while the CPU is sleeping. */
#define WDOG_CONFIG_SLEEP_Pos (0U) /*!< Position of SLEEP field. */
#define WDOG_CONFIG_SLEEP_Msk (0x1UL << WDOG_CONFIG_SLEEP_Pos) /*!< Bit mask of SLEEP field. */
#define WDOG_CONFIG_SLEEP_Pause (0UL) /*!< Pause watchdog while the CPU is asleep. */
#define WDOG_CONFIG_SLEEP_Run (1UL) /*!< Do not pause watchdog while the CPU is asleep. */

/* Register: WDOG_RR */
/* Description: Reload requests registers. */

/* Bits 31..0 : Reload register. */
#define WDOG_RR_RR_Pos (0U) /*!< Position of RR field. */
#define WDOG_RR_RR_Msk (0xFFFFFFFFUL << WDOG_RR_RR_Pos) /*!< Bit mask of RR field. */
#define WDOG_RR_RR_Reload (0x6E524635UL) /*!< Value to request a reload of the watchdog timer. */


#endif
